package acover;
import java.util.Vector;

/**
 * Statistik zur Berechnung der Mazahl der Methodenberdeckung.
 * @author Nicos Tegos
 */
public class ClassCoverageStatistic
{
	private Vector methodStatistics = new Vector(); // Statistiken zu allen Methoden der Klasse
	private String className;
	
	private int covered = 0; // Anzahl berdeckte Methoden
	private int uncovered = 0; // Anzahl nicht berdeckte Methoden
	
	/**
	 * Erstellt eine neue Statistik zur Methodenberdeckung.
	 * @param className Der vollqualifizierte Java-Name der Klasse.
	 */
	public ClassCoverageStatistic(String className) {
		this.className = className;
	}
	
	/**
	 * Fgt die Statistik zu einer einzelnen Methode der
	 * Klasse hinzu.
	 * @param stat Die Statistik einer Methode.
	 */
	public void addMethodCoverageStatistic(MethodCoverageStatistic stat) 
	{
		if( stat.covered() )
			++covered;
		else
			++uncovered;
		
		methodStatistics.add(stat);
	}

	/** Liefert den Klassennamen der analysierten Klasse. */
	public String getClassName() {
		return className;
	}
	
	/** Liefert die Statistiken zu allen Methoden der analysierten Klasse. 
	 * @return Vector(MethodCoverageStatistic) */
	public Vector getMethodStatistics() {
		return methodStatistics;
	}
	
	/**
	 * Gibt an, ob die Klasse vollstndig berdeckt ist. D.h. alle Methoden
	 * der Klasse mind. einmal ausgefhrt wurden. Auch ein Klasse, die keine
	 * ausfhrbaren Methoden implementiert gilt als berdeckt.
	 * @return <code>true</code>, wenn berdeckt.
	 */
	public boolean covered() {
		return uncovered == 0; // klassen ohne ausfhrbare methoden gelten als covered
	}
	
	/** Liefert die Anzahl der berdeckten/ausgefhrten Methoden. */
	public int getCovered() {
		return covered;
	}
	
	/** Liefert die Anzahl der nicht berdeckten/ausgefhrten Methoden. */
	public int getUncovered() {
		return uncovered;
	}
	
	/** Liefert die Anzahl aller analysierten Methoden. */
	public int getTotal() {
		return uncovered + covered;
	}
	
	/** Liefert den Anteil berdeckter/ausgefhrter Methoden. */
	public float getPercentageCovered() {
		if( getTotal() == 0 )
			return 1.0f;
		
		return (float)covered / getTotal();
	}
}
