package acover;

/**
 * Statistik, die die Anzahl der Aufrufe einer Methode enthlt.
 * @author Nicos Tegos
 */
public class MethodCoverageStatistic
{
	private String className; 
	private String[] modifiers; 
	private String returnType; 
	private String methodName;
	private String[] parameters;
	private String[] exceptions;
	private int runs;
	
	/**
	 * Erstellt eine neue Statistik fr eine Methode.
	 * @param className Name der Klasse, in der die Methode implementiert ist.
	 * @param modifiers Modifier der Methode z.B. <code>public</code>.
	 * @param returnType Vollqualifizierter Klassenname des Rckgabewertes.
	 * @param methodName Name der Methode.
	 * @param parameters Vollqualifizierte Klassennamen aller Parameter
	 * in der Reihenfolge ihrer Definition, wenn keine dann leeres Array.
	 * @param exceptions Vollqualifizierte Klassennamen aller geworfenen Ausnahmen, wenn keine
	 * dann leeres Array.
	 * @param runs
	 */
	public MethodCoverageStatistic(
			String className, 
			String[] modifiers, 
			String returnType, 
			String methodName,
			String[] parameters,
			String[] exceptions,
			int runs) 
	{
		this.className = className; 
		this.modifiers = modifiers; 
		this.returnType = returnType; 
		this.methodName = methodName;
		this.parameters = parameters;
		this.exceptions = exceptions;
		
		this.runs = runs;
	}

	/** Liefert die Anzahl der Aufrufe der Methode. */
	public int getRuns() {
		return runs;
	}
	
	/** 
	 * Gibt an, ob die Methode berdeckt ist. D.h. mind. einmal ausgefhrt wurde. 
	 * @return <code>true</code>, wenn berdeckt.
	 */
	public boolean covered() {
		return runs > 0;
	}

	/** 
	 * Liefert den vollqualifizierten Klassennamen der Klasse, 
	 * die die Methode implementiert. 
	 */
	public String getClassName() {
		return className;
	}

	/**
	 * Liefert vollqualifizierte Klassennamen aller geworfenen Ausnahmen, wenn keine
	 * dann leeres Array.
	 */
	public String[] getExceptions() {
		return exceptions;
	}

	/** Liefert den Namen der Methode */
	public String getMethodName() {
		return methodName;
	}

	/** Liefert Modifier der Methode z.B. <code>public</code>. */
	public String[] getModifiers() {
		return modifiers;
	}

	/** 
	 * Liefert die vollqualifizierten Klassennamen aller Parameter
	 * in der Reihenfolge ihrer Definition, wenn keine dann leeres Array.
	 */
	public String[] getParameters() {
		return parameters;
	}

	/** Liefert den vollqualifizierten Klassennamen des Rckgabewertes. */
	public String getReturnType() {
		return returnType;
	}
}
