package acover;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import javassist.Translator;

/**
 * Diese Klasse Instrumentiert das Testobjekt zur Laufzeit und
 * ermglicht das Inkrementieren eines Aufrufzhlers fr Methoden.
 * @author Nicos Tegos
 */
public class RuntimeCoverageTranslator implements Translator 
{
	public void start(ClassPool pool) throws NotFoundException,	CannotCompileException {
	}

	/**
	 * Realisiert die Instrumentierung. Fgt in jede Methode, die in der XML-Datenbasis
	 * enthalten ist eine zustzliche Zeile, die das Inkrementieren des Aufrufzhlers
	 * fr die Methode bewirkt.
	 * @see #addRun(String, String, String)
	 */
	public void onLoad(ClassPool pool, String className) throws NotFoundException, CannotCompileException 
	{
		try {
			if( !RuntimeCoverageGenerator.getInstance().contains(className) )
				return;
			
			CtClass clazz = pool.get(className);
			CtMethod[] methods = clazz.getDeclaredMethods();
			for(int i = 0; i < methods.length; ++i) 
			{
				String methodName = methods[i].getName();
				
				CtClass[] parameters = methods[i].getParameterTypes();
				String parameterNames[] = new String[parameters.length];
				for(int j = 0; j < parameters.length; ++j)
					parameterNames[j] = parameters[j].getName();
				
				if( RuntimeCoverageGenerator.getInstance().contains(className, methodName, parameterNames) )
				{
					String insertStr = getClass().getName() 
					+ ".addRun(\"" + className + "\", \"" 
					+ methodName + "\", \"";
					
					for (int j = 0; j < parameterNames.length; j++) {
						insertStr += parameterNames[j];
						if(j < parameterNames.length-1)
							insertStr += ",";
					}
					
					insertStr += "\");";
					
					methods[i].insertBefore(insertStr);
				}
			}
			
		} catch(Throwable t) {
			t.printStackTrace();
			/*ignore*/
		}		
	}

	/**
	 * Callback Funktion, die in jede Methode einer geladenen Klasse, die analysiert werden soll, 
	 * eingefgt wird. Somit wird sie immer aufgerufen, wenn die Methode ausgefhrt wird.
	 * @see RuntimeCoverageGenerator#addRun(String, String, String[])
	 */
	public static void addRun(String className, String methodName, String parameterNames) {
		if( parameterNames.length() == 0 )
			RuntimeCoverageGenerator.getInstance().addRun(className, methodName, new String[0]);
		else
			RuntimeCoverageGenerator.getInstance().addRun(className, methodName, parameterNames.split(","));
	}
	
	/*public String getJavaMethodString(CtClass clazz, CtMethod method) throws NotFoundException 
	{
		String ret = Modifier.toString( method.getModifiers() )
			+ " " + method.getReturnType().getName()
			+ " " + clazz.getName()
			+ "." + method.getName() 
			+ "(";
		
		CtClass[] parameters = method.getParameterTypes();
		for(int i = 0; i < parameters.length; ++i)
		{
			ret += parameters[i].getName();
			if(i < parameters.length-1)
				ret += ",";
		}
		
		ret += ")";
		
		CtClass[] exceptions = method.getExceptionTypes();
		if( exceptions.length > 0 )
		{	
			ret += " throws ";
			for(int i = 0; i < exceptions.length; ++i)
			{
				ret += exceptions[i].getName();
				if(i < exceptions.length-1)
					ret += ",";
			}
		}
		
		return ret;
	}*/
}
