package acover;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Hilfsklasse fr die Arbeit mit XML-Dateien.
 * @author Nicos Tegos
 */
public class XMLUtilities {

	/** Parsed ein XML-Dokument mit dem SAXParser. */
	public static Document parseStream(InputStream stream, boolean silent) throws Exception
	{
		// Eingabedatenquelle aus dem Datenstrom erzeugen
		InputSource source = new InputSource(stream);
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		// factory.setValidating(true); nur mit dtd
		DocumentBuilder xmlBuilder = factory.newDocumentBuilder();
		
		if (silent)
		{
			xmlBuilder.setErrorHandler(new ErrorHandler() {
				public void error(
						SAXParseException exception) throws SAXException {
					throw exception;
				}
				
				public void fatalError(
						SAXParseException exception) throws SAXException {
					throw exception;
				}
				
				public void warning(
						SAXParseException exception) throws SAXException {
					throw exception;
				}
			});
		}
		
		// Alles konfiguriert - wir knnen nun parsen.
		return xmlBuilder.parse(source);
	}

	/**
	 * Liest eine XML-Datei ein.
	 * @param path Vollstndiger Pfad der XML-Datei.
	 * @return Das zugehrige Dokument.
	 * @throws Exception Entweder FileNotFoundException, oder Exception beim parsen.
	 */
	public static Document readXMLDoc(String path) throws Exception 
	{
		File file = new File(path);
	    Document xmlDoc = parseStream(new FileInputStream(file), false);
	    
	    return xmlDoc;
	 }

	/** Schreibt ein XML-Dokument in eine Datei */
	public static void writeDocToXMLFile(Document doc, String xmlFilePath) throws IOException
	{
	    if( doc == null )
	    	return;
	    
	    try {
	    	File file = new File(xmlFilePath);
	    	StreamResult result = new StreamResult(file);
	
	        // Write the DOM document to the file
	        Transformer xformer = TransformerFactory.newInstance().newTransformer();
	        xformer.transform(new DOMSource(doc), result);
	    } catch (TransformerConfigurationException e) {
	    } catch (TransformerException e) {
	    }
	}

}
