/*
 * Created on 08.01.2006
 */
package atosj.action;

import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;

import atosj.Messages;
import atosj.editors.Changeable;
import atosj.exception.AtosJException;

/**
 * Speichert oder verwirft die angegebenen Ressourcen und zeigt dazu einen
 * Fortschrittsdialog.
 * @author Nicos Tegos
 */
public class SaveRunner implements IRunnableWithProgress
{
	private Vector toDiscard;
	private Vector toSave;
	
	/**
	 * @param toDiscard Vector(Changeable) alle Ressourcen die verworfen werden sollen.
	 * @param toSave Vector(Changeable) alle Ressourcen die gespeichert werden sollen.
	 */
	public SaveRunner(Vector toDiscard, Vector toSave)
	{
		this.toDiscard = toDiscard;
		this.toSave = toSave;
	}
	
	/**
	 * Speichert oder verwirft die angegebenen Ressourcen und zeigt dazu einen
	 * Fortschrittsdialog.
	 */
	public void run(IProgressMonitor monitor)	throws InvocationTargetException, InterruptedException 
	{
		monitor.beginTask( Messages.getString("SaveRunner.0"), toDiscard.size() + toSave.size() ); //$NON-NLS-1$
				
		for(int i = 0; i < toSave.size(); ++i)
		{
			Changeable resource = (Changeable) toSave.get(i); 
			try {
				monitor.subTask(Messages.getString("SaveRunner.1") + resource.getName()); //$NON-NLS-1$
				resource.save();
				monitor.worked(1);
			}
			catch(AtosJException ex) {
				// TODO MessageDialog.openError(null, "Ressource speichern", ex.getMessage() );
			}
		}
		
		for(int i = 0; i < toDiscard.size(); ++i)
		{
			Changeable resource = (Changeable) toDiscard.get(i); 
		
			try {
				monitor.subTask(Messages.getString("SaveRunner.2") + resource.getName()); //$NON-NLS-1$
				resource.discard();
				monitor.worked(1);
			}
			catch(AtosJException ex) {
				// TODO MessageDialog.openError(null, "Ressource speichern", ex.getMessage() );
			}
		}
		
		monitor.done();
	}
}