/*
 * Created on 07.12.2005
 */
package atosj.action.editor;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;

import atosj.Messages;
import atosj.action.EditorAction;
import atosj.app.AtosJApp;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.exception.AtosJException;
import atosj.listener.InputListener;
import atosj.listener.StateChangeListener;
import atosj.util.DefaultImageDescriptor;

/**
 * Verwirft die nderungen im aktuell geffneten Editor, 
 * d.h. ldt den letzten gespeicherten Zustand.
 * @author Nicos Tegos
 */
public class DiscardEditorAction extends EditorAction implements StateChangeListener, InputListener
{
	public static final String ID = "SAVE_EDITOR_ACTION"; //$NON-NLS-1$
	
	private Editor editor;
	
	public DiscardEditorAction() 
	{
		super(Messages.getString("DiscardEditorAction.1"), ID); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'W'); // lieber ein 'W' damit es nicht mit dem 'D'isable im editor ins gehege kommt
		setEnabled(false);
		setToolTipText(Messages.getString("DiscardEditorAction.2")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_DISCARD) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_DISCARD_DIS) );
		
		AtosJApp.getInstance().addInputListener(this);
	}
	
	public void editorActivated(Editor _editor) 
	{
		if(editor != null)
			editor.getInput().removeStateChangeListener(this);
		
		editor = _editor;
		
		if(editor == null)
		{
			setEnabled(false);
		}
		else
		{
			editor.getInput().addStateChangeListener(this);
			setEnabled( editor.getInput().isModified() );
		}
	}
	
	/**
	 * Verwirft die aktuellen nderungen des editierten Objektes.
	 * @see Changeable
	 */
	public void run() 
	{
		try
		{
			editor.getInput().discard();
			editor.getUndoManager().clear();
		}
		catch(AtosJException e)
		{
			MessageDialog.openError(null, Messages.getString("DiscardEditorAction.3"), e.getMessage() ); //$NON-NLS-1$
		}
	}

	/* (non-Javadoc)
	 * @see atosj.listener.StateChangeListener#stateChanged(atosj.editors.Changeable, java.lang.String, java.lang.Object, java.lang.Object)
	 */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) {
		if(state == StateChangeListener.STATE_MODIFIED)
			setEnabled( ((Boolean)newValue).booleanValue() );
	}

	/* (non-Javadoc)
	 * @see atosj.listener.InputListener#inputChanged(java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Object oldObj, Object newObj) {
		if(newObj == null)
			setEnabled(false);
	}
}
