/*
 * Created on 14.04.2006
 */
package atosj.action.editor;

import org.eclipse.swt.SWT;

import atosj.Messages;
import atosj.action.EditorAction;
import atosj.app.AtosJApp;
import atosj.editors.Editor;
import atosj.editors.UndoManager;
import atosj.listener.UndoManagerListener;
import atosj.util.DefaultImageDescriptor;

/**
 * Nimmt die nderungen, die durch Undo gemacht wurden wieder zurck.
 * @author Nicos Tegos
 * @see atosj.action.editor.UndoEditorAction
 */
public class RedoEditorAction extends EditorAction implements UndoManagerListener 
{
	private static final String ID = "REDO_ACTION"; //$NON-NLS-1$
	private Editor currentEditor;
	
	public RedoEditorAction() {
		super(Messages.getString("RedoEditorAction.1"), ID); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'Y');
		setToolTipText(Messages.getString("RedoEditorAction.2")); //$NON-NLS-1$
		setEnabled(false);
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REDO) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REDO_DIS) );
	}

	/* (non-Javadoc)
	 * @see atosj.listener.EditorListener#editorActivated(atosj.editors.Editor)
	 */
	public void editorActivated(Editor editMe) 
	{
		if(currentEditor != null)
			currentEditor.removeUndoManagerListener(this);
		
		currentEditor = editMe;
		if(currentEditor == null)
		{
			setEnabled(false);
			setToolTipText(Messages.getString("RedoEditorAction.3")); //$NON-NLS-1$
		}
		else
		{
			currentEditor.addUndoManagerListener(this);
			setEnabled(currentEditor.getUndoManager() != null
					&& currentEditor.getUndoManager().hasRedoElements() );
			setToolTipText( currentEditor.getUndoManager().getCurrentRedName() );
		}
	}

	/* (non-Javadoc)
	 * @see atosj.listener.UndoManagerListener#undoManagerChanged(atosj.editors.UndoManager)
	 */
	public void undoManagerChanged(UndoManager mngr) {
		setEnabled( mngr.hasRedoElements() );
		setToolTipText( mngr.getCurrentRedName() );
	}
	
	/**
	 * Stellt die zuletzt rckgngiggemachte nderung des Editors wieder her.
	 */
	public void run() {
		currentEditor.getUndoManager().redo();
	}
}