package atosj.action.editor;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;

import atosj.Messages;
import atosj.action.EditorAction;
import atosj.app.AtosJApp;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.exception.AtosJException;
import atosj.listener.InputListener;
import atosj.listener.StateChangeListener;
import atosj.util.DefaultImageDescriptor;

/**
 * Speichert den aktuell geffneten Editor.
 * @author Nicos Tegos
 */
public class SaveEditorAction extends EditorAction implements StateChangeListener, InputListener
{
	public static final String ID = "SAVE_EDITOR_ACTION"; //$NON-NLS-1$
	
	private Editor editor;
	
	public SaveEditorAction() 
	{
		super(Messages.getString("SaveEditorAction.1"), ID); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'S');
		setToolTipText(Messages.getString("SaveEditorAction.2")); //$NON-NLS-1$
		setEnabled(false);
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_SAVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_SAVE_DIS) );
		
		AtosJApp.getInstance().addInputListener(this);
	}
	
	public void editorActivated(Editor _editor) 
	{
		if(editor != null)
			editor.getInput().removeStateChangeListener(this);
		
		editor = _editor;
		
		if(editor == null)
		{
			setEnabled(false);
		}
		else
		{
			editor.getInput().addStateChangeListener(this);
			setEnabled( editor.getInput().isModified() );
		}
	}
	
	/**
	 * Speichert den aktuell geffneten Editor.
	 */
	public void run() 
	{
		try
		{
			editor.getInput().save();
		}
		catch(AtosJException e)
		{
			MessageDialog.openError(null, Messages.getString("SaveEditorAction.3"), e.getMessage() ); //$NON-NLS-1$
		}
	}

	/* (non-Javadoc)
	 * @see atosj.listener.StateChangeListener#stateChanged(atosj.editors.Changeable, java.lang.String, java.lang.Object, java.lang.Object)
	 */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) {
		if(state == StateChangeListener.STATE_MODIFIED)
			setEnabled( ((Boolean)newValue).booleanValue() );
	}

	/* (non-Javadoc)
	 * @see atosj.listener.InputListener#inputChanged(java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Object oldObj, Object newObj) {
		if(newObj == null)
			setEnabled(false);
	}
}
