/*
 * Created on 14.04.2006
 */
package atosj.action.editor;

import org.eclipse.swt.SWT;

import atosj.Messages;
import atosj.action.EditorAction;
import atosj.app.AtosJApp;
import atosj.editors.Editor;
import atosj.editors.UndoManager;
import atosj.listener.UndoManagerListener;
import atosj.util.DefaultImageDescriptor;

/**
 * Macht die jeweils letzte nderung im aktuellen Editor rckgngig.
 * @author Nicos Tegos
 */
public class UndoEditorAction extends EditorAction implements UndoManagerListener 
{
	private static final String ID = "UNDO_ACTION"; //$NON-NLS-1$
	private Editor currentEditor;
	
	public UndoEditorAction() {
		super(Messages.getString("UndoEditorAction.1"), ID); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'Z');
		setToolTipText(Messages.getString("UndoEditorAction.2")); //$NON-NLS-1$
		setEnabled(false);
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_UNDO) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_UNDO_DIS) );
	}

	/* (non-Javadoc)
	 * @see atosj.listener.EditorListener#editorActivated(atosj.editors.Editor)
	 */
	public void editorActivated(Editor editMe) 
	{
		if(currentEditor != null)
			currentEditor.removeUndoManagerListener(this);
		
		currentEditor = editMe;
		if(currentEditor == null)
		{
			setEnabled(false);
			setToolTipText(Messages.getString("UndoEditorAction.3")); //$NON-NLS-1$
		}
		else
		{
			currentEditor.addUndoManagerListener(this);
			setEnabled(currentEditor.getUndoManager() != null
					&& currentEditor.getUndoManager().hasUndoElements() );
			setToolTipText( currentEditor.getUndoManager().getCurrentUndoName() );
		}
	}

	/* (non-Javadoc)
	 * @see atosj.listener.UndoManagerListener#undoManagerChanged(atosj.editors.UndoManager)
	 */
	public void undoManagerChanged(UndoManager mngr) {
		setEnabled( mngr.hasUndoElements() );
		setToolTipText( mngr.getCurrentUndoName() );
	}
	
	/**
	 * Macht die letzte nderung im aktuellen Editor rckgngig.
	 * @see UndoManager 
	 */
	public void run() {
		currentEditor.getUndoManager().undo();
	}
}
