package atosj.action.pack;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.editors.pack.PackageTreeDir;
import atosj.exception.AtosJException;
import atosj.model.pack.HTSPackage;

/**
 * Erstellt ein neues Testpaket.
 * @author Nicos Tegos
 */
public class NewPackageAction extends TreeEntryAction {

	public static final String ID = "NEWPACKAGE_ACTION"; //$NON-NLS-1$
		
	public NewPackageAction(PackageTreeDir dir) {
		super(Messages.getString("NewPackageAction.1"), dir); //$NON-NLS-1$
		setId(ID);
	}
	
	private PackageTreeDir getPackageTreeDir() {
		return (PackageTreeDir) getTreeEntry();
	}
	
	/**
	 * Erstellt ein neues Testpaket. Zeigt dazu einen Dialog zur
	 * Eingabe des Namens fr das neue Paket.
	 * @see atosj.app.Project#createNewPackage(String)
	 */
	public void run() 
	{
		// paketnamen erfragen
		InputDialog nameDlg = new InputDialog(
				Display.getCurrent().getActiveShell(), 
				Messages.getString("NewPackageAction.2"),  //$NON-NLS-1$
				Messages.getString("NewPackageAction.3"), //$NON-NLS-1$
				"", //$NON-NLS-1$
			new IInputValidator() {
			public String isValid(String text) 
			{
				String trimmed = text.trim();
				if( trimmed.equals("") ) //$NON-NLS-1$
					return Messages.getString("NewPackageAction.6"); //$NON-NLS-1$
				
				if( AtosJApp.getInstance().getInput().containsPackage(trimmed) )
					return Messages.getString("NewPackageAction.7") + trimmed + Messages.getString("NewPackageAction.8"); //$NON-NLS-1$ //$NON-NLS-2$
				
				return null;
			}
		});
		
		// paket erstellen
		if( nameDlg.open() == InputDialog.OK )
		{
			// eventuelles .pak am ende entfernen
			String newPackageName = nameDlg.getValue();
			if (newPackageName.length() > 4 &&
					newPackageName.substring(newPackageName.length()-4,newPackageName.length()).equalsIgnoreCase(".pak")) //$NON-NLS-1$
				newPackageName = newPackageName.substring(0, newPackageName.length()-4);
			
			try 
			{
				HTSPackage newPackage = AtosJApp.getInstance().getInput().createNewPackage( newPackageName );
				getPackageTreeDir().addPackage(newPackage);
			} catch (AtosJException e) {
				MessageDialog.openError(null, Messages.getString("NewPackageAction.10"), Messages.getString("NewPackageAction.11") //$NON-NLS-1$ //$NON-NLS-2$
						+ e.getMessage() );
			}
		}
	}
}