package atosj.action.pack;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.editors.pack.PackageTreeEntry;
import atosj.model.pack.HTSPackage;

/**
 * Benennt ein Testpaket um.
 * @author Nicos Tegos
 */
public class RenamePackageAction extends TreeEntryAction {

	public RenamePackageAction(PackageTreeEntry entry) {
		super(Messages.getString("RenamePackageAction.0"), entry); //$NON-NLS-1$
	}
	
	private HTSPackage getPackage() {
		return (HTSPackage) getTreeEntry().getWrappedObject();
	}

	/**
	 * Benennt ein Testpaket um.
	 * Zeigt dazu vorher einen Dialog zur Namenseingabe.
	 * @see HTSPackage#rename(String)
	 */
	public void run() 
	{
		//	 paketnamen erfragen
		InputDialog nameDlg = new InputDialog(
				Display.getCurrent().getActiveShell(), 
				Messages.getString("RenamePackageAction.1"),  //$NON-NLS-1$
				Messages.getString("RenamePackageAction.2"), //$NON-NLS-1$
				getPackage().getName(),
				new IInputValidator() {
					public String isValid(String text) 
					{
						String trimmed = text.trim();
						if( trimmed.equals("") ) //$NON-NLS-1$
							return Messages.getString("RenamePackageAction.4"); //$NON-NLS-1$
						
						if( AtosJApp.getInstance().getInput().containsPackage(trimmed) )
							return Messages.getString("RenamePackageAction.5") + trimmed + Messages.getString("RenamePackageAction.6"); //$NON-NLS-1$ //$NON-NLS-2$
						
						return null;
					}
				});
		
		// paket umbenennen
		if( nameDlg.open() == InputDialog.OK )
		{
			// eventuelles .pak am ende entfernen
			String newPackageName = nameDlg.getValue();
			if (newPackageName.length() > 4 &&
					newPackageName.substring(newPackageName.length()-4,newPackageName.length()).equalsIgnoreCase(".pak")) //$NON-NLS-1$
				newPackageName = newPackageName.substring(0, newPackageName.length()-4);
			
			if( getPackage().rename(newPackageName) == false )
			{
				MessageDialog.openError(null, Messages.getString("RenamePackageAction.8"), Messages.getString("RenamePackageAction.9")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
	
}
