package atosj.action.pack;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.dialog.RunPackageDlg;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.pack.PackageTreeEntry;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;

/**
 * Zeigt den Dialog zur Ausfhrung eines Testpakets.
 * @author Nicos Tegos
 */
public class RunPackageAction extends TreeEntryAction 
{
	public static final String id = "RUN_ACTION"; //$NON-NLS-1$
	
	public RunPackageAction(PackageTreeEntry entry) {
		super(Messages.getString("RunPackageAction.1"), entry); //$NON-NLS-1$
		setId(id);
	}
	
	/**
	 * Zeigt den Dialog zur Ausfhrung eines Testpakets. 
	 * Zeigt zuvor Dialoge zur Speicherung evtl. genderter Ressourcen.
	 * @see RunPackageDlg
	 * @see SaveResourcesDlg
	 */
	public void run() 
	{
		HTSPackage pack = (HTSPackage)getTreeEntry().getWrappedObject();
		
		if(pack == null)
			return;
		
		// evtl. genderte enthaltene scripte, urfdatei und sich selbst speichern
		Vector modified = new Vector();
		
		if( pack.isModified() )
			modified.add(pack);
		
		if( AtosJApp.getInstance().getInput().getURFFile().isModified() )
			modified.add( AtosJApp.getInstance().getInput().getURFFile() );
		
		Vector scripts = pack.getScripts();
		
		for(int i = 0; i < scripts.size(); ++i)
		{
			if( ((HTSScript)scripts.get(i)).isModified() )
				modified.add( scripts.get(i) );
		}
		
		if( !modified.isEmpty() )
		{
			SaveResourcesDlg saveDlg = new SaveResourcesDlg( Display.getDefault().getActiveShell(), modified );
			int result = saveDlg.open();
			if(result == SaveResourcesDlg.ID_CANCEL)
			{
				return; // abbrechen
			}
			else
			{
				try{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( false, false, new SaveRunner(saveDlg.getElementsToDiscard(), saveDlg.getElementsToSave()) );
				}
				catch(Throwable e) {
					// hopefully this will never happen
				}
			}
		}
		
		// auf leer prfen
		if( pack.getNumberOfScripts() == 0 )
		{
			MessageDialog.openError(null, Messages.getString("RunPackageAction.2"), //$NON-NLS-1$
				Messages.getString("RunPackageAction.3")); //$NON-NLS-1$
			
			// abbrechen
			return;
		}
		
		// Integrittsprfung
		if(pack.getValidState() == HTSPackage.INVALID)
		{
			MessageDialog.openError(null, Messages.getString("RunPackageAction.4"), //$NON-NLS-1$
					Messages.getString("RunPackageAction.5") //$NON-NLS-1$
					+ pack.getMessage() );
			return;
		}

		if(pack.getValidState() == HTSPackage.VALID_WITH_WARNING)
		{
			// auskommentiert weil in der Praxis zu aufwendig dauernd den Dialog wegclicken zu mssen
			// MessageDialog.openWarning( null, "Testpaket ausfhren", pack.getMessage() );
			// trotzdem weiter
		}
		
		new RunPackageDlg(Display.getDefault().getActiveShell(), pack).open();
	}
}
