/*
 * Created on 05.12.2005
 */
package atosj.action.project;

import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.app.AtosJApp;
import atosj.app.Project;
import atosj.dialog.SaveResourcesDlg;
import atosj.exception.AtosJException;
import atosj.listener.InputListener;

/**
 * Schliet das aktuell geffnete Projekt.
 * @author Nicos Tegos
 */
public class CloseProjectAction extends Action implements InputListener{
	public static final String ID = "CLOSE_PROJECT_ACTION"; //$NON-NLS-1$
	
	public CloseProjectAction() {
		super(Messages.getString("CloseProjectAction.1")); //$NON-NLS-1$
		setId(ID);
		
		setEnabled(AtosJApp.getInstance().getInput() != null);
		AtosJApp.getInstance().addInputListener(this);
	}
	
	
	
	/**
	 * Schliet das aktuelle Projekt und dessen Editoren. Enthlt das Projekt ungespeicherte
	 * Ressourcen, wird der Nutzer zuvor zur Speicherung aufgefordert.
	 * @return <code>true</code> wenn das projekt geschlossen wurde, <code>false</code> wenn
	 * das aktuelle Projekt weiter geffnet bleiben soll.
	 * @see SaveResourcesDlg
	 */
	public static boolean closeProject() 
	{
		Project project = AtosJApp.getInstance().getInput();
		if(project == null)
			return true;
		
		// vor dem Schlieen alle genderten Resourcen speichern
		Vector modifiedResources = project.getModifiedResources();
		if( !modifiedResources.isEmpty() )
		{
			SaveResourcesDlg dlg = new SaveResourcesDlg(Display.getDefault().getActiveShell(), modifiedResources);
			if( dlg.open() == SaveResourcesDlg.ID_SAVE )
			{
				Vector toSave = dlg.getElementsToSave();
				Vector toDiscard = dlg.getElementsToDiscard();
				
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(toDiscard, toSave));
				} catch(Throwable t) {
					
				}
			}
			else // nutzer abbruch
				return false;
		}
		
		// die geffneten Editoren merken
		project.setEditedResources( AtosJApp.getInstance().getMainWindow().getEditedObjects() );
		project.setActiveResource( AtosJApp.getInstance().getMainWindow().getActiveObject() );
		
		try {
			project.save();
		} catch (AtosJException e) { /* ignorieren*/ }
		
		AtosJApp.getInstance().setInput(null);
		return true;
	}
	
	/**
	 * Ruft <code>closeProjekt</code>.
	 * @see #closeProject()
	 */
	public void run() {
		closeProject();
	}

	/* (non-Javadoc)
	 * @see atosj.listener.InputListener#inputChanged(java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Object oldObj, Object newObj) {
		setEnabled(newObj != null);
	}
}
