/*
 * Created on 17.11.2005
 */
package atosj.action.project;

import java.io.File;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.Project;
import atosj.exception.AtosJException;

/**
 * ffnet ein vorhandenes Projekt.
 * @author Nicos Tegos
 */
public class OpenProjectAction extends Action 
{
	public static final String id = "OPEN_ACTION"; //$NON-NLS-1$
	
	public OpenProjectAction() {
		super(Messages.getString("OpenProjectAction.1")); //$NON-NLS-1$
		setId(id);
	}
	
	/**
	 * Erstellt eine neue Aktion.
	 * @param title Der Titel der Aktion wie er in der GUI angezeigt wird, 
	 * beispielsweise der Name der Projektdatei.
	 */
	public OpenProjectAction(String title) {
		super(title);
	}
	
	/**
	 * Schliet zuvor das aktuelle Projekt und ffnet danach das angegebene
	 * Projekt. Kann das angegebene Projekt nicht geladen werden, wird der
	 * Nutzer gefragt, ob die backup Datei des Projektes geladen werden soll.
	 * @param path Der vollstndige Pfad der Projektdatei.
	 */
	protected void openProject(String path) 
	{
		// aktuelles Projekt schliessen
		if( !CloseProjectAction.closeProject() )
			return;
		
		Project p = null;
		try 
		{
			p = new Project(path);
			AtosJApp.getInstance().setInput(p);
		} 
		catch (AtosJException e) 
		{
			if( MessageDialog.openQuestion(null, Messages.getString("OpenProjectAction.2"),  //$NON-NLS-1$
					Messages.getString("OpenProjectAction.3") //$NON-NLS-1$
					+ e.getMessage()
					+ Messages.getString("OpenProjectAction.4"))  //$NON-NLS-1$
					== true 
				)
			{
				// backup Datei laden
				try 
				{
					String backupDir = new File(path).getParent();
					String backupName = new File(path).getName();
					backupName = backupName.substring(0, backupName.length()-4) + ".bak"; //.apf endung durch .bak ersetzen //$NON-NLS-1$
					String backupPath = backupDir + File.separator + backupName;
					
					// kaputte Datei sichern
					String corruptPath = path + ".corrupt"; //$NON-NLS-1$
					boolean corruptFileExists = Project.copyFile(path, corruptPath);
					
					if( Project.copyFile(backupPath, path) )
					{
						p = new Project(path);
						AtosJApp.getInstance().setInput(p);
					}
					else
					{
						throw new AtosJException(Messages.getString("OpenProjectAction.7")  //$NON-NLS-1$
								+ backupPath
								+ Messages.getString("OpenProjectAction.8")); //$NON-NLS-1$
					}
					
					if(corruptFileExists)
					{
						MessageDialog.openInformation(null, Messages.getString("OpenProjectAction.9"), Messages.getString("OpenProjectAction.10")  //$NON-NLS-1$ //$NON-NLS-2$
								+ Messages.getString("OpenProjectAction.11")  //$NON-NLS-1$
								+ corruptPath );
					}
					else
					{
						MessageDialog.openInformation(null, Messages.getString("OpenProjectAction.12"), Messages.getString("OpenProjectAction.13")); //$NON-NLS-1$ //$NON-NLS-2$
					}
				} 
				catch (AtosJException e1) 
				{
					MessageDialog.openError(null, Messages.getString("OpenProjectAction.14"), Messages.getString("OpenProjectAction.15") //$NON-NLS-1$ //$NON-NLS-2$
							+ e.getMessage() );
				}
			}
		}
	}
	
	/**
	 * ffnet ein vorhandenes Projekt. Dazu wird ein Dialog zur Auswahl der Projektdatei
	 * gezeigt. Schliet zuvor das aktuelle Projekt.
	 * @see #openProject(String)
	 */
	public void run() 
	{
		// aktuelles Projekt schliessen
		if( !CloseProjectAction.closeProject() )
			return;
			
		FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN);
		dlg.setText(Messages.getString("OpenProjectAction.16")); //$NON-NLS-1$
		dlg.setFilterExtensions( new String[] {"*.apf"} ); //$NON-NLS-1$
		dlg.setFilterNames( new String[] {Messages.getString("OpenProjectAction.0")} ); //$NON-NLS-1$
		
		String path = dlg.open();
		if(path == null) // Dialog abgebrochen
			return;
		
		openProject(path);
	}
}
