/*
 * Created on 28.11.200
 */
package atosj.action.script;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.dialog.DeleteScriptDlg;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.script.ScriptTreeEntry;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;

/**
 * Lscht eine Testsequenz.
 * @author Nicos Tegos
 */
public class DeleteScriptAction extends TreeEntryAction 
{
	public static final String id = "DELTE_SCRIPT_ACTION"; //$NON-NLS-1$
	
	public DeleteScriptAction(ScriptTreeEntry entry) {
		super(Messages.getString("DeleteScriptAction.1"), entry); //$NON-NLS-1$
		setId(id);
	}
	
	protected HTSScript getScript() {
		return (HTSScript) getTreeEntry().getWrappedObject();
	}
	
	/**
	 * Lscht eine Testsequenz aus dem Projekt. Zuvor wird ein Dialog zur
	 * Speicherung aller genderten Pakete gezeigt, danach ein Dialog zur
	 * Besttigung des Lschvorgangs.
	 * @see atosj.app.Project#deleteScript(HTSScript)
	 * @see SaveResourcesDlg
	 */
	public void run() 
	{
		// vor dem Lschen alle genderten Pakete speichern
		Vector modifiedPaks = getModifiedPackages();
		if( !modifiedPaks.isEmpty() )
		{
			SaveResourcesDlg dlg = new SaveResourcesDlg(Display.getDefault().getActiveShell(), modifiedPaks);
			if( dlg.open() == SaveResourcesDlg.ID_SAVE )
			{
				Vector toSave = dlg.getElementsToSave();
				Vector toDiscard = dlg.getElementsToDiscard();
				
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(toDiscard, toSave));
				} catch(Throwable t) {
					
				}
			}
			else // nutzer abbruch
				return;
		}
		
		// Lschen durch Nutzer besttigen lassen
		if( new DeleteScriptDlg( Display.getCurrent().getActiveShell(), AtosJApp.getInstance().getInput(), getScript() ).open() 
				== DeleteScriptDlg.ID_NO )
		{
			// lschen abgelehnt
			return;
		}
		
		// script im projekt lschen
		if( AtosJApp.getInstance().getInput().deleteScript(getScript()) )
		{
			// Das Lschen kann zur nderung von Paketen fhren, daher alle genderten Pakete speichern
			modifiedPaks = getModifiedPackages();
			if( !modifiedPaks.isEmpty() )
			{
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(new Vector(0), modifiedPaks));
				} catch(Throwable t) {
					
				}
			}
			
			// script im baum lschen
			getTreeEntry().getParent().removeChild( getTreeEntry() );
		}
		else
		{
			MessageDialog.openInformation(null, Messages.getString("DeleteScriptAction.2"), Messages.getString("DeleteScriptAction.3") + getScript().getName()  //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("DeleteScriptAction.4")); //$NON-NLS-1$
		}
	}
	
	/**
	 * @return Vector(HTSPackage), alle genderten Pakete des aktuellen Projektes.
	 */
	private Vector getModifiedPackages() 
	{
		Vector modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
		Iterator it = modifiedResources.iterator();
		while( it.hasNext() )
		{
			if( !(it.next() instanceof HTSPackage) )
				it.remove();
		}
		
		return modifiedResources;
	}
}
