package atosj.action.script;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.editors.script.ScriptTreeDir;
import atosj.exception.AtosJException;
import atosj.model.script.HTSScript;

/**
 * Erstellt eine neue Testsequenz.
 * @author Nicos Tegos
 */
public class NewScriptAction extends TreeEntryAction {

	public static final String ID = "NEWSCRIPT_ACTION"; //$NON-NLS-1$
		
	public NewScriptAction(ScriptTreeDir dir) {
		super(Messages.getString("NewScriptAction.1"), dir); //$NON-NLS-1$
		setId(ID);
	}
	
	private ScriptTreeDir getScriptTreeDir() {
		return (ScriptTreeDir) getTreeEntry();
	}
	
	/**
	 * Erstellt eine neue Testsequenz. Zeigt dazu einen Dialog zur
	 * Eingabe des Namens fr die neue Testsequenz.
	 * @see atosj.app.Project#createNewScript(String)
	 */
	public void run() 
	{
		// scriptnamen erfragen
		InputDialog nameDlg = new InputDialog(
				Display.getCurrent().getActiveShell(), 
				Messages.getString("NewScriptAction.2"),  //$NON-NLS-1$
				Messages.getString("NewScriptAction.3"), //$NON-NLS-1$
				"", //$NON-NLS-1$
			new IInputValidator() {
			public String isValid(String text) 
			{
				String trimmed = text.trim();
				if( trimmed.equals("") ) //$NON-NLS-1$
					return Messages.getString("NewScriptAction.6"); //$NON-NLS-1$
				
				if( AtosJApp.getInstance().getInput().containsScript(trimmed) )
					return Messages.getString("NewScriptAction.7") + trimmed + Messages.getString("NewScriptAction.8"); //$NON-NLS-1$ //$NON-NLS-2$
				
				return null;
			}
		});
		
		// script erstellen
		if( nameDlg.open() == InputDialog.OK )
		{
			String newScriptName = nameDlg.getValue();
			if (newScriptName.length() > 4 &&
					newScriptName.substring(newScriptName.length()-4,newScriptName.length()).equalsIgnoreCase(".hts")) //$NON-NLS-1$
				newScriptName = newScriptName.substring(0, newScriptName.length()-4);
			
			try 
			{
				HTSScript newScript = AtosJApp.getInstance().getInput().createNewScript( newScriptName );
				getScriptTreeDir().addScript(newScript);
			} catch (AtosJException e) {
				MessageDialog.openError(null, Messages.getString("NewScriptAction.10"), Messages.getString("NewScriptAction.11") //$NON-NLS-1$ //$NON-NLS-2$
						+ e.getMessage() );
			}
		}
	}
}
