package atosj.action.script;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.script.ScriptTreeEntry;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;

/**
 * Benennt eine Testsequenz um.
 * @author Nicos Tegos
 */
public class RenameScriptAction extends TreeEntryAction {

	public RenameScriptAction(ScriptTreeEntry entry) {
		super(Messages.getString("RenameScriptAction.0"), entry); //$NON-NLS-1$
	}

	private HTSScript getScript() {
		return (HTSScript) ((ScriptTreeEntry) getTreeEntry()).getWrappedObject();
	}
	
	/**
	 * Benennt eine Testsequenz um.
	 * Zeigt dazu vorher einen Dialog zur Namenseingabe.
	 * @see HTSScript#rename(String)
	 */
	public void run() 
	{
		// vor dem Umbenennen alle genderten Resourcen speichern
		Vector modifiedPaks = getModifiedPackages();
		if( !modifiedPaks.isEmpty() )
		{
			SaveResourcesDlg dlg = new SaveResourcesDlg(Display.getDefault().getActiveShell(), modifiedPaks);
			if( dlg.open() == SaveResourcesDlg.ID_SAVE )
			{
				Vector toSave = dlg.getElementsToSave();
				Vector toDiscard = dlg.getElementsToDiscard();
				
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(toDiscard, toSave));
				} catch(Throwable t) {
					
				}
			}
			else // nutzer abbruch
				return;
		}
		
		// scriptnamen erfragen
		InputDialog nameDlg = new InputDialog(
				Display.getCurrent().getActiveShell(), 
				Messages.getString("RenameScriptAction.1"),  //$NON-NLS-1$
				Messages.getString("RenameScriptAction.2"), //$NON-NLS-1$
				getScript().getName(),
			new IInputValidator() {
			public String isValid(String text) 
			{
				String trimmed = text.trim();
				if( trimmed.equals("") ) //$NON-NLS-1$
					return Messages.getString("RenameScriptAction.4"); //$NON-NLS-1$
				
				if( AtosJApp.getInstance().getInput().containsScript(trimmed) )
					return Messages.getString("RenameScriptAction.5") + trimmed + Messages.getString("RenameScriptAction.6"); //$NON-NLS-1$ //$NON-NLS-2$
				
				return null;
			}
		});
		
		// script umbenennen
		if( nameDlg.open() == InputDialog.OK )
		{
			String newScriptName = nameDlg.getValue();
			if (newScriptName.length() > 4 &&
					newScriptName.substring(newScriptName.length()-4,newScriptName.length()).equalsIgnoreCase(".hts")) //$NON-NLS-1$
				newScriptName = newScriptName.substring(0, newScriptName.length()-4);
			
			if(	getScript().rename(newScriptName) == false )
			{
				MessageDialog.openError(null, Messages.getString("RenameScriptAction.8"), Messages.getString("RenameScriptAction.9")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		
		// alle genderten pakete speichern
		modifiedPaks = getModifiedPackages();
		if( !modifiedPaks.isEmpty() )
		{
			try
			{
				new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
							false, 
							false,
							new SaveRunner(new Vector(0), modifiedPaks));
			} catch(Throwable t) {
				
			}
		}
	}
	
	private Vector getModifiedPackages() 
	{
		Vector modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
		Iterator it = modifiedResources.iterator();
		while( it.hasNext() )
		{
			if( !(it.next() instanceof HTSPackage) )
				it.remove();
		}
		
		return modifiedResources;
	}
}
