/*
 * Created on 19.11.2005
 */
package atosj.action.script;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.action.TreeEntryAction;
import atosj.app.AtosJApp;
import atosj.dialog.RunScriptDlg;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.script.ScriptTreeEntry;
import atosj.model.script.HTSScript;

/**
 * Zeigt den Dialog zur Ausfhrung einer Testsequenz.
 * @author Nicos Tegos
 */
public class RunScriptAction extends TreeEntryAction 
{
	public static final String id = "RUN_ACTION"; //$NON-NLS-1$
	
	private HTSScript script;
	
	public RunScriptAction(ScriptTreeEntry entry) {
		super(Messages.getString("RunScriptAction.1"), entry); //$NON-NLS-1$
		this.script = (HTSScript)entry.getWrappedObject();
		setId(id);
	}
	
	public HTSScript getScript() {
		return (HTSScript) getTreeEntry().getWrappedObject();
	}
	
	/**
	 * Zeigt den Dialog zur Ausfhrung einer Testsequenz. 
	 * Zeigt zuvor Dialoge zur Speicherung evtl. genderter Ressourcen.
	 * @see RunScriptDlg
	 * @see SaveResourcesDlg
	 */
	public void run() 
	{
		if(script == null)
			return;
		
		// evtl. nderungen vorher speichern bzw. verwerfen bzw. abbrechen
		Vector modified = new Vector(2);
		if( script.isModified() )
			modified.add(script);
		if( AtosJApp.getInstance().getInput().getURFFile().isModified() )
			modified.add( AtosJApp.getInstance().getInput().getURFFile() );
		
		if( !modified.isEmpty() )
		{
			SaveResourcesDlg saveDlg = new SaveResourcesDlg( Display.getDefault().getActiveShell(), modified );
			int result = saveDlg.open();
			if(result == SaveResourcesDlg.ID_CANCEL)
			{
				return; // abbrechen
			}
			else
			{
				try{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( false, false, new SaveRunner(saveDlg.getElementsToDiscard(), saveDlg.getElementsToSave()) );
				}
				catch(Throwable e) {
					// hopefully this will never happen
				}
			}
		}
		
		// auf leer prfen
		if( script.getNumOfCommands() == 0 )
		{
			MessageDialog.openError(null, Messages.getString("RunScriptAction.2"), //$NON-NLS-1$
				Messages.getString("RunScriptAction.3")); //$NON-NLS-1$
			// abbrechen
			return;
		}
		
		if(script.getValidState() == HTSScript.INVALID) 
		{
			MessageDialog.openError(null, Messages.getString("RunScriptAction.4"), //$NON-NLS-1$
					Messages.getString("RunScriptAction.5") //$NON-NLS-1$
					+ script.getMessage() );
			// abbrechen
			return;
		}
		
		if(script.getValidState() == HTSScript.VALID_WITH_WARNING)
		{
			MessageDialog.openWarning( null, Messages.getString("RunScriptAction.6"), script.getMessage() ); //$NON-NLS-1$
			// trotzdem weiter
		}
		
		new RunScriptDlg(Display.getCurrent().getActiveShell(), script).open();
	}
}
