package atosj.app;

import java.io.File;
import java.io.InputStream;
import java.util.Vector;

import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

import atosj.gui.AtosJMainWindow;
import atosj.listener.InputListener;
import atosj.Messages;

/**
 * Singleton Klasse der ATOSj Applikation.
 * Bietet Zugriff auf das aktuelle Projekt, verwaltet Bilder usw.
 * @author Nicos Tegos
 */
public class AtosJApp 
{
	public static final String APP_NAME = "ATOSj"; // Der name der Applikation so wie er als Fenstertitel oder in anderen ausgaben erscheinen soll
	
	// Konstanten fr Bildernamen
	public static final String IMG_ID = "id.gif";
	public static final String IMG_JAR = "jar.gif";
	public static final String IMG_FOLDER = "folder.gif";
	public static final String IMG_ERROR = "error.gif";
	public static final String IMG_WARNING = "warning.gif";
	public static final String IMG_PACKAGE_WARNING = "package_warning.gif";
	public static final String IMG_SCRIPT_WARNING = "script_warning.gif";
	public static final String IMG_PACKAGE_ERROR = "package_error.gif";
	public static final String IMG_SCRIPT_ERROR = "script_error.gif";
	public static final String IMG_PACKAGE = "package.gif";
	public static final String IMG_SCRIPT = "script.gif";
	public static final String IMG_SCRIPT_FOLDER = "script_folder.gif";
	public static final String IMG_PACKAGE_FOLDER = "package_folder.gif";
	public static final String IMG_REMOVE = "remove.gif";
	public static final String IMG_REMOVE_DIS = "remove_dis.gif";
	public static final String IMG_ARROW_UP = "arrow_up.gif";
	public static final String IMG_ARROW_UP_DIS = "arrow_up_dis.gif";
	public static final String IMG_ARROW_DOWN = "arrow_down.gif";
	public static final String IMG_ARROW_DOWN_DIS = "arrow_down_dis.gif";
	public static final String IMG_ARROW_TOP = "arrow_top.gif";
	public static final String IMG_ARROW_TOP_DIS = "arrow_top_dis.gif";
	public static final String IMG_ARROW_BOTTOM = "arrow_bottom.gif";
	public static final String IMG_ARROW_BOTTOM_DIS = "arrow_bottom_dis.gif";
	public static final String IMG_SPLASH = "splash.bmp";
	public static final String IMG_ICON = "gator.gif";
	public static final String IMG_UNDO = "undo.gif";
	public static final String IMG_UNDO_DIS = "undo_dis.gif";
	public static final String IMG_REDO = "redo.gif";
	public static final String IMG_REDO_DIS = "redo_dis.gif";
	public static final String IMG_SAVE = "save.gif";
	public static final String IMG_SAVE_DIS = "save_dis.gif";
	public static final String IMG_DISCARD = "discard.gif";
	public static final String IMG_DISCARD_DIS = "discard_dis.gif";
	public static final String IMG_ECLIPSE = "eclipse.gif";
	public static final String IMG_SWING = "swing.gif";
	public static final String IMG_ADD = "add.gif";
	public static final String IMG_ADD_DIS = "add_dis.gif";
	public static final String IMG_EYE = "eye.gif";
	public static final String IMG_RECORD = "record.gif";
	public static final String IMG_STOP = "stop.gif";
	public static final String IMG_EDIT = "edit.gif";
	public static final String IMG_PAUSE = "pause.gif";
		
	private static AtosJApp instance;
	private AtosJMainWindow mainWindow;
	private Project currentProject;
	private Vector inputListeners = new Vector(); // Vector<InputListener>
	private ImageRegistry imgReg;
	private ColorRegistry colorReg;
	
	/**
	 * Startet die Applikation, d.h. ffnet das Hauptfenster.
	 * @see AtosJMainWindow
	 */
	public void start() 
	{
		mainWindow = new AtosJMainWindow();
		mainWindow.open();
	}

	/**
	 * Gibt die Singleton Applikationsinstanz zurck, 
	 * initialisiert diese zuvor, wenn ntig.
	 */
	public static AtosJApp getInstance() {
		if(instance == null)
			instance = new AtosJApp();
		return instance;
	}
	
	/**
	 * Gibt das aktuelle Projekt zurck.
	 * @return Das aktuelle Projekt oder <code>null</code>, falls
	 * kein Projekt geffnet.
	 */
	synchronized public Project getInput() {
		return currentProject;
	}
	
	/**
	 * Setzt das akutelle Projekt.
	 * @param proj Das neue Projekt oder null fr ein geschlossenes Projekt.
	 */
	synchronized public void setInput(Project proj)
	{
		Project oldProject = currentProject;
		currentProject  = proj;
		for(int i = 0; i < inputListeners.size(); ++i)
		{
			((InputListener)inputListeners.get(i)).inputChanged(oldProject, currentProject);
		}
		
		// EIGENTLICH: muss der mdb gezeigt werden bevor die Listener aufgerufen 
		// werden denn evtl. wird Editor#setFocus gerufen und das funktioniert
		// nur wenn der mdb und damit die Editoren sichtbar sind
		// ABER: Es sieht scheisse aus, denn es dauert eh sich die gespeicherten
		// editoren ffnen und das sieht man dann, darum belassen wir es
		// vorerst bei der ursprnglichen reihenfolge, was zur folge hat
		// dass die acceleratoren des 1. editors nach dem ffnen erst 
		// funktionieren wenn der nutzer auf die Tabelle im editor klickt
		if(proj != null)
		{	
			mainWindow.showMDB(true);
			mainWindow.addPathToRecentList( proj.getPath() );
			mainWindow.getShell().setText(APP_NAME + " - " + proj.getName());
		}
		else
		{
			mainWindow.showMDB(false);
		}
	}
	
	/**
	 * Fgt einen neuen Listener hinzu, falls dieser noch nicht enthalten ist.
	 * @param listener Der neue Listener.
	 */
	synchronized public void addInputListener(InputListener listener) {
		if( !inputListeners.contains(listener) )
			inputListeners.add(listener);
	}
	
	/**
	 * Entfernt den angegebenen Listener.
	 * @param listener Der zu entfernende Listener.
	 */
	synchronized public void removeInputListener(InputListener listener) {
		inputListeners.remove(listener);
	}
	
	/**
	 * Gibt das Hauptfenster zurck.
	 * @return Das Hauptfenster.
	 */
	public AtosJMainWindow getMainWindow() {
		return mainWindow;
	}
	
	/**
	 * Erstellt eine neue <code>Image Instanz</code> und fgt sie
	 * der <code>ImageRegistry</code> hinzu.
	 * @param name eindeutiger Name des Bildes z.B. "hammer.gif",
	 * ist gleichzeitig auch der Schlssel fr die <code>ImageRegistry</code>.
	 * Das Bild wird aus dem Paket "atosj.images" geladen.
	 */
	private void initImage(String name) 
	{
		InputStream stream = getClass().getClassLoader().getResourceAsStream("atosj/images/" + name);
		imgReg.put(
				name,
				new Image(Display.getDefault(), stream )
		);
	}
	
	/**
	 * Ermittelt das <code>Image</code> mit dem angegebenen Namen.
	 * Initialisiert das <code>Image</code> beim ersten Aufruf.
	 * @param name eindeutiger Name des Bildes z.B. "hammer.gif".
	 * Das Bild wird aus dem Paket "atosj.images" geladen.
	 * @return Das Bild.
	 */
	public Image getImage(String name)
	{
		if(imgReg == null)
			imgReg = new ImageRegistry( Display.getDefault() );
		
		Image img = imgReg.get(name);
		if(img == null)
		{
			initImage(name);
			return imgReg.get(name);
		}
		else
		{
			return img;
		}
	}
	
	/**
	 * Ermittelt die angegebene Farbe aus der <code>ColorRegistry</code>.
	 * Initialisiert die Farbe beim ersten Aufruf.
	 * @param rgb RGB-Wert der Farbe.
	 * @return Die zum RGB-wert entsprechende <code>Color</code>-Instanz,
	 * diese ist shared und darf nicht disposed werden.
	 */
	public Color getColor(RGB rgb) {
		if(colorReg == null)
			colorReg = new ColorRegistry( mainWindow.getShell().getDisplay() );
		
		Color color = colorReg.get(rgb.toString());
		if(color == null)
		{
			colorReg.put(rgb.toString(), rgb);
			return colorReg.get(rgb.toString());
		}
		else
		{
			return color;
		}
	}
	
	/**
	 * Gibt alle Ressourcen aus der <code>ImageRegistry</code> frei.
	 * <em>Muss</em> beim Beenden der Applikation gerufen werden, sonst
	 * akkumulieren sich die verbrauchten Betriebssystem Ressourcen, bis
	 * keine mehr frei sind.
	 */
	public void dispose() {
		if(imgReg != null)
			imgReg.dispose();
	}
	
	/**
	 * Startet die Applikation.
	 * @param args Argumente werden ignoriert.
	 * @see #start()
	 */
	public static void main(String[] args) {
		Messages.setPrefPath(System.getProperty("user.dir")+ File.separator + "user.prf");
		AtosJApp.getInstance().start();
	}
}
