/*
 * Created on 29.11.2005
 */
package atosj.app;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.Vector;

import org.eclipse.jface.action.MenuManager;

import atosj.action.project.OpenProjectAction;
import atosj.exception.ParseException;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Verwaltet eine Datei, welche die 5 zuletzt geffneten Projekte speichert.
 * @author Nicos Tegos
 */
public class RecentFileList 
{
	/**
	 * Diese Aktion bernimmt das ffnen eines, in der <code>RecentFileList</code>
	 * gespeicherten Projektes.
	 * @author Nicos Tegos
	 */
	private class OpenRecentProjectAction extends OpenProjectAction
	{
		private File path;
		
		/**
		 * Erstellt die Aktion mit dem Namen der Projektdatei
		 * als Titel der Aktion.
		 * @param path Der Pfad der Projektdatei, die geffnet
		 * werden soll
		 */
		public OpenRecentProjectAction(File path) 
		{
			super( path.getName() );
			this.path = path;
			setId( path.getAbsolutePath() );
			setToolTipText( path.getAbsolutePath() );
		}
		
		/**
		 * Schliet das aktuelle Projekt und ffnet das assoziierte
		 * Projekt.
		 * @see OpenProjectAction#openProject(String)
		 */
		public void run() {
			openProject( path.getAbsolutePath() );
		}
	}
	
	private LinkedList files = new LinkedList(); // LinkedList<File>
	private int limit = 5;
	private MenuManager mngr;
	
	/**
	 * ffnet die Liste der 5 letzten Projekte aus der Datei "recent.lst"
	 * im aktuellen Verzeichnis. Erstellt ein Men zum ffnen der
	 * Projekte.
	 * @param mngr Der <code>MenuManager</code>, der die zuletzt geffneten Projekte
	 * beinhalten soll.
	 */
	public RecentFileList(MenuManager mngr) 
	{
		this.mngr = mngr;
		
		try
		{
			CSVParser parser = new CSVParser("." + File.separator + "recent.lst");
			CSVLine line = null;
			while( (line = parser.getNextLine() ) != null )
			{
				File f = new File(line.getElementUnquoted(0));
				if( f.exists() )
					files.add(f);
			}
			
			initMenu();
		}
		catch(IOException e){}
		catch(ParseException e) {}
	}
	
	public void initMenu() 
	{
		for(int i = 0; i < files.size(); ++i)
			mngr.add( new OpenRecentProjectAction((File)files.get(i)) );
	}

	/**
	 * Fgt path an die Spitze der Liste der zuletzt geffneten Projekte an
	 * und updated den assoziierten <code>MenuManager</code>.
	 * @param path Der absolute Pfad der Projektdatei.
	 */
	public void addPath(String path) 
	{
		File newFile = new File(path);
		if( !newFile.exists() )
			return;
		
		// bei berschreiten des limits das letzte file lschen
		if( files.size() > limit )
		{
			if( !files.isEmpty() )
			{
				File removed = (File) files.removeLast();
				mngr.remove( removed.getAbsolutePath() );
			}
		}
		
		// lschen falls schonmal vorhanden
		if( files.remove(newFile) )
		{
			mngr.remove(newFile.getAbsolutePath());
		}
		
		// neue action als erste einfgen
		if( !files.isEmpty() )
		{
			File firstFile = (File) files.getFirst();
			mngr.insertBefore(firstFile.getAbsolutePath(), new OpenRecentProjectAction(newFile) );
			files.addFirst(newFile);
		}
		else
		{
			mngr.add( new OpenRecentProjectAction(newFile) );
			files.addFirst(newFile);
		}
		
		mngr.update(true);
	}
	
	/**
	 * Speichert die Liste der zuletzt geffneten Projekte im aktuellen Verzeichnis unter
	 * dem Namen "recent.lst" ab.
	 */
	public void save() 
	{
		try
		{
			FileWriter writer = new FileWriter("." + File.separator + "recent.lst");
			Vector v = new Vector(1);
			for(int i = 0; i < files.size(); ++i)
			{
				v.removeAllElements();
				File f = (File) files.get(i);
				v.add( "\"" + f.getAbsolutePath() + "\"" );
				writer.write( new CSVLine(v).toString() + "\n" );
			}
			writer.close();
		}
		catch(IOException e) {}
	}
}
