/*
 * Created on 06.09.2005
 */
package atosj.component;

import java.io.Serializable;

import atosj.Messages;

/**
 * Diese Klasse stellt einen eindeutigen Identifikator fr ein Component dar.
 * Er besteht aus dem Namen des Components, dessen HTS Typ und dem Namen
 * des Fensters in dem das Component liegt.
 * @author Nicos Tegos
 */
public class ComponentId implements Serializable
{
	private String name;
	private String className;
	private String windowName;
	
	private String id; // wird nur einmal im konstruktor gebaut
	
	/**
	 * Erstellt eine ID.
	 * @param name Name des Components.
	 * @param className HTS Typ des Components, siehe TYPE_* Konstanten in <code>HTSSyntax</code>.
	 * @param windowName Name des Fensters in dem das Component liegt.
	 */
	public ComponentId(String name, String className, String windowName /*Rectangle bounds*/) 
	{
		this.name = name;
		this.className = className;
		this.windowName = windowName;
		
		setId();
	}

	/**
	 * Erstellt die Id einmalig im Konstruktor.
	 */
	private void setId() {
		id = name + ";" + className + ";" + windowName; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return
		   Messages.getString("ComponentId.2") + windowName //$NON-NLS-1$
		+Messages.getString("ComponentId.3") + className //$NON-NLS-1$
		+Messages.getString("ComponentId.4") + name; //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return id.hashCode();
	}
	
	/** 
	 * Vergleicht eine <code>ComponentID</code> dem Empfnger.
	 * @return <code>true</code>, wenn der Name, der Typ und der
	 * Fenstername gleich sind, <code>false</code> sonst.
	 */
	public boolean equals(Object arg0) 
	{
		if( arg0 instanceof ComponentId && arg0 != null )
		{
			ComponentId toCompare = (ComponentId) arg0;
			return id.equals(toCompare.id);
		}
		
		return false;
	}

	/**
	 * Gibt den HTS Typ zurck.
	 */
	public String getClassName() {
		return className;
	}
	/**
	 * Gibt den Namen des Components zurck.
	 */
	public String getName() {
		return name;
	}
	/**
	 * Gibt den Fensternamen, in dem das Component liegt, zurck.
	 */
	public String getWindowName() {
		return windowName;
	}
}
