/*
 * Created on 06.09.2005
 */
package atosj.component;

import java.io.Serializable;


/**
 * Diese Klasse erstellt zu einem Component den entsprechenden ATOSj-Wrapper,
 * der Klasse <code>MComponent</code>.
 * @author Nicos Tegos
 * @see atosj.component.MComponent
 */
public abstract class ComponentResolver implements Serializable
{
	/**
	 * Erstellt den Wrapper zu dem angegebenen Component.
	 * @param component ein graph. Objekt, das entsprechend 
	 * der Architektur einem <code>MComponent</code> zugeordnet wird.
	 * @param id Der eindeutige Identifikator fr dieses Component,
	 * welcher sich ber die gesamte Laufzeit des Testobjekts nicht ndert.
	 * @return Das entsprechende <code>MComponent</code> oder <code>null</code> wenn 
	 * <code>component</code> nicht zu den untersttzten Klassen gehrt.
	 */
	public abstract MComponent getMComponent(Object component, ComponentId id);
	
	/**
	 * Gibt an zu welchem der HTS Typ <code>component</code> gehrt.
	 * @param component Das Component, alle graph. Objekte einer GUI Bibliothek erlaubt.
	 * @return Der Elementtyp, siehe TYPE_* Konstanten aus <code>HTSSyntax</code>.
	 * @see atosj.model.hts.parser.HTSSyntax
	 */
	public abstract String getTypeName(Object component);
	
	/**
	 * Ermittelt die Namen der HTS Typen, 
	 * die von diesem Resolver untersttzt werden.
	 * @return Die HTS Typen, siehe siehe TYPE_* Konstanten aus <code>HTSSyntax</code>.
	 * @see atosj.model.hts.parser.HTSSyntax
	 */
	public abstract String[] getTypeNames();
	
	/**
	 * Gibt an ob der HTS Typ ein zusammengesetztes Component reprsentiert.
	 * Ein zusammengesetztes Component beinhaltet andere Components,
	 * auf die aber nicht gesondert zugegriffen wird. Zu den zusammengesetzten
	 * HTS Typen gehren COMBOBOX, LIST, TABLE, TABFOLDER und TREE.
	 * @param type Der HTS Typ, siehe TYPE_* Konstanten aus <code>HTSSyntax</code>.
	 * @return <code>true</code> wenn dieser Typ die Ausfhrung von Kommandos 
	 * auf Subelementen zult, <code>false</code> sonst.
	 */
	public abstract boolean isCollectionComponent(String type);
	
	/**
	 * Gibt an ob der angegebene HTS Typ von diesem Resolver untersttzt wird.
	 * @param type Der HTS Typ, siehe TYPE_* Konstanten aus <code>HTSSyntax</code>.
	 * @return <code>true</code> wenn dieser Resolver <code>type</code> untersttzt,
	 * <code>false</code> sonst.
	 */
	public boolean isSupportedType(String type) 
	{
		String[] typeNames = getTypeNames();
		for (int i = 0; i < typeNames.length; i++) {
			if( typeNames[i].equals(type) )
				return true;
		}
		
		return false;
	}
}
