/*
 * Created on 17.06.2006
 */
package atosj.component;

import java.util.HashMap;
import java.util.Vector;

import atosj.util.WindowMonitor;

/**
 * Diese Klasse sammelt alle Listener, die durch <code>ICommandCreator</code> fr
 * das Aufzeichnen von Nutzeraktion im Record Modus initialisiert werden. Aufrufe
 * der Listener Methoden werden durch an die tatschlichen Listener weitergeleitet.
 * Dies sichert, dass alle Listener einfach entfernt werden knnen ohne Kenntnis ber
 * den Listener Typ.
 * @author Nicos Tegos
 */
public abstract class EventMonitorWrapper {

	private HashMap componentToListenerMap = new HashMap(); // HashMap<Object, Vector<Object> merkt sich alle listener die zu einem Component registriert wurden
	
	public EventMonitorWrapper() {
	}
	
	/**
	 * Fgt einen Listener fr das Component hinzu. <em>ACHTUNG:</em>
	 * Dieser Listener wird nicht dirket an das Component angehangen sondern
	 * wird vom <code>EventMonitorWrapper</code> benachritigt.
	 * @param component Das Component.
	 * @param listener Der Listener fr das Component, dieser sollte vom
	 * Component untersttzt werden, wird er nicht untersttzt ist er
	 * wirkungslos.
	 */
	public void addListener(Object component, Object listener) 
	{
		Vector listeners = (Vector) componentToListenerMap.get(component);
		if(listeners == null)
		{
			listeners = new Vector();
			listeners.add(listener);
			componentToListenerMap.put(component, listeners);
		}
		else
		{
			if( !listeners.contains(listener) )
				listeners.add(listener);
		}
	}
	
	/**
	 * Entfernt den Listener fr das Component hinzu.
	 * @param component Das Component.
	 * @param listener Der zu entfernende Listener.
	 * @see #addListener(Object, Object)
	 */
	public void removeListener(Object component, Object listener)
	{
		Vector listeners = (Vector) componentToListenerMap.get(component);
		if(listeners != null)
			listeners.remove(listener);
		
		componentToListenerMap.remove(component);
	}
	
	/**
	 * Gibt an ob die Quelle eines Events auch tatschlich ein Component,
	 * d.h. ein graphisches Objekt war.
	 * @param source Die Ereignisquelle.
	 * @return <code>true</code>, wenn die Ereignisquelle ein Component war,
	 * <code>false</code> sonst.
	 */
	protected abstract boolean isComponent(Object source);
	
	/**
	 * Gibt alle hier registrierten Listener zu <code>component</code> zurck.
	 * @param component Das Component.
	 * @return Vector(Object) alle Listener des Components.
	 */
	protected Vector getListeners(Object component) {
		if( isComponent(component) )
			WindowMonitor.getInstance().getComponentFinder().initCreator(component);
		
		return (Vector) componentToListenerMap.get(component);
	}
}
