package atosj.component;

import java.util.Vector;

import atosj.Messages;

/**
 * Diese Klasse erstellt neue Kommandos fr ein Script anhand
 * von Nutzeraktionen, die ber Listener registriert werden.
 * @author Nicos Tegos
 */
public interface ICommandCreator 
{
	/**
	 * Titel des Dialogs zum Erstllen von TEST Kommandos.
	 * @see atosj.dialog.AWTTestDialog
	 * @see atosj.dialog.SWTTestCreatorDlg
	 */
	public static final String TEST_ACTION_DIALOG_TITLE = Messages.getString("ICommandCreator.0"); //$NON-NLS-1$
	
	/**
	 * Gibt das <code>MComponent</code> zurck, fr das Kommandos aufgezeichnet werden sollen.
	 */
	MComponent getMComponent();
	
	/**
	 * Entfernt alle Listener.
	 */
	void dispose();

	/**
	 * Erstellt ein ACTION Kommando und gibt es an den Receiver weiter.
	 * @param actionType Der Aktionstyp siehe <code>HTSSyntax</code>, z.B. SELECT
	 * @param actionParameters Vector(String) optionale Parameter fr 
	 * einen bestimmten Aktionstyp, z.B. der Text bei EDIT. 
	 * Auch <code>null</code> oder leere Vectoren erlaubt.
	 * @param subitemPath Vector(String) der Subelementpfad, 
	 * <code>null</code> oder leere Vectoren erlaubt.
	 */
	void publishActionCommand(String actionType,
			Vector actionParameters, Vector subitemPath);
	
	/**
	 * Gibt alle TEST Kommandos zurck, die den aktuellen Zustand
	 * des gewrappten Components reprsentieren.
	 * @return Vector(HTSTest) Die TEST Kommandos.
	 * @see atosj.model.hts.command.HTSTest
	 */
	Vector getTestCommands();
}