/*
 * Created on 28.04.2006
 */
package atosj.component;

import atosj.exception.ComponentException;

/**
 * Dieses Interface reprsentiert den HTS TYP COMPONENT, Basistyp aller HTS Typen.
 * @author Nicos Tegos
 */
public interface IComponent 
{
	public static final int NO_MODIFIER = 0;
	public static final int MODIFIER_ALT = 1 << 0;
	public static final int MODIFIER_ALT_GR = 1 << 1;
	public static final int MODIFIER_SHIFT = 1 << 2;
	public static final int MODIFIER_CTRL = 1 << 3;
	
	/**
	 * Simuliert einen Klick auf ein graph. Element.
	 * @param numClicks Anzahl der Klicks (2 sind ein Doppelklick).
	 * @param left wenn <code>true</code> Linksklick, sonst Rechtsklick.
	 * @param modifiers Die AtosJ ModifierKey Konstanten verknpft durch
	 * bitweises oder, siehe <code>MODIFIER_*</code>. Oder <code>NO_MODIFIER</code> falls kein 
	 * Modifier gedrckt.
	 * @throws ComponentException wenn das Component nicht bearbeitbar ist.
	 */
	void click(int numClicks, boolean left, int modifiers) throws ComponentException; 
	
	/**
	 * Simuliert einen Tastendruck (semantische Funktion entspricht der Aktion PRESSKEY
	 * aus der HTS Syntax).
	 * @param character Das Zeichen, das eingegeben werden soll.
	 * @param modifiers Die ATOSj modifier Konstanten. Stehen fr die modifier, 
	 * die gleichzeitig mit dem Buchstaben gedrckt werden sollen z.B. MODIFIER_CTRL.
	 * Modifier knnen nicht allein gedrckt werden.
	 * @throws ComponentException wenn das Component nicht bearbeitbar ist.
	 */
	void pressKey(char character, int modifiers) throws ComponentException;
	
	/**
	 * @return <code>true</code> wenn das Element aktiviert ist, <code>false</code> wenn nicht (ausgegraut).
	 */
	boolean isEnabled() throws ComponentException;
	
    /**
     * @return <code>true</code> wenn das Element sichtbar ist, <code>false</code> wenn nicht.
     */
	boolean isVisible() throws ComponentException;
    
    /**
     * @return <code>true</code> wenn das Element den Fokus hat, <code>false</code> wenn nicht.
     */
	boolean hasFocus() throws ComponentException;
	
	/**
	 * Fr Custom-Components knnen Eigenschaften gesetzt werden.
	 * @param property Der Name der Eigenchaft.
	 * @param value Der neue Wert.
	 * @throws ComponentException wenn die Eigenschaft unbekannt ist oder wenn der Wert
	 * nicht korrekt ist oder bei sonstigen Fehlern.
	 */
	void setProperty(String property, String value) throws ComponentException;
	
	/**
	 * Liest die Eigenschaft eines Custom-Components aus.
	 * @param property Der Name der Eigenschaft.
	 * @return Der aktuelle Wert der Eigenschaft.
	 * @throws ComponentException Wenn die Eigenschaft unbekannt ist oder gerade nicht gelesen
	 * werden kann oder bei sonstigen Fehlern.
	 */
	String getProperty(String property) throws ComponentException;
}
