/*
 * Created on 28.04.2006
 */
package atosj.component;

import atosj.exception.ComponentException;

/**
 * Dieses Interface reprsentiert den HTS TYP LIST.
 * @author Nicos Tegos
 */
public interface IList extends IComponent
{
	/**
	 * Whlt das erste Element mit dem angegebenen Text aus der Liste aus.
	 * @param item Der Text.
	 * @throws ComponentException wenn der Text nicht gefunden wurde.
	 */
	void select(String item) throws ComponentException;
	
	/**
	 * Whlt den angegebenen Index aus der Liste aus.
	 * @param index Der index, 0-basiert.
	 * @throws ComponentException wenn der Index nicht im gltigen bereich liegt
	 */
	void select(int index) throws ComponentException;

	/**
	 * Deselektiert den Eintrag mit dem angegebenen Index, falls selektiert.
	 * @param index Der index, 0-basiert.
	 * @throws ComponentException wenn der Index nicht im gltigen bereich liegt,
	 * oder wenn die Liste keine Mehrfachauswahl erlaubt.
	 */
	void deselect(int index) throws ComponentException;
	
	/**
	 * Deselektiert den ersten Eintrag mit dem angegebenen Text, falls selektiert.
	 * @param item Der Text.
	 * @throws ComponentException wenn der Text nicht gefunden wurde,
	 * oder wenn die Liste keine Mehrfachauswahl erlaubt.
	 */
	void deselect(String item) throws ComponentException;
	
	/**
	 * Fgt der Auswahl den Eintrag mit dem angegebenen Index hinzu.
	 * @param index Der Index 0-basiert
	 * @throws ComponentException wenn der Index nicht im gltigen Bereich liegt,
	 * oder wenn die Liste keine Mehrfachauswahl erlaubt.
	 */
	void addSelect(int index) throws ComponentException;
	
	/**
	 * Fgt der Auswahl den Eintrag mit dem angegebenen Text hinzu.
	 * @param item Der Text.
	 * @throws ComponentException wenn der Text nicht gefunden wurde,
	 * oder wenn die Liste keine Mehrfachauswahl erlaubt.
	 */
	void addSelect(String item) throws ComponentException;
	
	/**
	 * @return Die Anzahl der Eintrge in der Liste.
	 */
	int getItemCount() throws ComponentException;
	
	/**
	 * Gibt den Text des Eintrages am angegebenen Index zurck.
	 * @param index Der Index des Eintrages, 0-basiert.
	 * @return Der Text des angegebenen Eintrages.
	 * @throws ComponentException wenn der index nicht im gltigen bereich liegt
	 */
	String getText(int index) throws ComponentException;
}
