/*
 * Created on 28.04.2006
 */
package atosj.component;

import atosj.exception.ComponentException;

/**
 * Dieses Interface reprsentiert den HTS-Typ Menu.
 * @author Nicos Tegos
 */
public interface IMenu extends IComponent
{
	/**
	 * Whlt einen Eintrag im Men aus.
	 * @param path Der Pfad mit den Labels der Menpunkte, beginnend beim
	 * obersten Menpunkt.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde.
	 */
	void select(String[] path) throws ComponentException;
	
	/**
	 * Setzt den Zustand auf <code>check</code>, falls dieser Zustand nicht schon vorher
	 * so war.
	 * @param path Der Pfad mit den Labels der Menpunkte, beginnend beim
	 * obersten Menpunkt.
	 * @param check der neue Zustand.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde oder wenn der Menpunkt nicht gecheckt werden kann.
	 */
	void setChecked(String[] path, boolean check) throws ComponentException;
	
	/**
	 * Liest den Zustand eines Checkbox Menpunktes.
	 * @param path Der Pfad mit den Labels der Menpunkte, beginnend beim
	 * obersten Menpunkt.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde oder wenn der Menpunkt nicht checkbar ist.
	 */
	boolean isChecked(String[] path) throws ComponentException;
	
	/**
	 * Liest den Enabled-Zustand eines Menpunktes.
	 * @param path Der Pfad mit den Labels der Menpunkte, beginnend beim
	 * obersten Menpunkt.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde.
	 */
	boolean isEnabled(String[] path) throws ComponentException;
}
