/*
 * Created on 28.04.2006
 */
package atosj.component;

import atosj.exception.ComponentException;

/**
 * Dieses Interface reprsentiert den HTS-Typ TABLE.
 * @author Nicos Tegos
 */
public interface ITable extends IComponent
{
	/**
	 * Whlt den Zeile aus der Tabelle aus.
	 * @param row Die Zeile, 0-basiert.
	 * @throws ComponentException wenn row nicht im gltigen Bereich liegt.
	 */
	void select(int row) throws ComponentException;

	/**
	 * Deselektiert den Eintrag mit der angegebenen Zeile, falls selektiert.
	 * @param row Die Zeile, 0-basiert.
	 * @throws ComponentException wenn <code>row</code> nicht im gltigen Bereich liegt,
	 * oder wenn die Tabelle keine Mehrfachauswahl erlaubt.
	 */
	void deselect(int row) throws ComponentException;
	
	/**
	 * Fgt der Auswahl den angegeben Eintrag hinzu, falls noch nicht selektiert.
	 * @param row Die Zeile, 0-basiert.
	 * @throws ComponentException wenn <code>row</code> nicht im gltigen Bereich liegt,
	 * oder wenn die Tabelle keine Mehrfachauswahl erlaubt.
	 */
	void addSelect(int row) throws ComponentException;
	
	/**
	 * @return Die Anzahl der Eintrge in der Tabelle.
	 */
	int getItemCount() throws ComponentException;
	
	/**
	 * @return Die Anzahl der Spalten in der Tabelle.
	 */
	int getColCount() throws ComponentException;
	
	/**
	 * Gibt die Tabellenzelle an der angegebenen Position zurck.
	 * @param row Die Zeile, 0-basiert.
	 * @param col Die Spalte, 0-basiert.
	 * @return Die angegebene Tabellenzelle.
	 * @throws ComponentException wenn <code>row</code> oder <code>col</code> 
	 * nicht im gltigen Bereich liegen.
	 */
	ITableCell getTableCell(int row, int col) throws ComponentException;
}
