/*
 * Created on 28.04.2006
 */
package atosj.component;

import atosj.exception.ComponentException;

/**
 * Dieses Interface reprsentiert den HTS-Typ TREE.
 * @author Nicos Tegos
 */
public interface ITree extends IComponent
{
	
	/**
	 * Whlt einen Eintrag im Baum aus.
	 * @param path Der Pfad mit den Labels der Knoten, beginnend beim
	 * Wurzelknoten.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde.
	 */
	void select(String[] path) throws ComponentException;
	
	/**
	 * Whlt einen Eintrag im Baum ab.
	 * @param path Der Pfad mit den Labels der Knoten, beginnend beim
	 * Wurzelknoten.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde oder wenn der Baum keine Mehrfachauswahl erlaubt.
	 */
	void deselect(String[] path) throws ComponentException;
	
	/**
	 * Fgt den Eintrag im Baum zur aktuellen Auswahl hinzu.
	 * @param path Der Pfad mit den Labels der Knoten, beginnend beim
	 * Wurzelknoten.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde oder wenn der Baum keine Mehrfachauswahl erlaubt.
	 */
	void addSelect(String[] path) throws ComponentException;
	
	/**
	 * Gibt den Knoten des Baumes mit dem angegebenen Pfad zurck.
	 * @param path Der Pfad mit den Labels der Knoten, beginnend beim
	 * Wurzelknoten.
	 * @return Die Zelle des Eintrages.
	 * @throws ComponentException wenn der angegebene Pfad nicht gefunden
	 * wurde.
	 */
	ITreeCell getTreeCell(String[] path) throws ComponentException;
}
