/*
 * Created on 11.06.2006
 */
package atosj.component;

import java.util.Vector;

import atosj.model.hts.parser.HTSSyntax;

/**
 * Erstellt Kommandos sich ndernder Selektionen in Components, die eine Selektion von
 * enthaltenen Elementen zulassen, wie z.B. Baum oder Liste.
 * @author Nicos Tegos
 */
public class ListCommandCreatorHelper 
{
	private Object[] selection;
	private ICommandCreator creator;
	
	public ListCommandCreatorHelper(IListCommandCreator creator) {
		this.creator = creator;
		selection = getSelection();
	}
	
	private Object[] getSelection() {
		return ((IListCommandCreator)creator).getSelection();
	}
	
	private Vector convertSelectionObjectToPath(Object selectionObject) {
		return ((IListCommandCreator)creator).convertSelectionObjectToPath(selectionObject);
	}

	/**
	 * Erstellt die ACTION Kommandos, ermittelt aus der Differenz zwischen
	 * alter und neuer Selection. Darf nur nach einer nderung der Selekion gerufen werden.
	 */
	public void selectionChanged() 
	{
		Object[] newSelection = getSelection();
		
		if( newSelection.length == 1 && !contains(newSelection[0], selection) )
		{
			creator.publishActionCommand(HTSSyntax.ACTION_SELECT, null, convertSelectionObjectToPath(newSelection[0]));
		}
		else // es wurde was weggenommen, oder hinzugefgt oder beides
		{
			// die hinzugekommenen finden
			for(int i = 0; i < newSelection.length; ++i)
			{
				boolean found = false;
				for(int j = 0; j < selection.length; ++j)
				{
					if( newSelection[i].equals(selection[j]) )
					{
						found = true;
						break;
					}
				}
				
				if(!found) // nicht in der alten selection gefunden
					creator.publishActionCommand(HTSSyntax.ACTION_ADDSELECT, null, convertSelectionObjectToPath(newSelection[i]));
			}
			
			// die deselektierten finden
			for(int i = 0; i < selection.length; ++i)
			{
				boolean found = false;
				for(int j = 0; j < newSelection.length; ++j)
				{
					if( selection[i].equals(newSelection[j]) )
					{
						found = true;
						break;
					}
				}
				
				if(!found) // nicht in der neuen selection gefunden
					creator.publishActionCommand(HTSSyntax.ACTION_DESELECT, null, convertSelectionObjectToPath(selection[i]));
			}
		}
		
		selection = newSelection; 		
	}
	
	/**
	 * Ermittelt ob ein bestimmtes Element in einer Selektion enthalten ist.
	 * @param val Das gesuchte Element.
	 * @param selection Die Selektion.
	 * @return <code>true</code>, wenn <code>val</code> in <code>selection</code> enthalten ist, 
	 * <code>false</code> sonst.
	 */
	private boolean contains(Object val, Object[] selection) 
	{
		for (int i = 0; i < selection.length; i++) {
			if( val.equals(selection[i]) )
				return true;
		}
		return false;
	}
	
	/**
	 * Convenience Funktion die ein int[] in ein Integer[] umwandelt.
	 */
	public static Integer[] convertToIntegerArray(int[] selection) {
		Integer[] ret = new Integer[selection.length];
		for (int i = 0; i < selection.length; i++) {
			ret[i] = new Integer(selection[i]);
		}
		
		return ret;
	}
}