package  atosj.component.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.AbstractButton;

import atosj.component.ComponentId;
import atosj.component.IButton;
import atosj.component.ICommandCreator;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>javax.swing.AbstractButton</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingButton extends SwingComponent implements IButton 
{
	/**
	 * Generiert ACTION Kommandos und TEST Kommandos fr Components der Klasse javax.swing.AbstractButton.
	 */
	protected class ButtonCommandCreator extends SwingCommandCreator implements ActionListener
	{
		public ButtonCommandCreator(SwingButton button, IHTSCommandReceiver receiver) {
			super(button, receiver);
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((IButton)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((IButton)getMComponent()).getNum();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}

		/**
		 * Zeichnet das Drcken des Knopfes auf.
		 */
		public void actionPerformed(ActionEvent e) {
			publishActionCommand(HTSSyntax.ACTION_SELECT, null, null);
		}
	};
	
	public SwingButton(AbstractButton button, ComponentId id) {
		super(button, id);
	}

	protected AbstractButton getButton() {
		return (AbstractButton) getAWTComponent();
	}

	public String getText() {
		return getButton().getText();
	}
	
	public void select() throws ComponentException {
		click(1, true, NO_MODIFIER);
	}

	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
   
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ButtonCommandCreator(this, receiver);
	}
}
