/*
 * Created on 12.09.2005
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.plaf.basic.ComboPopup;

import atosj.component.ComponentId;
import atosj.component.ICombobox;
import atosj.component.ICommandCreator;
import atosj.component.IEditable;
import atosj.component.ILabel;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;
;

/**
 * Wrapperklasse fr alle von javax.swing.JComboBox abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingComboBox extends SwingComponent implements ICombobox
{
	/**
	 * Generiert ACTION Kommandos und TEST Kommandos fr Components der Klasse javax.swing.JComboBox.
	 */
	private class ComboBoxCommandCreator extends SwingCommandCreator implements ItemListener, ActionListener
	{
		public ComboBoxCommandCreator(SwingComboBox combo, IHTSCommandReceiver receiver) 
		{
			super(combo, receiver);
			
			addListener(this);
		}
		
		private JComboBox getComboBox() {
			return ((SwingComboBox)getMComponent()).getComboBox();
		}
		
		public Vector getTestCommands() 
		{
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ICombobox)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((ICombobox)getMComponent()).getNum();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				int num = ((ICombobox)getMComponent()).getItemCount();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}

		public void itemStateChanged(ItemEvent e) 
		{
			if( e.getStateChange() == ItemEvent.SELECTED )
			{
				int index = getComboBox().getSelectedIndex();
				if(index != -1)
				{
					Vector subitemPath = new Vector(1);
					subitemPath.add( "" + index );
					publishActionCommand(HTSSyntax.ACTION_SELECT, null, subitemPath);
				}
			}
		}
		
		public void actionPerformed(ActionEvent e) 
		{
			if( e.getActionCommand().equals("comboBoxEdited") );
			{
				try {
					Vector actionParams = new Vector(1);
					actionParams.add( '"' + ((SwingComboBox)getMComponent()).getText() + '"' );
					publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
				} 
				catch (ComponentException e1) {}
			}
		}
	};
	
	public SwingComboBox(JComboBox component, ComponentId id) {
		super(component, id);
	}
	
	private JComboBox getComboBox() {
		return (JComboBox) getAWTComponent();
	}
	
	public String getText() throws ComponentException
	{
		if( getComboBox().isEditable() ) 
		{
			Component c = getComboBox().getEditor().getEditorComponent();
			
			MComponent editor = getResolver().getMComponent( c, getComponentId() );
			if(editor instanceof ILabel)
				return ((ILabel)editor).getText();
			else
				throw new InvalidOperationException(this, OP_GET_TEXT, true);
		}
		else
		{
			SwingList list = getPopupList(false);
			int index = list.getList().getSelectedIndex();
			if(index == -1)
				return "";
			
			return list.getText(index);
		}
	}
	
	public void setText(String text) throws ComponentException
	{
		assertEditable();
		
		if ( getComboBox().isEditable() ) 
		{
			// am editor setzen
			Component c = getComboBox().getEditor().getEditorComponent();
			MComponent editor = getResolver().getMComponent(c, getComponentId());
			if(editor instanceof IEditable)
			{
				((IEditable)editor).setText(text);
				// abschliessen durch enter drcken
				editor.pressKey(KeyEvent.CHAR_UNDEFINED, KeyEvent.VK_ENTER, 0);
			}
			else
				throw new InvalidOperationException(this, OP_SET_TEXT, true);
		}
		else 	
		{
			throw new InvalidOperationException(this, OP_SET_TEXT, false);
		}
	}
	
	/**
	 * Ermittelt den Wrapper fr die Dropdown Liste der Combobox.
	 * @param showPopup <code>true</code>, wenn die Liste gezeigt werden soll.
	 * @return Den Wrapper fr die Popupliste der Combobox. 
	 */
	private SwingList getPopupList(boolean showPopup) throws ComponentException 
	{
		ComboPopup popup = (ComboPopup) getComboBox().getAccessibleContext().getAccessibleChild(0 /*popup*/);
		
		JList list = popup.getList();
		if (showPopup) 
		{
			getComboBox().requestFocus();
			getComboBox().showPopup();
			//sync();
			
			list.requestFocus();
			//sync();
		}
		
		return new SwingList(list, getComponentId());
	}

	public void select(int index) throws ComponentException {
		getPopupList(true).select(index);		
	}

	public void select(String text) throws ComponentException {
		getPopupList(true).select(text);		
	}

	public int getItemCount() throws ComponentException {
		return getComboBox().getItemCount();
	}

	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ComboBoxCommandCreator(this, receiver);
	}
}
