/*
 * Created on 09.04.2006
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;

import javax.swing.AbstractButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JTree;
import javax.swing.text.JTextComponent;

import atosj.component.ComponentId;
import atosj.component.CustomComponentResolver;
import atosj.component.MComponent;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Liefert alle <code>MComponents</code> zu Klassen der Swing Architektur.
 * @author Nicos Tegos
 */
public class SwingComponentResolver extends CustomComponentResolver {

	public SwingComponentResolver(String filePath) {
		super(filePath);
	}
	
	public MComponent getMComponent(Object component, ComponentId id) 
	{
		// zuerst berprfen ob ein custom component definiert wurde
		MComponent customComponent = super.getMComponent(component, id);
		if(customComponent != null)
			return customComponent;
		
		// kein custom component also weiter mit den standard components
		if(component instanceof JTextComponent)
		{
			return new SwingTextComponent( (JTextComponent)component, id);
		}
		if (component instanceof JLabel) 
		{
			return new SwingLabel((JLabel) component, id);
		}
		else if (component instanceof JMenu)	{
			return new SwingMenu((JMenu) component, id, this);
		}
		else if (component instanceof JRadioButton) 
		{
			return new SwingRadioButton((JRadioButton) component, id);
		} // ACHTUNG eine extra Klasse fr JCheckBox gibt es nicht, denn diese wird vollstndig durch JToggleButton bernommen
		else if (component instanceof JToggleButton) 
		{
			return new SwingToggleButton((JToggleButton) component, id);
		}
		else if (component instanceof AbstractButton) 
		{
			return new SwingButton((AbstractButton) component, id);
		} 
		else if (component instanceof Frame) 
		{
			return new SwingFrame((Frame) component, id);
		} 
		else if (component instanceof Dialog) 
		{
			return new SwingDialog((Dialog) component, id);
		}
		else if (component instanceof JSpinner)
		{
			return new SwingSpinner( (JSpinner)component, id);
		}
		else if (component instanceof JTable)
		{
			return new SwingTable( (JTable)component, id);
		}
		else if (component instanceof JTree)
		{
			return new SwingTree( (JTree)component, id);
		}
		else if (component instanceof JList)
		{
			return new SwingList( (JList)component, id);
		}
		else if (component instanceof JTabbedPane)
		{
			return new SwingTAbbedPane( (JTabbedPane)component, id);
		}
		else if (component instanceof JComboBox)
		{
			return new SwingComboBox( (JComboBox)component, id);
		}
		else if (component instanceof Component)
		{
			return new SwingComponent( (Component) component, id);
		}
		
		// kein entsprechendes SwingComponent gefunden.
		return null;
	}
	
	public String getTypeName(Object component) 
	{
		// zuerst berprfen ob ein custom component definiert wurde
		String customClassName = super.getTypeName(component);
		if( !customClassName.equals("") )
			return customClassName;
		
		// kein custom component also weiter mit den standard namen
		if(component instanceof JTextComponent)
		{
			return HTSSyntax.TYPE_EDITBOX;
		}
		if (component instanceof JLabel) 
		{
			return HTSSyntax.TYPE_LABEL;
		}
		else if (component instanceof JMenu)	{
			return HTSSyntax.TYPE_MENU;
		}
		else if (component instanceof JSpinner) 
		{
			return HTSSyntax.TYPE_SPINNER;
		}
		else if (component instanceof JCheckBox) 
		{
			return HTSSyntax.TYPE_CHECKBOX;
		}
		else if (component instanceof JToggleButton) 
		{
			return HTSSyntax.TYPE_RADIOBUTTON;
		}
		else if (component instanceof JToggleButton) 
		{
			return HTSSyntax.TYPE_BUTTON;
		}
		else if (component instanceof AbstractButton) 
		{
			return HTSSyntax.TYPE_BUTTON;
		} 
		else if (component instanceof Frame) 
		{
			return HTSSyntax.TYPE_WINDOW;
		} 
		else if (component instanceof Dialog) 
		{
			return HTSSyntax.TYPE_WINDOW;
		}
		else if (component instanceof JTable)
		{
			return HTSSyntax.TYPE_TABLE;
		}
		else if (component instanceof JTree)
		{
			return HTSSyntax.TYPE_TREE;
		}
		else if (component instanceof JList)
		{
			return HTSSyntax.TYPE_LIST;
		}
		else if (component instanceof JTabbedPane)
		{
			return HTSSyntax.TYPE_TABFOLDER;
		}
		else if (component instanceof JComboBox)
		{
			return HTSSyntax.TYPE_COMBOBOX;
		}
		
		return HTSSyntax.TYPE_COMPONENT;
	}
	
	public String[] getTypeNames() {
		String[] customNames = super.getTypeNames();
		String[] myNames = new String[]
			{
				HTSSyntax.TYPE_BUTTON,
				HTSSyntax.TYPE_CHECKBOX,
				HTSSyntax.TYPE_COMBOBOX,
				HTSSyntax.TYPE_COMPONENT,
				HTSSyntax.TYPE_SPINNER,
				HTSSyntax.TYPE_EDITBOX,
				HTSSyntax.TYPE_LABEL,
				HTSSyntax.TYPE_LIST,
				HTSSyntax.TYPE_MENU,
				HTSSyntax.TYPE_RADIOBUTTON,
				HTSSyntax.TYPE_TABFOLDER,
				HTSSyntax.TYPE_TABLE,
				HTSSyntax.TYPE_TREE,
				HTSSyntax.TYPE_WINDOW,
			};
		// custom namen und standardnamen zusammenfassen
		String[] allNames = new String[customNames.length + myNames.length];
		System.arraycopy(customNames, 0, allNames, 0, customNames.length);
		System.arraycopy(myNames, 0, allNames, customNames.length, myNames.length);
		
		return allNames;
	}

	public boolean isCollectionComponent(String type) 
	{
		if( super.isCollectionComponent(type)
				|| type.equals(HTSSyntax.TYPE_COMBOBOX) 
				|| type.equals(HTSSyntax.TYPE_LIST)
				|| type.equals(HTSSyntax.TYPE_MENU)
				|| type.equals(HTSSyntax.TYPE_TABFOLDER)
				|| type.equals(HTSSyntax.TYPE_TABLE)
				|| type.equals(HTSSyntax.TYPE_TREE)
		)
			return true;
		
		return false;
	}
}
