package atosj.component.swing;

import java.awt.Dialog;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IWindow;
import atosj.event.awt.CloseMainWindowEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>java.awt.Dialog</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingDialog extends SwingComponent implements IWindow 
{
	private class DialogCommandCreator extends SwingCommandCreator
	{
		public DialogCommandCreator(SwingDialog dialog, IHTSCommandReceiver receiver) 
		{
			super(dialog, receiver);
				
			addListener( new WindowAdapter() {
				public void windowClosing(WindowEvent arg0) {
					publishActionCommand(HTSSyntax.ACTION_CLOSE, null, null);
				}
			});
		}
	}

	public SwingDialog(Dialog window, ComponentId id) {
		super(window, id);
	}
	
	private Dialog getDialog() {
		return (Dialog) getComponent();
	}
	
	public void close() throws ComponentException {
		if( prepareWindowOperation() && !getDialog().isUndecorated() )
			new CloseMainWindowEvent(this).fire();
		else
			throw new InvalidOperationException(this, OP_CLOSE, false);
	}

	public void maximize() throws ComponentException{
		throw new InvalidOperationException(this, OP_MAXIMIZE, false);
	}

	public void minimize() throws ComponentException {
		throw new InvalidOperationException(this, OP_MINIMIZE, false);
	}

	public void normalize() throws ComponentException {
		throw new InvalidOperationException(this, OP_NORMALIZE, false);
	}
	
	public boolean prepareWindowOperation()	{
		return getAWTComponent().requestFocusInWindow();
	}

	public String getText() throws ComponentException {
		return getDialog().getTitle();
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new DialogCommandCreator(this, receiver);
	}
}
