/*
 * Created on 17.06.2006
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.util.Vector;

import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import atosj.component.EventMonitorWrapper;

import com.sun.java.accessibility.util.SwingEventMonitor;

/**
 * Realisiert das Multi-Casting von Ereignissen an die durch die <code>SwingCommandCreator</code> 
 * initialisierten Listener. Implementiert alle Listenertypen der Swing Architektur und registriert
 * sich beim <code>SwingEventMonitor</code> 
 * @author Nicos Tegos
 * @see com.sun.java.accessibility.util.SwingEventMonitor
 */
public class SwingEventMonitorWrapper extends EventMonitorWrapper
implements ActionListener, AdjustmentListener, AncestorListener, 
CaretListener, CellEditorListener, ChangeListener, ComponentListener,
ContainerListener, DocumentListener, FocusListener, InternalFrameListener,
ItemListener, KeyListener, ListDataListener, ListSelectionListener, 
MenuListener, MouseListener, MouseMotionListener, PopupMenuListener, PropertyChangeListener,
TableModelListener, TextListener, TreeExpansionListener, TreeModelListener,
TreeSelectionListener, UndoableEditListener, VetoableChangeListener,
WindowListener
{

	public SwingEventMonitorWrapper() 
	{
		super();
		
		SwingEventMonitor.addActionListener(this);
		SwingEventMonitor.addAdjustmentListener(this);
		SwingEventMonitor.addAncestorListener(this);
		SwingEventMonitor.addCaretListener(this);
		SwingEventMonitor.addCellEditorListener(this);
		SwingEventMonitor.addChangeListener(this);
		SwingEventMonitor.addComponentListener(this);
		SwingEventMonitor.addContainerListener(this);
		SwingEventMonitor.addDocumentListener(this);
		SwingEventMonitor.addFocusListener(this);
		SwingEventMonitor.addInternalFrameListener(this);
		SwingEventMonitor.addItemListener(this);
		SwingEventMonitor.addKeyListener(this);
		SwingEventMonitor.addListDataListener(this);
		SwingEventMonitor.addListSelectionListener(this);
		SwingEventMonitor.addMenuListener(this);
		SwingEventMonitor.addMouseListener(this);
		SwingEventMonitor.addMouseMotionListener(this);
		SwingEventMonitor.addPopupMenuListener(this);
		// FIXME: das geht nicht, aus unerfindlichen Grnden strzt das Testobjekt unter WinXP beim Aufrufen
		// des PropertyChangeListeners ab
		SwingEventMonitor.addPropertyChangeListener(this);
		SwingEventMonitor.addTableModelListener(this);
		SwingEventMonitor.addTextListener(this);
		SwingEventMonitor.addTreeExpansionListener(this);
		SwingEventMonitor.addTreeModelListener(this);
		SwingEventMonitor.addTreeSelectionListener(this);
		SwingEventMonitor.addUndoableEditListener(this);
		SwingEventMonitor.addVetoableChangeListener(this);
		SwingEventMonitor.addWindowListener(this);
	}

	public void actionPerformed(ActionEvent arg0) 
	{
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ActionListener)
					((ActionListener)listeners.get(i)).actionPerformed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.AdjustmentListener#adjustmentValueChanged(java.awt.event.AdjustmentEvent)
	 */
	public void adjustmentValueChanged(AdjustmentEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof AdjustmentListener)
					((AdjustmentListener)listeners.get(i)).adjustmentValueChanged(arg0);
			}
		}	
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.AncestorListener#ancestorAdded(javax.swing.event.AncestorEvent)
	 */
	public void ancestorAdded(AncestorEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof AncestorListener)
					((AncestorListener)listeners.get(i)).ancestorAdded(arg0);
			}
		}	
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.AncestorListener#ancestorMoved(javax.swing.event.AncestorEvent)
	 */
	public void ancestorMoved(AncestorEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof AncestorListener)
					((AncestorListener)listeners.get(i)).ancestorMoved(arg0);
			}
		}	
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.AncestorListener#ancestorRemoved(javax.swing.event.AncestorEvent)
	 */
	public void ancestorRemoved(AncestorEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof AncestorListener)
					((AncestorListener)listeners.get(i)).ancestorRemoved(arg0);
			}
		}	
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.CaretListener#caretUpdate(javax.swing.event.CaretEvent)
	 */
	public void caretUpdate(CaretEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof CaretListener)
					((CaretListener)listeners.get(i)).caretUpdate(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.CellEditorListener#editingCanceled(javax.swing.event.ChangeEvent)
	 */
	public void editingCanceled(ChangeEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof CellEditorListener)
					((CellEditorListener)listeners.get(i)).editingCanceled(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.CellEditorListener#editingStopped(javax.swing.event.ChangeEvent)
	 */
	public void editingStopped(ChangeEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof CellEditorListener)
					((CellEditorListener)listeners.get(i)).editingStopped(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ChangeListener)
					((ChangeListener)listeners.get(i)).stateChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ComponentListener)
					((ComponentListener)listeners.get(i)).componentHidden(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ComponentListener)
					((ComponentListener)listeners.get(i)).componentMoved(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ComponentListener)
					((ComponentListener)listeners.get(i)).componentResized(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ComponentListener)
					((ComponentListener)listeners.get(i)).componentShown(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ContainerListener#componentAdded(java.awt.event.ContainerEvent)
	 */
	public void componentAdded(ContainerEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ContainerListener)
					((ContainerListener)listeners.get(i)).componentAdded(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ContainerListener#componentRemoved(java.awt.event.ContainerEvent)
	 */
	public void componentRemoved(ContainerEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ContainerListener)
					((ContainerListener)listeners.get(i)).componentRemoved(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
	 */
	public void changedUpdate(DocumentEvent arg0) {
		/* zu diesem Listener gibt es leider kein source */
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
	 */
	public void insertUpdate(DocumentEvent arg0) {
		/* zu diesem Listener gibt es leider kein source */
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
	 */
	public void removeUpdate(DocumentEvent arg0) {
		/* zu diesem Listener gibt es leider kein source */
	}

	/* (non-Javadoc)
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof FocusListener)
					((FocusListener)listeners.get(i)).focusGained(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
	 */
	public void focusLost(FocusEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof FocusListener)
					((FocusListener)listeners.get(i)).focusLost(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameActivated(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameActivated(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameActivated(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameClosed(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameClosed(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameClosed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameClosing(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameClosing(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameClosing(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameDeactivated(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameDeactivated(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameDeactivated(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameDeiconified(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameDeiconified(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameDeiconified(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameIconified(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameIconified(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameIconified(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameOpened(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameOpened(InternalFrameEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof InternalFrameListener)
					((InternalFrameListener)listeners.get(i)).internalFrameOpened(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ItemListener)
					((ItemListener)listeners.get(i)).itemStateChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof KeyListener)
					((KeyListener)listeners.get(i)).keyPressed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof KeyListener)
					((KeyListener)listeners.get(i)).keyReleased(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof KeyListener)
					((KeyListener)listeners.get(i)).keyTyped(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListDataListener#contentsChanged(javax.swing.event.ListDataEvent)
	 */
	public void contentsChanged(ListDataEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ListDataListener)
					((ListDataListener)listeners.get(i)).contentsChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListDataListener#intervalAdded(javax.swing.event.ListDataEvent)
	 */
	public void intervalAdded(ListDataEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ListDataListener)
					((ListDataListener)listeners.get(i)).intervalAdded(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListDataListener#intervalRemoved(javax.swing.event.ListDataEvent)
	 */
	public void intervalRemoved(ListDataEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ListDataListener)
					((ListDataListener)listeners.get(i)).intervalRemoved(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof ListSelectionListener)
					((ListSelectionListener)listeners.get(i)).valueChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.MenuListener#menuCanceled(javax.swing.event.MenuEvent)
	 */
	public void menuCanceled(MenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MenuListener)
					((MenuListener)listeners.get(i)).menuCanceled(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.MenuListener#menuDeselected(javax.swing.event.MenuEvent)
	 */
	public void menuDeselected(MenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MenuListener)
					((MenuListener)listeners.get(i)).menuDeselected(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.MenuListener#menuSelected(javax.swing.event.MenuEvent)
	 */
	public void menuSelected(MenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MenuListener)
					((MenuListener)listeners.get(i)).menuSelected(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseListener)
					((MouseListener)listeners.get(i)).mouseClicked(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseListener)
					((MouseListener)listeners.get(i)).mouseEntered(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseListener)
					((MouseListener)listeners.get(i)).mouseExited(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseListener)
					((MouseListener)listeners.get(i)).mousePressed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseListener)
					((MouseListener)listeners.get(i)).mouseReleased(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseMotionListener)
					((MouseMotionListener)listeners.get(i)).mouseDragged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof MouseMotionListener)
					((MouseMotionListener)listeners.get(i)).mouseMoved(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.PopupMenuListener#popupMenuCanceled(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuCanceled(PopupMenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof PopupMenuListener)
					((PopupMenuListener)listeners.get(i)).popupMenuCanceled(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.PopupMenuListener#popupMenuWillBecomeInvisible(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuWillBecomeInvisible(PopupMenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof PopupMenuListener)
					((PopupMenuListener)listeners.get(i)).popupMenuWillBecomeInvisible(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.PopupMenuListener#popupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuWillBecomeVisible(PopupMenuEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof PopupMenuListener)
					((PopupMenuListener)listeners.get(i)).popupMenuWillBecomeVisible(arg0);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof PropertyChangeListener)
					((PropertyChangeListener)listeners.get(i)).propertyChange(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
	 */
	public void tableChanged(TableModelEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TableModelListener)
					((TableModelListener)listeners.get(i)).tableChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.TextListener#textValueChanged(java.awt.event.TextEvent)
	 */
	public void textValueChanged(TextEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TextListener)
					((TextListener)listeners.get(i)).textValueChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeExpansionListener#treeCollapsed(javax.swing.event.TreeExpansionEvent)
	 */
	public void treeCollapsed(TreeExpansionEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeExpansionListener)
					((TreeExpansionListener)listeners.get(i)).treeCollapsed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeExpansionListener#treeExpanded(javax.swing.event.TreeExpansionEvent)
	 */
	public void treeExpanded(TreeExpansionEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeExpansionListener)
					((TreeExpansionListener)listeners.get(i)).treeExpanded(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeModelListener#treeNodesChanged(javax.swing.event.TreeModelEvent)
	 */
	public void treeNodesChanged(TreeModelEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeModelListener)
					((TreeModelListener)listeners.get(i)).treeNodesChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeModelListener#treeNodesInserted(javax.swing.event.TreeModelEvent)
	 */
	public void treeNodesInserted(TreeModelEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeModelListener)
					((TreeModelListener)listeners.get(i)).treeNodesInserted(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeModelListener#treeNodesRemoved(javax.swing.event.TreeModelEvent)
	 */
	public void treeNodesRemoved(TreeModelEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeModelListener)
					((TreeModelListener)listeners.get(i)).treeNodesRemoved(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeModelListener#treeStructureChanged(javax.swing.event.TreeModelEvent)
	 */
	public void treeStructureChanged(TreeModelEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeModelListener)
					((TreeModelListener)listeners.get(i)).treeStructureChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof TreeSelectionListener)
					((TreeSelectionListener)listeners.get(i)).valueChanged(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.UndoableEditListener#undoableEditHappened(javax.swing.event.UndoableEditEvent)
	 */
	public void undoableEditHappened(UndoableEditEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof UndoableEditListener)
					((UndoableEditListener)listeners.get(i)).undoableEditHappened(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.beans.VetoableChangeListener#vetoableChange(java.beans.PropertyChangeEvent)
	 */
	public void vetoableChange(PropertyChangeEvent arg0) throws PropertyVetoException {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof VetoableChangeListener)
					((VetoableChangeListener)listeners.get(i)).vetoableChange(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowActivated(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowClosed(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowClosing(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowDeactivated(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowDeiconified(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowIconified(arg0);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent arg0) {
		Vector listeners = getListeners( arg0.getSource() );
		if(listeners != null)
		{
			for(int i = 0; i < listeners.size(); ++i)
			{
				if(listeners.get(i) instanceof WindowListener)
					((WindowListener)listeners.get(i)).windowOpened(arg0);
			}
		}
	}

	/**
	 * Gibt <code>true</code> zurck, wenn die Ereignisquelle von <code>java.awt.Component</code>
	 * abgeleitet ist, <code>false</code> sonst.
	 * @param source Die Ereignisquelle.
	 */
	protected boolean isComponent(Object source) {
		if(source instanceof Component)
			return true;
		
		return false;
	}
}
