/*
 * Created on 09.04.2006
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Rectangle;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;

import atosj.Messages;
import atosj.component.NamingStrategy;

/**
 * Die Benanumgsstrategie fr Components der Swing Bibliothek.
 * @author Nicos Tegos
 */
public class SwingNamingStrategy extends NamingStrategy
{
	/**
	 * Gibt den Klarnamen des Components zurck, dieser hngt vom
	 * Typ des Components ab.
	 * @param c Muss mindestens vom Typ <code>java.awt.Component</code> sein.
	 * @return ein nicht leerer String, der den Klarnamen 
	 * des graph. Objektes enthlt.
	 */
	public String getName(Object c)	{
		String name = ""; //$NON-NLS-1$
		
		if(c instanceof Frame)
		{
			name = ((Frame)c).getTitle();
		}
		else
		if(c instanceof Dialog)
		{
			name = ((Dialog)c).getTitle();
		}
		else
		{
			name = ((Component)c).getName();
		}
		
		if( name == null || name.equals("") ) //$NON-NLS-1$
		{
			if(c instanceof JLabel)
			{
				name = ((JLabel)c).getText();
			}
			else
			if(c instanceof AbstractButton)
			{
				name = ((AbstractButton)c).getText();
			}
			
			// versuch ein Label fr das component zu finden
			if( name == null || name.equals("") ) //$NON-NLS-1$
			{
				if(c instanceof JComponent)
				{
					JLabel label = (JLabel) ((JComponent) c).getClientProperty("labeledBy");
					if(label != null)
						name = label.getText();
					
					// this is a hack for a 1.3 / 1.4 bug - see the test
					if( name != null && name.equals("File name") )
						name = "File Name";
				}
				
				// letzte Mglichkeit die Position des Elements, im root Element
				if( name == null || name.equals("") ) //$NON-NLS-1$
				{
					Rectangle rect = SwingUtilities.convertRectangle((Component)c, ((Component)c).getBounds(), null);
					name = boundsToName(rect.x, rect.y, rect.width, rect.height);
				}
			}
		}
		
		return name.trim();
	}
}
