package atosj.component.swing;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IEditbox;
import atosj.component.ISpinner;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>javax.swing.JSpinner</code> abgeleiteten Klassen.
 * @author Volker Janetschek
  */
public class SwingSpinner extends SwingComponent implements ISpinner
{
	/**
	 * Zeichnet die nderung des Textes im Spinner auf. 
	 * Generiert TEST Kommandos zu den Zustnden des Spinners (TEXT, NUM, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class JSpinnerCommandCreator extends SwingCommandCreator implements ChangeListener, FocusListener
	{
		protected boolean modified = false; // gibt an ob des Textfeld zwischen 2 aufeinander folgenden Focus gained und Focus lost ereignissen gendert wurde

		public JSpinnerCommandCreator(SwingSpinner text, IHTSCommandReceiver receiver) 
		{
			super(text, receiver);
			// TODO das funktioniert hier leider nicht, da der SwingEventMonitor nur Events von den
			// Basiscomponents, aus denen sich der Spinner zusammensetzt, weiterleitet.
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((IEditbox)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((IEditbox)getMComponent()).getNum();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
		
		public void stateChanged(ChangeEvent e) {
			modified = true;			
		}

		public void focusGained(FocusEvent arg0) {
			modified = false;
			
		}

		public void focusLost(FocusEvent arg0) 
		{
			System.out.println(Messages.getString("SwingSpinner.1") + modified); //$NON-NLS-1$
			if(modified)
			{
				try 
				{
					Vector params = new Vector(1);
					params.add(	((SwingSpinner)getMComponent()).getText() );
					publishActionCommand(HTSSyntax.ACTION_EDIT, params, null);
				} catch (ComponentException e) { /* egal */	}
			}
			
			modified = false;
		}
	};
	
	
	private SwingTextComponent textComp = null;
	
	public SwingSpinner(JSpinner component, ComponentId id) {
		super(component, id);
	}
	
	private SwingTextComponent getEditor() throws InvalidOperationException	{
		if (textComp == null)	{
			JComponent editorComponent = getSpinner().getEditor();
			
			if (!(editorComponent instanceof JSpinner.DefaultEditor))
				//TODO: korrekte Exception werfen, falls unbekannter Editor
				throw new InvalidOperationException(this, OP_SET_TEXT, true);
			
			JSpinner.DefaultEditor defaultEditor = (JSpinner.DefaultEditor)(getSpinner().getEditor());
			
			textComp = new SwingTextComponent(defaultEditor.getTextField(), getComponentId());
		}
		return textComp;
	}


	public String getText() throws ComponentException {
		assertReadable();
		return getEditor().getText();
	}
	
	public void assertEditable() throws ComponentException 
	{
		getEditor().assertEditable();
	}

	public void setText(String text) throws ComponentException
	{
		getEditor().setText(text);
	}

	private JSpinner getSpinner() {
		return (JSpinner) getAWTComponent();
	}

	public float getNum() throws ComponentException 
	{
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new JSpinnerCommandCreator(this, receiver);
	}
}
