package atosj.component.swing;

import java.util.Vector;

import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.ITabFolder;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>javax.swing.JTabbedPane</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingTAbbedPane extends SwingComponent implements ITabFolder
{
	/**
	 * Zeichnet den Wechsel zwischen den Tabs auf.
	 * Generiert TEST Kommandos zu den Zustnden der Combobox (TEXT,  
	 * ITEMCOUNT, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	private class TabbedPaneCommandCreator extends SwingCommandCreator implements ChangeListener
	{
		public TabbedPaneCommandCreator(SwingTAbbedPane tab, IHTSCommandReceiver receiver) 
		{
			super(tab, receiver);
			
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ITabFolder)getMComponent()).getText();
				values.add('"' + txt + '"');
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				int num = ((ITabFolder)getMComponent()).getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}

		public void stateChanged(ChangeEvent e) {
			int selection = ((JTabbedPane)e.getSource()).getSelectedIndex();
			if(selection != -1)
			{
				Vector subitemPath = new Vector(1);
				subitemPath.add( "" + selection ); //$NON-NLS-1$
				publishActionCommand(HTSSyntax.ACTION_SELECT, null, subitemPath);
			}
		}
	};
	
	public SwingTAbbedPane(JTabbedPane pane, ComponentId id) {
		super(pane, id);
	}
	
	private JTabbedPane getTabbedPane() {
		return (JTabbedPane) getAWTComponent();
	}

	private void revealSubItem(int index) {
		// TODO weiss nicht ob hier was geht
	}

	/**
	 * berprft, ob das angegebene Tab existiert.
	 * @param index Der Index des Tabs.
	 * @throws ComponentException wenn kein Tab an <code>index</code> existiert.
	 */
	private void assertIndex(int index) throws ComponentException {
		if( index < 0 || index >= getItemCount() )
		{
			int count = getItemCount();
			if( count > 0 )
				throw new ComponentException( Messages.getString("SwingTAbbedPane.2") + index + Messages.getString("SwingTAbbedPane.3") + count); //$NON-NLS-1$ //$NON-NLS-2$
			else
				throw new ComponentException( Messages.getString("SwingTAbbedPane.4") + index + Messages.getString("SwingTAbbedPane.5")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public void select(int index) throws ComponentException {
		assertEditable();
		assertIndex(index);
		revealSubItem(index);
		
		// also das pane schmeisst, bei setSelectedIndex selber ein ChangeEvent
		// mehr knnen wir wohl nicht tun
		getTabbedPane().setSelectedIndex(index);
	}

	public void select(String title) throws ComponentException {
		for(int i = 0; i < getTabbedPane().getTabCount(); ++i)
		{
			if(	getTabbedPane().getTitleAt(i).equals(title) )
				select(i);
		}
		
		throw new ComponentException(Messages.getString("SwingTAbbedPane.6") + title + Messages.getString("SwingTAbbedPane.7")); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public String getText() throws ComponentException {
		assertReadable();
		int index = getTabbedPane().getSelectedIndex(); 
		if(index == -1) // kein tab ausgewhlt
			return ""; //$NON-NLS-1$
		else
			return getTabbedPane().getTitleAt(index);
	}
	
	public String getText(int index) throws ComponentException {
		assertReadable();
		assertIndex(index);
		return getTabbedPane().getTitleAt(index);
	}

	public int getItemCount() throws ComponentException {
		return getTabbedPane().getTabCount();
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TabbedPaneCommandCreator(this, receiver);
	}
}
