/*
 * Created on 27.12.2005
 */
package atosj.component.swing;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IListCommandCreator;
import atosj.component.ITable;
import atosj.component.ITableCell;
import atosj.component.ListCommandCreatorHelper;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>javax.swing.JTable</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingTable extends SwingComponent implements ITable
{
	/**
	 * Zeichnet das Selektieren und Editieren von Eintrgen der Tabelle auf.
	 * Generiert TEST Kommandos zu den Zustnden der Tabelle/Tabellenzellen (TEXT, NUM, 
	 * ITEMCOUNT, CHECKSTATE, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TableCommandCreator extends SwingCommandCreator implements PropertyChangeListener, ListSelectionListener, IListCommandCreator
	{
		private ListCommandCreatorHelper helper;
		public TableCommandCreator(SwingTable table, IHTSCommandReceiver receiver) 
		{
			super(table, receiver);
			helper = new ListCommandCreatorHelper(this);
			
			// leider mssen wir hier den Listener lokal anhngen
			table.getTable().getSelectionModel().addListSelectionListener(this);
			addListener(this);
		}
		
		public Vector getTestCommands() 
		{
			SwingTable table = (SwingTable) getMComponent();
			Vector commands = super.getTestCommands();
			
			// ITEMCOUNT
			try {
				Vector values = new Vector(1);
				int num = table.getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			
			// Fr alle Spalten in den ausgewhlten Zeilen TEXT
			try {
				Object[] selection = getSelection();
				int colCount = table.getColCount();
				for(int i = 0; i < selection.length; ++i)
				{
					int row = ((Integer)selection[i]).intValue();
					for(int col = 0; col < colCount; ++col)
					{
						Vector values = new Vector(1);
						String txt = table.getTableCell(row , col).getText();
						values.add( '"' + txt + '"' );
						Vector path = new Vector(2);
						path.add( Integer.toString(row) );
						path.add( Integer.toString(col) );
						commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, path) );
					}
				}
			} catch(ComponentException e){}
			
			return commands;
		}
		
		public void propertyChange(PropertyChangeEvent evt) 
		{
			// TODO richtiges property herausfinden
			//System.out.println("Table Property Change: " + evt);
			Object source = evt.getSource();
			if (source instanceof JTable)	
			{
				JTable table = (JTable) source;
				int row = table.getEditingRow();
				int col = table.getEditingColumn();
				if (row == -1 || col == -1 ) return;
				Vector subItemPath = new Vector(2);
				subItemPath.add("" + row); //$NON-NLS-1$
				subItemPath.add("" + col); //$NON-NLS-1$
				Object value = table.getValueAt(row, col);
				
				if (value instanceof String || value instanceof Integer)	{
					Vector actionParams = new Vector(1);
					actionParams.add('"' + value.toString() + '"');
					publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, subItemPath);
				}
				else if (value instanceof Boolean)	
				{
					Boolean boolVal = (Boolean) value;
					if (boolVal.booleanValue() == true)
						publishActionCommand(HTSSyntax.ACTION_CHECK, null, subItemPath);
					else
						publishActionCommand(HTSSyntax.ACTION_UNCHECK, null, subItemPath);
					
				}
				//else ScriptRecorder.println(evt.getPropertyName()+": "+source.getClass());
				
			}
		}
		
		public void valueChanged(ListSelectionEvent e) {
			helper.selectionChanged();			
		}
		
		public Object[] getSelection() {
			return ListCommandCreatorHelper.convertToIntegerArray( ((SwingTable)getMComponent()).getTable().getSelectedRows() );
		}
		
		public Vector convertSelectionObjectToPath(Object selectionObject) {
			Vector path = new Vector(1);
			// das selectionObject ist ein listenindex als Integer
			path.add( selectionObject.toString() );
			return path;
		}
	}
	
	public SwingTable(JTable component, ComponentId id) {
		super(component, id);
	}

	private JTable getTable() {
		return (JTable) getAWTComponent();
	}
	
	/**
	 * Rckt die angegebene Tabellenzelle in den sichtbaren Bereich.
	 * @param row Zeilenindex der Tabellenzelle.
	 * @param col Spaltenindex der Tabellenzelle.
	 */
	private void revealSubitem(int row, int col) {
		getTable().scrollRectToVisible( getTable().getCellRect(row, col, false) );
	}
	
	/**
	 * berprft, ob die angegebene Tabellenzelle existiert.
	 * @param row Zeilenindex der Tabellenzelle.
	 * @param col Spaltenindex der Tabellenzelle.
	 * @throws ComponentException wenn die Tabellenzelle nicht existiert.
	 */
	private void assertSubitem(int row, int col) throws ComponentException 
	{
		if( row < 0 || row >= getTable().getRowCount() )
		{
			int count = getItemCount();
			if( count > 0 )
				throw new ComponentException( Messages.getString("SwingTable.3") + row + Messages.getString("SwingTable.4") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
			else
				throw new ComponentException( Messages.getString("SwingTable.5") + row + Messages.getString("SwingTable.6")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		if( col < 0 || col >= getTable().getColumnCount() )
		{
			int count = getTable().getColumnCount();
			if( count > 0 )
				throw new ComponentException( Messages.getString("SwingTable.7") + col + Messages.getString("SwingTable.8") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
			else
				throw new ComponentException( Messages.getString("SwingTable.9") + col + Messages.getString("SwingTable.10")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	public void select(int row) throws ComponentException {
		assertEditable();
		assertSubitem(row, 0);
		revealSubitem(row, 0);
		
		new SwingTableCell(this, row, 0).click(1, true, NO_MODIFIER);
	}

	public void deselect(int row) throws ComponentException {
		assertEditable();
		assertSubitem(row, 0);
		
		if( getTable().getSelectionModel().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_DESELECT, false);
		
		revealSubitem(row, 0);
		new SwingTableCell(this, row, 0).click(1, true, MODIFIER_CTRL);		
	}

	public void addSelect(int row) throws ComponentException {
		assertEditable();
		assertSubitem(row, 0);
		
		if( getTable().getSelectionModel().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_ADDSELECT, false);
		
		revealSubitem(row, 0);
		new SwingTableCell(this, row, 0).click(1, true, MODIFIER_CTRL);
	}

	public int getItemCount() throws ComponentException {
		return getTable().getRowCount();
	}
	
	public int getColCount() throws ComponentException {
		return getTable().getColumnCount();
	}

	public ITableCell getTableCell(int row, int col) throws ComponentException {
		assertSubitem(row, col);
		revealSubitem(row, col);
		return new SwingTableCell(this, row, col);
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TableCommandCreator(this, receiver);
	}
}
