package atosj.component.swing;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IComponent;
import atosj.component.IEditbox;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper sowohl fr ein- als auch fr mehrzeilige Textfelder, alle von 
 * <code>javax.swing.JTextComponent</code> abgeleitet.
 * @author Nicos Tegos
 */
public class SwingTextComponent extends SwingComponent implements IEditbox
{
	/**
	 * Zeichnet alle nderungen des Textes auf.
	 * Generiert TEST Kommandos zu den Zustnden der Combobox (TEXT, NUM, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TextComponentCommandCreator extends SwingCommandCreator implements FocusListener
	{
		protected boolean modified = false; // gibt an ob des Textfeld zwischen 2 aufeinander folgenden Focus gained und Focus lost ereignissen gendert wurde
		public TextComponentCommandCreator(SwingTextComponent text, IHTSCommandReceiver receiver) 
		{
			super(text, receiver);
			
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((IEditbox)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((IEditbox)getMComponent()).getNum();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
		
		public void focusGained(FocusEvent e) {
			modified = false;
		}

		public void focusLost(FocusEvent e) {
			generateEditAction( (JTextComponent)e.getComponent() );
		}
		
		protected void onKeyPressed(KeyEvent e) {
			if( e.getKeyChar() != KeyEvent.CHAR_UNDEFINED )
				modified = true;
			
			// wir generieren nur eine Edit Aktion nach dem Druck von Enter, wenn es ein
			// einzeiliges Textfeld ist.
			if( e.getComponent() instanceof JTextField && e.getKeyCode() == KeyEvent.VK_ENTER )
				generateEditAction( (JTextField)e.getComponent() );
			
			// TODO der Supercall funktioniert nicht so richtig super.onKeyPressed(e);
		}
		
		protected void generateEditAction(JTextComponent textField) {
			if(modified)
			{
				Vector actionParams = new Vector(1);
				actionParams.add( '"' + textField.getText().replaceAll("\r", "") + '"' ); //$NON-NLS-1$ //$NON-NLS-2$
				publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
			}
			modified = false;
		}
	};
	
	public SwingTextComponent(JTextComponent component, ComponentId id) {
		super(component, id);
	}

	public String getText() throws ComponentException {
		assertReadable();
		return getTextComponent().getText();
	}
	
	public void assertEditable() throws ComponentException 
	{
		super.assertEditable();
		if( !getTextComponent().isEditable() )
			throw new ComponentException(Messages.getString("SwingTextComponent.3")); //$NON-NLS-1$
	}

	public void setText(String text) throws ComponentException
	{
		assertEditable();
		selectAllText();

		if(text.length() > 0) 
		{
			for (int i = 0; i < text.length(); i++)
				pressKey( text.charAt(i), IComponent.NO_MODIFIER );
		}
		else // gesamten inhalt lschen durch drcken der delete Taste
			pressKey(KeyEvent.CHAR_UNDEFINED, KeyEvent.VK_DELETE, 0);
	}

	private JTextComponent getTextComponent() {
		return (JTextComponent) getAWTComponent();
	}

	/**
	 * Selektiert den gesamten Text durch Drcken der
	 * Tastenkombination Strg + A.
	 * @throws ComponentException, wenn nicht selektiert werden konnte.
	 */
	private void selectAllText() throws ComponentException
	{
		// wir versuchen alles zu selecten durch Strg + A
		// ein Doppelklick geht nicht, denn dieser selected u.U. nur einzelne worte
		pressKey('a', MODIFIER_CTRL);
		
		if (getTextComponent().getSelectionStart() != 0
			|| getTextComponent().getSelectionEnd() != getTextComponent().getText().length()) {
			throw new ComponentException(Messages.getString("SwingTextComponent.4")); //$NON-NLS-1$
		}
	}

	public float getNum() throws ComponentException 
	{
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TextComponentCommandCreator(this, receiver);
	}
}
