/*
 * @author: Jean Lazarou @date: 22 juil. 03
 */
package atosj.component.swing;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JToggleButton;

import atosj.component.ComponentId;
import atosj.component.ICheckbox;
import atosj.component.ICommandCreator;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>javax.swing.JToggleButton</code> abgeleiteten Klassen.
 * Diese werden wie Checkboxen behandelt.
 * @author Nicos Tegos
 */
public class SwingToggleButton extends SwingButton implements ICheckbox
{
	/**
	 * Zeichnet das Drcken des Knopfes auf.
	 */
	protected class ToggleButtonCommandCreator extends ButtonCommandCreator
	{
		public ToggleButtonCommandCreator(SwingToggleButton button, IHTSCommandReceiver receiver) {
			super(button, receiver);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				boolean val = ((ICheckbox)getMComponent()).isChecked();
				values.add( val ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE );
				commands.add( createTestCommand(HTSSyntax.READ_CHECKSTATE, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
		
		public void actionPerformed(ActionEvent e) {
			// im Gegensatz zur Superimplementation machen wir kein SELECT sonder CHECK oder UNCHECK
			publishActionCommand(((JToggleButton)e.getSource()).isSelected() ? HTSSyntax.ACTION_CHECK : HTSSyntax.ACTION_UNCHECK, null, null);
		}
	};
	
	public SwingToggleButton(JToggleButton button, ComponentId id) {
		super(button, id);
	}

	public boolean isChecked() throws ComponentException {
		assertReadable();
		return getToggleButtonComponent().isSelected();
	}
	
	public void setChecked(boolean checked) throws ComponentException
	{
		assertEditable();
		// den status nur setzen wenn er noch nicht gesetzt ist
		if( isChecked() != checked)
			select();
	}

	private JToggleButton getToggleButtonComponent() {
		return (JToggleButton) getAWTComponent();
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ToggleButtonCommandCreator(this, receiver);
	}
}
