/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;

import atosj.component.ComponentId;
import atosj.component.IButton;
import atosj.component.ICheckbox;
import atosj.component.ICommandCreator;
import atosj.component.IRadio;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Button</code> abgeleiteten Klassen.
 * Funktionalitt hngt vom &quot;style&quot; des Knopfes ab.
 * @author Nicos Tegos
 */
public class SWTButton extends SWTControl implements ICheckbox, IRadio
{
	/**
	 * Zeichnet das Drcken des Knopfes auf. Generiert TEST Kommandos zu den Zustnden
	 * des Knopfes (TEXT, CHECKSTATE, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ButtonCommandCreator extends ControlCommandCreator implements SelectionListener
	{
		public ButtonCommandCreator(SWTButton button, IHTSCommandReceiver receiver) {
			super(button, receiver);
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((IButton)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((IButton)getMComponent()).getNum();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				boolean val = ((ICheckbox)getMComponent()).isChecked();
				values.add( val ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE );
				commands.add( createTestCommand(HTSSyntax.READ_CHECKSTATE, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}

		public void widgetSelected(SelectionEvent e) {
			Button b = (Button)e.widget;
			
			if( (b.getStyle() & SWT.RADIO) != 0 )
			{
				// bei radio gibt es auch ein event fr den button der unchecked wird
				// wenn die selection wechselt, dieses event interessiert uns nicht
				if( b.getSelection() )
					publishActionCommand(HTSSyntax.ACTION_CHECK, null, null);
			}
			else
			if( (b.getStyle() & SWT.CHECK) != 0 )
			{
				publishActionCommand(b.getSelection() ? HTSSyntax.ACTION_CHECK : HTSSyntax.ACTION_UNCHECK, null, null);
			}
			else
			{
				publishActionCommand(HTSSyntax.ACTION_SELECT, null, null);
			}
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
	};
	
	public SWTButton(Button component, ComponentId id) {		
		super(component, id);
	}

	private Button getButton() {
		return (Button) getComponent();
	}
	
	public void setChecked(final boolean checked) throws ComponentException 
	{
		if( (getStyle() & SWT.CHECK) == 0 )
			throw new InvalidOperationException(this, OP_SET_CHECKED, true);
		
		assertEditable();
		if( isChecked() != checked )
			select();
	}
	
	public boolean isChecked() throws ComponentException 
	{
		if( (getStyle() & (SWT.CHECK | SWT.RADIO)) == 0 )
			throw new InvalidOperationException(this, OP_GET_CHECKED, true);
		
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getButton().getSelection());
			}
		};
		return ((Boolean)action.go(this, OP_GET_CHECKED)).booleanValue();
	}
	
	public String getText() throws ComponentException{
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return getButton().getText();
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}
	
	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}

	public void select() throws ComponentException {
		assertEditable();
		click(1, true, NO_MODIFIER);		
	}

	public void check() throws ComponentException {
		if( (getStyle() & SWT.RADIO) == 0 )
			throw new InvalidOperationException(this, OP_SET_CHECKED, true);
		
		assertEditable();
		if( isChecked() != true )
			click(1, true, NO_MODIFIER);
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ButtonCommandCreator(this, receiver);
	}
}
