package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICombobox;
import atosj.component.ICommandCreator;
import atosj.event.swt.NotifyEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Combo</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTCombo extends SWTControl implements ICombobox 
{
	/**
	 * Zeichnet das nderungen des Textes und Auswahl von Listeneintrgen auf. 
	 * Generiert TEST Kommandos zu den Zustnden der Combobox (TEXT, NUM, 
	 * ITEMCOUNT, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ComboCommandCreator extends ControlCommandCreator implements SelectionListener, FocusListener, ModifyListener
	{
		private boolean modified = false; // gibt an ob des Textfeld zwischen 2 aufeinander folgenden Focus gained und Focus lost ereignissen gendert wurde
		public ComboCommandCreator(SWTCombo combo, IHTSCommandReceiver receiver) 
		{
			super(combo, receiver);
			addListener(this);
		}
		
		private Combo getCombo() {
			return ((SWTCombo)getMComponent()).getCombo();
		}

		public void modifyText(ModifyEvent e) {
			modified = true;
		}

		public void focusGained(FocusEvent e) {
			modified = false;
		}

		public void focusLost(FocusEvent e) 
		{
			if( modified && (getCombo().getStyle() & SWT.READ_ONLY) == 0 )
			{
				Vector actionParams = new Vector(1);
				actionParams.add( '"' + getCombo().getText() + '"' );
				publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
			}
			
			modified = false;
		}

		public void widgetSelected(SelectionEvent e) 
		{
			int index = getCombo().getSelectionIndex();
			if(index != -1)
			{
				Vector subitemPath = new Vector(1);
				subitemPath.add( "" + getCombo().getSelectionIndex() ); //$NON-NLS-1$
				publishActionCommand(HTSSyntax.ACTION_SELECT, null, subitemPath);
			}
			
			modified = false;
		}

		public void widgetDefaultSelected(SelectionEvent e) {
			// wenn enter im textfeld gedrckt wurde
			if( modified && (getCombo().getStyle() & SWT.READ_ONLY) == 0 )
			{
				Vector actionParams = new Vector(1);
				actionParams.add( '"' + getCombo().getText() + '"' );
				publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
			}
			
			modified = false;
		}
		
		protected void onKeyPressed(KeyEvent e) {
			// wir wollen keine extra keyEvents capturen
		}
		
		public Vector getTestCommands() 
		{
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ICombobox)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((ICombobox)getMComponent()).getNum();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				int num = ((ICombobox)getMComponent()).getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
	};
	
	public SWTCombo(Combo component, ComponentId id) {
		super(component, id);
	}
	
	private Combo getCombo() {
		return (Combo) getComponent();
	}
	
	/**
	 * berprft, ob der angegebene Listenindex existiert.
	 * @param index Der Listenindex.
	 * @throws ComponentException, wenn der Listenindex nicht existiert.
	 */
	private void assertSubitem(final int index) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				if( index < 0 || index >= getCombo().getItemCount() )
				{
					int count = getCombo().getItemCount();
					if( count > 0 )
						throw new ComponentException( Messages.getString("SWTCombo.3") + index + Messages.getString("SWTCombo.4") + (getCombo().getItemCount()-1)); //$NON-NLS-1$ //$NON-NLS-2$
					else
						throw new ComponentException( Messages.getString("SWTCombo.5") + index + Messages.getString("SWTCombo.6")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Sucht in der Liste der Combobox nach dem Eintrag mit dem
	 * angegebenen Text.
	 * @param text Text des Eintrages.
	 * @return Index des Eintrages.
	 * @throws ComponentException, wenn kein Eintrag gefunden wurde.
	 */
	private int findItem(final String text) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				for(int i = 0; i < getItemCount(); ++i)
					if( getCombo().getItem(i).equals(text) )
						return new Integer(i);
				
				throw new ComponentException(Messages.getString("SWTCombo.7") + text + Messages.getString("SWTCombo.8")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		};
		
		return ((Integer)action.go()).intValue();
	}
	
	public void select(final int index) throws ComponentException 
	{
		assertEditable();
		assertSubitem(index);
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				// wir knnen nicht garantieren, dass wir das ganze mit mouse events hinbekommen
				// theoretisch knnte man ganz rechts auf die clientArea der Combobox klicken,
				// um den button zu erwischen, der die liste ffnet, doch dann kann man die liste
				// nicht erwischen um dort zu selektieren (sie ist betriebssystem intern)
				getCombo().select(index);
				new NotifyEvent(SWTCombo.this).fire(SWT.Selection);
				
				return null;
			}
		};
		
		action.go(this, OP_SELECT);
	}

	public void select(String text) throws ComponentException {
		select( findItem(text) );
	}

	public void setText(final String text) throws ComponentException 
	{
		assertEditable();
				
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable
			{
				// wenn editierbar dann setzen wir den text in der editbox
				if( (getCombo().getStyle() & SWT.READ_ONLY) != 0 )
					throw new ComponentException(Messages.getString("SWTCombo.9"));				 //$NON-NLS-1$
				return null;
			}
		};
		
		action.go(this, OP_SET_TEXT);
		
		// wir brauchen vorher nicht den kompletten Text zu
		// selektieren da dies automatisch beim setzen des Focus geschieht
		if(text.length() > 0)
		{
			for(int i = 0; i < text.length(); ++i)
				pressKey(text.charAt(i), NO_MODIFIER);
		}
		else // vorhanden Text lschen
			pressKey('\0', SWT.DEL, 0);

	}

	public int getItemCount() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer( getCombo().getItemCount() );
			}
		};
		
		return ((Integer)action.go(this, OP_GET_ITEMCOUNT)).intValue();
	}

	/*public String getText(final int index) throws ComponentException 
	{
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return getCombo().getItem(index);
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}*/
	
	public String getText() throws ComponentException 
	{
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return getCombo().getText();
			}
		};
		
		return (String)action.go(this, OP_GET_TEXT);
	}

	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ComboCommandCreator(this, receiver);
	}
}
