/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Widget;

import atosj.component.ComponentFinder;
import atosj.component.CustomComponentResolver;
import atosj.exception.ComponentException;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.SWTWindowMonitor;

/**
 * Der <code>ComponentFinder</code> fr die SWT Architektur.
 * @author Nicos Tegos
 */
public class SWTComponentFinder extends ComponentFinder 
{
	/**
	 * Erstellt einen <code>SWTComponentFinder</code> mit einem <code>SWTComponentResolver</code>
	 * und einer <code>SWTNamingStrategy</code>.
	 */
	public SWTComponentFinder(SWTComponentResolver resolver, IHTSCommandReceiver receiver) {
		super( resolver, new SWTNamingStrategy(), receiver );
	}
	
	public Object[] getChildren(final Object component) 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			protected Object performAction() throws Throwable 
			{
				Widget[] children = null;
				
				if( (getResolver() instanceof CustomComponentResolver) 
						&& ((CustomComponentResolver)getResolver()).isCustomComponent(component) )
				{
					children = null;
				}
				else
				if(component instanceof ToolBar) 
				{
					children = ((ToolBar)component).getItems();
				}
				else if(component instanceof CoolBar)
				{
					children = ((CoolBar)component).getItems();
				}
				else if(component instanceof Shell)
				{
					Menu menu = ((Shell)component).getMenuBar();
					if(menu != null)
					{
						Widget[] origChildren = ((Shell)component).getChildren();
						children = new Widget[origChildren.length+1];
						System.arraycopy(origChildren, 0, children, 0, origChildren.length);
						children[children.length-1] = menu;
					}
					else
						children = ((Shell)component).getChildren();
				}
				else if(component instanceof Composite)
				{
					if (component instanceof Table
						|| component instanceof Tree
						|| component instanceof Combo)
						children = null;
					else
						children = ((Composite)component).getChildren();
				}
				
				
				return children;
			}
		};
		
		try {
			return (Widget[]) action.go();
		} catch (ComponentException e) {
			return null;
		}
	}

	public Object getWindow(final Object component) 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return SWTWindowMonitor.getShell( (Widget)component );
			}
		};
		
		try {
			return action.go();
		} catch (ComponentException e) {
			return null;
		}
	}
}
