/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Widget;

import atosj.component.ComponentId;
import atosj.component.CustomComponentResolver;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Liefert alle <code>MComponents</code> zu Klassen der SWT Architektur.
 * @author Nicos Tegos
 */
public class SWTComponentResolver extends CustomComponentResolver 
{
	public SWTComponentResolver(String filePath) {
		super(filePath);
	}
	
	public MComponent getMComponent(Object component, ComponentId id) 
	{
		// zuerst berprfen ob ein custom component definiert wurde
		MComponent customComponent = super.getMComponent(component, id);
		if(customComponent != null)
			return customComponent;
		
		// kein custom component also weiter mit den standard components
		if (component instanceof Label)
		{
			return new SWTLabel( (Label)component, id);
		}
		else
		if (component instanceof Spinner)
		{
			return new SWTSpinner( (Spinner)component, id);
		}
		else
		if (component instanceof Button)
		{
			return new SWTButton( (Button)component, id);
		}
		else
		if (component instanceof ToolItem)
		{
			return new SWTToolItem( (ToolItem)component, id);
		}
		else
		if (component instanceof Text)
		{
			return new SWTTextField( (Text)component, id);
		}
		else
		if (component instanceof List)
		{
			return new SWTList( (List)component, id);
		}
		else
		if	(component instanceof Table)
		{
			return new SWTTable( (Table)component, id);
		}	
		else
		if 	(component instanceof Tree)
		{
			return new SWTTree( (Tree)component, id);
		}	
		else
		if 	(component instanceof Menu)
		{
			return new SWTMenu( (Menu)component, id);
		}
		else
		if 	(component instanceof TabFolder)
		{
			return new SWTTabFolder( (TabFolder)component, id);
		}
		else
		if	(component instanceof Combo)
		{
			return new SWTCombo( (Combo)component, id);
		}
		else
		if 	(component instanceof Shell)
		{
			return new SWTShell( (Shell)component, id);
		} 
		
		// low level items TreeItem, MenuItem, TableItem werden nicht gesondert aufgefhrt
		return new SWTComponent( (Widget)component, id );
	}

	public String getTypeName(final Object component) 
	{
		// zuerst berprfen ob ein custom component definiert wurde
		String customClassName = super.getTypeName(component);
		if( !customClassName.equals("") )
			return customClassName;
		
		// kein custom component also weiter mit den standard namen
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			protected Object performAction() throws Throwable 
			{				
				if (component instanceof Label)
				{
					return HTSSyntax.TYPE_LABEL;
				}
				else
				if (component instanceof Button)
				{
					Button b = (Button) component;
					if( (b.getStyle() & SWT.CHECK) == SWT.CHECK)
						return HTSSyntax.TYPE_CHECKBOX;
					if( (b.getStyle() & SWT.RADIO) == SWT.RADIO)
						return HTSSyntax.TYPE_RADIOBUTTON;
					if( (b.getStyle() & SWT.TOGGLE) == SWT.TOGGLE)
						return HTSSyntax.TYPE_CHECKBOX;
					
					return HTSSyntax.TYPE_BUTTON;
				}
				else
				if (component instanceof ToolItem)
				{
					ToolItem i = (ToolItem) component;
					if( (i.getStyle() & SWT.CHECK) == SWT.CHECK)
						return HTSSyntax.TYPE_CHECKBOX;
					if( (i.getStyle() & SWT.RADIO) == SWT.RADIO)
						return HTSSyntax.TYPE_RADIOBUTTON;
					
					return HTSSyntax.TYPE_BUTTON;
				}
				else
				if (component instanceof Spinner)
				{
					return HTSSyntax.TYPE_SPINNER;
				}
				else
				if (component instanceof Text)
				{
					return HTSSyntax.TYPE_EDITBOX;
				}
				else
				if (component instanceof Tree)
				{
					return HTSSyntax.TYPE_TREE;
				}
				else
				if (component instanceof Table)
				{
					return HTSSyntax.TYPE_TABLE;
				}
				else
				if (component instanceof List)
				{
					return HTSSyntax.TYPE_LIST;
				}
				else
				if (component instanceof Menu)
				{
					return HTSSyntax.TYPE_MENU;
				}
				else
				if (component instanceof Shell)
				{
					return HTSSyntax.TYPE_WINDOW;
				}
				else
				if (component instanceof TabFolder)
				{
					return HTSSyntax.TYPE_TABFOLDER;
				}
				else
				if (component instanceof Combo)
				{
					return HTSSyntax.TYPE_COMBOBOX;
				}
				
				return HTSSyntax.TYPE_COMPONENT;
			}
		};
		
		try {
			return (String) action.go();
		} catch (ComponentException e) {
			return HTSSyntax.TYPE_COMPONENT;
		}
	}

	public String[] getTypeNames() 
	{
		String[] customNames = super.getTypeNames();
		String[] myNames = new String[]
			{
				HTSSyntax.TYPE_BUTTON,
				HTSSyntax.TYPE_CHECKBOX,
				HTSSyntax.TYPE_COMBOBOX,
				HTSSyntax.TYPE_COMPONENT,
				HTSSyntax.TYPE_SPINNER,
				HTSSyntax.TYPE_EDITBOX,
				HTSSyntax.TYPE_LABEL,
				HTSSyntax.TYPE_LIST,
				HTSSyntax.TYPE_MENU,
				HTSSyntax.TYPE_RADIOBUTTON,
				HTSSyntax.TYPE_TABFOLDER,
				HTSSyntax.TYPE_TABLE,
				HTSSyntax.TYPE_TREE,
				HTSSyntax.TYPE_WINDOW,
			};
		// custom namen und standardnamen zusammenfassen
		String[] allNames = new String[customNames.length + myNames.length];
		System.arraycopy(customNames, 0, allNames, 0, customNames.length);
		System.arraycopy(myNames, 0, allNames, customNames.length, myNames.length);
		
		return allNames;
	}

	public boolean isCollectionComponent(String type) 
	{
		if( super.isCollectionComponent(type)
				|| type.equals(HTSSyntax.TYPE_COMBOBOX) 
				|| type.equals(HTSSyntax.TYPE_LIST)
				|| type.equals(HTSSyntax.TYPE_MENU)
				|| type.equals(HTSSyntax.TYPE_TABFOLDER)
				|| type.equals(HTSSyntax.TYPE_TABLE)
				|| type.equals(HTSSyntax.TYPE_TREE)
		)
			return true;
		
		return false;
	}
}
