/*
 * Created on 25.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;

import atosj.component.ComponentId;
import atosj.event.swt.NotifyEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Control</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public abstract class SWTControl extends SWTComponent 
{
	/**
	 * Zeichnet das Tastendrcke und Mausklicks auf. 
	 * Generiert TEST Kommandos zu den Zustnden des Controls (ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ControlCommandCreator extends SWTCommandCreator 
	{
		public ControlCommandCreator(SWTControl comp, IHTSCommandReceiver receiver) {
			super(comp, receiver);
		}

		protected void hookTestListener(MouseListener listener) {
			// erst hier ist es mglich Mouse- und KeyListener anzuhngen in der Klasse Widget nicht.
			addListener(listener);		
		}

		protected void hookKeyListener(KeyListener listener) {
			addListener(listener);
		}

		protected void hookMouseListener(MouseListener listener) {
			addListener(listener);
		}
		
	}
	
	public SWTControl(Control component, ComponentId id) {
		super(component, id);
	}
	
	protected Control getControl() {
		return (Control) getComponent();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.swt.SWTComponent#getPoint()
	 */
	public Point getPoint() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				Rectangle rect = getControl().getBounds();
				Point middle = getControl().toDisplay(rect.width/2, 
						rect.height/2);
				
				return middle;
			}
		};
		return (Point)action.go();
	}
	
	public void requestFocus() throws ComponentException 
	{
		if( hasFocus() )
			return;
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				boolean ret = getControl().setFocus();
				if(ret)
					new NotifyEvent(SWTControl.this).fire(SWT.FocusIn);
				
				return new Boolean(ret);
			}
		};
		
		if( !((Boolean)action.go()).booleanValue() ) // der Focus wurde nicht gesetzt
			throw new InvalidOperationException(this, OP_SET_FOCUS, false);
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#hasFocus()
	 */
	public boolean hasFocus() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getControl().isFocusControl());
			}
		};
		return ((Boolean)action.go(this, OP_GET_FOCUS)).booleanValue();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#isEnabled()
	 */
	public boolean isEnabled() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getControl().isEnabled());
			}
		};
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#isVisible()
	 */
	public boolean isVisible() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getControl().isVisible());
			}
		};
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
}
