/*
 * Created on 14.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.widgets.Item;

import atosj.component.ComponentId;
import atosj.component.IButton;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Wrapper fr alle von <code>Item</code> abgeleiteten Klassen.
 * Die Funktionen <code>isEnabled</code>  und <code>isVisible</code>
 * mssen in den Unterklassen implementiert werden.
 * @author Nicos Tegos
 */
public abstract class SWTItem extends SWTComponent implements IButton
{
	public SWTItem(Item component, ComponentId id) {
		super(component, id);
	}
	
	protected Item getItem() {
		return (Item) getComponent();
	}
	
	public String getText() throws ComponentException {
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return ((Item)getSWTComponent()).getText();
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}
	
	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public void select() throws ComponentException {
		assertEditable();
		click(1, true, NO_MODIFIER);
	}
	
}
