/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.widgets.Label;

import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.ILabel;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Label</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTLabel extends SWTControl implements ILabel 
{
	/**
	 * Generiert TEST Kommandos zu den Zustnden des Labels (TEXT, NUM, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class LabelCommandCreator extends ControlCommandCreator
	{
		public LabelCommandCreator(SWTLabel comp, IHTSCommandReceiver receiver) {
			super(comp, receiver);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ILabel)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((ILabel)getMComponent()).getNum();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
	}
	public SWTLabel(Label component, ComponentId id) {
		super(component, id);
	}
	
	public String getText() throws ComponentException {
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return ((Label)getSWTComponent()).getText();
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}

	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new LabelCommandCreator(this, receiver);
	}
}
