/*
 * Created on 06.05.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.List;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IList;
import atosj.component.IListCommandCreator;
import atosj.component.ListCommandCreatorHelper;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>List</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTList extends SWTControl implements IList 
{
	/**
	 * Zeichnet die Auswahl von Eintrgen auf.
	 * Generiert TEST Kommandos zu den Zustnden der Liste (TEXT, ITEMCOUNT, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ListCommandCreator extends ControlCommandCreator implements SelectionListener, IListCommandCreator
	{
		private ListCommandCreatorHelper helper;
		
		public ListCommandCreator(SWTList list, IHTSCommandReceiver receiver) {
			super(list, receiver);
			helper = new ListCommandCreatorHelper(this); 
			
			addListener(this);
		}

		public void widgetSelected(SelectionEvent e) {
			helper.selectionChanged();
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
		public Object[] getSelection() 
		{
			SWTThreadSafeAction action = new SWTThreadSafeAction() {
				protected Object performAction() throws Throwable {
					return ListCommandCreatorHelper.convertToIntegerArray( ((SWTList)getMComponent()).getList().getSelectionIndices() );
				}
			};
			
			try {
				return (Object[])action.go();
			} catch (ComponentException e) {
				// irgendwas ist schiefgelaufen z.B. list schon disposed
				return new Object[0];
			}
		}

		public Vector convertSelectionObjectToPath(Object selectionObject) {
			Vector path = new Vector(1);
			// das selectionObject ist ein listenindex als Integer
			path.add( selectionObject.toString() );
			return path;
		}
		
		public Vector getTestCommands() 
		{
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				int num = ((IList)getMComponent()).getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			Object[] selection = getSelection();
			for(int i = 0; i < selection.length; ++i)
			{
				try {
					Vector values = new Vector(1);
					String txt = ((IList)getMComponent()).getText( ((Integer)selection[i]).intValue() );
					values.add( '"' + txt + '"' );
					commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, convertSelectionObjectToPath(selection[i])) );
				} catch(ComponentException e){}
			}
			
			return commands;
		}
	};
	
	public SWTList(List component, ComponentId id) {
		super(component, id);
	}
	
	private List getList() {
		return (List) getComponent();
	}
	
	/**
	 * Rckt den Listeneintrag an Position <code>index</code>
	 * in den sichtbaren Bereich.
	 * @param index Der Index des Eintrages, 0-basiert.
	 */
	private void revealSubitem(final int index) throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				// Code aus org.eclipse.jface.viewers.ListViewer#reveal(Object)
				List list = getList();
				int count = list.getItemCount();
				int height = list.getItemHeight();
				Rectangle rect = list.getClientArea();
				int topIndex = list.getTopIndex();
				int visibleCount = Math.max ((rect.x + rect.height) / height, 1);
				int bottomIndex = Math.min (topIndex + visibleCount + 1, count - 1);
				if ((topIndex <= index) && (index <= bottomIndex)) return null;
				int newTop = Math.min (Math.max (index - (visibleCount / 2), 0), count - 1);
				list.setTopIndex(newTop);
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Stellt die Gltigkeit des angegebenen Listenindex sicher.
	 * @param index Der Listenindex.
	 * @throws ComponentException wenn der Listenindex ungltig ist.
	 */
	private void assertSubitem(final int index) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				if( index < 0 || index >= getList().getItemCount() )
				{
					int count = getList().getItemCount();
					if( count > 0 )
						throw new ComponentException( Messages.getString("SWTList.1") + index + Messages.getString("SWTList.2") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
					else
						throw new ComponentException( Messages.getString("SWTList.3") + index + Messages.getString("SWTList.4")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * berprft, ob die Liste die Auswahl mehrerer Eintrge erlaubt.
	 * @throws ComponentException wenn die Liste keine Mehrfachauswahl untersttzt.
	 */
	private void assertMulti() throws ComponentException
	{
		if( (getStyle() & SWT.MULTI) == 0 )
			throw new ComponentException(Messages.getString("SWTList.5")); //$NON-NLS-1$
	}
	
	/**
	 * Gibt an, ob der Eintrag bei <code>row</code> ausgewhlt ist.
	 * @param row Der Index des Eintrags.
	 * @return <code>true</code> wenn ausgewhlt, <code>false</code> sonst.
	 */
	private boolean isSelected(final int row) throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				int[] selection = getList().getSelectionIndices();
				for (int i = 0; i < selection.length; i++) 
				{
					if(selection[i] == row)
						return Boolean.TRUE;
				}
				
				return Boolean.FALSE;
			}
		};
		
		return ((Boolean)action.go()).booleanValue();
	}
	
	/**
	 * Simuliert einen Klick auf die Mitte des angegebenen Listeneintrags.
	 * Der Eintrag muss vorher sichtbar gemacht werden.
	 * @param index Der Index des Eintrages, 0-basiert.
	 * @see #revealSubitem(int)
	 */
	public void clickOnItem(final int index, final int modifier) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				List list = getList();
				int x = list.getBounds().width / 2;
				// unter der Bedingung, dass wir vorher revealen 
				// ist der aktuelle index immer >= topIndex, 
				// somit ist y nicht negativ
				int y = (index - list.getTopIndex()) * list.getItemHeight();
				if( y == 0)
					y = list.getItemHeight() / 2;
				Point p = list.toDisplay(x, y);
				
				click(1, true, modifier, p.x, p.y);
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Sucht nach dem Listeneintrag mit dem angegebenen Text.
	 * @param text Der Text des gesuchten Eintrages.
	 * @return den Index des gefundenen Eintrages.
	 * @throws ComponentException wenn kein Eintrag mit <code>text</code> gefunden wurde.
	 */
	private int findItem(final String text) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				for(int i = 0; i < getItemCount(); ++i)
					if( getList().getItem(i).equals(text) )
						return new Integer(i);
				
				throw new ComponentException(Messages.getString("SWTList.6") + text + Messages.getString("SWTList.7")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		};
		
		return ((Integer)action.go()).intValue();
	}

	public void select(String item) throws ComponentException {
		select( findItem(item) );
	}

	public void select(int index) throws ComponentException {
		assertEditable();
		assertSubitem(index);
		revealSubitem(index);
		
		clickOnItem(index, NO_MODIFIER);
	}

	public void deselect(int index) throws ComponentException 
	{
		assertEditable();
		assertSubitem(index);
		assertMulti();
		
		revealSubitem(index);
		if( isSelected(index) )
			clickOnItem(index, MODIFIER_CTRL);
	}

	public void deselect(String item) throws ComponentException {
		deselect( findItem(item) );
	}

	public void addSelect(int index) throws ComponentException 
	{
		assertEditable();
		assertSubitem(index);
		assertMulti();
		
		revealSubitem(index);
		if( !isSelected(index) )
			clickOnItem(index, MODIFIER_CTRL);
	}

	public void addSelect(String item) throws ComponentException {
		addSelect( findItem(item) );
	}

	public int getItemCount() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer( getList().getItemCount() );
			}
		};
		
		return ((Integer)action.go(this, OP_GET_ITEMCOUNT)).intValue();
	}

	public String getText(final int index) throws ComponentException 
	{
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return getList().getItem(index);
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ListCommandCreator(this, receiver);
	}
}
