package atosj.component.swt;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Widget;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IMenu;
import atosj.exception.ComponentException;
import atosj.exception.ComponentNotFoundException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Menu</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTMenu extends SWTComponent implements IMenu 
{
	/**
	 * Zeichnet die Auswahl eines Untermenpunktes auf. 
	 * Generiert TEST Kommandos zu den Zustnden der Untermenpunkte (TEXT, 
	 * CHECKSTATE, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class MenuCommandCreator extends SWTCommandCreator implements SelectionListener
	{
		private HashMap itemToPathMap = new HashMap(); // HashMap<JMenuItem, Vector<String>> mapped ein item auf seinen subitemPath
		public MenuCommandCreator(SWTMenu menu, IHTSCommandReceiver receiver) {
			super(menu, receiver);
			
			SWTThreadSafeAction action = new SWTThreadSafeAction() {
				protected Object performAction() throws Throwable {
					initMap( ((SWTMenu)getMComponent()).getMenu(), new Vector());
					return null;
				}
			};
			
			try {
				action.go();
			} catch (ComponentException e) {
				// tja da knnen wir dann auch nix machen
			}
		}
		
		private void initMap(Menu parent, Vector path)
		{
			MenuItem[] items = parent.getItems();
			
			for(int i = 0; i < items.length; ++i) 
			{
				MenuItem item = items[i];
				if( item.getMenu() != null ) // ein kaskadierendes men
				{
					// der menpunkt selbst interessiert uns nicht, aber dessen Kinder
					// TODO leider ist das Root men mit im pfad enthalten, da sein erstes item
					// das Men selbst ist
					Vector myPath = new Vector(path);
					myPath.add( '"' + item.getText() + '"');
					initMap( item.getMenu(), myPath);
				}
				else
				{
					Vector myPath = new Vector(path);
					myPath.add( '"' + item.getText() + '"');
					itemToPathMap.put( item, myPath );
					addListener(item, this);
				}
			}
		}

		public void widgetSelected(SelectionEvent e) 
		{
			MenuItem item = (MenuItem)e.widget;
			if( (item.getStyle() & SWT.CHECK) == SWT.CHECK )
			{
				publishActionCommand( item.getSelection() ? HTSSyntax.ACTION_CHECK : HTSSyntax.ACTION_UNCHECK, 
						null, 
						(Vector) itemToPathMap.get(item)
					);			
			}
			else if( (item.getStyle() & SWT.RADIO) == SWT.RADIO )
			{
				if( item.getSelection() )
					publishActionCommand( HTSSyntax.ACTION_CHECK, null, (Vector) itemToPathMap.get(item) );
			}
			else
			{
				publishActionCommand( HTSSyntax.ACTION_SELECT, null, (Vector) itemToPathMap.get(item) );
			}
		}

		public void widgetDefaultSelected(SelectionEvent arg0) {}

		protected void hookTestListener(MouseListener listener) {
			// TODO Auto-generated method stub
			
		}

		protected void hookKeyListener(KeyListener listener) {
			// TODO Auto-generated method stub
			
		}

		protected void hookMouseListener(MouseListener listener) {
			// TODO Auto-generated method stub
			
		}
	}
	
	public SWTMenu(Menu component, ComponentId id) {
		super(component, id);
	}
	
	private Menu getMenu() {
		return (Menu) getComponent();
	}

	/**
	 * Sucht einen Menpunkt entsprechend des Suchpfades, und macht
	 * ihn, durch Klicken auf die Obermens sichtbar.
	 * @param path Der Suchpfad.
	 * @return Das MenuItem.
	 * @throws ComponentException wenn nicht gefunden.
	 */
	private SWTMenuItem revealSubitem(final String[] path) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return new SWTMenuItem( findItem( getMenu(), path, 0, true), createIdForSubItem(path) );
			}
		};
		
		return (SWTMenuItem) action.go();
	}
	
	/**
	 * Rekursive Suche nach einem Menpunkt. Sucht immer in der aktuellen Menebene
	 * nach einem Menpunkt, der die Bezeichnung trgt, die an Position <code>searchIndex</code>
	 * im Suchpfad steht. <em>ACHTUNG:<em> luft nicht im SWT Thread.
	 * @param parent Das <code>Menu</code> dessen Kinder durchsucht werden 
	 * oder das <code>MenuItem</code> dessen Kinder durchsucht werden.
	 * @param search Der Suchpfad, Text aller Obermens und des gesuchten Menpunktes,
	 * beginnend mit dem Wurzelmen.
	 * @param searchIndex Der aktuelle Index im Suchpfad, alle vorhergenden items wurden schon gefunden.
	 * @param reveal wenn <code>true</code> dann werden die Mens durch Klicken aufgeklappt.
	 * @return Das gesuchte <code>MenuItem</code>.
	 * @throws ComponentException wenn das Item nicht gefunden wurde.
	 */
	private MenuItem findItem(Widget parent, String[] search, int searchIndex, boolean reveal) throws ComponentException
	{
        String text = (String) search[searchIndex];
		
        MenuItem[] items = getChildren(parent);
        for (int i = 0; i < items.length; i++) {
            MenuItem item = items[i];
            if( item.getText().equals(text) )
            {
            	if( search.length-1 == searchIndex ) // wir haben das letzte gefunden
            		return item;
            	else
            	{
            		if( (item.getStyle() & SWT.CASCADE) != 0 )
            		{
            			if(reveal)
            			{
            				String[] currentPath = new String[searchIndex+1];
            				System.arraycopy(search, 0, currentPath, 0, currentPath.length);
            				new SWTMenuItem(item, createIdForSubItem(currentPath) ).select();
            			}
            			return findItem(item, search, ++searchIndex, reveal);
            		}
            		else // zu frh gefunden (ein Menitem vor ende des Pfades, das kein untermen enthlt)
            		    throw new ComponentException(Messages.getString("SWTMenu.0") + (i+1) + Messages.getString("SWTMenu.1") + search[i] + Messages.getString("SWTMenu.2")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            	}
            }
        }
        
        throw new ComponentNotFoundException( getComponentId(), Messages.getString("SWTMenu.3") + (searchIndex+1) + Messages.getString("SWTMenu.4") + search[searchIndex] + Messages.getString("SWTMenu.5") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }
	
	/**
	 * Sucht zum angegebenen <code>Menu</code> oder <code>MenuItem</code>
	 * alle Untermenpunkte bzw. alle Menpunkte der selben Ebene.
	 * <em>ACHTUNG:</em> luft nicht im SWT Thread.
	 * @param parent Das <code>Menu</code> oder <code>MenuItem</code>.
	 * @return Die
	 */
	private MenuItem[] getChildren(Widget parent) 
	{
		if(parent instanceof Menu)
			return ((Menu)parent).getItems();
		if(parent instanceof MenuItem)
		{
			Menu cascade = ((MenuItem)parent).getMenu();
			if(cascade != null)
				return cascade.getItems();
			else
				return new MenuItem[0];
		}
		return null;
	}
	
	private ComponentId createIdForSubItem(String[] path) {
		// ComponentId fr das SubElement erstellen
		String pathStr = ""; //$NON-NLS-1$
		for(int i = 0; i < path.length; ++i) 
		{
			pathStr += path[i];
			if(i < path.length-1)
				pathStr += "->"; //$NON-NLS-1$
		}
		
		ComponentId subId = new ComponentId(
				getComponentId().getName() + Messages.getString("SWTMenu.8") + pathStr, //$NON-NLS-1$
				getComponentId().getClassName(),
				getComponentId().getWindowName()
		);
		
		return subId;
	}
	
	public void select(String[] path) throws ComponentException {
		assertEditable();
		
		revealSubitem(path).select();
	}

	public void setChecked(String[] path, boolean check) throws ComponentException {
		assertEditable();
		
		revealSubitem(path).setChecked(check);
	}

	public boolean isChecked(final String[] path) throws ComponentException {
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return new Boolean( new SWTMenuItem( findItem( getMenu(), path, 0, false), createIdForSubItem(path) ).isChecked() );
			}
		};
		
		return ((Boolean)action.go()).booleanValue();
	}
	
	public boolean isEnabled(final String[] path) throws ComponentException {
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return new Boolean( new SWTMenuItem( findItem( getMenu(), path, 0, false), createIdForSubItem(path) ).isEnabled() );
			}
		};
		
		return ((Boolean)action.go()).booleanValue();
	}
	
	public Point getPoint() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				// leider hat die SWT Crew den public modifier der getBounds 
				// Funktion auskommentiert (die schweine) darum versuchen wir
				// es so:
				Method getBounds = getMenu().getClass().getMethod("getBounds", new Class[0]); //$NON-NLS-1$
				// Rectangle is already relative to Display
				Rectangle rect = (Rectangle) getBounds.invoke( getMenu(), new Object[0] );
				
				return new Point(rect.x + rect.width/2, rect.y + rect.height/2);
			}
		};
		
		return (Point) action.go();
	}
	
	public boolean isEnabled() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getMenu().isEnabled());
			}
		};
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	public boolean isVisible() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getMenu().isVisible());
			}
		};
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	public boolean hasFocus() throws ComponentException 
	{
		// Wir returnen immer false, weil die Klasse Menu
		// keine Funktion zum Abrufen des Fokuszustands hat
		return false;
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new MenuCommandCreator(this, receiver);
	}
}
