package atosj.component.swt;

import java.lang.reflect.Method;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.MenuItem;

import atosj.component.ComponentId;
import atosj.component.ICheckbox;
import atosj.component.IRadio;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Wrapperklasse fr alle von <code>MenuItem</code> abgeleiteten Klassen.
 * Diese Klasse reprsentiert einen Menpunkt.
 * Diese Klasse darf nur von einem Men erzeugt werden.
 * @author Nicos Tegos
 */
public class SWTMenuItem extends SWTItem implements ICheckbox, IRadio {

	public SWTMenuItem(MenuItem component, ComponentId id) {
		super(component, id);
	}

	protected MenuItem getMenuItem() {
		return (MenuItem) getComponent();
	}
	
	public Point getPoint() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			protected Object performAction() throws Throwable 
			{
				// leider hat die SWT Crew den public modifier der getBounds 
				// Funktion auskommentiert (die schweine) darum versuchen wir
				// es so:
				Method getBoundsItem = getMenuItem().getClass().getMethod("getBounds", new Class[0]);
				Rectangle itemRect = (Rectangle) getBoundsItem.invoke( getMenuItem(), new Object[0] );
				
				// beim Menu sind die bounds schon relativ zum Display
				Method getBoundsParent = getMenuItem().getParent().getClass().getMethod("getBounds", new Class[0]);
				Rectangle parentRect = (Rectangle) getBoundsParent.invoke( getMenuItem().getParent(), new Object[0] );
				
				return new Point(parentRect.x + itemRect.x+ itemRect.width/2, 
						parentRect.y + itemRect.y + itemRect.height/2);
			}
		};
		
		return (Point) action.go();
	}
	
	public boolean isChecked() throws ComponentException {
		if( (getStyle() & (SWT.CHECK | SWT.RADIO)) == 0 )
			throw new InvalidOperationException(this, OP_GET_CHECKED, true);
		
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getMenuItem().getSelection());
			}
		};
		return ((Boolean)action.go(this, OP_GET_CHECKED)).booleanValue();
	}
	
	
	public void setChecked(final boolean checked) throws ComponentException {
		if( (getStyle() & (SWT.CHECK | SWT.RADIO)) == 0 )
			throw new InvalidOperationException(this, OP_SET_CHECKED, true);
		
		assertEditable();
		if( isChecked() != checked )
		{
			click(1, true, NO_MODIFIER);
		}
	}
	
	public void check() throws ComponentException {
		setChecked(true);		
	}
	
	public boolean isEnabled() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getMenuItem().isEnabled());
			}
		};
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	public boolean isVisible() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getMenuItem().getParent().isVisible());
			}
		};
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	public boolean hasFocus() throws ComponentException 
	{
		// Wir returnen immer false, weil sowohl die Klasse Menu als auch Menuitem
		// keine Funktion zum Abrufen des Fokuszustands haben
		return false;
	}
}
