/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Decorations;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Widget;

import atosj.Messages;
import atosj.component.NamingStrategy;
import atosj.exception.ComponentException;

/**
 * Die Benanumgsstrategie fr Components der SWT Bibliothek.
 * @author Nicos Tegos
 */
public class SWTNamingStrategy extends NamingStrategy 
{
	public static final String ATOSJ_COMPONENT_NAME_KEY = "ATOSJ_COMPONENT_NAME_KEY";  //$NON-NLS-1$
	
	/**
	 * Gibt den Klarnamen des Components zurck, dieser hngt vom
	 * Typ des Components ab.
	 * @param w Muss mindestens vom Typ <code>org.eclipse.swt.widgets.Widget</code> sein.
	 * @return ein nicht leerer String, der den Klarnamen 
	 * des graph. Objektes enthlt.
	 */
	public String getName(final Object w)	
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				String name = ""; //$NON-NLS-1$
				// wir suchen den von Programmierer evtl. vergebenen
				// Namen, dieser muss ber setData mit einem
				// bestimmten Schlssel vergeben worden sein
				Object value = ((Widget)w).getData(ATOSJ_COMPONENT_NAME_KEY);
				if(value != null && value instanceof String)
				{
					name = (String) value;
				}
				else
				if (w instanceof Decorations)	
				{
					name = ((Decorations)w).getText();
				}
				else
				if(w instanceof Button)
				{
					name = ((Button)w).getText();
				}
				else
				if(w instanceof Label)
				{
					name = ((Label)w).getText();
				}
				else
				if(w instanceof Item)
				{
					name = ((Item)w).getText();
				}
				// TODO getBounds geht leider nicht da zur konstruktionszeit der Shell
				// alle KindElemente noch die Bounds(0,0,0,0) haben,
				// wir sollte in der open Funktion unseren TopLevelWindow Listener benachrichtigen
				else
				if(w instanceof Control)
				{
					// TODO eine einheitliche getBounds Funktion fr alle Elemente erstellen,
					// die die Position im Fenster angibt
					Rectangle bounds = ((Control)w).getBounds();
					name = boundsToName(bounds.x, bounds.y, bounds.width, bounds.height);
				}
				
				if( name.equals("") ) //$NON-NLS-1$
					name = ((Widget)w).toString();
				
				return name.trim();
			}
		};
		
		
		try {
			return (String)action.go();
		} catch (ComponentException e) {
			return Messages.getString("SWTNamingStrategy.3"); //$NON-NLS-1$
		}
	}
}
