package atosj.component.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.ShellListener;
import org.eclipse.swt.widgets.Shell;

import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IWindow;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Shell</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTShell extends SWTControl implements IWindow 
{
	/**
	 * Zeichnet das Schlieen, Maximieren, Minimieren und Normalisieren des
	 * Fensters auf. Generiert TEST Kommandos zu den Zustnden der Fensters (TEXT,  
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ShellCommandCreator extends ControlCommandCreator implements ShellListener, ControlListener
	{
		private boolean maximized = false;
		public ShellCommandCreator(SWTShell shell, IHTSCommandReceiver receiver) 
		{
			super(shell, receiver);
			
			addListener(this);
		}
	
		public void shellActivated(ShellEvent e) {
			maximized = ((Shell)e.widget).getMaximized();
		}

		public void shellClosed(ShellEvent e) {
			// TODO das funktioniert leider nicht, weil die listener
			// disposed werden bevor dieser aufruf stattfindet?
			publishActionCommand(HTSSyntax.ACTION_CLOSE, null, null);			
		}

		public void shellDeactivated(ShellEvent e) {
			// uninteressant
		}
		
		public void shellDeiconified(ShellEvent e) {
			publishActionCommand(((Shell)e.widget).getMaximized() ? HTSSyntax.ACTION_MAXIMIZE : HTSSyntax.ACTION_NORMALIZE, null, null);	
			
		}
		
		public void shellIconified(ShellEvent e) {
			publishActionCommand(HTSSyntax.ACTION_MINIMIZE, null, null);
		}

		public void controlMoved(ControlEvent e) {
			// uninteressant
		}

		public void controlResized(ControlEvent e) 
		{
			Shell shell = (Shell)e.widget;
			// es wird auch ein resize event geliefert bei iconify
			if( !shell.getMinimized() && maximized != shell.getMaximized() )
				publishActionCommand( shell.getMaximized() ? HTSSyntax.ACTION_MAXIMIZE : HTSSyntax.ACTION_NORMALIZE, null, null);
			
			maximized = shell.getMaximized();
		}
	};
	
	public SWTShell(Shell component, ComponentId id) {
		super(component, id);
	}
	
	private Shell getShell() {
		return (Shell) getComponent();
	}

	public void close() throws ComponentException 
	{
		assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable
			{
				if( (getShell().getStyle() & SWT.CLOSE) == 0 )
					throw new InvalidOperationException(SWTShell.this, OP_CLOSE, true);
				
				getShell().close();
				return null;
			}
		};
		
		action.go(this, OP_CLOSE);
	}

	public void maximize() throws ComponentException 
	{
		assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			protected Object performAction() throws Throwable
			{
				if( (getShell().getStyle() & SWT.MAX) == 0 )
					throw new InvalidOperationException(SWTShell.this, OP_MAXIMIZE, true);
				
				if( !getShell().getMaximized() )
					getShell().setMaximized(true);
				
				return null;
			}
		};
		
		action.go(this, OP_MAXIMIZE);
	}

	public void minimize() throws ComponentException 
	{
		assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable
			{
				if( (getShell().getStyle() & SWT.MIN) == 0 )
					throw new InvalidOperationException(SWTShell.this, OP_MINIMIZE, true);
				
				if( !getShell().getMinimized() )
					getShell().setMinimized(true);
				
				return null;
			}
		};
		
		action.go(this, OP_MINIMIZE);
	}

	public void normalize() throws ComponentException 
	{
		assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable
			{
				if( getShell().getMinimized() )
					getShell().setMinimized(false);
				else
				if( getShell().getMaximized() )
					getShell().setMaximized(false);
				// sonst ist schon normalized
				
				return null;
			}
		};
		
		action.go(this, OP_NORMALIZE);

	}

	public String getText() throws ComponentException 
	{
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable
			{
				return getShell().getText();
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ShellCommandCreator(this, receiver);
	}
}
