/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Spinner;

import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.ISpinner;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>javax.swing.JSpinner</code> abgeleiteten Klassen.
 * @author Volker Janetschek
 */
public class SWTSpinner extends SWTControl implements ISpinner
{
	/**
	 * Zeichnet die nderung des Textes im Spinner auf.
	 * Generiert TEST Kommandos zu den Zustnden des Spinners (TEXT, NUM, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class SpinnerCommandCreator extends ControlCommandCreator implements SelectionListener
	{
		public SpinnerCommandCreator(SWTSpinner Spinner, IHTSCommandReceiver receiver) {
			super(Spinner, receiver);
			addListener(this);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ISpinner)getMComponent()).getText();
				values.add('"' + txt + '"');
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}			
			try {
				Vector values = new Vector(1);
				float num = ((ISpinner)getMComponent()).getNum();
				values.add("" + num);
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
						
			return commands;
		}

		public void widgetSelected(SelectionEvent e) {
			Vector actionParams = new Vector(1);
			
			if (getSpinner().getDigits()==0)
				actionParams.add( "\""  + (int)getSpinnerValue()  + "\"" );
			else
				actionParams.add( "\""  + getSpinnerValue()  + "\"" );
			publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
	};
	
	public SWTSpinner(Spinner component, ComponentId id) {		
		super(component, id);
	}

	private Spinner getSpinner() {
		return (Spinner) getComponent();
	}
	
	private float getSpinnerValue()	{
		return getSpinner().getSelection()/(float)Math.pow(10, getSpinner().getDigits());
	}
	
	public String getText() throws ComponentException{
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				if (getSpinner().getDigits() == 0)
					return ""+getSpinner().getSelection();
				else
					return ""+getSpinnerValue();
				
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}
	
	public float getNum() throws ComponentException {
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return new Float(getSpinnerValue());
			}
		};
		
		return ((Float) (action.go(this, OP_GET_NUM))).floatValue();
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new SpinnerCommandCreator(this, receiver);
	}

	public void setText(String text) throws ComponentException {

		float val;
		try	{
			val = Float.parseFloat(text);
		} catch (NumberFormatException e)	{
			//TODO: passende Ableitung von AtosJException
			throw new InvalidOperationException(this, OP_SET_TEXT, true);
		}
		
		int digits = ((Integer) new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer (getSpinner().getDigits());
			}}.go()).intValue();
	
		val = val * (float)Math.pow(10, digits);
		final int intVal = (int)val;
		
		//berprfen, ob zuviele Nachkommastellenangegeben wurden
		if (val-intVal!=0)
			throw new InvalidOperationException(this, OP_SET_TEXT, true);
		
		assertEditable();
		requestFocus();
		new SWTThreadSafeAction() {

			protected Object performAction() throws Throwable {
				getSpinner().setSelection(intVal);
				return null;
			}}.go();
		
	}
}
