package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.ITabFolder;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>TabFolder</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTTabFolder extends SWTControl implements ITabFolder 
{
	/**
	 * Zeichnet den Wechsel zwischen den Tabs auf.
	 * Generiert TEST Kommandos zu den Zustnden der Combobox (TEXT,  
	 * ITEMCOUNT, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TabFolderCommandCreator extends ControlCommandCreator implements SelectionListener
	{
		public TabFolderCommandCreator(SWTTabFolder tab, IHTSCommandReceiver receiver) 
		{
			super(tab, receiver);
			
			addListener(this);
		}

		public void widgetSelected(SelectionEvent e) 
		{
			TabFolder tab = (TabFolder)e.widget;
			int selection = tab.getSelectionIndex();
			if(selection != -1) // nur gltige indices
			{
				Vector subitemPath = new Vector(1);
				subitemPath.add( "" + selection ); //$NON-NLS-1$
				publishActionCommand(HTSSyntax.ACTION_SELECT, null, subitemPath);
			}
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((ITabFolder)getMComponent()).getText();
				values.add('"' + txt + '"');
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				int num = ((ITabFolder)getMComponent()).getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
	};
	
	public SWTTabFolder(TabFolder component, ComponentId id) {
		super(component, id);
	}
	
	private TabFolder getTabFolder() {
		return (TabFolder) getComponent();
	}

	/**
	 * berprft, ob das angegebene Tab existiert.
	 * @param index Der Index des Tabs.
	 * @throws ComponentException wenn kein Tab an <code>index</code> existiert.
	 */
	private void assertSubitem(final int index) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				if( index < 0 || index >= getTabFolder().getItemCount() )
				{
					int count = getTabFolder().getItemCount();
					if( count > 0 )
						throw new ComponentException( Messages.getString("SWTTabFolder.2") + index + Messages.getString("SWTTabFolder.3") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
					else
						throw new ComponentException( Messages.getString("SWTTabFolder.4") + index + Messages.getString("SWTTabFolder.5")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Sucht nach dem Tab mit dem angegebenen Text.
	 * @param text Der Text des gesuchten Tabs.
	 * @return den Index des gefundenen Tabs.
	 * @throws ComponentException wenn kein Tab mit <code>text</code> gefunden wurde.
	 */
	private int findItem(final String text) throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				for(int i = 0; i < getItemCount(); ++i)
					if( getTabFolder().getItem(i).getText().equals(text) )
						return new Integer(i);
				
				throw new ComponentException(Messages.getString("SWTTabFolder.6") + text + Messages.getString("SWTTabFolder.7")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		};
		
		return ((Integer)action.go()).intValue();
	}
	
	public void select(final int index) throws ComponentException 
	{
		assertEditable();
		assertSubitem(index);
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				// es gibt hier keine chance das selektieren
				// durch einen mausklick zu simulieren, getBounds fr TabItem
				// ist nicht verfgbar
				getTabFolder().setSelection(index);
				
				Event event = new Event ();
				event.item = getTabFolder().getItem(index);
				event.time = (int) System.currentTimeMillis();
				getTabFolder().notifyListeners(SWT.Selection, event);
				
				return null;
			}
		};
		
		action.go(this, OP_SELECT);
	}

	public void select(String title) throws ComponentException {
		select( findItem(title) );
	}

	public String getText() throws ComponentException 
	{
		assertReadable();
				
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				TabItem[] items = getTabFolder().getSelection();
				if( items.length == 0 )
					return ""; //$NON-NLS-1$
				else
					return items[0].getText();
			}
		};
		
		return (String)action.go(this, OP_GET_TEXT);
	}

	public String getText(final int index) throws ComponentException 
	{
		assertReadable();
		assertSubitem(index);
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return getTabFolder().getItem(index).getText();
			}
		};
		
		return (String)action.go(this, OP_GET_TEXT);
	}

	public int getItemCount() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer( getTabFolder().getItemCount() );
			}
		};
		
		return ((Integer)action.go(this, OP_GET_ITEMCOUNT)).intValue();
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TabFolderCommandCreator(this, receiver);
	}
}
