/*
 * Created on 05.05.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IListCommandCreator;
import atosj.component.ITable;
import atosj.component.ITableCell;
import atosj.component.ListCommandCreatorHelper;
import atosj.exception.ComponentException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>Table</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTTable extends SWTControl implements ITable
{
	/**
	 * Zeichnet das Selektieren und Editieren von Eintrgen der Tabelle auf.
	 * Generiert TEST Kommandos zu den Zustnden der Tabelle/Tabellenzellen (TEXT, NUM, 
	 * ITEMCOUNT, CHECKSTATE, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TableCommandCreator extends ControlCommandCreator implements SelectionListener, IListCommandCreator
	{
		private boolean textModified = false;
		private ListCommandCreatorHelper helper;
		
		public TableCommandCreator(SWTTable table, IHTSCommandReceiver receiver) {
			super(table, receiver);
			helper = new ListCommandCreatorHelper(this); 
			
			// capture selection change
			addListener(this);
			
			// capture TextCell editing
			Control[] children = table.getTable().getChildren();
			FocusListener textCellFocusListener =  new FocusListener() {
				public void focusGained(FocusEvent e) {
					textModified = false;
				}

				public void focusLost(FocusEvent e) 
				{
					if(textModified)
					{
						Text textCell = (Text) e.widget;
						Vector actionParams = new Vector(1);
						actionParams.add( '"'  + textCell.getText() + '"' );
						
						// das tableitem anhand der koordinaten des mittelpunktes, des textfeldes herausfinden
						Table table = ((SWTTable)getMComponent()).getTable();
						Rectangle textCellBounds = textCell.getBounds();
						Point pCell = new Point(textCellBounds.x + textCellBounds.width / 2, 
								textCellBounds.y + textCellBounds.height / 2); 
						TableItem item = table.getItem(pCell);
						if(item != null)
						{
							int row = -1;
							int col = -1;
							// die zeile herausfinden
							for(int i = 0; i < table.getItemCount(); ++i)
							{
								if(table.getItem(i) ==  item)
									row = i;
							}
							// die spalte herausfinden
							for(int i = 0; i < table.getColumnCount(); ++i)
							{
								if( item.getBounds(i).contains(pCell) )
									col = i;
							}
							
							if(row != -1 && col != -1)					
							{
								Vector subitemPath = new Vector(2);
								subitemPath.add("" + row); //$NON-NLS-1$
								subitemPath.add("" + col); //$NON-NLS-1$
								publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, subitemPath);
							}
						}
					}
					
					textModified = false;
				}
			};
			ModifyListener textCellModifyListener = new ModifyListener() {

				public void modifyText(ModifyEvent e) {
					textModified = true;
				}
				
			};
			
			KeyListener textCellKeyListener = new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					// editing durch escape drcken abgebrochen
					if(e.keyCode == SWT.ESC)
						textModified = false;
				}
			};
			
			for(int i = 0; i < children.length; ++i)
			{
				if(children[i] instanceof Text)
				{
					addListener(children[i], textCellFocusListener);
					addListener(children[i], textCellModifyListener);
					addListener(children[i], textCellKeyListener);
				}
			}
					
		}

		public void widgetSelected(SelectionEvent e) {
			helper.selectionChanged();
		}

		public void widgetDefaultSelected(SelectionEvent e) {
		}
		
		public Object[] getSelection() 
		{
			SWTThreadSafeAction action = new SWTThreadSafeAction() {
				protected Object performAction() throws Throwable {
					return ListCommandCreatorHelper.convertToIntegerArray( ((SWTTable)getMComponent()).getTable().getSelectionIndices() );
				}
			};
			
			try {
				return (Object[])action.go();
			} catch (ComponentException e) {
				// irgendwas ist schiefgelaufen z.B. list schon disposed
				return new Object[0];
			}
		}
		
		public Vector convertSelectionObjectToPath(Object selectionObject) {
			Vector path = new Vector(1);
			// das selectionObject ist ein listenindex als Integer
			path.add( selectionObject.toString() );
			return path;
		}
		
		public Vector getTestCommands() 
		{
			SWTTable table = (SWTTable) getMComponent();
			Vector commands = super.getTestCommands();
			
			// ITEMCOUNT
			try {
				Vector values = new Vector(1);
				int num = table.getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			// Fr alle Spalten in den ausgewhlten Zeilen TEXT
			try {
				Object[] selection = getSelection();
				int colCount = table.getColCount();
				for(int i = 0; i < selection.length; ++i)
				{
					int row = ((Integer)selection[i]).intValue();
					for(int col = 0; col < colCount; ++col)
					{
						Vector values = new Vector(1);
						String txt = table.getTableCell(row , col).getText();
						values.add( '"' + txt + '"' );
						Vector path = new Vector(2);
						path.add( Integer.toString(row) );
						path.add( Integer.toString(col) );
						commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, path) );
					}
				}
			} catch(ComponentException e){}
			
			return commands;
		}
	}
	
	public SWTTable(Table component, ComponentId id) {
		super(component, id);
	}

	private Table getTable() {
		return (Table) getComponent();
	}
	
	/**
	 * Rckt die angegebene Tabellenzelle in den sichtbaren Bereich, <em>threadsafe</em>.
	 * @param row Zeilenindex der Tabellenzelle.
	 * @param col Spaltenindex der Tabellenzelle.
	 */
	private void revealSubitem(final int row, final int col) throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				getTable().showItem( getTable().getItem(row) );
				getTable().showColumn( getTable().getColumn(col) );
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * berprft, ob die angegebene Tabellenzelle existiert, <em>threadsafe</em>.
	 * @param row Zeilenindex der Tabellenzelle.
	 * @param col Spaltenindex der Tabellenzelle.
	 * @throws ComponentException wenn die Tabellenzelle nicht existiert.
	 */
	private void assertSubitem(final int row, final int col) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				if( row < 0 || row >= getTable().getItemCount() )
				{
					int count = getTable().getItemCount();
					if( count > 0 )
						throw new ComponentException( Messages.getString("SWTTable.3") + row + Messages.getString("SWTTable.4") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
					else
						throw new ComponentException( Messages.getString("SWTTable.5") + row + Messages.getString("SWTTable.6")); //$NON-NLS-1$ //$NON-NLS-2$
					
				}
				
				if( col < 0 || col >= getTable().getColumnCount() )
				{
					int count = getTable().getColumnCount();
					if( count > 0 )
						throw new ComponentException( Messages.getString("SWTTable.7") + col + Messages.getString("SWTTable.8") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
					else
						throw new ComponentException( Messages.getString("SWTTable.9") + col + Messages.getString("SWTTable.10")); //$NON-NLS-1$ //$NON-NLS-2$
					
				}
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * berprft, ob die Tabelle die Auswahl mehrerer Eintrge erlaubt.
	 * @throws ComponentException wenn die Tabelle keine Mehrfachauswahl untersttzt.
	 */
	private void assertMulti() throws ComponentException
	{
		if( (getStyle() & SWT.MULTI) == 0 )
			throw new ComponentException(Messages.getString("SWTTable.11")); //$NON-NLS-1$
	}
	
	private boolean isSelected(final int row) throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				int[] selection = getTable().getSelectionIndices();
				for (int i = 0; i < selection.length; i++) 
				{
					if(selection[i] == row)
						return Boolean.TRUE;
				}
				
				return Boolean.FALSE;
			}
		};
		
		return ((Boolean)action.go()).booleanValue();
	}
	
	public void select(int row) throws ComponentException {
		assertEditable();
		assertSubitem(row, 0);
		revealSubitem(row, 0);
		
		new SWTTableCell(this, row, 0).click(1, true, NO_MODIFIER);
	}

	public void deselect(int row) throws ComponentException 
	{
		assertEditable();
		assertSubitem(row, 0);
		assertMulti();
		
		revealSubitem(row, 0);
		if( isSelected(row) )
			new SWTTableCell(this, row, 0).click(1, true, MODIFIER_CTRL);
	}

	public void addSelect(int row) throws ComponentException 
	{
		assertEditable();
		assertSubitem(row, 0);
		assertMulti();
		
		revealSubitem(row, 0);
		if( !isSelected(row) )
			new SWTTableCell(this, row, 0).click(1, true, MODIFIER_CTRL);
	}

	public int getItemCount() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer( getTable().getItemCount() );
			}
		};
		
		return ((Integer)action.go(this, OP_GET_ITEMCOUNT)).intValue();
	}
	
	public int getColCount() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer( getTable().getColumnCount() );
			}
		};
		
		return ((Integer)action.go(this, OP_GET_COLCOUNT)).intValue();
	}

	public ITableCell getTableCell(int row, int col) throws ComponentException {
		return new SWTTableCell(this, row, col);
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TableCommandCreator(this, receiver);
	}
}
