/*
 * Created on 05.05.2006
 */
package atosj.component.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.IEditable;
import atosj.component.ITableCell;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Untersttz die Manipulation und das Auslesen einer Tabellenzelle.
 * Die Tabellenzelle ist ein virtuelles Konstrukt und hat keine direkte Entsprechung in SWT
 * sondern ist Teil der Tabelle. Diese Klasse sollte nur durch die entsprechende Tabelle erzeugt werden.
 * @author Nicos Tegos
 */
public class SWTTableCell extends SWTComponent implements ITableCell 
{
	private int row;
	private int col;
			
	public SWTTableCell(SWTTable table, int row, int col)
	{
		super(table.getSWTComponent(), 
				new ComponentId(
				table.getComponentId().getName() + Messages.getString("SWTTableCell.0") + row + Messages.getString("SWTTableCell.1") + col + ")", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				table.getComponentId().getClassName(),
				table.getComponentId().getWindowName()
			)
		);
		
		this.row = row;
		this.col = col;
	}

	private Table getTable() {
		return (Table) getComponent();
	}
	
	public Point getPoint() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				Rectangle rect = getTable().getItem(row).getBounds(col);
				return getTable().toDisplay(rect.x + rect.width/2, 
						rect.y + rect.height/2);
			}
		};
		return (Point)action.go();
	}
	
	public boolean isVisible() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTable().isVisible() );
			}
		};
		
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	public boolean isEnabled() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTable().isEnabled() );
			}
		};
		
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	public boolean hasFocus() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTable().isFocusControl() );
			}
		};
		
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	public void setText(final String text) throws ComponentException 
	{
		// wir knnen leider nur schauen ob die Tabelle die bedingungen erfllt
		assertEditable();
		// wir aktivieren den Zelleneditor durch einfachen Mausklick (hoffentlich)
		click(1, true, NO_MODIFIER);
		
		// wir berprfen ob der Editor auch aktiv ist, krass style
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				Control[] children = getTable().getChildren();
				for(int i = 0; i < children.length; ++i)
				{
					// Tja ja die CellEditor Components sind Kinder der Tabelle
					if( children[i].isFocusControl() )
					{
						MComponent editor = getResolver().getMComponent(children[i], getComponentId());
						// der Name assertEditable ist etwas irrefhrend es muss explizit geprft werden
						// ob die Funktion setText verfgbar ist
						if(editor instanceof IEditable )
						{
							((IEditable)editor).setText(text);
							// enter drcken, um den editor zu schliessen
							editor.pressKey('\0', SWT.CR, 0);
							return null;
						}
						else
							throw new InvalidOperationException(editor, OP_SET_TEXT, true);
					}
				}
				
				return null;
			}
		};
	
		action.go();	
	}

	public void setChecked(boolean check) throws ComponentException {
		throw new InvalidOperationException(this, OP_SET_CHECKED, true);
	}

	public boolean isChecked() throws ComponentException {
		throw new InvalidOperationException(this, OP_GET_CHECKED, true);
	}

	public String getText() throws ComponentException 
	{
		assertReadable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				return getTable().getItem(row).getText(col);
			}
		};
		
		return (String)action.go(this, OP_GET_TEXT);
	}

	public float getNum() throws ComponentException 
	{
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
}
