/*
 * Created on 25.04.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IEditbox;
import atosj.event.swt.NotifyEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper sowohl fr ein- als auch fr mehrzeilige Textfelder, alle von <code>Text</code>.
 * @author Nicos Tegos
 */
public class SWTTextField extends SWTControl implements IEditbox 
{
	/**
	 * Zeichnet alle nderungen des Textes auf.
	 * Generiert TEST Kommandos zu den Zustnden des Textfeldes (TEXT, NUM, 
	 * ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TextFieldCommandCreator extends ControlCommandCreator implements ModifyListener, FocusListener
	{
		boolean modified = false; // gibt an ob des Textfeld zwischen 2 aufeinander folgenden Focus gained und Focus lost ereignissen gendert wurde
		public TextFieldCommandCreator(SWTTextField text, IHTSCommandReceiver receiver) 
		{
			super(text, receiver);
			
			addListener(this);
		}
		
		protected void onKeyPressed(KeyEvent e) {
			// nur fr einzeilige Textfelder
			if( (getTextField().getStyle() & SWT.MULTI) == 0 
					&& e.keyCode == SWT.CR )
				generateCommand();
		}
		
		public void modifyText(ModifyEvent e) {
			modified = true;
		}

		public void focusGained(FocusEvent e) {
			modified = false;
		}

		public void focusLost(FocusEvent e) 
		{
			generateCommand();
		}
		
		private Text getTextField() {
			return ((SWTTextField)getSWTComponent()).getTextField();
		}
		
		protected void generateCommand() 
		{
			if(modified)
			{
				Vector actionParams = new Vector(1);
				actionParams.add( '"'  + getTextField().getText().replaceAll("\r", "") + '"' ); //$NON-NLS-1$ //$NON-NLS-2$
				publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, null);
			}
			
			modified = false;
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				String txt = ((IEditbox)getMComponent()).getText();
				values.add( '"' + txt + '"' );
				commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, null) );
			} catch(ComponentException e){}
			
			try {
				Vector values = new Vector(1);
				float num = ((IEditbox)getMComponent()).getNum();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_NUM, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
	};
	
	public SWTTextField(Text component, ComponentId id) {
		super(component, id);
	}
	
	protected Text getTextField() {
		return (Text) getComponent();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.swt.SWTComponent#assertEditable()
	 */
	public void assertEditable() throws ComponentException {
		super.assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return new Boolean(getTextField().getEditable());
			}
		};

		Boolean result = (Boolean)action.go();
		if( !result.booleanValue() )
			throw new ComponentException(Messages.getString("SWTTextField.3")); //$NON-NLS-1$
	}
	
	public String getText() throws ComponentException{
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				return getTextField().getText();
			}
		};
		
		return (String) action.go(this, OP_GET_TEXT);
	}
	
	public void setText(final String text) throws ComponentException 
	{
		assertEditable();
		requestFocus();
		selectAllText();
		
		if(text.length() > 0)
		{
			for(int i = 0; i < text.length(); ++i)
				pressKey(text.charAt(i), NO_MODIFIER);
		}
		else // vorhanden Text lschen
			pressKey('\0', SWT.DEL, 0);
	}
	
	/**
	 * Selektiert den gesamten Text durch mit Hilfe der Funktion <code>setText</code>
	 * des Textfeldes und benachrichtig alle Listener.
	 */
	private void selectAllText() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() {
				// wir knnen nicht CTRL + A verwenden, da
				// dies sicher nicht fr alle Betriebssysteme gleich ist
				getTextField().selectAll();
				// wir benachrichtigen alle Listener ber die nderung der Selection
				new NotifyEvent(SWTTextField.this).fire(SWT.Selection);
				
				return null;
			}
		};
		
		action.go(this, OP_SET_TEXT);
	}
	
	public float getNum() throws ComponentException {
		try {
			return Float.parseFloat( getText().replace(',', '.') );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TextFieldCommandCreator(this, receiver);
	}
}
