/*
 * Created on 25.04.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import atosj.component.ComponentId;
import atosj.component.ICheckbox;
import atosj.component.ICommandCreator;
import atosj.component.IRadio;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>ToolItem</code> abgeleiteten Klassen.
 * Diese haben verschiedene Ausprgungen, entsprechend des verwendeten Stils des <code>ToolItems</code>.
 * @author Nicos Tegos
 */
public class SWTToolItem extends SWTItem implements ICheckbox, IRadio
{
	/**
	 * Zeichnet das Drcken eines <code>ToolItems</code> auf.
	 * Generiert TEST Kommandos zu den Zustnden des <code>ToolItems</code> (TEXT, NUM, 
	 * CHECKSTATE, ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class ToolitemCommandCreator extends SWTCommandCreator implements SelectionListener
	{
		public ToolitemCommandCreator(SWTToolItem item, IHTSCommandReceiver receiver) {
			super(item, receiver);
			addListener(this);
		}
		
		public void widgetSelected(SelectionEvent e) 
		{
			ToolItem item = (ToolItem)e.widget;
			if( (item.getStyle() & SWT.PUSH) != 0 )
			{
				publishActionCommand(HTSSyntax.ACTION_SELECT, null, null);
			}
			else
			if( (item.getStyle() & SWT.RADIO) != 0 )
			{
				// bei radio gibt es auch ein event fr den button der unchecked wird
				// wenn die selection wechselt, dieses event interessiert uns nicht
				if( item.getSelection() )
					publishActionCommand(HTSSyntax.ACTION_CHECK, null, null);
			}
			else
			{
				publishActionCommand(item.getSelection() ? HTSSyntax.ACTION_CHECK : HTSSyntax.ACTION_UNCHECK, null, null);
			}
		}

		public void widgetDefaultSelected(SelectionEvent e) { }


		protected void hookTestListener(final MouseListener listener) {
			SWTThreadSafeAction action = new SWTThreadSafeAction () {
				protected Object performAction() throws Throwable 
				{
					ToolBar parent = ((SWTToolItem)getMComponent()).getToolItem().getParent();
					addListener(parent, listener); 
					return null;
				}
			};
			
			try {
				action.go();
			} catch (ComponentException e) {}
		}

		protected void hookKeyListener(final KeyListener listener) {
			SWTThreadSafeAction action = new SWTThreadSafeAction () {
				protected Object performAction() throws Throwable 
				{
					ToolBar parent = ((SWTToolItem)getMComponent()).getToolItem().getParent();
					addListener(parent, listener); 
					return null;
				}
			};
			
			try {
				action.go();
			} catch (ComponentException e) {}
		}

		protected void hookMouseListener(final MouseListener listener) {
			SWTThreadSafeAction action = new SWTThreadSafeAction () {
				protected Object performAction() throws Throwable 
				{
					ToolBar parent = ((SWTToolItem)getMComponent()).getToolItem().getParent();
					addListener(parent, listener); 
					return null;
				}
			};
			
			try {
				action.go();
			} catch (ComponentException e) {}
		}
		
		protected void onTestEvent(MouseEvent e) {
			Rectangle rect = ((SWTToolItem)getMComponent()).getToolItem().getBounds();
			// wir zeigen den Testdialog nur, wenn der MouseClick innerhalb unseres
			// ToolItems passiert ist. Es knnen auch MouseEvents an anderen items in
			// der ToolBar entstanden sein, diese ignorieren wir.
			if( rect.contains(e.x, e.y) )
				super.onTestEvent(e);
		}
		
		public Vector getTestCommands() {
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				boolean val = ((ICheckbox)getMComponent()).isChecked();
				values.add( val ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE );
				commands.add( createTestCommand(HTSSyntax.READ_CHECKSTATE, values, null) );
			} catch(ComponentException e){}
			
			return commands;
		}
	};
	
	public SWTToolItem(ToolItem component, ComponentId id) {
		super(component, id);
	}
	
	protected ToolItem getToolItem() {
		return (ToolItem) getComponent();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.swt.SWTComponent#getPoint()
	 */
	public Point getPoint() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				Rectangle rect = getToolItem().getBounds();
				return getToolItem().getParent().toDisplay(rect.x + rect.width/2, 
						rect.y + rect.height/2);
			}
		};
		return (Point)action.go();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#isChecked()
	 */
	public boolean isChecked() throws ComponentException {
		if( (getStyle() & (SWT.CHECK | SWT.RADIO)) == 0 )
			throw new InvalidOperationException(this, OP_GET_CHECKED, true);
		
		assertReadable();
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getToolItem().getSelection());
			}
		};
		return ((Boolean)action.go(this, OP_GET_CHECKED)).booleanValue();
	}
	
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#setChecked(boolean)
	 */
	public void setChecked(final boolean checked) throws ComponentException {
		if( (getStyle() & (SWT.CHECK | SWT.RADIO)) == 0 )
			throw new InvalidOperationException(this, OP_SET_CHECKED, true);
		
		assertEditable();
		if( isChecked() != checked )
			click(1, true, NO_MODIFIER);
	}
	
	public void check() throws ComponentException {
		setChecked(true);		
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#isEnabled()
	 */
	public boolean isEnabled() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getToolItem().isEnabled());
			}
		};
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#isVisible()
	 */
	public boolean isVisible() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getToolItem().getParent().isVisible());
			}
		};
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#hasFocus()
	 */
	public boolean hasFocus() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getToolItem().getParent().isFocusControl() );
			}
		};
		return ((Boolean)action.go(this, OP_GET_FOCUS)).booleanValue();
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ToolitemCommandCreator(this, receiver);
	}
}
