package atosj.component.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TreeItem;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.IEditable;
import atosj.component.ITreeCell;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Leitet Aktionen auf einem Baumknoten an seinen Baum weiter.
 * Der Baumknoten ist ein virtuelles Konstrukt und hat keine direkte Entsprechung in SWT
 * sondern ist Teil eines <code>Tree</code>. Diese Klasse sollte nur durch den entsprechenden
 * Baum erzeugt werden.
 * @author Nicos Tegos
 */
public class SWTTreeItem extends SWTItem implements ITreeCell 
{
	private SWTTree parent;
	
	public SWTTreeItem(SWTTree parent, TreeItem component, String path[]) {
		super(
			component, 
			new ComponentId(
					parent.getComponentId().getName() + Messages.getString("SWTTreeItem.0") + pathToString(path) + ")", //$NON-NLS-1$ //$NON-NLS-2$
					parent.getComponentId().getClassName(),
					parent.getComponentId().getWindowName()
			)
		);
		
		this.parent = parent;
	}
	
	private static String pathToString(String path[]) {
		String ret = ""; //$NON-NLS-1$
		for (int i = 0; i < path.length; i++) {
			ret += path[i];
			if(i < path.length-1)
				ret += ',';
		}
		
		return ret;
	}
	
	public boolean isVisible() throws ComponentException {
		// bernehmen den visiblestate des Baumes in dem sich das item befindet
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTreeItem().getParent().isVisible() );
			}
		};
		
		return ((Boolean)action.go(this, OP_GET_VISIBLE)).booleanValue();
	}
	
	public boolean isEnabled() throws ComponentException {
		// bernehmen den enabledstate des Baumes in dem sich das item befindet
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTreeItem().getParent().isEnabled() );
			}
		};
		
		return ((Boolean)action.go(this, OP_GET_ENABLED)).booleanValue();
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.MComponent#hasFocus()
	 */
	public boolean hasFocus() throws ComponentException 
	{
		// bernehmen den focusstate des Baumes in dem sich das item befindet
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean( getTreeItem().getParent().isFocusControl() );
			}
		};
		return ((Boolean)action.go(this, OP_GET_FOCUS)).booleanValue();
	}
	
	private TreeItem getTreeItem() {
		return (TreeItem) getComponent();
	}
	
	public Point getPoint() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				Rectangle rect = getTreeItem().getBounds();
				return getTreeItem().getParent().toDisplay(rect.x + rect.width/2, 
						rect.y + rect.height/2);
			}
		};
		return (Point)action.go();
	}

	public void setText(final String text) throws ComponentException 
	{
		// wir knnen leider nur schauen ob die Tabelle die bedingungen erfllt
		assertEditable();
		// wir aktivieren den Zelleneditor durch einfachen Mausklick (hoffentlich)
		click(1, true, NO_MODIFIER);
		
		// wir berprfen ob der Editor auch aktiv ist, krass style
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				Control[] children = parent.getTree().getChildren();
				for(int i = 0; i < children.length; ++i)
				{
					// Tja ja die CellEditor Components sind Kinder des Baums
					if( children[i].isFocusControl() )
					{
						MComponent editor = getResolver().getMComponent(children[i], getComponentId());
						if(editor instanceof IEditable )
						{
							editor.assertEditable(); 
							((IEditable)editor).setText(text);
							// enter drcken, um den editor zu schliessen
							editor.pressKey('\0', SWT.CR, 0);
							return null;
						}
						else
							throw new InvalidOperationException(editor, OP_SET_TEXT, true);
					}
				}
				
				return null;
			}
		};
	
		action.go();	
	}
}
