package atosj.dialog;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.swing.SwingCommandCreator;
import atosj.model.hts.command.HTSTest;

/**
 * Liefert alle TEST Kommandos zur Anzeige in der Tabelle.
 */
class TestTableModel extends AbstractTableModel{
	
	private static final String[] columnNames = new String[] { Messages.getString("AWTTestDialog.0") }; //$NON-NLS-1$
	private HTSTest[] testCommands;
	
	public TestTableModel(ICommandCreator creator) {
		Vector v = creator.getTestCommands();
		testCommands = (HTSTest[]) v.toArray(new HTSTest[v.size()]);
	}
	
	public int getColumnCount() {
		return columnNames.length;
	}
	
	public int getRowCount() {
		return testCommands.length;
	}
	
	public Object getValueAt(int rowIndex, int columnIndex) {
		
		switch(columnIndex) {
		case 0:
			return testCommands[rowIndex].toCaptureDlgString();
		default:
			return ""; //$NON-NLS-1$
		}
	}
	
	
	public String getColumnName(int column) {
		return columnNames[column];
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
	public Class getColumnClass(int columnIndex) {
		switch(columnIndex) {
		default:
			return String.class;
		}
	}
	
	public HTSTest getElementAt(int index) {
		return testCommands[index];
	}
}

/**
 * Zeigt eine Tabelle mit allen gltigen Zustandsprfungen (TEST Kommmandos)
 * fr ein Swing-Component. Der Dialog erscheint im Kontext des Testobjektes.
 * Aus dieser Tabelle kann der Tester Kommandos auswhlen, damit diese
 * spter einer Testsequenz hinzugefgt werden knnen
 * @author Nicos Tegos
 */
public class AWTTestDialog extends JDialog
{
	public static final int OK = 1;
	public static final int CANCEL = 2;
	
	JTable table;
	TestTableModel model;
	private int returnCode;
	
    private HTSTest[] selectedElements;

    public int getReturnCode() {
    	return returnCode;
    }

    /**
     * Erstellt einen neuen Dialog zur Anzeige aller Zustandsprfungen 
     * eines Components.
     * @param locationComp Das Component, ber dem der Dialog positioniert werden soll.
     * @param creator Der <code>CommandCreator</code> des Components.
     */
    public AWTTestDialog(Component locationComp, SwingCommandCreator creator) 
    {
        super(JOptionPane.getFrameForComponent(locationComp), SwingCommandCreator.TEST_ACTION_DIALOG_TITLE, true);

        //Create and initialize the buttons.
        JButton cancelButton = new JButton(Messages.getString("AWTTestDialog.2")); //$NON-NLS-1$
        cancelButton.addActionListener( new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				returnCode = CANCEL;
				setVisible(false);
			}
        });
        //
        JButton setButton = new JButton(Messages.getString("AWTTestDialog.3")); //$NON-NLS-1$
        setButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) 
			{
				// die markierten elemente sammeln
				int[] selectedRows = table.getSelectedRows();
				selectedElements = new HTSTest[selectedRows.length];
				for(int i =0; i < selectedElements.length; ++i)
					selectedElements[i] = model.getElementAt(selectedRows[i]);
				
				returnCode = OK;
				setVisible(false);
			}
        });
        getRootPane().setDefaultButton(setButton);
        
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout( new BoxLayout(buttonPane, BoxLayout.LINE_AXIS) );
        buttonPane.add( Box.createHorizontalGlue() );
        buttonPane.add(cancelButton);
        buttonPane.add( Box.createHorizontalStrut(10) );
        buttonPane.add(setButton);
        
        table = new JTable();
		table.setRowHeight(20);
		table.setName(Messages.getString("AWTTestDialog.4")); //$NON-NLS-1$
		table.setColumnSelectionAllowed(false);
		model = new TestTableModel(creator);
		table.setModel(model);
		
		JPanel infoPane = new JPanel();
		infoPane.setLayout( new BoxLayout(infoPane, BoxLayout.X_AXIS) );
		
		ComponentId id = creator.getMComponent().getComponentId();
		Box labelBox = Box.createVerticalBox();
		infoPane.add(labelBox);
		infoPane.add( Box.createHorizontalStrut(10) );
		Box valueBox = Box.createVerticalBox();
		infoPane.add(valueBox);
		labelBox.add( new JLabel(Messages.getString("AWTTestDialog.5")) ); //$NON-NLS-1$
		valueBox.add( new JLabel(id.getWindowName()) );
		valueBox.add( Box.createHorizontalGlue() );
		labelBox.add( new JLabel(Messages.getString("AWTTestDialog.6")) ); //$NON-NLS-1$
		valueBox.add( new JLabel(id.getClassName()) );
		infoPane.add( Box.createHorizontalGlue() );
		labelBox.add( new JLabel(Messages.getString("AWTTestDialog.7")) ); //$NON-NLS-1$
		valueBox.add( new JLabel(id.getName()) );
		infoPane.add( Box.createHorizontalGlue() );
		
		JPanel dlgArea = new JPanel();
		dlgArea.setLayout( new BoxLayout(dlgArea, BoxLayout.Y_AXIS) );
		dlgArea.setBorder( BorderFactory.createEmptyBorder(10, 10, 10, 10) );
		
		dlgArea.add(infoPane);
		dlgArea.add( Box.createVerticalStrut(10) );
		dlgArea.add( new JScrollPane(table) );
		dlgArea.add( Box.createVerticalStrut(10) );
		dlgArea.add(buttonPane);
        
        getContentPane().add(dlgArea);
      
        pack();
        setLocationRelativeTo(locationComp);
        setVisible(true);
    }

    /**
     * Liefert alle aus der Tabelle ausgewhlten TEST Kommandos.
     */
	public HTSTest[] getSelectedElements() {
		return selectedElements;
	}
}
