/*
 * Created on 01.12.2005
 */
package atosj.dialog;

import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.editors.ChangeableNameComparator;
import atosj.gui.BaseLabelProvider;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.util.UniversalViewerSorter;

/**
 * Dialog zum Auswahl von Skripten eines Projektes.
 * Diese knnen dann z.B. einem Paket hinzugefgt werden.
 * @author Nicos Tegos
 */
public class AddScriptDlg extends Dialog 
{
	public static final int ID_ADD = 0;
	public static final int ID_CANCEL = 1;
	
	private HTSPackage pack;
	private Vector scripts; // Vector<HTSScript> die hinzuzufgenden scripte
	private Vector addableScripts; // Vector<HTSScript> alle scripte im Projekt
	private CheckboxTableViewer scriptTV;
	
	private PositionWidget posWidget;
	private int currentPos;
	private int maxPos;
	
	/**
	 * Erstellt einen neuen Dialog zur Auswahl von Testsequenzen.
	 * Der Dialog muss mit der Methode <code>open</code> geffnet werden.
	 * @param parent Das Vaterfenster des Dialogs oder <code>null</code>
	 * @param pack Das Paket dem Skripte hinzugefgt werden sollen.
	 * @param maxPos siehe <code>PositionWidget</code>
	 * @param currentPos siehe <code>PositionWidget</code>
	 * @see PositionWidget
	 */
	 public AddScriptDlg(Shell parent, HTSPackage pack, int maxPos, int currentPos) {
		super(parent);
		
		this.pack = pack;
		this.maxPos = maxPos;
		this.currentPos = currentPos;
		
		addableScripts = AtosJApp.getInstance().getInput().getScripts();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		if( !addableScripts.isEmpty() )
			createButton(parent, ID_ADD, Messages.getString("AddScriptDlg.0"), true); //$NON-NLS-1$
		
		createButton(parent, ID_CANCEL, Messages.getString("AddScriptDlg.1"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_ADD: addPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	/**
	 * Vermerkt alle ausgewhlten Skripte in einer internen Variable,
	 * damit diese auch nach dem Schlieen des Dialogs verfgbar sind.
	 */
	protected void addPressed() 
	{
		Object[] scriptArray = scriptTV.getCheckedElements();
		scripts = new Vector(scriptArray.length);
		for(int i = 0; i < scriptArray.length; ++i)
			scripts.add(scriptArray[i]);
		
		if( scripts.isEmpty() )
		{
			MessageDialog.openInformation(null, Messages.getString("AddScriptDlg.2"), Messages.getString("AddScriptDlg.3") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("AddScriptDlg.4")); //$NON-NLS-1$
			
			return; // abbruch
		}
		
		setReturnCode(ID_ADD);
		close();
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(Messages.getString("AddScriptDlg.5")); //$NON-NLS-1$
		return super.createContents(parent);
	}
	
	/**
	 * Erstellt einen <code>CheckboxTableViewer</code> fr die Auswahl der Skripte.
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		if( addableScripts.isEmpty() )
		{
			Label lbl = new Label(dlgArea, SWT.NULL);
			lbl.setText(Messages.getString("AddScriptDlg.6") + //$NON-NLS-1$
					Messages.getString("AddScriptDlg.7")); //$NON-NLS-1$
		}
		else
		{
			Label lbl = new Label(dlgArea, SWT.NULL);
			lbl.setText(Messages.getString("AddScriptDlg.8")); //$NON-NLS-1$
			
			Table table = new Table(dlgArea, SWT.CHECK|SWT.BORDER);
			GridData gd = new GridData(GridData.FILL_BOTH);
			gd.widthHint = 350;
			table.setLayoutData(gd);
			
			TableColumn col = new TableColumn(table, SWT.NULL);
			col.setText(Messages.getString("AddScriptDlg.9")); //$NON-NLS-1$
			col.setWidth(300);
			
			scriptTV = new CheckboxTableViewer(table);
			scriptTV.setLabelProvider( new BaseLabelProvider() {
				public String getColumnText(Object element, int columnIndex) {
					if(columnIndex == 0)
						return ((HTSScript)element).getName();
					
					return ""; //$NON-NLS-1$
				}
			});
			scriptTV.setContentProvider( new ArrayContentProvider() );
			// nach Name sortieren
			scriptTV.setSorter( new UniversalViewerSorter(new ChangeableNameComparator()) );
						
			scriptTV.setInput(addableScripts);
			
			posWidget = new PositionWidget(dlgArea, maxPos, currentPos);
			posWidget.getContent().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		}
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Liefert alle vom Nutzer ausgwewhlten Skripte.
	 * @return Vector(HTSScript) die ausgewhlten Scripte.
	 */
	public Vector getScripts() {
		return scripts;
	}
	
	/**
	 * Liefert die Einfgeposition der Skripte im Paket.
	 * @return Die Einfgeposition, 0-basiert.
	 */
	public int getPosition() {
		return posWidget.getPosition();
	}
}
