package atosj.dialog;

import java.util.HashSet;
import java.util.Iterator;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.GC;

/**
 * <code>ContentProvider</code> fr den <code>TableViewer</code> zur Angabe des Classpath des Testobjekts
 * im Projekt erstellen Dialog.
 * Als Input wird ein <code>HashSet</code> von Strings verwendet.
 * @author Nicos Tegos
 * @see atosj.dialog.ProjectDlg
 */
class ClassPathContentProvider extends ArrayContentProvider
{
	private TableViewer viewer;
	private HashSet input; // HashSet<String> die Klassenpfade, absolut
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ArrayContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		this.viewer = (TableViewer) viewer;
		this.input = (HashSet) newInput;
	}
	
	/**
	 * Fgt den angegebenen Pfad hinzu, falls noch nicht enthalten.
	 * Fgt auch einen entsprechenden Eintrag im <code>TableViewer</code> hinzu und
	 * selektiert diesen.
	 * @param path Der Pfad.
	 */
	public void add(String path) {
		if(input != null)
		{
			boolean added = input.add(path);
			if(added && viewer != null)
			{
				viewer.add(path);
				viewer.setSelection( new StructuredSelection(path) );
				viewer.reveal(path);
				adaptHorizontalSize();
			}
		}
	}
	
	/**
	 * Entfernt den angegebenen Pfad.
	 * Entfernt auch den entsprechenden Eintrag im <code>TableViewer</code>.
	 * @param path Der Pfad.
	 */
	public void remove(String path) {
		if(input != null)
		{
			input.remove(path);
			if(viewer != null)
			{
				viewer.remove(path);
				adaptHorizontalSize();
			}
		}
	}
	
	/**
	 * Passt die Spaltenbreite der Tabelle an den lngsten Eintrag an.
	 */
	private void adaptHorizontalSize() {
		int maxLen = 0;
		for (Iterator iter = input.iterator(); iter.hasNext();)
		{
			int len = ((String) iter.next()).length();
			if(len > maxLen)
				maxLen = len;
		}
		
		if(maxLen > 0)
		{
			GC gc = new GC( viewer.getTable() );
			viewer.getTable().getColumn(0).setWidth( gc.getFontMetrics().getAverageCharWidth() * maxLen + 25 );
			gc.dispose();
		}
	}
}
