/*
 * Created on 08.12.2005
 */
package atosj.dialog;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.TableItem;

import atosj.app.AtosJApp;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.command.HTSComment;
import atosj.model.hts.command.HTSDisable;

/**
 * <code>LabelProvider</code> fr die Statistiken zur Ausfhrung von Kommandos.
 * @author Nicos Tegos
 */
public class CommandStatisticsTableLabelProvider extends BaseLabelProvider
{
	private static final RGB commentRGB = new RGB(52, 137, 52);
	private static final RGB disabledRGB = new RGB(114, 114, 114);
	private TableViewer tv;
	
	/**
	 * Erstellt einen neuen <code>LabelProvider</code> fr die angegebene Tabelle.
	 * @param tv Die Tabelle.
	 */
	public CommandStatisticsTableLabelProvider(TableViewer tv) {
		this.tv = tv;
	}
	
	/**
	 * Liefert den Text fr die einzelnen Spalten der Tabelle.
	 * @param element Ein <code>CommandStatisticsTableElement</code>
	 * @param columnIndex Der Spaltenindex mit folgender Bedeutung:<br>
	 * 0 = Nummer<br>
	 * 1 = Kommando<br>
	 * 2 = Status<br>
	 * 3 = Durchlufe<br>
	 * 4 = Dauer<br>
	 * @see CommandStatisticsTableElement
	 */
	public String getColumnText(Object element, int columnIndex) 
	{
		CommandStatisticsTableElement tableElement = (CommandStatisticsTableElement) element;
		switch(columnIndex)
		{
			case 0: 
				setFontColor(tableElement);
				return tableElement.getLineNumber();
			case 1: return tableElement.getCommandLine();
			case 2:
				TableItem item = (TableItem) tv.testFindItem(element);
				item.setBackground(columnIndex, AtosJApp.getInstance().getColor(tableElement.getStatusColor()) );
				return tableElement.getStatus();
			case 3: return tableElement.getRuns();
			case 4: return tableElement.getAvgDuration();
			default: return "";
		}
	}
	
	public boolean isLabelProperty(Object element, String property) 
	{
		for(int i = 0; i < CommandStatisticsTableContentProvider.PROPERTIES.length; ++i)
		{
			if( property.equals(CommandStatisticsTableContentProvider.PROPERTIES[i]) )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Die Textfarbe der Tabellenzeile von <code>element</code>.
	 * Grn fr Statistiken zu Kommentaren, grau fr deaktivierte Kommandos und
	 * schwarz fr alle anderen.
	 * @param element Die Statistik.
	 */
	private void setFontColor(CommandStatisticsTableElement element) 
	{
		TableItem item = (TableItem) tv.testFindItem(element);

		if( element.getElement() instanceof HTSComment )
		{
			item.setForeground( 0, AtosJApp.getInstance().getColor(commentRGB) );
			item.setForeground( 1, AtosJApp.getInstance().getColor(commentRGB) );
		}
		else
		if( element.getElement() instanceof HTSDisable )
		{
			item.setForeground( 0, AtosJApp.getInstance().getColor(disabledRGB) );
			item.setForeground( 1, AtosJApp.getInstance().getColor(disabledRGB) );
		}
		else
			item.setForeground(null);
	}
}
