package atosj.dialog;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;


import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.ToolBar;

import acover.HTMLConverter;
import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.TestObject;
import atosj.app.UserPreferences;
import atosj.exception.AtosJException;
import atosj.gui.TableResizeListener;
import atosj.rmi.RMIConnectionManager;
import atosj.util.AcceleratorHook;
import atosj.util.DefaultImageDescriptor;

/**
 * Fgt einen Paketstring zur Liste der Pakete hinzu.
 */
class AddPackageAction extends Action
{
	private TableViewer tv;
	
	public AddPackageAction(TableViewer tv) 
	{
		this.tv = tv;
		
		setAccelerator(SWT.CTRL | 'N');
		setText(Messages.getString("CoverageDlg.0")); //$NON-NLS-1$
		setToolTipText(Messages.getString("CoverageDlg.1")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD_DIS) );
	}
	
	public void run() 
	{
		InputDialog dlg = new InputDialog(tv.getTable().getShell(), Messages.getString("CoverageDlg.2"),  //$NON-NLS-1$
				Messages.getString("CoverageDlg.3"), //$NON-NLS-1$
				"", //$NON-NLS-1$
				new IInputValidator() 
				{
					public String isValid(String str) 
					{
						if(str.length() == 0)
							return Messages.getString("CoverageDlg.5"); //$NON-NLS-1$
						return null;
					}
				});
		
		if( dlg.open() == InputDialog.OK )
			((ClassPathContentProvider)tv.getContentProvider()).add( dlg.getValue() );			
	}
}

/**
 * Entfernt einen Paketstring aus der Liste der Pakete.
 */
class RemovePackageAction extends Action
{
	private TableViewer tv;
	
	public RemovePackageAction(TableViewer tv) 
	{
		this.tv = tv;
		
		setAccelerator(SWT.DEL);
		setText(Messages.getString("CoverageDlg.6")); //$NON-NLS-1$
		setToolTipText(Messages.getString("CoverageDlg.7")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE_DIS) );
		
		setEnabled( !tv.getSelection().isEmpty() );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				setEnabled( !event.getSelection().isEmpty() );				
			}
			
		});
	}
	
	public void run() {
		StructuredSelection ssel = (StructuredSelection) tv.getSelection();
		ClassPathContentProvider provider = (ClassPathContentProvider) tv.getContentProvider();
		
		Iterator it = ssel.iterator();
		while( it.hasNext() )
			provider.remove( (String)it.next() );
			
	}
}

/**
 * Ein Dialog zur Eingabe aller Pakete des Testobjekts, die einer
 * berdeckungsanalyse unterzogen werden sollen. Bietet auch die Mglichkeit
 * die XML-Datenbasis fr die Analyse zu erstellen und die Ergebnisse der Analyse anzuzeigen.
 * @author Nicos Tegos
 */
public class CoverageDlg extends Dialog 
{
	public static final int ID_OK = 0;
	public static final int ID_GEN = 1;
	public static final int ID_REP = 2;
	
	private TableViewer classPathTV;
	private ClassPathContentProvider classPathProvider = new ClassPathContentProvider(); // der ClassPathProvider wird missbraucht, da wir auch nur mit Strings operieren, die nur einmal in der Liste enthalten sein drfen.
	private HashSet classPath;
	
	private Button doCoverChk;
	
	private TestObject testee;
	
	private AddPackageAction addPackageAction;
	private RemovePackageAction removePackageAction;
	private MenuManager menu;
		
	/**
	 * Erstellt einen Dialog zur berdeckungsanalyse.
	 * @param parent Das Vaterfenster.
	 */
	public CoverageDlg(Shell parent) {
		super(parent);
		testee = AtosJApp.getInstance().getInput().getTestObject();
		classPath = new HashSet( testee.getCoveragePath() );
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(Messages.getString("CoverageDlg.8")); //$NON-NLS-1$
	}
	
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_GEN, Messages.getString("CoverageDlg.9"), false); //$NON-NLS-1$
		createButton(parent, ID_REP, Messages.getString("CoverageDlg.10"), false); //$NON-NLS-1$
		createButton(parent, ID_OK, Messages.getString("CoverageDlg.11"), true); //$NON-NLS-1$
	}
	
	protected void buttonPressed(int id) {
		switch(id)
		{
			case ID_OK: okPressed();
			break;
			case ID_GEN: genPressed();
			break;
			case ID_REP: repPressed();
			break;
		}	
	}
	
	protected void okPressed() {
		testee.setCoveragePath( new Vector(classPath) );
		testee.setCoverage( doCoverChk.getSelection() );
		super.okPressed();
	}
	
	/**
	 * Fhrt den Generator fr die XML-Datenbasis aus. Die Datenbasis wird im Ordner COV
	 * des Projektes erstellt.
	 * @see acover.DatabaseGenerator
	 */
	private void genPressed() 
	{
		if( classPath.isEmpty() ) 
		{
			MessageDialog.openError(getShell(), Messages.getString("CoverageDlg.12"), Messages.getString("CoverageDlg.13")); //$NON-NLS-1$ //$NON-NLS-2$
			classPathTV.getTable().setFocus();
			return;
		}
		
		String cp = ""; //$NON-NLS-1$
		// FIXME: bessere Variante ist es das ATOSj Home Verzeichnis ber eine environment variable zu setzen
		String[] atosjCp = System.getProperty("java.class.path").split(File.pathSeparator); //$NON-NLS-1$
		for (int i = 0; i < atosjCp.length; i++) {
			if( atosjCp[i].contains("atosj") ) //$NON-NLS-1$
				cp += new File(atosjCp[i]).getAbsolutePath();
		}
		
		cp += File.pathSeparator + testee.getClassPath();
		
		String cover = ""; //$NON-NLS-1$
		Iterator it = classPath.iterator();
		while( it.hasNext() )
			cover += " " + (String) it.next(); //$NON-NLS-1$
		
		// wir starten einen neuen prozess da wir den klassenpfad fr das testobjekt nicht nachladen knnen/wollen
		String commandLine = "java -cp \"" + cp + "\" acover.DatabaseGenerator \"" + testee.getCoverageDatabasePath() + "\" " + cover; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				
		try 
		{
			Process proc = RMIConnectionManager.getInstance().createProcess(commandLine, null, testee.getProjectDir(), null, null);
			try 
			{
				int ret = proc.waitFor();
				switch(ret)
				{
					case 0: break;	// alles in ordnung
					default: MessageDialog.openError(getShell(), Messages.getString("CoverageDlg.22"), Messages.getString("CoverageDlg.23") ); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
			} catch (InterruptedException e) {
			}
		} catch (AtosJException e) {
			System.out.println("KUCKUCK"); //$NON-NLS-1$
			MessageDialog.openError(getShell(), Messages.getString("CoverageDlg.25"), e.getMessage() ); //$NON-NLS-1$
		}
	}
	
	/**
	 * Ruft den Generator fr den HTML-Report der berdeckungsanalyse auf. Wenn ein Browser in den Einstellungen
	 * fr ATOSj spezifiziert wurde, wird dieser aufgerufen, um das Dokument anzuzeigen.
	 * @see HTMLConverter
	 */
	private void repPressed()
	{
		try 
		{
			HTMLConverter converter  = new HTMLConverter( testee.getCoverageDatabasePath() );
			converter.createHTMLOutput();
			
			String htmlReader = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.HTML_READER_PATH);
			if(htmlReader != null)
			{
				try {
					Runtime.getRuntime().exec(htmlReader + " \"" + testee.getCoverageDatabasePath().replaceAll("xml", "html") + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				} catch(IOException e) {
					MessageDialog.openError(null, Messages.getString("CoverageDlg.30"), Messages.getString("CoverageDlg.31")); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			else
			{
				MessageDialog.openError(null, Messages.getString("CoverageDlg.32"), Messages.getString("CoverageDlg.33")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		} catch (Exception e) {
			MessageDialog.openError(getShell(), Messages.getString("CoverageDlg.34"), Messages.getString("CoverageDlg.35") ); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Composite classpathArea = new Composite(dlgArea, SWT.NULL);
		classpathArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		classpathArea.setLayout(gl);
		
		classPathTV = new TableViewer(classpathArea, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		classPathTV.setContentProvider( classPathProvider );
		classPathTV.setInput(classPath);
		
		Table table = classPathTV.getTable();
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 150;
		table.setLayoutData(gd);
		
		new TableResizeListener(table, new int[] {100}, 40);
		new TableColumn(table, SWT.NULL);
						
		addPackageAction = new AddPackageAction(classPathTV);
		removePackageAction = new RemovePackageAction(classPathTV);
		AcceleratorHook accHook = new AcceleratorHook( classPathTV.getTable() );
		accHook.add(addPackageAction);
		accHook.add(removePackageAction);
		
		// context men fr die tabelle
		menu = new MenuManager();
		menu.add(addPackageAction);
		menu.add(removePackageAction);
		menu.createContextMenu( classPathTV.getTable() );
		classPathTV.getTable().addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event event) {
				classPathTV.getTable().setMenu( menu.getMenu() );	
			}
			
		});
		
		// toolbar rechts neben der tabelle
		ToolBarManager tobMngr = new ToolBarManager(SWT.VERTICAL);
		tobMngr.add(addPackageAction);
		tobMngr.add(removePackageAction);
		ToolBar tob = tobMngr.createControl(classpathArea);
		tob.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		doCoverChk = new Button(dlgArea, SWT.CHECK);
		doCoverChk.setText(Messages.getString("CoverageDlg.36")); //$NON-NLS-1$
		doCoverChk.setSelection( testee.isCoverage() );
		
		return dlgArea;
	}
}
