/*
 * Created on 09.01.2006
 */
package atosj.dialog;

import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import atosj.Messages;
import atosj.app.Project;
import atosj.editors.ChangeableNameComparator;
import atosj.gui.BaseLabelProvider;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.util.UniversalViewerSorter;

/**
 * Dialog zum Lschen eines Skripts.
 * Zeigt alle Pakete in einer Liste an, in denen das angegebene Script enthalten ist.
 * @author Nicos Tegos
 */
public class DeleteScriptDlg extends Dialog 
{
	/** Der Nutzer hat den Dialog mit der Antwort "Ja" (Lschen besttigt) beendet. */
	public static final int ID_YES = 0;
	/** Der Nutzer hat den Dialog mit der Antwort "Nein" beendet. */
	public static final int ID_NO = 1;
	
	private HTSScript script;
	private Vector packagesContaining = new Vector();
	
	private ListViewer packageLV;
	
	/**
	 * Erstellt einen neuen Skript-Lschen-Dialog.
	 * @param parent Das Vaterfenster.
	 * @param project Das Projekt des Skripts.
	 * @param script Das zu lschende Skript.
	 */
	public DeleteScriptDlg(Shell parent, Project project, HTSScript script) {
		super(parent);
		this.script = script;
		
		Vector packages = project.getPackages();
		for(int i = 0; i < packages.size(); ++i)
		{
			if( ((HTSPackage)packages.get(i)).containsScript(script) )
				packagesContaining.add( packages.get(i) );
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_YES, Messages.getString("DeleteScriptDlg.0"), true); //$NON-NLS-1$
		createButton(parent, ID_NO, Messages.getString("DeleteScriptDlg.1"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_YES: yesPressed();
			break;
			case ID_NO: noPressed();
			break;
		}
	}
	
	protected void yesPressed() 
	{
		setReturnCode(ID_YES);
		close();
	}
	
	protected void noPressed() 
	{
		setReturnCode(ID_NO);
		close();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(Messages.getString("DeleteScriptDlg.2")); //$NON-NLS-1$
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Label imgLbl = new Label(dlgArea, SWT.NULL);
		imgLbl.setImage( getShell().getDisplay().getSystemImage(SWT.ICON_QUESTION) );
		
		if( packagesContaining.isEmpty() )
		{
			Label lbl = new Label(dlgArea, SWT.NULL);
			lbl.setText(Messages.getString("DeleteScriptDlg.3") + script.getName()  //$NON-NLS-1$
					+ Messages.getString("DeleteScriptDlg.4") ); //$NON-NLS-1$
		}
		else
		{
			Label lbl = new Label(dlgArea, SWT.NULL | SWT.WRAP);
			lbl.setText(Messages.getString("DeleteScriptDlg.5") + script.getName()  //$NON-NLS-1$
					+ Messages.getString("DeleteScriptDlg.6")); //$NON-NLS-1$
					
			
			packageLV = new ListViewer(dlgArea);
			packageLV.setLabelProvider( new BaseLabelProvider() {
				public String getText(Object element) {
						return ((HTSPackage)element).getName();
				}
			});
			packageLV.setContentProvider( new ArrayContentProvider() );
			// nach Name sortieren
			packageLV.setSorter( new UniversalViewerSorter( new ChangeableNameComparator()) );
			GridData gd = new GridData(GridData.FILL_BOTH);
			gd.horizontalSpan = 2;
			packageLV.getList().setLayoutData(gd);
			
			// input setzen
			packageLV.setInput(packagesContaining);
		}
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
}
