/*
 * Created on 22.05.2006
 */
package atosj.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;

/**
 * Ein Dialog, der eine Fehlernachricht darstellt, geteilt in einen
 * Bereich fr Kurzbeschreibung und einen Bereich fr Details. Der Bereich
 * fr Details ist ein scrollbares Textfeld.
 * @author Nicos Tegos
 */
public class ErrorDlg extends Dialog 
{
	private Label imgLbl;
	private Label messageLbl;
	private Text messageDetailTxt;
	
	private String title;
	private String message;
	private String messageDetail;
	
	/**
	 * Erstellt einen neuen Dialog zur Anzeige einer Fehlermeldung.
	 * @param parent Das Vaterfenster.
	 * @param title Dialogtitel.
	 * @param message Kurze Fehlernachricht.
	 * @param messageDetail Ausfhrliche Fehlernachricht.
	 */
	public ErrorDlg(Shell parent, String title, String message, String messageDetail) {
		super(parent);
		
		this.title = title;
		this.message = message;
		this.messageDetail = messageDetail;
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setSize(600, 400);
		shell.setText(title);
	}
	
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea = new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 15;
		gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Image errorImg = getShell().getDisplay().getSystemImage(SWT.ICON_ERROR);
		imgLbl = new Label(dlgArea, SWT.NULL);
		errorImg.setBackground(imgLbl.getBackground());
		imgLbl.setImage(errorImg);
		imgLbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		messageLbl = new Label(dlgArea, SWT.WRAP);
		messageLbl.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		messageLbl.setText(message);
		
		messageDetailTxt = new Text(dlgArea, SWT.BORDER | SWT.MULTI | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		messageDetailTxt.setBackground( getShell().getDisplay().getSystemColor(SWT.COLOR_WHITE) );
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 2;
		messageDetailTxt.setLayoutData(gd);
		messageDetailTxt.setText(messageDetail);
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, OK, Messages.getString("ErrorDlg.0"), true); //$NON-NLS-1$
	}
	

}
