package atosj.dialog;

import java.io.File;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;

import atosj.Messages;
import atosj.app.UserPreferences;
import atosj.exception.AtosJException;

/**
 * Dialog zur Eingabe globaler Einstellungen fr ATOSj.
 * PDF-Reader und HTML-Reader.
 * @author Nicos Tegos
 */
public class OptionsDlg extends Dialog 
{
	private Text pdfReaderTxt;
	private String origPdfPath;
	
	private Text htmlReaderTxt;
	private String origHtmlPath;
	
	private Combo languageSelection;
	private String language_short = ""; //$NON-NLS-1$
	
	public OptionsDlg(Shell shell) {
		super(shell);
		origPdfPath = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.PDF_READER_PATH);
		origHtmlPath = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.HTML_READER_PATH);
		language_short = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.LANGUAGE);
		
		//language settings not set, use default: 'de'
		if(language_short == null){
			language_short = "de"; //$NON-NLS-1$
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, OK, Messages.getString("OptionsDlg.2"), true); //$NON-NLS-1$
		createButton(parent, CANCEL, Messages.getString("OptionsDlg.3"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case OK: okPressed();
			break;
			case CANCEL: cancelPressed();
			break;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(Messages.getString("OptionsDlg.4")); //$NON-NLS-1$
				
		return super.createContents(parent);
	}
	
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Label lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("OptionsDlg.5")); //$NON-NLS-1$
		Composite pdfArea = new Composite(dlgArea, SWT.NULL);
		pdfArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		pdfArea.setLayout(gl);
		
		pdfReaderTxt = new Text(pdfArea, SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 250;
		pdfReaderTxt.setLayoutData(gd);
		pdfReaderTxt.setToolTipText(Messages.getString("OptionsDlg.6")); //$NON-NLS-1$
		if(origPdfPath != null)
			pdfReaderTxt.setText(origPdfPath);
		
		Button pdfDirBtn = new Button(pdfArea, SWT.PUSH);
		pdfDirBtn.setText("..."); //$NON-NLS-1$
		pdfDirBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN);
				dlg.setText(Messages.getString("OptionsDlg.8")); //$NON-NLS-1$
				
				String pdfPath = dlg.open();
				if(pdfPath != null)
					pdfReaderTxt.setText(pdfPath);
			}
		});
		
		lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("OptionsDlg.9")); //$NON-NLS-1$
		Composite htmlArea = new Composite(dlgArea, SWT.NULL);
		htmlArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		htmlArea.setLayout(gl);
		
		htmlReaderTxt = new Text(htmlArea, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 250;
		htmlReaderTxt.setLayoutData(gd);
		htmlReaderTxt.setToolTipText(Messages.getString("OptionsDlg.10")); //$NON-NLS-1$
		if(origHtmlPath != null)
			htmlReaderTxt.setText(origHtmlPath);
		
		Button htmlDirBtn = new Button(htmlArea, SWT.PUSH);
		htmlDirBtn.setText("..."); //$NON-NLS-1$
		htmlDirBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN);
				dlg.setText(Messages.getString("OptionsDlg.12")); //$NON-NLS-1$
				
				String htmlPath = dlg.open();
				if(htmlPath != null)
					htmlReaderTxt.setText(htmlPath);
			}
		});
		
		//language selection drop down menu
		lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("OptionsDlg.13")); //$NON-NLS-1$
		Composite languageArea = new Composite(dlgArea, SWT.NULL);
		languageArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		languageArea.setLayout(gl);
		
		languageSelection = new Combo(languageArea, SWT.READ_ONLY);
		languageSelection.setItems(new String[] { "de", "en" });//$NON-NLS-1$ //$NON-NLS-2$
		languageSelection.setSize(200, 200);
		languageSelection.setText(language_short);
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * berprft die Korrektheit der Nutzereingaben.
	 * @throws AtosJException bei fehlerhaften Eingaben, nicht existente Dateien.
	 */
	protected void checkIntegrity() throws AtosJException
	{
		String test = pdfReaderTxt.getText();
		if( !test.equals("") ) //$NON-NLS-1$
		{
			File f = new File(test);
			if( !f.isFile() )
			{
				pdfReaderTxt.setFocus();
				throw new AtosJException(Messages.getString("OptionsDlg.17")); //$NON-NLS-1$
			}
		}
		
		test = htmlReaderTxt.getText();
		if( !test.equals("") ) //$NON-NLS-1$
		{
			File f = new File(test);
			if( !f.isFile() )
			{
				htmlReaderTxt.setFocus();
				throw new AtosJException(Messages.getString("OptionsDlg.19")); //$NON-NLS-1$
			}
		}
	}
	
	/**
	 * Speichert die angegebenen Einstellugen in der Datei fr die
	 * Nutzerprferenzen.
	 * @see UserPreferences#save()
	 */
	protected void okPressed() 
	{
		try
		{
			//read language setting
			String tmp = languageSelection.getText();
			if (!language_short.equals(tmp)){
				language_short = tmp;
				MessageDialog.openWarning(getShell(), Messages.getString("OptionsDlg.11"), Messages.getString("OptionsDlg.14"));
			}
			
			checkIntegrity();
			UserPreferences.getInstance().setPreferedValue( UserPreferences.PDF_READER_PATH, pdfReaderTxt.getText() );
			UserPreferences.getInstance().setPreferedValue( UserPreferences.HTML_READER_PATH, htmlReaderTxt.getText() );
			UserPreferences.getInstance().setPreferedValue( UserPreferences.LANGUAGE, language_short );
			UserPreferences.getInstance().save();
			super.okPressed();
		}
		catch(AtosJException e)
		{
			MessageDialog.openError(getShell(), Messages.getString("OptionsDlg.1"), Messages.getString("OptionsDlg.0") //$NON-NLS-1$ //$NON-NLS-2$
					+ e.getMessage());
		}
	}
}
