/*
 * Created on 18.02.2006
 */
package atosj.dialog;

import java.util.Vector;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;

import atosj.model.script.HTSScript;

/**
 * Filtert Skripte nach ihren Eigenschaften.
 * @author Nicos Tegos
 */
public class PackageTableFilter extends ViewerFilter
{
	private boolean hideNoCleanup = false;
	private boolean hideInteractive = false;
	
	/**
	 * Neuer Filter fr Skripte.
	 * @param hideInteractive wenn <code>true</code>, sollen alle interaktiven Skripte rausgefiltert werden.
	 * @param hideNoCleanup wenn <code>true</code>, sollen alle Skripte ohne das Kommando CLEANUP rausgefiltert werden.
	 * @see HTSScript#isInteractive()
	 * @see HTSScript#hasCleanup()
	 */
	public PackageTableFilter(boolean hideInteractive, boolean hideNoCleanup) {
		this.hideInteractive = hideInteractive;
		this.hideNoCleanup = hideNoCleanup;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerFilter#filter(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object[])
	 */
	public Object[] filter(Viewer viewer, Object parent, Object[] elements) 
	{
		Vector filteredElements = new Vector(elements.length);
		for(int i = 0; i < elements.length; ++i)
		{
			if( select(viewer, parent, elements[i]) )
				filteredElements.add(elements[i]);
		}
		
		return filteredElements.toArray();
	}

	/**
	 * Gibt an ob das angegeben <code>ScriptStatisticsTableElement</code> gefiltert wird.
	 * @param viewer Wird ignoriert.
	 * @param parentElement Wird ignoriert.
	 * @param element Ein <code>ScriptStatisticsTableElement</code>
	 * @return <code>true</code>, wenn <code>element</code> die Filtervorschrift erfllt, 
	 * d.h. <em>NICHT</em> rausgefiltert wird.
	 * @see ScriptStatisticsTableElement
	 */
	 public boolean select(Viewer viewer, Object parentElement, Object element) {
		return select( ((ScriptStatisticsTableElement) element).getScript() );
	 }
	 
	 /**
	  * Gibt an, ob das angegebene Skript gefiltert wird.
	  * @return <code>true</code>, wenn das <code>script</code> die Filtervorschrift erfllt, 
	  * d.h. <em>NICHT</em> rausgefiltert wird.
	  */
	public boolean select(HTSScript script) 
	{
		if( hideInteractive && script.isInteractive() )
			return false;
		if( hideNoCleanup && !script.hasCleanup() )
			return false;
		
		return true;
	}
	
	/**
	 * Schalten den Filter fr Skripte ohne CLEANUP ein oder aus.
	 * @param hide wenn <code>true</code> werden alle Skripte ohne CLEANUP rausgefiltert.
	 */
	public void hideNoCleanup(boolean hide) {
		hideNoCleanup = hide;
	}
	
	/**
	 * Schaltet den Filter fr interaktive Skripte ein oder aus.
	 * @param hide wenn <code>true</code> werden alle interaktiven Skripte rausgefiltert.
	 */
	public void hideInteractive(boolean hide) {
		hideInteractive = hide;
	}
}