/*
 * Created on 13.01.2006
 */
package atosj.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import atosj.Messages;

/**
 * Dialog der eine Positionsauswahl vor Einfgeoperationen ermglicht.
 * @author Nicos Tegos
 */
public class PositionDlg extends Dialog 
{
	/** Der Dialog wurde durch Drcken des Knopfes "Hinzufgen" beendet. */
	public static final int ID_ADD = 0;
	/** Der Dialog wurde durch Drcken des Knopfes "Abbrechen" beendet. */
	public static final int ID_CANCEL = 1;
	
	private String title;
	
	private PositionWidget posWidget;
	private int currentPos;
	private int maxPos;
	
	/**
	 * Erstellt einen neuen Dialog zur Positionsangabe.
	 * @param title Der Dialogtitel.
	 * @param maxPos siehe <code>PositionWidget</code>
	 * @param currentPos siehe <code>PositionWidget</code>
	 * @see PositionWidget
	 */
	public PositionDlg(Shell parent, String title, int maxPos, int currentPos) {
		super(parent);
		
		this.title = title;
		this.maxPos = maxPos;
		this.currentPos = currentPos;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_ADD, Messages.getString("PositionDlg.0"), true); //$NON-NLS-1$
		createButton(parent, ID_CANCEL, Messages.getString("PositionDlg.1"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_ADD: addPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	protected void addPressed() 
	{
		setReturnCode(ID_ADD);
		close();
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(title);
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		posWidget = new PositionWidget(dlgArea, maxPos, currentPos);
		posWidget.getContent().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * @return Postionskonstante gem POS_* in <code>PositionWidget</code>.
	 * @see PositionWidget
	 */
	public int getPosition() {
		return posWidget.getPosition();
	}
}
