/*
 * Created on 10.01.2006
 */
package atosj.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import atosj.Messages;
import atosj.gui.SWTToolkit;

/**
 * Diese Klasse beinhaltet eine Reihe von Radiobuttons, mit
 * denen der Nutzer die Position einer Einfgeoperation in eine Liste von Elementen
 * bestimmen kann.
 * @author Nicos Tegos
 */
public class PositionWidget 
{
	public static final int NO_CURRENT_POS = -1;
	
	private int currentPos;
	private int maxPos;
	
	private int pos; // die Position, die zurckgegeben wird
	
	private Button beforeBtn;
	private Button afterBtn;
	private Button topBtn;
	private Button bottomBtn;
	
	private Group positionGroup;
	
	/** 
	 * Erstellt ein neues Widget zur Auswahl der Einfgeposition.
	 * Mgliche Auswahlmglichkeiten sind:
	 * <ul>
	 * <li>vor markiertem Element</li>
	 * <li>nach markiertem Element</li>
	 * <li>am Anfang</li>
	 * <li>am Ende</li>
	 * </ul>
	 * @param maxPos Dieser Wert wird zurckgegeben, wenn der Nutzer "am Ende"
	 * als Einfgeposition gewhlt hat.
	 * @param currentPos NO_CURRENT_POS, wenn dieser Wert unbelegt bleiben soll,
	 * dann fallen die Einfgepositionen "nach/vor markiertem Element" weg. Ansonsten
	 * gibt <code>currentPos</code> die Position des aktuell ausgewhlten Elements wieder,
	 * 0-basiert.
	 * @param parent wDas graph. Vaterelement.
	 */
	public PositionWidget(Composite parent, int maxPos, int currentPos) 
	{
		this.currentPos = currentPos;
		this.maxPos = maxPos;
		
		positionGroup = new Group(parent, SWT.NONE);
		positionGroup.setText(Messages.getString("PositionWidget.0")); //$NON-NLS-1$
		positionGroup.setLayout( new GridLayout(2, true) );
		
		topBtn = SWTToolkit.createButton(positionGroup, SWT.RADIO, Messages.getString("PositionWidget.1")); //$NON-NLS-1$
		topBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		topBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setPositionTop();
			}
		});
		
		bottomBtn = SWTToolkit.createButton(positionGroup, SWT.RADIO, Messages.getString("PositionWidget.2")); //$NON-NLS-1$
		bottomBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		bottomBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				setPositionBottom();
			}
		});
				
		if( hasCurrentPos() )
		{
			beforeBtn = SWTToolkit.createButton(positionGroup, SWT.RADIO, Messages.getString("PositionWidget.3")); //$NON-NLS-1$
			beforeBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
			beforeBtn.addSelectionListener( new SelectionAdapter() {
				public void widgetSelected(SelectionEvent arg0) {
					setPositionBefore();
				}
			});
			
			afterBtn = SWTToolkit.createButton(positionGroup, SWT.RADIO, Messages.getString("PositionWidget.4")); //$NON-NLS-1$
			afterBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
			afterBtn.addSelectionListener( new SelectionAdapter() {
				public void widgetSelected(SelectionEvent arg0) {
					setPositionAfter();
				}
			});
			
			setPositionAfter();
			afterBtn.setSelection(true);
		}
		else
		{
			setPositionBottom();
			bottomBtn.setSelection(true);
		}
	}
	
	private void setPositionTop() {	pos = 0; }
	private void setPositionBottom() {	pos = maxPos; }
	private void setPositionBefore() { pos = currentPos; }
	private void setPositionAfter() { pos = currentPos + 1; }
	
	private boolean hasCurrentPos() {
		return currentPos != NO_CURRENT_POS;
	}
	
	/**
	 * Liefert die Einfgeposition.
	 * @return Je nach Auswahl die neue Position,
	 * diese liegt im Intervall [0, maxPos].
	 */
	public int getPosition() {
		return pos;
	}
	
	/**
	 * Liefert das tatschliche Widget, fr Verwendung in Dialogen oder
	 * in anderen graphischen Bereichen.
	 */
	public Composite getContent() {
		return positionGroup;
	}
}
