/*
 * Created on 20.11.2005
 */
package atosj.dialog;

import java.io.File;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.app.Project;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.gui.TableResizeListener;
import atosj.listener.ReceiverListener;
import atosj.rmi.TestStarterReceiver;

/**
 * Ein Dialog zum Erstellen und Bearbeiten von Projekten. 
 * @author Nicos Tegos
 */
public class ProjectDlg extends Dialog 
{
	public static final int ID_OK = 0;
	public static final int ID_CANCEL = 1;
	private static final int ID_START = 2; // Id fr den Button, der das Testobjekt testweise startet
	
	private Button awtRadio;
	private Button swtRadio;
	
	private Text projectNameTxt;
	private Text projectDirTxt;
	
	private Text mainClassTxt;
	private Text mainClassDirTxt;
	private Text mainWindowTxt;
	private TableViewer classPathTV;
	
	private Project project; // das neu angelegte Projekt bei erfolgreichem return
	private String currentPath; // der Pfad der im letzten Directory oder FileDialog gewhlt wurde
	private HashSet classPath = new HashSet(); // HashSet<String> Menge der Klassenpfade
	private ClassPathContentProvider classPathProvider = new ClassPathContentProvider();
	
	private TestStarterReceiver receiver; // startet ein TestObjekt mit den aktuellen projektparametern
	
	private boolean editing;
	
	/**
	 * Erstellt einen neuen Dialog zum Bearbeiten eines bereits vorhandenen Projektes.
	 * @param parent Das Vaterfenster.
	 * @param toEdit Das zu bearbeitende Projekt.
	 */
	public ProjectDlg(Shell parent, Project toEdit) 
	{
		super(parent);
		
		this.project = toEdit;
		editing = toEdit != null;
		if(editing)
		{
			currentPath = project.getTestObject().getMainClassDir();
			classPath.addAll( project.getTestObject().getClassPathList() );
		}
		
		try 
		{
			receiver = new TestStarterReceiver();
			receiver.addListener( new ReceiverListener() {
				
				public void runningStateChanged(boolean running) {
					getButton(ID_START).setText(running ? Messages.getString("ProjectDlg.0") : Messages.getString("ProjectDlg.1"));					 //$NON-NLS-1$ //$NON-NLS-2$
				}
			});
		} catch (RemoteException e) {
			// wir fangen bei startPressed() ab, wenn receiver null ist
		}
	}
	
	/**
	 * Erstellt einen neuen Dialog zum Erstellen eines neuen Projektes.
	 * @param parent Das Vaterfenster.
	 */
	public ProjectDlg(Shell parent) {
		this(parent, null);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_START, Messages.getString("ProjectDlg.2"), false); //$NON-NLS-1$
		createButton(parent, ID_OK, editing ? Messages.getString("ProjectDlg.3") : Messages.getString("ProjectDlg.4"), true); //$NON-NLS-1$ //$NON-NLS-2$
		createButton(parent, ID_CANCEL, Messages.getString("ProjectDlg.5"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_OK: okPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
			case ID_START: startPressed();
			break;
		}
	}
	
	/**
	 * Erstellt ein neues bzw. speichert ein bearbeitetes Projekt, sofern
	 * die Angaben korrekt waren.
	 */
	protected void okPressed() 
	{
		try
		{
			checkIntegrity();
			
			if(editing)
				changeProject();
			else
				createProject();
			
			setReturnCode(ID_OK);
			close();
		}
		catch(AtosJException e)
		{
			MessageDialog.openError(getShell(), 
					editing ? Messages.getString("ProjectDlg.6") : Messages.getString("ProjectDlg.7"),  //$NON-NLS-1$ //$NON-NLS-2$
					(editing ?  Messages.getString("ProjectDlg.8") : Messages.getString("ProjectDlg.9")) //$NON-NLS-1$ //$NON-NLS-2$
					+ e.getMessage());
		}
	}
	
	/**
	 * Bricht den Dialog ab.
	 */
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}
	
	/**
	 * Startet das Testobjekt probeweise, um dem Nutzer die Mglichkeit zu geben
	 * seine Angabe bzgl. des Klassenpfades zu berprfen.
	 */
	protected void startPressed() 
	{
		if(receiver == null) // kann eigentlich nur bei rmi problemen entstehen
			return;
		
		if( receiver.isRunning() )
		{
			receiver.stop();
		}
		else
		{
			try {
				checkIntegrity();
			} catch(AtosJException e) {
				MessageDialog.openError(getShell(), Messages.getString("ProjectDlg.10"), Messages.getString("ProjectDlg.11") //$NON-NLS-1$ //$NON-NLS-2$
						+ e.getMessage());
				return;
			}
			
			boolean projectType = (awtRadio.getSelection() == true);
			String projectDir = projectDirTxt.getText();
			if( !projectDir.endsWith(File.separator) )
				projectDir += File.separator;
			projectDir += projectNameTxt.getText();
			
			// ein Behilfstestobjekt erstellen
			final TestObject testee = new TestObject
			(		projectNameTxt.getText(),
					mainClassDirTxt.getText(),
					mainClassTxt.getText(),
					mainWindowTxt.getText(),
					projectDir,
					new Vector(classPath),
					projectType,
					false,
					new Vector()
			);
			
			Thread starter = new Thread() {
				public void run() 
				{
					try {
						receiver.start(testee);
					} 
					catch(final AtosJException e) 
					{
						getShell().getDisplay().asyncExec( new Runnable() {
							public void run() {
								MessageDialog.openError(getShell(), Messages.getString("ProjectDlg.12"), Messages.getString("ProjectDlg.13") //$NON-NLS-1$ //$NON-NLS-2$
										+ e.getMessage());
							}
						});
					}
				}
			};
			
			starter.start();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#close()
	 */
	public boolean close() {
		if(receiver != null)
			receiver.dispose();
		return super.close();
	}

	/**
	 * Initialisiert den Inhalt der graph. Elemente.
	 */
	private void initContent() {
		if(editing)
		{
			// project angaben
			TestObject testee = project.getTestObject();
			if( project.isAwtProject() )
				awtRadio.setSelection(true);
			else
				swtRadio.setSelection(true);
			projectNameTxt.setText( project.getName() );
			projectNameTxt.setToolTipText( project.getName() );
			projectDirTxt.setText( testee.getProjectDir() );
			projectDirTxt.setToolTipText( testee.getProjectDir() );
			
			// testobject angaben
			mainClassTxt.setText( testee.getMainClass() );
			mainClassDirTxt.setText( testee.getMainClassDir() );
			mainWindowTxt.setText( testee.getMainWindow() );
			
			// input fr ClassPath setzen
			classPathTV.setInput(classPath);
		}
		else
		{
			// input fr ClassPath setzen
			classPathTV.setInput(classPath);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell shell) {
		shell.setText(editing ? Messages.getString("ProjectDlg.14") : Messages.getString("ProjectDlg.15")); //$NON-NLS-1$ //$NON-NLS-2$
		shell.setSize(640, 640);
		
		super.configureShell(shell);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		createProjectArea(dlgArea);
		createTesteeArea(dlgArea);
		
		initContent();
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	private void createProjectArea(Composite parent) 
	{
		Group projectArea =  new Group(parent, SWT.NULL);
		projectArea.setText(Messages.getString("ProjectDlg.16")); //$NON-NLS-1$
		projectArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		projectArea.setLayout( new GridLayout(2, false) );
		
		Label lbl = new Label(projectArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.17")); //$NON-NLS-1$
		
	    Composite projectType = new Composite(projectArea, SWT.NULL);
	    projectType.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	    RowLayout radioRowLayout = new RowLayout();
	    radioRowLayout.marginLeft = 50; 
	    radioRowLayout.marginRight = 100;
	    radioRowLayout.justify = true;
	    projectType.setLayout( radioRowLayout );
	    	
    	awtRadio = new Button(projectType, SWT.RADIO);	
    	awtRadio.setText("Swing"); //$NON-NLS-1$
    	awtRadio.setEnabled(!editing);
	
    	swtRadio = new Button(projectType, SWT.RADIO);	
    	swtRadio.setText("SWT"); //$NON-NLS-1$
		swtRadio.setEnabled(!editing);
		
		lbl = new Label(projectArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.20")); //$NON-NLS-1$
		if(editing)
		{
			projectNameTxt = new Text(projectArea, SWT.BORDER | SWT.READ_ONLY);
			projectNameTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
			projectNameTxt.setBackground( projectNameTxt.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND) );
		}
		else
		{
			projectNameTxt = new Text(projectArea, SWT.BORDER);
			projectNameTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		}
		
		lbl = new Label(projectArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.21")); //$NON-NLS-1$
		Composite dirArea = new Composite(projectArea, SWT.NULL);
		dirArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(editing ? 1 : 2, false);
		gl.marginWidth = gl.marginHeight = 0;
		dirArea.setLayout(gl);
		
		if(editing)
		{
			projectDirTxt = new Text(dirArea, SWT.BORDER | SWT.READ_ONLY);
			projectDirTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
			projectDirTxt.setBackground( projectDirTxt.getDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND) );
		}
		else
		{
			projectDirTxt = new Text(dirArea, SWT.BORDER);
			projectDirTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
			projectDirTxt.addModifyListener( new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					projectDirTxt.setToolTipText( projectDirTxt.getText() );
				}
			});
		}
				
		if(!editing)
		{
			Button projectDirBtn = new Button(dirArea, SWT.PUSH);
			projectDirBtn.setText("..."); //$NON-NLS-1$
			projectDirBtn.addSelectionListener( new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					DirectoryDialog dlg = new DirectoryDialog(e.display.getActiveShell());
					dlg.setText(Messages.getString("ProjectDlg.23")); //$NON-NLS-1$
					dlg.setMessage(Messages.getString("ProjectDlg.24")); //$NON-NLS-1$
					String dir = dlg.open();
					if(dir != null)
						projectDirTxt.setText(dir);
				}
			});
		}
	}
	
	private void createTesteeArea(Composite parent) 
	{
		Group testeeArea =  new Group(parent, SWT.NULL);
		testeeArea.setText(Messages.getString("ProjectDlg.25")); //$NON-NLS-1$
		testeeArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		testeeArea.setLayout( new GridLayout(2, false) );
		
		Label lbl = new Label(testeeArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.26")); //$NON-NLS-1$
		mainClassTxt = new Text(testeeArea, SWT.BORDER);
		mainClassTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		lbl = new Label(testeeArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.27")); //$NON-NLS-1$
		Composite dirArea = new Composite(testeeArea, SWT.NULL);
		dirArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		dirArea.setLayout(gl);
				
		mainClassDirTxt = new Text(dirArea, SWT.BORDER);
		mainClassDirTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		mainClassDirTxt.addModifyListener( new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				mainClassDirTxt.setToolTipText( mainClassDirTxt.getText() );
			}
			
		});
		
		Button mainClassDirBtn = new Button(dirArea, SWT.PUSH);
		mainClassDirBtn.setText("..."); //$NON-NLS-1$
		mainClassDirBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dlg = new DirectoryDialog(e.display.getActiveShell());
				dlg.setText(Messages.getString("ProjectDlg.29")); //$NON-NLS-1$
				dlg.setMessage(Messages.getString("ProjectDlg.30")); //$NON-NLS-1$
				dlg.setFilterPath(currentPath);
				
				String dir = dlg.open();
				if(dir != null)
				{
					currentPath = dir;
					mainClassDirTxt.setText(dir);
				}
			}
		});
		
		lbl = new Label(testeeArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.31")); //$NON-NLS-1$
		mainWindowTxt = new Text(testeeArea, SWT.BORDER);
		mainWindowTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		lbl = new Label(testeeArea, SWT.NULL);
		lbl.setText(Messages.getString("ProjectDlg.32")); //$NON-NLS-1$
		//TODO: Hinweis darauf, dass die SWT-libs nicht enthalten sein drfen
		lbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		Composite classpathArea = new Composite(testeeArea, SWT.NULL);
		classpathArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		classpathArea.setLayout(gl);
		
		classPathTV = new TableViewer(classpathArea, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		classPathTV.setContentProvider( classPathProvider );
		classPathTV.setLabelProvider( new ClassPathLabelProvider() );
		classPathTV.setSorter( new ClassPathSorter() );
		
		Table table = classPathTV.getTable();
		//table.addControlListener( new TableResizeListener(new int[] {100}, 40) );
		new TableResizeListener(table, new int[] {100}, 40);
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		new TableColumn(table, SWT.NULL);
						
		Composite buttonArea = new Composite(classpathArea, SWT.NULL);
		buttonArea.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		buttonArea.setLayout(gl);
		
		Button addJarBtn = new Button(buttonArea, SWT.PUSH);
		addJarBtn.setText(Messages.getString("ProjectDlg.33")); //$NON-NLS-1$
		addJarBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		addJarBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				FileDialog dlg = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN | SWT.MULTI);
				dlg.setText(Messages.getString("ProjectDlg.34")); //$NON-NLS-1$
				dlg.setFilterExtensions( new String[] {"*.jar"} ); //$NON-NLS-1$
				dlg.setFilterNames( new String[] {"Java Archive *.jar"} ); //$NON-NLS-1$
				dlg.setFilterPath(currentPath);
				
				String firstPath = dlg.open();
				if(firstPath != null)
				{
					currentPath = new File(firstPath).getParent();
					String[] fileNames = dlg.getFileNames();
					for(int i = 0; i < fileNames.length; ++i)
						classPathProvider.add( dlg.getFilterPath() + File.separator  + fileNames[i] );
				}
			}
		});
		
		Button addDirBtn = new Button(buttonArea, SWT.PUSH);
		addDirBtn.setText(Messages.getString("ProjectDlg.37")); //$NON-NLS-1$
		addDirBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		addDirBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dlg = new DirectoryDialog(e.display.getActiveShell());
				dlg.setText(Messages.getString("ProjectDlg.38")); //$NON-NLS-1$
				dlg.setMessage(Messages.getString("ProjectDlg.39")); //$NON-NLS-1$
				dlg.setFilterPath(currentPath);
				
				String dir = dlg.open();
				if(dir != null)
				{
					currentPath = dir;
					classPathProvider.add(dir);
				}
			}
		});
		
		Button removeBtn = new Button(buttonArea, SWT.PUSH);
		removeBtn.setText(Messages.getString("ProjectDlg.40")); //$NON-NLS-1$
		removeBtn.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		removeBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				StructuredSelection ssel = (StructuredSelection) classPathTV.getSelection();
				Iterator it = ssel.iterator();
				while( it.hasNext() )
					classPathProvider.remove( (String)it.next() );
			}
		});
	}
	
	/**
	 * berprft die Gltigkeit der Nutzerangaben.
	 * @throws AtosJException bei fehlerhaften Angaben.
	 */
	private void checkIntegrity() throws AtosJException
	{
		if(!editing)
		{
			if( !swtRadio.getSelection() && !awtRadio.getSelection() )
			{
				awtRadio.setFocus();
				throw new AtosJException(Messages.getString("ProjectDlg.41")); //$NON-NLS-1$
			}
			
			String test = projectNameTxt.getText();
			if( test.equals("") ) //$NON-NLS-1$
			{
				projectNameTxt.setFocus();
				throw new AtosJException(Messages.getString("ProjectDlg.43")); //$NON-NLS-1$
			}
			
			test = projectDirTxt.getText();
			if( test.equals("") ) //$NON-NLS-1$
			{
				projectDirTxt.setFocus();
				throw new AtosJException(Messages.getString("ProjectDlg.45")); //$NON-NLS-1$
			}
			else
			{
				File f = new File(test);
				if( !f.isDirectory() )
				{
					projectDirTxt.setFocus();
					throw new AtosJException(Messages.getString("ProjectDlg.46")); //$NON-NLS-1$
				}
			}
		}
		
		String test = mainClassTxt.getText();
		if( test.equals("") ) //$NON-NLS-1$
		{
			mainClassTxt.setFocus();
			throw new AtosJException(Messages.getString("ProjectDlg.48")); //$NON-NLS-1$
		}
		
		test = mainClassDirTxt.getText();
		if( test.equals("") ) //$NON-NLS-1$
		{
			mainClassDirTxt.setFocus();
			throw new AtosJException(Messages.getString("ProjectDlg.50")); //$NON-NLS-1$
		}
		else
		{
			File f = new File(test);
			if( !f.isDirectory() )
			{
				mainClassDirTxt.setFocus();
				throw new AtosJException(Messages.getString("ProjectDlg.51")); //$NON-NLS-1$
			}
		}
		
		test = mainWindowTxt.getText();
		if( test.equals("") ) //$NON-NLS-1$
		{
			mainWindowTxt.setFocus();
			throw new AtosJException(Messages.getString("ProjectDlg.53")); //$NON-NLS-1$
		}
		
		if( classPath.size() == 0)
		{
			throw new AtosJException(Messages.getString("ProjectDlg.54")); //$NON-NLS-1$
		}
		
		test = projectDirTxt.getText() + File.separator + projectNameTxt.getText();
		File f = new File(test);
		if( !f.isAbsolute() )
		{
			projectNameTxt.setFocus();
			throw new AtosJException(Messages.getString("ProjectDlg.55")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Speichert die Nutzernderungen nach dem Bearbeiten eines Projekts.
	 * @throws AtosJException wenn beim Speichern Fehler auftraten.
	 */
	private void changeProject()  throws AtosJException
	{
		TestObject testee = project.getTestObject();
		// sicherheitskopien der originalen werte
		String origMainClass = testee.getMainClass();
		String origMainClassDir = testee.getMainClassDir();
		String origMainWindow = testee.getMainWindow();
		Vector origClassPath = testee.getClassPathList();
		
		try
		{
			// Setzen der genderten werte
			testee.setMainWindow( mainWindowTxt.getText() );
			testee.setMainClass(mainClassTxt.getText() );
			testee.setMainClassDir( mainClassDirTxt.getText() );
			testee.setClassPathList( new Vector(classPath) );
			project.save(); // hopefully this fuckin' exception will never occur
			
			setReturnCode(ID_OK);
			close();
		}
		catch(AtosJException e)
		{
			// zurck zu den original werten
			testee.setMainClass(origMainClass);
			testee.setMainWindow(origMainWindow);
			testee.setMainClassDir(origMainClassDir);
			testee.setClassPathList(origClassPath);
			
			throw e;
		}
	}
	
	/**
	 * Erstellt ein neues Projekt.
	 * @throws AtosJException wenn beim Erstellen Fehler autraten.
	 */
	private void createProject() throws AtosJException  
	{
		boolean awtProject = (awtRadio.getSelection() == true);
		String projectDir = projectDirTxt.getText();
		if( !projectDir.endsWith(File.separator) )
			projectDir += File.separator;
		projectDir += projectNameTxt.getText();
		
		project = new Project
		(
			projectNameTxt.getText(), 
			projectDir, 
			mainClassTxt.getText(), 
			mainClassDirTxt.getText(),
			mainWindowTxt.getText(),
			new Vector(classPath),
			awtProject
		);
	}
	
	/**
	 * Liefert das neue bzw. das bearbeitete Projekt.
	 * @return Das Projekt oder <code>null</code>, wenn der Dialog mit Abbrechen beendet wurde.
	 */
	public Project getProject() {
		return project;
	}
}
