/*
 * Created on 08.12.2005
 */
package atosj.dialog;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.UserPreferences;
import atosj.gui.BalloonWindow;
import atosj.gui.SWTToolkit;
import atosj.gui.TableResizeListener;
import atosj.listener.PackagePlayerReceiverListener;
import atosj.model.pack.HTSPackage;
import atosj.rmi.HTSPlayer;
import atosj.util.ReportPrinter;
import atosj.util.Statistics;

/**
 * Dialog zur Ausfhrung eines Testpakets.
 * @author Nicos Tegos
 */
public class RunPackageDlg extends Dialog 
{
	private static final int ID_RUN = 0;
	private static final int ID_CANCEL = 2;
	private static final int ID_REPORT = 3;
	
	private HTSPackage pack;
	private HTSPlayer receiver;
	private String reportPath;
	
	private PackageTableFilter filter;
		
	private boolean haltOnError = false;
	private Button haltOnErrorChk;
	private boolean createReport = false;
	private Button createReportChk;
	private boolean haltOnEndOfScript = false;
	private Button haltOnEndOfScriptChk;
	private boolean ignoreInteractive = false;
	private Button ignoreInteractiveChk;
	private boolean ignoreNoCleanup = false;
	private Button ignoreNoCleanupChk;
	private int delay = 100;
	private Spinner delayTxt;
	private Label delayLbl;
		
	private TableViewer packageTV; // zeigt alle Kommandos des aktuellen skripts
	private Composite[] scriptAreas; // Array mit allen mit Tabellen fr kommandos pro script, fr stacklayout
	private TableViewer[] scriptTVs;
	
	private StackLayout scriptStack;
	private Composite scriptArea;
	
	private Button reportButton, runButton, escapeButton;
	private StdTextSetter[] stderrListeners, stdoutListeners;
	private StyledText stdTxt;
	
	/**
	 * Erstellt einen neuen Dialog zur Ausfhrung eines Testpakets.
	 * @param parent Das Vaterfenster.
	 * @param pack Das Paket, das ausgefhrt werden soll.
	 */
	public RunPackageDlg(Shell parent, HTSPackage pack) {
		super(parent);
		setShellStyle(SWT.TITLE | SWT.MIN | SWT.MAX | SWT.RESIZE | SWT.APPLICATION_MODAL );
		
		this.pack = pack;
		filter = new PackageTableFilter(ignoreInteractive, ignoreNoCleanup);
		
		setupReceiver();
	}
	
	/**
	 * Initialisiert den Listener, der auf den Start und das Ende der Paketausfhrung reagieren soll,
	 * sowie ber die Ausfhrung von Kommandos und Skripten unterrichtet wird. Dieser aktualisiert
	 * die graphischen Anzeigen des Dialogs, insbesondere die Ausfhrungsstatistiken.
	 * @see PackagePlayerReceiverListener
	 */
	private void setupReceiver() {
		try {
			receiver = new HTSPlayer(AtosJApp.getInstance().getInput().getTestObject(), pack);
		}
		catch(RemoteException e) {
			return;
		}
		
		receiver.addListener( new PackagePlayerReceiverListener() {
			private int scriptIndex = -1;
						
			public void runningStateChanged(final boolean running) 
			{
				haltOnErrorChk.setEnabled(running ? false : true);
				createReportChk.setEnabled(running ? false : true);
				haltOnEndOfScriptChk.setEnabled(running ? false : true);
				ignoreInteractiveChk.setEnabled(running ? false : true);
				ignoreNoCleanupChk.setEnabled(running ? false : true);
				delayTxt.setEnabled(running ? false : true);
				delayLbl.setEnabled(running ? false : true);
								
				if(running)
				{
					runButton.setText(Messages.getString("RunPackageDlg.0")); //$NON-NLS-1$
					scriptIndex = -1;
					stdTxt.setText(""); //$NON-NLS-1$
					// die StreamListener zurcksetzen
					for(int i = 0; i < pack.getNumberOfScripts(); ++i)
					{
						stdoutListeners[i].clear();
						stderrListeners[i].clear();
					}
					// die Statistiken fr einen neuen Lauf wieder auf Ungetestet setzen
					((StatisticsTableContentProvider)packageTV.getContentProvider()).reset();
					for(int i = 0; i < scriptTVs.length; ++i)
						((StatisticsTableContentProvider)scriptTVs[i].getContentProvider()).reset();
				}
				else
				{
					runButton.setText(Messages.getString("RunPackageDlg.2")); //$NON-NLS-1$
					// markierung der aktuellen zeile entfernen
					((StatisticsTableContentProvider)packageTV.getContentProvider()).removeHighlights();
					for(int i = 0; i < scriptTVs.length; ++i)
						((StatisticsTableContentProvider)scriptTVs[i].getContentProvider()).removeHighlights();
					
					// meldungen ausgeben und evtl. Report erstellen
					if( !receiver.isStoppedByUser() ) // nicht durch Nutzer beendet
					{
						if( receiver.hasError() )
						{
							new ErrorDlg(Display.getDefault().getActiveShell(), 
									Messages.getString("RunPackageDlg.3"), //$NON-NLS-1$
									Messages.getString("RunPackageDlg.4") + pack.getName() + Messages.getString("RunPackageDlg.5"), //$NON-NLS-1$ //$NON-NLS-2$
									receiver.getErrors() ).open();
						}
						else
						{
							MessageDialog.openInformation(null, Messages.getString("RunPackageDlg.6"), //$NON-NLS-1$
									Messages.getString("RunPackageDlg.7") + pack.getName() + Messages.getString("RunPackageDlg.8")); //$NON-NLS-1$ //$NON-NLS-2$
						}
						
						if(createReport)
						{
							String[] stdout = new String[pack.getNumberOfScripts()];
							String[] stderr = new String[pack.getNumberOfScripts()];
							for(int i = 0; i < pack.getNumberOfScripts(); ++i)
							{
								stdout[i] = stdoutListeners[i].getText();
								stderr[i] = stderrListeners[i].getText();
							}
							
							reportPath = ReportPrinter.getInstance().createPackagePdf(
									pack, 
									receiver.getScriptStatistics(), 
									receiver.getCommandStatistics(), 
									stdout, stderr);
							
							if(reportPath != null)
							{
								reportButton.setEnabled(true);
								reportButton.setToolTipText(reportPath);
							}
						}
					}
				}
			}

			public void commandStarted(int index) {
				((StatisticsTableContentProvider)scriptTVs[scriptIndex].getContentProvider()).setRunning(index);
			}

			public void scriptStarted(int index) {
				scriptIndex = index;
				// die Selection immer mitwandern lassen und damit auch die aktuelle scriptTabelle
				packageTV.setSelection( new StructuredSelection(receiver.getScriptStatistics()[index]) );
				
				((StatisticsTableContentProvider)packageTV.getContentProvider()).setRunning(index);
				setTopScript(scriptIndex);
			}

			public void scriptStatisticsReceived(Statistics statistics) {
				((StatisticsTableContentProvider)packageTV.getContentProvider()).setCurrentStatistics(statistics);
			}

			public void commandStatisticsReceived(Statistics statistics) {
				((StatisticsTableContentProvider)scriptTVs[scriptIndex].getContentProvider()).setCurrentStatistics(statistics);
			}
		});
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		reportButton = createButton(parent, ID_REPORT, Messages.getString("RunPackageDlg.9"), false); //$NON-NLS-1$
		reportButton.setEnabled(false);
		runButton = createButton(parent, ID_RUN, Messages.getString("RunPackageDlg.10"), true); //$NON-NLS-1$
		escapeButton = createButton(parent, ID_CANCEL, Messages.getString("RunPackageDlg.11"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_RUN: runPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
			case ID_REPORT: reportPressed();
			break;
		}
	}
	
	/**
	 * Startet den PDF Reader und zeigt den Bericht zur Testausfhrung an.
	 */
	protected void reportPressed() {
		String pdfReader = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.PDF_READER_PATH);
		if(pdfReader != null)
		{
			try {
				Runtime.getRuntime().exec(pdfReader + " \"" + reportPath + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
			} catch(IOException e) {
				MessageDialog.openError(null, Messages.getString("RunPackageDlg.14"), Messages.getString("RunPackageDlg.15")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		else
		{
			MessageDialog.openError(null, Messages.getString("RunPackageDlg.16"), Messages.getString("RunPackageDlg.17")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Startet das Abspielen des Pakets.
	 * @see HTSPlayer#start(Shell, PackageTableFilter, int, boolean, boolean, StreamListener[], StreamListener[])
	 */
	protected void runPressed() 
	{
		if( receiver.isRunning() )
		{
			receiver.userStop();
		}
		else
		{
			Runnable run = new Runnable()
			{
				public void run() {
					receiver.start(
							getShell(),
							filter,
							delay, 
							haltOnError,
							haltOnEndOfScript,
							stdoutListeners,
							stderrListeners
					);
				}
			};
			
			new Thread(run).start();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#close()
	 */
	public boolean close() 
	{
		if( receiver.isRunning() )
		{
			MessageDialog questionDlg = new MessageDialog(
					Display.getDefault().getActiveShell(), 
					Messages.getString("RunPackageDlg.18"),  //$NON-NLS-1$
					null, 
					Messages.getString("RunPackageDlg.19"),  //$NON-NLS-1$
					MessageDialog.QUESTION, 
					new String[] {Messages.getString("RunPackageDlg.20"), Messages.getString("RunPackageDlg.21")},  //$NON-NLS-1$ //$NON-NLS-2$
					1);
			
			if(questionDlg.open() == 1) // nicht beenden
				return false;
		}
		
		// MOLTO importante sonst wird das Hauptprogramm beim Beenden nicht geschlossen
		receiver.dispose();
		
		if( getShell().getMaximized() )
			UserPreferences.getInstance().setPreferedValue(UserPreferences.PACKAGE_DLG_STATE, UserPreferences.STATE_WND_MAXIMIZED);
		else
			UserPreferences.getInstance().setPreferedValue(UserPreferences.PACKAGE_DLG_RECT, getShell().getBounds() );
		
		setReturnCode(ID_CANCEL);
		return super.close();
	}
	
	protected void cancelPressed() 
	{
		close();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#open()
	 */
	public int open() {
		if(receiver == null)
		{
			MessageDialog.openError(null, Messages.getString("RunPackageDlg.22"), Messages.getString("RunPackageDlg.23") ); //$NON-NLS-1$ //$NON-NLS-2$
			return ID_CANCEL;
		}
		
		return super.open();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(Messages.getString("RunPackageDlg.24") + pack.getName() + Messages.getString("RunPackageDlg.25")); //$NON-NLS-1$ //$NON-NLS-2$
		getShell().setImage( AtosJApp.getInstance().getImage(AtosJApp.IMG_ICON) );
		
		getShell().setBounds( (Rectangle) UserPreferences.getInstance().getPreferedValue(UserPreferences.PACKAGE_DLG_RECT) );
		if( UserPreferences.getInstance().getPreferedValue(UserPreferences.PACKAGE_DLG_STATE).equals(UserPreferences.STATE_WND_MAXIMIZED) )
			getShell().setMaximized(true);
				
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		createOptionsArea(dlgArea);
		createPackageTableArea(dlgArea);
		
		// scriptstack initialisieren
		Vector scripts = pack.getScripts();
		
		scriptTVs = new TableViewer[scripts.size()];
		scriptAreas = new Composite[scripts.size()];
		
		scriptArea = new Composite(dlgArea, SWT.NULL);
		scriptArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		scriptStack = new StackLayout();
		scriptStack.marginHeight = scriptStack.marginWidth = 0;
		scriptArea.setLayout(scriptStack);
		
		for(int i = 0; i < scripts.size(); ++i)
			createScriptTableArea(scriptArea, i);
		
		createStdArea(dlgArea);
		
		// inputs initialisieren
		initContents();
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	private void createOptionsArea(Composite parent) 
	{
		Group optionsArea =  new Group(parent, SWT.NULL);
		optionsArea.setText(Messages.getString("RunPackageDlg.26")); //$NON-NLS-1$
		optionsArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		optionsArea.setLayout( new GridLayout(2, true) );
		
		haltOnErrorChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunPackageDlg.27")); //$NON-NLS-1$
		haltOnErrorChk.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		haltOnErrorChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				haltOnError = haltOnErrorChk.getSelection();
			}
		});
		haltOnErrorChk.setSelection(haltOnError);
		
		haltOnEndOfScriptChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunPackageDlg.28")); //$NON-NLS-1$
		haltOnEndOfScriptChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				haltOnEndOfScript = haltOnEndOfScriptChk.getSelection();
			}
		});
		haltOnEndOfScriptChk.setSelection(haltOnEndOfScript);
		
		createReportChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunPackageDlg.29")); //$NON-NLS-1$
		createReportChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				createReport = createReportChk.getSelection();
			}
		});
		createReportChk.setSelection(createReport);
		
		ignoreInteractiveChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunPackageDlg.30")); //$NON-NLS-1$
		ignoreInteractiveChk.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		ignoreInteractiveChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ignoreInteractive = ignoreInteractiveChk.getSelection();
				filter.hideInteractive(ignoreInteractive);
				packageTV.refresh();
			}
		});
		ignoreInteractiveChk.setSelection(ignoreInteractive);
		
		// Spinner fr die Verzgerungszeit
		Composite delayArea = new Composite(optionsArea, SWT.NULL);
		GridLayout gl = new GridLayout(2, false);
		gl.marginHeight = gl.marginWidth = 0;
		delayArea.setLayout(gl);
		delayTxt = new Spinner(delayArea, SWT.BORDER);
		delayTxt.setMinimum(0);
		delayTxt.setMaximum(1000000);
		delayTxt.setIncrement(100);
		delayTxt.setSelection(delay);
		delayTxt.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				delay = delayTxt.getSelection();
			}
		});
		
		delayLbl = new Label(delayArea, SWT.NULL);
		delayLbl.setText(Messages.getString("RunPackageDlg.31")); //$NON-NLS-1$
		
		ignoreNoCleanupChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunPackageDlg.32")); //$NON-NLS-1$
		ignoreNoCleanupChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ignoreNoCleanup = ignoreNoCleanupChk.getSelection();
				filter.hideNoCleanup(ignoreNoCleanup);
				packageTV.refresh();
			}
		});
		ignoreNoCleanupChk.setSelection(ignoreNoCleanup);
	}
	
	private void createScriptTableArea(Composite parent, int index) 
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		tableArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		tableArea.setLayout(gl);
						
		final TableViewer scriptTV = new TableViewer(tableArea, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.READ_ONLY);
		scriptTV.setColumnProperties( CommandStatisticsTableContentProvider.PROPERTIES);
		scriptTV.setLabelProvider( new CommandStatisticsTableLabelProvider(scriptTV) );
		scriptTV.setContentProvider( new CommandStatisticsTableContentProvider(index) );
				
		Table table = scriptTV.getTable();
		// fehler onDoubleclick anzeigen
		table.addMouseListener( new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) {
				StructuredSelection ssel = (StructuredSelection) scriptTV.getSelection();
				if( !ssel.isEmpty() )
				{
					CommandStatisticsTableElement element = (CommandStatisticsTableElement) ssel.getFirstElement();
					if( element.hasError() )
					{
						final BalloonWindow exWindow = new BalloonWindow(getShell(), SWT.TITLE | SWT.CLOSE);
						exWindow.setImage(AtosJApp.getInstance().getImage( AtosJApp.IMG_ERROR) );
						exWindow.setTitle(Messages.getString("RunPackageDlg.33")); //$NON-NLS-1$
						exWindow.setText( element.getError() );
						exWindow.setLocation( scriptTV.getTable().toDisplay(e.x, e.y) );
						exWindow.getShell().addShellListener( new ShellAdapter() {
							public void shellDeactivated(ShellEvent arg0) {
								exWindow.close();
							}
						});
						exWindow.open();
					}
				}
			}
		});
		
		new TableResizeListener(table, new int[] {10, 42, 16, 16, 16}, 50);
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.34")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.35")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.36")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.37")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.38")); //$NON-NLS-1$
		
		scriptAreas[index] = tableArea;
		scriptTVs[index] = scriptTV;
	}
	
	private void createPackageTableArea(Composite parent) 
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		tableArea.setLayoutData(gd);
		GridLayout gl = new GridLayout(1, false);
		gl.marginHeight = gl.marginWidth = 0;
		tableArea.setLayout(gl);
		
		packageTV = new TableViewer(tableArea, SWT.BORDER| SWT.MULTI | SWT.FULL_SELECTION);
		packageTV.setColumnProperties( ScriptStatisticsTableContentProvider.PROPERTIES );
		packageTV.setContentProvider( new ScriptStatisticsTableContentProvider() );
		packageTV.setLabelProvider( new ScriptStatisticsTableLabelProvider(packageTV) );
		packageTV.addFilter(filter);
		packageTV.addSelectionChangedListener( new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent e) {
					StructuredSelection ssel = (StructuredSelection) e.getSelection();
					if( !ssel.isEmpty() )
					{
						ScriptStatisticsTableElement element =  (ScriptStatisticsTableElement) ssel.getFirstElement();
						setTopScript( element.getIndex() );
						scriptArea.setVisible(true);
					}
					else
					{
						scriptArea.setVisible(false);
					}
			}
			
		});
		
		Table table = packageTV.getTable();
		// fehler onDoubleclick anzeigen
		table.addMouseListener( new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) {
				StructuredSelection ssel = (StructuredSelection) packageTV.getSelection();
				if( !ssel.isEmpty() )
				{
					ScriptStatisticsTableElement element = (ScriptStatisticsTableElement) ssel.getFirstElement();
					if( element.hasError() )
					{
						final BalloonWindow exWindow = new BalloonWindow(getShell(), SWT.TITLE | SWT.CLOSE);
						exWindow.setImage(AtosJApp.getInstance().getImage( AtosJApp.IMG_ERROR) );
						exWindow.setTitle(Messages.getString("RunPackageDlg.39")); //$NON-NLS-1$
						exWindow.setText( element.getError() );
						exWindow.setLocation( packageTV.getTable().toDisplay(e.x, e.y) );
						exWindow.getShell().addShellListener( new ShellAdapter() {
							public void shellDeactivated(ShellEvent arg0) {
								exWindow.close();
							}
						});
						exWindow.open();
					}
				}
			}
		});
		
		new TableResizeListener(table, new int[] {10, 34, 12, 12, 20, 12}, 50);
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.40")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.41")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.42")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.43")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.44")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunPackageDlg.45")); //$NON-NLS-1$
	}
	
	private void createStdArea(Composite parent) 
	{
		Composite stdArea = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		stdArea.setLayout(gl);
		stdArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		stdTxt = new StyledText(stdArea, SWT.BORDER | SWT.READ_ONLY | SWT.V_SCROLL | SWT.H_SCROLL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 80;
		stdTxt.setLayoutData(gd);
		
		stdoutListeners = new StdTextSetter[pack.getNumberOfScripts()];
		stderrListeners = new StdTextSetter[pack.getNumberOfScripts()];
		for(int i = 0; i < pack.getNumberOfScripts(); ++i)
		{
			stdoutListeners[i] = new StdTextSetter( stdTxt, stdTxt.getDisplay().getSystemColor(SWT.COLOR_BLUE) );
			stderrListeners[i] = new StdTextSetter( stdTxt, stdTxt.getDisplay().getSystemColor(SWT.COLOR_RED) );
		}
	}
	
	private void setTopScript(int index) {
		scriptStack.topControl = scriptAreas[index];
		scriptArea.layout();
	}
	
	private void initContents() 
	{
		packageTV.setInput(receiver);
		packageTV.setSelection( new StructuredSelection(receiver.getScriptStatistics()[0]) );
		for(int i = 0; i < pack.getNumberOfScripts(); ++i)
		{
			scriptTVs[i].setInput(receiver);
			//if(i == 0)
				//setTopScript(i);
		}
	}
}
;