package atosj.dialog;

import java.util.Iterator;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.swt.SWTCommandCreator;
import atosj.gui.SWTToolkit;
import atosj.model.hts.command.HTSTest;

/**
 * Liefert die Beschriftung der einzelnen Tabelleneintrge.
 * @see atosj.model.hts.command.HTSTest#toCaptureDlgString()
 */
class TestActionLabelProvider extends LabelProvider implements ITableLabelProvider
{

	public Image getColumnImage(Object arg0, int arg1) {
		return null;
	}

	public String getColumnText(Object element, int col) {
		if(col == 0)
			return ((HTSTest)element).toCaptureDlgString();
		
		return ""; //$NON-NLS-1$
	}
	
}

/**
 * Zeigt eine Tabelle mit allen gltigen Zustandsprfungen (TEST Kommmandos)
 * fr ein SWT-Component. Der Dialog erscheint im Kontext des Testobjektes.
 * Aus dieser Tabelle kann der Tester Kommandos auswhlen, damit diese
 * spter einer Testsequenz hinzugefgt werden knnen
 * @author Nicos Tegos
 */
public class SWTTestCreatorDlg extends Dialog
{
	private TableViewer testActionTV;
	private SWTCommandCreator creator;
	private HTSTest[] selectedElements;
	
	/**
     * Erstellt einen neuen Dialog zur Anzeige aller Zustandsprfungen 
     * eines Components.
     * @param shell Das Fenster, in dem der Dialog erstellt werden soll.
     * @param creator Der <code>CommandCreator</code> des Components.
     */
	public SWTTestCreatorDlg(Shell shell, SWTCommandCreator creator) {
		super(shell);
		this.creator = creator;
	}

	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(SWTCommandCreator.TEST_ACTION_DIALOG_TITLE);
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, OK, Messages.getString("SWTTestCreatorDlg.1"), true); //$NON-NLS-1$
		createButton(parent, CANCEL, Messages.getString("SWTTestCreatorDlg.2"), true); //$NON-NLS-1$
	}
	
	/**
	 * Erstellt die Tabelle zur Auswahl der TEST-Kommandos.
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea = new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 15;
		gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		createInfoArea(dlgArea);
		
		testActionTV = new TableViewer(dlgArea, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		testActionTV.setContentProvider( new ArrayContentProvider() );
		testActionTV.setLabelProvider( new TestActionLabelProvider() );
		
		Table table = testActionTV.getTable();
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("SWTTestCreatorDlg.3")); //$NON-NLS-1$
		col.setWidth(400);
		/*col = new TableColumn(table, SWT.NULL);
		col.setText("Wert");
		col.setWidth(200);*/
		
		testActionTV.setInput( creator.getTestCommands() );
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Erstellt den Bereich zur Anzeige der ComponentID.
	 * @param parent Das graph. Vaterelement.
	 */
	private void createInfoArea(Composite parent) 
	{
		Composite infoArea = new Composite(parent, SWT.NULL);
		infoArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		infoArea.setLayout( new GridLayout(2, false) );
		
		ComponentId id = creator.getMComponent().getComponentId();
		SWTToolkit.createLabel(infoArea, Messages.getString("SWTTestCreatorDlg.4")); //$NON-NLS-1$
		SWTToolkit.createLabel(infoArea, id.getWindowName());
		SWTToolkit.createLabel(infoArea, Messages.getString("SWTTestCreatorDlg.5")); //$NON-NLS-1$
		SWTToolkit.createLabel(infoArea, id.getClassName());
		SWTToolkit.createLabel(infoArea, Messages.getString("SWTTestCreatorDlg.6")); //$NON-NLS-1$
		SWTToolkit.createLabel(infoArea, id.getName());
	}
	
	/**
	 * Speichert die ausgewhlten Kommandos in einer Variable,
	 * damit sie auch nach dem Schlieen des Dialogs noch verfgbar sind.
	 */
	protected void okPressed() 
	{
		StructuredSelection ssel = (StructuredSelection)testActionTV.getSelection();
		selectedElements = new HTSTest[ssel.size()];
		int i = 0;
		for (Iterator iter = ssel.iterator(); iter.hasNext();) {
			selectedElements[i] = (HTSTest) iter.next();
			++i;
		}
		
		super.okPressed();
	}

	/**
     * Liefert alle aus der Tabelle ausgewhlten TEST Kommandos.
     */
	public HTSTest[] getSelectedElements() {
		return selectedElements;
	}
}
