/*
 * Created on 08.12.2005
 */
package atosj.dialog;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.TableItem;

import atosj.app.AtosJApp;
import atosj.util.Statistics;

/**
 * Liefert Statistiken die zur Laufzeit eines Skriptes gendert werden knnen.
 * @author Nicos Tegos
 **/
public abstract class StatisticsTableContentProvider implements IStructuredContentProvider
{
	private TableViewer tv;
	private StatisticsTableElement[] sortedElements; // Vector<StatisticsTableElement>, Ordnung korrespondiert mit der Zeilennummer
	
	public Object[] getElements(Object arg0) {
		return sortedElements;
	}
	
	public void dispose() {
		// do nothing
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) 
	{
		if(newInput == null)
			return;
		
		tv = (TableViewer) viewer;
		sortedElements = createStatisticTableElements(newInput);
				
		removeHighlights();
	}
	
	/**
	 * Aktualisiert den Tabelleneintrag nach der Ausfhrung anhand der Statistik.
	 */
	public void setCurrentStatistics(Statistics statistics) 
	{
		// refresh labels
		tv.update(sortedElements[statistics.getLine()], (String[]) tv.getColumnProperties());
	}
	
	/**
	 * Highlighted das Element in der angegebenen Zeile. Die Zeile wird hellblau hinterlegt.
	 * @param line Index eines Kommandos in einem Skript oder Index eines Skripts in einem Paket.
	 */
	public void setRunning(int line) {
		removeHighlights();
		
		tv.update(sortedElements[line], (String[]) tv.getColumnProperties());
		
		// aktuelle zeile highlighten, unbedingt die testFindItem-Methode benutzen, wegen evtl. stimmen vielleicht 
		// die Zeilennummern nicht mit den nummern der items berein Filter
		TableItem item = (TableItem) tv.testFindItem(sortedElements[line]);
		item.setBackground( AtosJApp.getInstance().getColor(StatisticsTableElement.RGB_IS_RUNNING) );
		
		tv.reveal(sortedElements[line]);
	}
	
	/**
	 * Entfernt alle Highlights in der Tabelle.
	 * @see #setRunning(int)
	 */
	public void removeHighlights() {
		TableItem[] items = tv.getTable().getItems();
    	for(int i = 0; i < items.length; ++i) {
    		items[i].setBackground(null);
    	}
	}
	
	/**
	 * Aktualisiert die komplette Tabelle und entfernt alle Highlights.
	 */
	public void reset() {
		removeHighlights();
		tv.refresh();
	}
	
	/**
	 * Liefert die Statistiken fr das Eingabeelement.
	 * @param input Das Eingabeelement, beispielsweise ein <code>HTSPlayer</code>
	 * @return Die Statistiken aller im Eingabeelement enthaltenen Elemente.
	 */
	public abstract StatisticsTableElement[] createStatisticTableElements(Object input);
}
