/*
 * Created on 18.02.2006
 */
package atosj.dialog;

import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;

import atosj.listener.StreamListener;
import atosj.util.Snooze;

/**
 * Sammelt die Ausgaben des Testobjektes und stellt sie zur Laufzeit in einem Textfeld live dar.
 * @author Nicos Tegos
 */
public class StdTextSetter implements StreamListener 
{
	private StyledText txt;
	private Color foreground;
	private StringBuffer wholeStr = new StringBuffer(); // der gesamte Text des Streams
	private boolean running = false;
	
	/**
	 * Erstellt einen neuen Listener fr den Stream des Testobjektes.
	 * @param txt Der <code>StyledText</code> fr die Textausgabe.
	 * @param foreground Die Schriftfarbe, der Ausgabe dieses Listeners.
	 */
	public StdTextSetter(StyledText txt, Color foreground) {
		this.txt = txt;
		this.foreground = foreground;
	}

	/* (non-Javadoc)
	 * @see atosj.listener.StreamListener#charsReceived(char[])
	 */
	public void charsReceived(final char[] chars) 
	{
		running = true;	
		
		Runnable run = new Runnable() {
			public void run() {
				int start = txt.getCharCount();
				txt.append( new String(chars) );
				StyleRange sr = new StyleRange(
						start,
						chars.length, 
						foreground,
						txt.getBackground()
				);
				txt.setStyleRange(sr);
			}
		};
		
		txt.getDisplay().asyncExec(run);
		wholeStr.append(chars);
	}

	/* (non-Javadoc)
	 * @see atosj.listener.StreamListener#streamFinished()
	 */
	public void streamFinished() {
		running = false;
	}
	
	/**
	 * Wartet auf die Beendigung des Prozesses und gibt dann den
	 * gesamten String des Streams aus.
	 * @return Der gesamte String des Streams.
	 */
	public String getText() {
		while(running)
			new Snooze(100);

		return wholeStr.toString();
	}
	
	/**
	 * Lscht den Inhalt des bisher gesammelten Textes.
	 */
	public void clear() {
		wholeStr.delete( 0, wholeStr.length() );
	}
}
