/*
 * Created on 17.04.2006
 */
package atosj.dialog.addcmd;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.PositionWidget;
import atosj.exception.ParseException;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.parser.HTSParser;
import atosj.model.script.HTSScript;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Ein Dialog zum Erstellen eines HTS Kommandos nur ber textuelle Eingabe.
 * Der gleiche Dialog kann auch zum Editieren eines bestehenden Kommandos
 * genutzt werden.
 * @author Nicos Tegos
 */
public class AddCmdAdvancedDlg extends Dialog {

	public static final int ID_ADD = 0;
	public static final int ID_CANCEL = 1;
	
	private boolean isEdit;
	
	private HTSScript script;
	private HTSCommand cmd; // das neu generierte kommando
	
	private Text inputTxt;
	private Canvas errorCanvas;
	private Text errorTxt;
	
	private int currentPos;
	private int maxPos;
	private PositionWidget posWidget;
	
	/**
	 * Erstellen eines neuen Kommandos.
	 * @param script Das script dem das neue Kommando hinzugefgt werden soll.
	 * @param maxPos @see PositionWidget
	 * @param currentPos @see PositionWidget
	 **/
	public AddCmdAdvancedDlg(Shell parent, HTSScript script, int maxPos, int currentPos) {
		super(parent);
		
		this.isEdit = false;
		this.script = script;
		this.currentPos = currentPos;
		this.maxPos = maxPos;
	}
	
	/**
	 * Editieren eines bestehenden Kommandos.
	 * @param script Das script in dem sich das Originalkommando befindet.
	 * @param cmd Das zu editierende Kommando.
	 */
	public AddCmdAdvancedDlg(Shell parent, HTSScript script, HTSCommand cmd) 
	{
		super(parent);
		
		this.isEdit = true;
		this.script = script;
		this.cmd = cmd;
		this.currentPos = -1; // ungltig in dieser Variante
		this.maxPos = -1; // ungltig in dieser Variante
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_ADD, isEdit ? Messages.getString("AddCmdAdvancedDlg.0") : Messages.getString("AddCmdAdvancedDlg.1"), true); //$NON-NLS-1$ //$NON-NLS-2$
		createButton(parent, ID_CANCEL, Messages.getString("AddCmdAdvancedDlg.2"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_ADD: addPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	protected void addPressed() 
	{
		if( parseCommand() != null )
		{
			setReturnCode(ID_ADD);
			close();
		}
		else
		{
			MessageDialog.openError(Display.getDefault().getActiveShell(), 
					isEdit ? Messages.getString("AddCmdAdvancedDlg.3") : Messages.getString("AddCmdAdvancedDlg.4"),  //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("AddCmdAdvancedDlg.5")); //$NON-NLS-1$
			inputTxt.setFocus();
		}
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText( isEdit 
				? Messages.getString("AddCmdAdvancedDlg.6") + script.getName() + Messages.getString("AddCmdAdvancedDlg.7") //$NON-NLS-1$ //$NON-NLS-2$
				: Messages.getString("AddCmdAdvancedDlg.8") + script.getName() + Messages.getString("AddCmdAdvancedDlg.9")); //$NON-NLS-1$ //$NON-NLS-2$
				
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Composite inputArea = new Composite(dlgArea, SWT.NULL);
		gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		inputArea.setLayout(gl);
		inputArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		if( isEdit ) // wir editieren ein bestehende Kommando -> Anzeige des Originalkommandos
		{
			Label origLbl = new Label(inputArea, SWT.NULL);
			origLbl.setText(Messages.getString("AddCmdAdvancedDlg.10")); //$NON-NLS-1$
			Text originalTxt = new Text(inputArea, SWT.BORDER | SWT.READ_ONLY);
			originalTxt.setText( cmd.toString() );
			originalTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		}
		
		Label lbl = new Label(inputArea, SWT.NULL);
		lbl.setText( isEdit ? Messages.getString("AddCmdAdvancedDlg.11") : Messages.getString("AddCmdAdvancedDlg.12")); // beschriftung je nach ediert oder neu //$NON-NLS-1$ //$NON-NLS-2$
		inputTxt = new Text(inputArea, SWT.BORDER);
		inputTxt.setText( isEdit ? cmd.toString() : "" ); // beschriftung je nach ediert oder neu //$NON-NLS-1$
		inputTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		// Korrektheit der Eingabe berprfen
		inputTxt.addModifyListener( new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				parseCommand();
			}
		});
		inputTxt.setFocus(); // wichtig bei edit, denn sonst hat das Textfeld mit dem Original den focus 
		
		errorCanvas = new Canvas(inputArea, SWT.NULL);
		errorCanvas.addPaintListener( new PaintListener() {
			public void paintControl(PaintEvent e) {
				Image errorImg = AtosJApp.getInstance().getImage(AtosJApp.IMG_ERROR);
				e.gc.drawImage(errorImg, 0, 0);
			}
		});
		errorCanvas.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		errorTxt = new Text(inputArea, SWT.BORDER | SWT.MULTI | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 500;
		gd.heightHint = 100;
		errorTxt.setLayoutData(gd);
		
		// erstmaliger parsedurchgang
		parseCommand();
		
		// Einfgeposition nur bei Erstellen eines Kommandos wichtig nicht bei editieren
		if(!isEdit)
		{
			posWidget = new PositionWidget(dlgArea, maxPos, currentPos);
			posWidget.getContent().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		}
			
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Setzt den member cmd auf das neue Kommando, wenn erfolgreich oder auf null, 
	 * wenn nicht erfolgreich. Aktualisiert Fehlernachrichten.
	 */
	private HTSCommand parseCommand() 
	{
		// leerer String liefert null bei CSVParser.getNextLine(), deshalb extra behandelt
		if( inputTxt.getText().equals("") ) //$NON-NLS-1$
		{
			errorCanvas.setVisible(true);
			errorTxt.setVisible(true);
			errorTxt.setText( Messages.getString("AddCmdAdvancedDlg.15") ); //$NON-NLS-1$
			cmd = null;
		}
		else // nicht leerer String
		{
			try 
			{
				CSVLine line = new CSVParser(inputTxt.getText(), true).getNextLine();
				cmd =  HTSParser.getCommandFromLine(line, AtosJApp.getInstance().getInput().getTestObject().getResolver() );
				errorCanvas.setVisible(false);
				errorTxt.setVisible(false);
			} catch (ParseException err) 
			{
				errorCanvas.setVisible(true);
				errorTxt.setVisible(true);
				errorTxt.setText( err.getMessage() );
				cmd = null;
			}
		}
		
		return cmd;
	}
	
	/**
	 * Liefert das neu erstellte Kommando. Nur gltig
	 * nach beenden mit Add-Button.
	 */
	public HTSCommand getCommand() {
		return cmd;
	}
	
	/**
	 * Liefert den Einfgeindex (0-basiert). Nur gltig
	 * nach beenden mit Add-Button und wenn dieser Dialog ein
	 * neues Kommando erstellen soll, nicht beim Editieren.
	 */
	public int getPosition() {
		return posWidget.getPosition();
	}

}
