/*
 * Created on 05.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.PositionWidget;
import atosj.dialog.addcmd.action.ActionSection;
import atosj.dialog.addcmd.test.CompareSection;
import atosj.dialog.addcmd.test.TestSection;
import atosj.exception.ParseException;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.InvalidCommand;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.script.HTSScript;
import atosj.util.CSVLine;

/**
 * Graphischer Dialog zum Hinzufgen eines Kommandos zu einem Script.
 * @author Nicos Tegos
 */
public class AddCmdDlg extends Dialog 
{
	public static final int ID_ADD = 0;
	public static final int ID_CANCEL = 1;
	
	private HTSScript script;
	private ComboViewer cmdCV;
	private String[] commands = new String[] { HTSSyntax.CMD_ACTION, HTSSyntax.CMD_CLEAN,	HTSSyntax.CMD_COMM,
		HTSSyntax.CMD_COMP, HTSSyntax.CMD_COPY, HTSSyntax.CMD_DEL, HTSSyntax.CMD_ENDLOOP, HTSSyntax.CMD_ENDWHILE,
		HTSSyntax.CMD_EXISTS, HTSSyntax.CMD_LAUNCH, HTSSyntax.CMD_LOOP, HTSSyntax.CMD_MESS, HTSSyntax.CMD_QUEST, 
		HTSSyntax.CMD_READ,	HTSSyntax.CMD_START, HTSSyntax.CMD_TEST, HTSSyntax.CMD_WAIT, HTSSyntax.CMD_WEX, 
		HTSSyntax.CMD_WHILE };
		
	private Composite parametersArea;
	private StackLayout parametersStack;
	
	private CommandSection actionSection;
	private CommandSection readSection;
	private CommandSection testSection;
	private CommandSection startSection;
	private CommandSection loopSection;
	private CommandSection endloopSection;
	private CommandSection waitSection;
	private CommandSection questionSection;
	private CommandSection launchSection;
	private CommandSection cleanupSection;
	private CommandSection copySection;
	private CommandSection deleteSection;
	private CommandSection existsSection;
	private CommandSection windowExistsSection;
	private CommandSection compareSection;
	private CommandSection messageSection;
	private CommandSection commentSection;
	private CommandSection whileSection;
	private CommandSection endwhileSection;
	
	private boolean isEdit;
	private HTSCommand cmd; // das neu generierte kommando
	
	private int currentPos;
	private int maxPos;
	private PositionWidget posWidget;
	
	/**
	 * Erstell einen neuen Dialog zum Erstellen eines neuen Kommandos.
	 * @param script Das script dem das neue Kommando hinzugefgt werden soll.
	 * @param maxPos Die maximale Einfgeposition des Kommandos im Skript.
	 * @param currentPos Die Position des aktuell ausgewhlten Kommandos in der Skripttabelle.
	 * @see PositionWidget
	 **/
	public AddCmdDlg(Shell parent, HTSScript script, int maxPos, int currentPos) {
		super(parent);
		
		this.isEdit = false;
		this.script = script;
		this.currentPos = currentPos;
		this.maxPos = maxPos;
	}
	
	/**
	 * Erstellt einen neuen Dialog zum Bearbeiten eines Kommandos.
	 * @param script Das Skript, dem das genderte Kommando hinzugefgt werden soll.
	 * @param toEdit Das Kommando, das gendert werden soll.
	 **/
	public AddCmdDlg(Shell parent, HTSScript script, HTSCommand toEdit) {
		super(parent);
		
		this.isEdit = true;
		this.script = script;
		this.currentPos = -1;
		this.maxPos = -1;
		this.cmd = toEdit;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_ADD, Messages.getString("AddCmdDlg.0"), true); //$NON-NLS-1$
		createButton(parent, ID_CANCEL, Messages.getString("AddCmdDlg.1"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_ADD: addPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	protected void addPressed() 
	{
		CommandSection section = (CommandSection) parametersStack.topControl;
		
		if(section == null)
		{
			MessageDialog.openError(null, Messages.getString("AddCmdDlg.2"), Messages.getString("AddCmdDlg.3")); //$NON-NLS-1$ //$NON-NLS-2$
			return;
		}
		
		if( section.checkIntegrity() )
		{
			String commandName = (String) ((StructuredSelection)cmdCV.getSelection()).getFirstElement();
			Vector parameters = new Vector();
			parameters.add(commandName);
			parameters.addAll( section.getCommandParameters() );
			
			try 
			{
				cmd = HTSParser.getCommandFromLine( new CSVLine(parameters), AtosJApp.getInstance().getInput().getTestObject().getResolver() );
			} 
			catch (ParseException e) 
			{
				try
				{
					cmd = new InvalidCommand(e);
				}
				catch(ParseException e2) 
				{
					// wil never happen
				}
			}
			
			setReturnCode(ID_ADD);
			close();
		}	
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell shell) 
	{
		shell.setText(isEdit ? (Messages.getString("AddCmdDlg.4") + script.getName() + Messages.getString("AddCmdDlg.5")) : (Messages.getString("AddCmdDlg.6") + script.getName() + Messages.getString("AddCmdDlg.7"))); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		shell.setSize(640, 640);
		super.configureShell(shell);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		createCommandCombo(dlgArea);
				
		parametersArea = new Composite(dlgArea, SWT.NULL);
		parametersArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		
		parametersStack = new StackLayout();
		parametersStack.marginHeight = parametersStack.marginWidth = 0;
		parametersArea.setLayout(parametersStack);
		
		if( !isEdit )
		{
			posWidget = new PositionWidget(dlgArea, maxPos, currentPos);
			posWidget.getContent().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		}
		else
		{
			initOrigCmd();
		}
			
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Erstellt eine Combobox zur Auswahl aller HTS-Kommandos
	 * Initialisiert Listener, der bei der Auswahl eines Kommandos
	 * das entsprechende Widget zur Eingabe anzeigt.
	 */
	private void createCommandCombo(Composite area) 
	{
		Composite comboArea = new Composite(area, SWT.NULL);
		comboArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginHeight = 20;
		comboArea.setLayout(gl);
		
		comboArea.addListener(SWT.Paint, new Listener() {
			public void handleEvent(Event e) {
				if (e.widget.isDisposed())
					return;
				Rectangle bounds = ((Composite)e.widget).getBounds();
				GC gc = e.gc;
				gc.setForeground( AtosJApp.getInstance().getColor( new RGB(152, 170, 203) ) ); 
				gc.setBackground( e.display.getSystemColor(SWT.COLOR_WHITE) );
				gc.fillGradientRectangle(0, 0, bounds.width, bounds.height,	false);
			}
		});
		
		Label lbl = new Label(comboArea, SWT.NULL);
		lbl.setBackground( AtosJApp.getInstance().getColor( new RGB(152, 170, 203) ) );
		//FontData fd = lbl.getDisplay().getSystemFont().getFontData()[0];
		//Font font = new Font(lbl.getDisplay(), fd.getName(), fd.getHeight(), SWT.BOLD);
		//lbl.setFont(font);
		lbl.setText(Messages.getString("AddCmdDlg.8")); //$NON-NLS-1$
		
		cmdCV = new ComboViewer(comboArea, SWT.READ_ONLY);
		cmdCV.getCombo().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		cmdCV.setLabelProvider( new BaseLabelProvider() {
			/* (non-Javadoc)
			 * @see atosj.gui.BaseLabelProvider#getText(java.lang.Object)
			 */
			public String getText(Object element) {
				if( element.equals(HTSSyntax.CMD_ACTION) )
					return element + Messages.getString("AddCmdDlg.9"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_CLEAN) )
					return element + Messages.getString("AddCmdDlg.10"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_COMM) )
					return element + Messages.getString("AddCmdDlg.11"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_COMP) )
					return element + Messages.getString("AddCmdDlg.12"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_COPY) )
					return element + Messages.getString("AddCmdDlg.13"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_DEL) )
					return element + Messages.getString("AddCmdDlg.14"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_ENDLOOP) )
					return element + Messages.getString("AddCmdDlg.15"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_EXISTS) )
					return element + Messages.getString("AddCmdDlg.16"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_LAUNCH) )
					return element + Messages.getString("AddCmdDlg.17"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_LOOP) )
					return element + Messages.getString("AddCmdDlg.18"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_MESS) )
					return element + Messages.getString("AddCmdDlg.19"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_QUEST) )
					return element + Messages.getString("AddCmdDlg.20"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_READ) )
					return element + Messages.getString("AddCmdDlg.21"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_START) )
					return element + Messages.getString("AddCmdDlg.22"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_TEST) )
					return element + Messages.getString("AddCmdDlg.23"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_WAIT) )
					return element + Messages.getString("AddCmdDlg.24"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_WEX) )
					return element + Messages.getString("AddCmdDlg.25"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_WHILE) )
					return element + Messages.getString("AddCmdDlg.26"); //$NON-NLS-1$
				if( element.equals(HTSSyntax.CMD_ENDWHILE) )
					return element + Messages.getString("AddCmdDlg.27"); //$NON-NLS-1$
				
				return ""; //$NON-NLS-1$
			}
		});
		cmdCV.setContentProvider( new ArrayContentProvider() );
		
		cmdCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				StructuredSelection ssel = (StructuredSelection)e.getSelection();
				Object element = ssel.getFirstElement();
				if(element == null)
					return;
				
				if( element.equals(HTSSyntax.CMD_ACTION) )
				{
					if(actionSection == null)
					{
						actionSection = new ActionSection(parametersArea);
						actionSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = actionSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_READ) )
				{
					if(readSection == null)
					{
						readSection = new ReadSection(parametersArea);
						readSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = readSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_TEST) )
				{
					if(testSection == null)
					{
						testSection = new TestSection(parametersArea);
						testSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = testSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_START) )
				{
					if(startSection == null)
					{
						startSection = new StartSection(parametersArea);
						startSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = startSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_LOOP) )
				{
					if(loopSection == null)
					{
						loopSection = new LoopSection(parametersArea);
						loopSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = loopSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_ENDLOOP) )
				{
					if(endloopSection == null)
					{
						// das kommando endloop beinhaltet keine Paramter
						// also bekommt es eine leere Section
						endloopSection = new EmptySpecificationWidget(parametersArea);
						endloopSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = endloopSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_WAIT) )
				{
					if(waitSection == null)
					{
						waitSection = new WaitSection(parametersArea);
						waitSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = waitSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_QUEST) )
				{
					if(questionSection == null)
					{
						questionSection = new QuestionSection(parametersArea);
						questionSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = questionSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_CLEAN) )
				{
					if(cleanupSection == null)
					{
						cleanupSection = new CleanupSection(parametersArea);
						cleanupSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = cleanupSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_LAUNCH) )
				{
					if(launchSection == null)
					{
						launchSection = new LaunchSection(parametersArea);
						launchSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = launchSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_COPY) )
				{
					if(copySection == null)
					{
						copySection = new CopySection(parametersArea);
						copySection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = copySection;
					parametersArea.layout();
				}	
				else
				if( element.equals(HTSSyntax.CMD_DEL) )
				{
					if(deleteSection == null)
					{
						deleteSection = new DeleteSection(parametersArea);
						deleteSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = deleteSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_EXISTS) )
				{
					if(existsSection == null)
					{
						existsSection = new ExistsSection(parametersArea);
						existsSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = existsSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_WEX) )
				{
					if(windowExistsSection == null)
					{
						windowExistsSection = new WindowExistsSection(parametersArea);
						windowExistsSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = windowExistsSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_COMP) )
				{
					if(compareSection == null)
					{
						compareSection = new CompareSection(parametersArea);
						compareSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = compareSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_MESS) )
				{
					if(messageSection == null)
					{
						messageSection = new MessageSection(parametersArea);
						messageSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = messageSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_COMM) )
				{
					if(commentSection == null)
					{
						commentSection = new CommentSection(parametersArea);
						commentSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = commentSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_WHILE) )
				{
					if(whileSection == null)
					{
						// der witz ist das das while genau dieselben Parameter 
						// hat wie Compare, darum nutzen wir einfach die CompareSection
						whileSection = new CompareSection(parametersArea);
						whileSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = whileSection;
					parametersArea.layout();
				}
				else
				if( element.equals(HTSSyntax.CMD_ENDWHILE) )
				{
					if(endwhileSection == null)
					{
						// das kommando endwhile beinhaltet keine Paramter
						// also bekommt es eine leere Section
						endwhileSection = new EmptySpecificationWidget(parametersArea);
						endwhileSection.setLayoutData( new GridData(GridData.FILL_BOTH) );
					}
					
					parametersStack.topControl = endwhileSection;
					parametersArea.layout();
				}
			}
			
		});
		
		cmdCV.setInput(commands);
	}
	
	private void initOrigCmd() 
	{
		cmdCV.setSelection( new StructuredSelection(cmd.getCommandKey()) );
		CommandSection section = (CommandSection) parametersStack.topControl;
		section.setOrigParameters( cmd.getParameters() );
	}
	
	/**
	 * Liefert das neu erstellte Kommando. Nur gltig
	 * nach Beenden mit Add-Button.
	 */
	public HTSCommand getCommand() {
		return cmd;
	}
	
	/**
	 * Liefert den Einfgeindex (0-basiert). Nur gltig
	 * nach Beenden mit Add-Button.
	 */
	public int getPosition() {
		return posWidget.getPosition();
	}
}
