package atosj.dialog.addcmd;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.PositionWidget;

/**
 * Dialog fr die Spezifikation eines Elements eines Subelementpfades.
 */
public class AddPathElementDlg extends Dialog 
{
	public static final int ID_ADD = 0;
	public static final int ID_CANCEL = 1;
	
	private Text inputTxt;
	private Object input;
	private boolean editing;
	
	private Button numRadio, strRadio;
	
	private PositionWidget posWidget;
	private int currentPos;
	private int maxPos;
	
	/**
	 * Erstellt einen Dialog zum Erstellen eines neuen Elements
	 * eines Subelementpfades.
	 * @param parent Das Vaterfenster.
	 * @param maxPos Die maximale Einfgeposition des Elements im Pfad.
	 * @param currentPos Die Position des aktuell ausgewhlten Elements in der Tabelle.
	 */
	public AddPathElementDlg(Shell parent, int maxPos, int currentPos) {
		super(parent);
		
		this.maxPos = maxPos;
		this.currentPos = currentPos;
		editing = false;
	}
	
	/**
	 * Erstellt einen Dialog zum Editieren eines Elements eines Subelementpfades.
	 * @param parent Das Vaterfenster.
	 * @param input Das zu editierende Element.
	 */
	public AddPathElementDlg(Shell parent, Object input) {
		super(parent);
		this.input = input;
		editing = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_ADD, Messages.getString("AddPathElementDlg.0"), true); //$NON-NLS-1$
		createButton(parent, ID_CANCEL, Messages.getString("AddPathElementDlg.1"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_ADD: addPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	protected void addPressed() 
	{
		if( numRadio.getSelection() ) // Zahl
		{
			try 
			{
				int i = Integer.parseInt(inputTxt.getText());
				if(i < 0)
				{
					inputTxt.setFocus();
					MessageDialog.openError(null, getTitle(), Messages.getString("AddPathElementDlg.2")); //$NON-NLS-1$
					return; // abbruch
				}
				
				input = new Integer(i);
			}
			catch(NumberFormatException e) {
				inputTxt.setFocus();
				MessageDialog.openError(null, getTitle(), Messages.getString("AddPathElementDlg.3")); //$NON-NLS-1$
				return; // abbruch
			}
		}
		else // Zeichenkette
		{
			if( inputTxt.getText().equals("") ) //$NON-NLS-1$
			{
				inputTxt.setFocus();
				MessageDialog.openError(null, getTitle(), Messages.getString("AddPathElementDlg.5")); //$NON-NLS-1$
				return; // abbruch
			}
			
			input = inputTxt.getText();
		}
		
		setReturnCode(ID_ADD);
		close();
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}

	protected void configureShell(Shell shell) {
		shell.setText( getTitle() );
		super.configureShell(shell);
	}
	
	private String getTitle() {
		return editing ? Messages.getString("AddPathElementDlg.6") : Messages.getString("AddPathElementDlg.7"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Label lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("AddPathElementDlg.8")); //$NON-NLS-1$
		Composite radioArea =  new Composite(dlgArea, SWT.NULL);
		radioArea.setLayoutData( new GridData(GridData.HORIZONTAL_ALIGN_CENTER) );
		gl = new GridLayout(2, false);
		gl.marginHeight = gl.marginBottom = 0;
		radioArea.setLayout(gl);
		numRadio = new Button(radioArea, SWT.RADIO);
		numRadio.setText(Messages.getString("AddPathElementDlg.9")); //$NON-NLS-1$
		strRadio = new Button(radioArea, SWT.RADIO);
		strRadio.setText(Messages.getString("AddPathElementDlg.10")); //$NON-NLS-1$
				
		lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("AddPathElementDlg.11")); //$NON-NLS-1$
		inputTxt = new Text(dlgArea, SWT.BORDER);
		inputTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		if(editing)
		{	
			inputTxt.setText( input.toString() );
			if(input instanceof String)
				strRadio.setSelection(true);
			else
				numRadio.setSelection(true);
		}
		else
		{
			numRadio.setSelection(true);
			
			posWidget = new PositionWidget(dlgArea, maxPos, currentPos);
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalSpan = 2;
			posWidget.getContent().setLayoutData(gd);
		}
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Liefert das Element des Subelementpfades. Es kann ein String
	 * oder ein Integer sein.
	 */
	public Object getPathElement() {
		return input;
	}
	
	/**
	 * Liefert die Einfgeposition des Elements in seinem Pfad (0-basiert).
	 */
	public int getPosition() {
		return posWidget == null ? 0 : posWidget.getPosition();
	}
}
