/*
 * Created on 05.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import atosj.gui.BaseLabelProvider;
import atosj.util.UniversalViewerSorter;

/**
 * Basisklasse zur graph. Eingabe von HTSKommandos.
 * @author Nicos Tegos
 */
public abstract class CommandSection extends Composite 
{
	/**
	 * Erstellt ein neues <code>Composite</code> zur Eingabe von Kommandoparametern.
	 * Setzt das Layout standardmig auf ein 2-spaltiges GridLayout.
	 */
	public CommandSection(Composite parent) 
	{
		super(parent, SWT.NULL);
		
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		setLayout(gl);
	}
	
	/**
	 * Wenn ein vorhandenes Kommando bearbeitet werden soll,
	 * werden hier die graph. Elemente auf die dem Original
	 * entsprechenden Werte gesetzt.
	 * @param parameters Die Parameter des Kommandos. Parameter in Anfhrungszeichen
	 * stehen fr Zeichenketten, alle anderen fr Zahlen und Schlsselwrter.
	 */
	public abstract void setOrigParameters(String[] parameters);

	/**
	 * @return Vector<String> mit allen Parametern fr das Kommando ausser
	 * dem Kommandonamen selbst. Erst aufrufen wenn vorher checkIntgrity
	 * true geliefert hat.
	 */
	public abstract Vector getCommandParameters();
	
	/**
	 * Gibt eine Warnung aus wenn der Inhalt eines bestimmten Elements nicht korrekt ist.
	 * @return <code>true</code> wenn alle Felder dieses 
	 * <code>Composites</code> mit gltigem Inhalt belegt sind.
	 */
	public abstract boolean checkIntegrity();

	/**
	 * Erstellt einen <codeComboViewer></code> mit einem <code>ArrayContentProvider</code> und
	 * einem <code>BaseLabelProvider</code>, Stil FILL_HORIZONTAL und READ_ONLY.
	 * @param labelText Ein Label mit dem angegebenen Text wird erstellt, darf nicht <code>null</code> sein.
	 * @return den neu erstellten <code>ComboViewer</code>
	 * @see BaseLabelProvider
	 */
	protected ComboViewer createComboViewer(String labelText) {
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(labelText);
		
		ComboViewer cv = new ComboViewer(this, SWT.READ_ONLY);
		cv.setContentProvider( new ArrayContentProvider() );
		cv.setLabelProvider( new BaseLabelProvider() );
		cv.setSorter( new UniversalViewerSorter() );
		cv.getCombo().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		return cv;
	}

	/**
	 * Liefert das aktuell ausgewhlte Element der Combobox. 
	 * Liefert einen leeren String, wenn kein Element ausgewhlt wurde.
	 * Es wird ein als input des Viewers ein String[] vorausgesetzt. 
	 */
	protected String getSelectedElement(ComboViewer cv) {
		String ret = (String) ((StructuredSelection)cv.getSelection()).getFirstElement();
		if(ret == null)
			return "";
		else
			return ret;
	}
}
