package atosj.dialog.addcmd;

import java.util.Collection;
import java.util.HashSet;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;

interface ComponentIdSetterListener {
	void idSet(ComponentId id);
}

public class ComponentIdSetter 
{
	private ComboViewer windowTitleCV; 
	private ComboViewer typeCV; 
	private ComboViewer nameCV;
	
	private Vector ids = AtosJApp.getInstance().getInput().getURFFile().getIds();
	private Vector listeners = new Vector();
	
	public ComponentIdSetter(ComboViewer windowTitleCV, ComboViewer typeCV, ComboViewer nameCV) 
	{
		super();
		
		this.windowTitleCV = windowTitleCV;
		this.typeCV = typeCV;
		this.nameCV = nameCV;
		
		windowTitleCV.setInput( getWindowTitles() );
		windowTitleCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				onWindowSelection();
			}
		});
		typeCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				onTypeSelection();
			}
		});
	}
	
	public ComponentIdSetter(ComboViewer windowTitleCV, ComboViewer typeCV, ComboViewer nameCV, ComponentId id) 
	{
		super();
		
		this.windowTitleCV = windowTitleCV;
		this.typeCV = typeCV;
		this.nameCV = nameCV;
		
		windowTitleCV.setInput( getWindowTitles() );
		typeCV.setInput( id.getWindowName() );
		nameCV.setInput( getNames(id.getWindowName(), id.getClassName()) );
		
		windowTitleCV.setSelection( new StructuredSelection(id.getWindowName()) );
		typeCV.setSelection( new StructuredSelection(id.getClassName()) );
		nameCV.setSelection( new StructuredSelection(id.getName()) );
		
		windowTitleCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				onWindowSelection();
			}
		});
		typeCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				onTypeSelection();
			}
		});
	}
	
	private String getSelection(ComboViewer viewer) {
		return (String) ((StructuredSelection)viewer.getSelection()).getFirstElement();
	}
	
	private void onWindowSelection() {
		String title = (String) ((StructuredSelection)windowTitleCV.getSelection()).getFirstElement();
		
		if(title != null)
		{
			typeCV.setInput( getTypes(title) );
			nameCV.setInput( new String[0] );
		}
		else
		{
			typeCV.setInput( new String[0] );
			nameCV.setInput( new String[0] );
		}
	}
	
	private void onTypeSelection() 
	{
		String title =(String) ((StructuredSelection)windowTitleCV.getSelection()).getFirstElement();
		String type = (String) ((StructuredSelection)typeCV.getSelection()).getFirstElement();
		
		if(type != null && title != null)
			nameCV.setInput( getNames(title, type) );
		else
			nameCV.setInput( new String[0] );
	}
	
	private void onIdSelection() {
		
	}
	
	private Collection getWindowTitles() 
	{
		HashSet titles = new HashSet();
		for(int i = 0; i < ids.size(); ++i)
			titles.add( ((ComponentId)ids.get(i)).getWindowName() );
		
		return titles;
	}

	private Collection getTypes(String windowTitle) 
	{
		HashSet types = new HashSet();
		for(int i = 0; i < ids.size(); ++i)
		{
			ComponentId id = (ComponentId)ids.get(i);
			if( id.getWindowName().equals(windowTitle) )
				types.add(id);
		}
		
		return types;
	}
	
	private Collection getNames(String windowTitle, String type) 
	{
		HashSet names = new HashSet();
		for(int i = 0; i < ids.size(); ++i)
		{
			ComponentId id = (ComponentId)ids.get(i);
			if( id.getWindowName().equals(windowTitle) && id.getClassName().equals(type) )
				names.add(id);
		}
		
		return names;
	}
	
	public boolean checkIntegrity() 
	{
		if( getSelection(typeCV) == null )
		{
			windowTitleCV.getCombo().setFocus();
			MessageDialog.openError(null, Messages.getString("ComponentIdSetter.0"), Messages.getString("ComponentIdSetter.1")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if( getSelection(typeCV) == null )
		{
			typeCV.getCombo().setFocus();
			MessageDialog.openError(null, Messages.getString("ComponentIdSetter.2"), Messages.getString("ComponentIdSetter.3")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if( getSelection(nameCV) == null )
		{
			nameCV.getCombo().setFocus();
			MessageDialog.openError(null, Messages.getString("ComponentIdSetter.4"), Messages.getString("ComponentIdSetter.5")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		return true;
	}
	
	public ComponentId getComponentId() {
		return new ComponentId( getSelection(nameCV), getSelection(typeCV), getSelection(windowTitleCV) );
	}
}
