/*
 * Created on 23.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das COPY Kommando.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSCopy
 */
public class CopySection extends CommandSection {

	private ComboViewer srcDirCV;
	private Text srcFileTxt;
	private ComboViewer destDirCV;
	private Text destFileTxt;
	private Button forceChk;
	private Button ifsrcexistsChk;
	
	private static final String[] dirs = { HTSSyntax.FILEOP_ABS, HTSSyntax.FILEOP_BIN, HTSSyntax.FILEOP_ENV, HTSSyntax.FILEOP_LOG, HTSSyntax.FILEOP_REF, HTSSyntax.FILEOP_TGT };
	
	public CopySection(Composite parent) {
		super(parent);
		
		createContents();
		fillCombos();
	}
	
	private void createContents() {
		// COPY, <QUELLVERZEICHNIS>, <QUELLDATEI>, <ZIELVERZEICHNIS>,
		// <ZIELDATEI> [,FORCE] [,IFSRCEXISTS]
		
		srcDirCV = createComboViewer(Messages.getString("CopySection.0")); //$NON-NLS-1$
		
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("CopySection.1")); //$NON-NLS-1$
		srcFileTxt = new Text(this, SWT.BORDER);
		srcFileTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		destDirCV = createComboViewer(Messages.getString("CopySection.2")); //$NON-NLS-1$
		
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("CopySection.3")); //$NON-NLS-1$
		destFileTxt = new Text(this, SWT.BORDER);
		destFileTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		ifsrcexistsChk = new Button(this, SWT.CHECK);
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		ifsrcexistsChk.setLayoutData(gd);
		ifsrcexistsChk.setText(Messages.getString("CopySection.4")); //$NON-NLS-1$
		
		forceChk = new Button(this, SWT.CHECK);
		gd = new GridData();
		gd.horizontalSpan = 2;
		forceChk.setLayoutData(gd);
		forceChk.setText(Messages.getString("CopySection.5")); //$NON-NLS-1$
	}
	
	private void fillCombos() 
	{
		srcDirCV.setInput(dirs);
		destDirCV.setInput(dirs);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		srcDirCV.setSelection( new StructuredSelection(parameters[0]) );
		srcFileTxt.setText( HTSParser.stripQuotes(parameters[1]) );
		destDirCV.setSelection( new StructuredSelection(parameters[2]) );
		destFileTxt.setText( HTSParser.stripQuotes(parameters[3]) );
		
		ifsrcexistsChk.setSelection(true);
		for(int i = 4; i < parameters.length; ++i)
		{
			if( parameters[i].equals(HTSSyntax.COPY_FORCE) )
			{
				forceChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.COPY_IFSRCEXISTS) )
			{
				ifsrcexistsChk.setSelection(false);
			}
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector ret = new Vector(6);
		
		ret.add( getSelectedElement(srcDirCV) );
		ret.add( '"' + srcFileTxt.getText() + '"');
		ret.add( getSelectedElement(destDirCV) );
		ret.add( '"' + destFileTxt.getText() + '"');
		if( forceChk.getSelection() )
			ret.add(HTSSyntax.COPY_FORCE);
		if( !ifsrcexistsChk.getSelection() )
			ret.add(HTSSyntax.COPY_IFSRCEXISTS);
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( getSelectedElement(srcDirCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CopySection.7"), Messages.getString("CopySection.8")); //$NON-NLS-1$ //$NON-NLS-2$
			srcDirCV.getCombo().setFocus();
			return false;
		}
		
		if( srcFileTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CopySection.10"), Messages.getString("CopySection.11")); //$NON-NLS-1$ //$NON-NLS-2$
			srcFileTxt.setFocus();
			return false;
		}
		
		if( getSelectedElement(destDirCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CopySection.13"), Messages.getString("CopySection.14")); //$NON-NLS-1$ //$NON-NLS-2$
			destDirCV.getCombo().setFocus();
			return false;
		}
		
		if( destFileTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CopySection.16"), Messages.getString("CopySection.17")); //$NON-NLS-1$ //$NON-NLS-2$
			destFileTxt.setFocus();
			return false;
		}
		
		return true;
	}

}
