/*
 * Created on 24.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das DELETE Kommando.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSDelete
 */
public class DeleteSection extends CommandSection {

	private ComboViewer dirCV;
	private Text fileTxt;
	private Button forceChk;
	private Button ifexistsChk;
	
	private static final String[] dirs = { HTSSyntax.FILEOP_ABS, HTSSyntax.FILEOP_BIN, HTSSyntax.FILEOP_ENV, HTSSyntax.FILEOP_LOG, HTSSyntax.FILEOP_REF, HTSSyntax.FILEOP_TGT };
	
	public DeleteSection(Composite parent) {
		super(parent);
				
		createContents();
		fillCombos();
	}
	
	private void createContents() {
		// COPY, <QUELLVERZEICHNIS>, <QUELLDATEI>, <ZIELVERZEICHNIS>,
		// <ZIELDATEI> [,FORCE] [,IFSRCEXISTS]
		
		dirCV = createComboViewer(Messages.getString("DeleteSection.0")); //$NON-NLS-1$
		
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("DeleteSection.1")); //$NON-NLS-1$
		fileTxt = new Text(this, SWT.BORDER);
		fileTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		ifexistsChk = new Button(this, SWT.CHECK);
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		ifexistsChk.setLayoutData(gd);
		ifexistsChk.setText(Messages.getString("DeleteSection.2")); //$NON-NLS-1$
		
		forceChk = new Button(this, SWT.CHECK);
		gd = new GridData();
		gd.horizontalSpan = 2;
		forceChk.setLayoutData(gd);
		forceChk.setText(Messages.getString("DeleteSection.3")); //$NON-NLS-1$
	}
	
	private void fillCombos() 
	{
		dirCV.setInput(dirs);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		dirCV.setSelection( new StructuredSelection(parameters[0]) );
		fileTxt.setText( HTSParser.stripQuotes(parameters[1]) );
		
		ifexistsChk.setSelection(true);
		for(int i = 2; i < parameters.length; ++i)
		{
			if( parameters[i].equals(HTSSyntax.DELETE_FORCE) )
			{
				forceChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.DELETE_IFEXISTS) )
			{
				ifexistsChk.setSelection(false);
			}
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector ret = new Vector(4);
		
		ret.add( getSelectedElement(dirCV) );
		ret.add( '"' + fileTxt.getText() + '"');
		if( forceChk.getSelection() )
			ret.add(HTSSyntax.DELETE_FORCE);
		if( !ifexistsChk.getSelection() )
			ret.add(HTSSyntax.DELETE_IFEXISTS);
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( getSelectedElement(dirCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("DeleteSection.5"), Messages.getString("DeleteSection.6")); //$NON-NLS-1$ //$NON-NLS-2$
			dirCV.getCombo().setFocus();
			return false;
		}
		
		if( fileTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("DeleteSection.8"), Messages.getString("DeleteSection.9")); //$NON-NLS-1$ //$NON-NLS-2$
			fileTxt.setFocus();
			return false;
		}
		
		return true;
	}

}
