/*
 * Created on 24.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das EXISTS Kommando.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSExists
 */
public class ExistsSection extends CommandSection {

	private ComboViewer dirCV;
	private Text fileTxt;
	private Button isDirChk;
	
	private static final String[] dirs = { HTSSyntax.FILEOP_ABS, HTSSyntax.FILEOP_BIN, HTSSyntax.FILEOP_ENV, HTSSyntax.FILEOP_LOG, HTSSyntax.FILEOP_REF, HTSSyntax.FILEOP_TGT };
	
	public ExistsSection(Composite parent) {
		super(parent);

		createContents();
		fillCombos();
	}
	
	private void createContents() {
		// COPY, <QUELLVERZEICHNIS>, <QUELLDATEI>, <ZIELVERZEICHNIS>,
		// <ZIELDATEI> [,FORCE] [,IFSRCEXISTS]
		
		dirCV = createComboViewer(Messages.getString("ExistsSection.0")); //$NON-NLS-1$
		
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("ExistsSection.1")); //$NON-NLS-1$
		fileTxt = new Text(this, SWT.BORDER);
		fileTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		isDirChk = new Button(this, SWT.CHECK);
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		isDirChk.setLayoutData(gd);
		isDirChk.setText(Messages.getString("ExistsSection.2")); //$NON-NLS-1$
	}
	
	private void fillCombos() 
	{
		dirCV.setInput(dirs);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		dirCV.setSelection( new StructuredSelection(parameters[0]) );
		fileTxt.setText( HTSParser.stripQuotes(parameters[1]) );
				
		for(int i = 2; i < parameters.length; ++i)
		{
			if( parameters[i].equals(HTSSyntax.EXISTS_DIRECTORY) )
				isDirChk.setSelection(true);
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector ret = new Vector(3);
		
		ret.add( getSelectedElement(dirCV) );
		ret.add( '"' + fileTxt.getText() + '"');
		if( isDirChk.getSelection() )
			ret.add(HTSSyntax.EXISTS_DIRECTORY);
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( getSelectedElement(dirCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("ExistsSection.4"), Messages.getString("ExistsSection.5")); //$NON-NLS-1$ //$NON-NLS-2$
			dirCV.getCombo().setFocus();
			return false;
		}
		
		if( fileTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("ExistsSection.7"), Messages.getString("ExistsSection.8")); //$NON-NLS-1$ //$NON-NLS-2$
			fileTxt.setFocus();
			return false;
		}
		
		return true;
	}

}
