/*
 * Created on 31.01.2006
 */
package atosj.dialog.addcmd;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Control;

import atosj.app.AtosJApp;
import atosj.util.Snooze;

/**
 * Faded die Hintergrundfarbe eines Controls von RGB(152, 170, 203) 
 * zu RGB(255, 255, 255).
 * @author Nicos Tegos
 */
public class Fader implements Runnable 
{
	private class ColorSetter implements Runnable
	{
		private Control ctrl;
		private Color clr;
		
		public ColorSetter(Control ctrl, Color clr) {
			this.ctrl = ctrl;
			this.clr = clr;
		}
		
		public void run() {
			if( !ctrl.isDisposed() && ctrl.isVisible() )
				ctrl.setBackground(clr);
		}
		
		public void setColor(Color clr) {
			this.clr = clr;
		}
	}
	
	private Control ctrl;
	private int steps;
	private int timePerSteps;
	
	private int currentSteps = 0;
	private ColorSetter fadeSetter;
	private ColorSetter defaultSetter;
	private Thread currentThread = new Thread(this);
	
	/**
	 * @param ctrl Das Control desssen Hintergrundfarbe gefaded werden soll.
	 * @param steps Anzahl der Schritte die beim faden verwendet werden.
	 * @param timePerStep Wartezeit zwischen den Schritten in Millisekunden.
	 */
	public Fader(Control ctrl, int steps, int timePerStep)
	{
		this.ctrl = ctrl;
		this.steps = steps;
		this.timePerSteps = timePerStep;
		
		fadeSetter = new ColorSetter(ctrl, AtosJApp.getInstance().getColor( new RGB(152, 170, 203) ) );
		defaultSetter = new ColorSetter(ctrl, ctrl.getBackground() );
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	public void run()
	{
		 RGB startRGB = new RGB(152, 170, 203);
		 float increaseRed = (255.0f - startRGB.red) / steps;
		 float increaseGreen = (255.0f - startRGB.green) / steps;
		 float increaseBlue = (255.0f - startRGB.blue) / steps;
		 
		while(currentSteps <= steps)
		{
			if( !ctrl.isDisposed() )
			{
				RGB currentRGB = new RGB(
						startRGB.red + (int)(increaseRed*currentSteps), 
						startRGB.green + (int)(increaseGreen*currentSteps), 
						startRGB.blue + (int)(increaseBlue*currentSteps));
				fadeSetter.setColor( AtosJApp.getInstance().getColor(currentRGB) );
				ctrl.getDisplay().syncExec(fadeSetter);
				new Snooze(timePerSteps);
			}
			
			++currentSteps;
		}
		 
		 if( !ctrl.isDisposed() )
			ctrl.getDisplay().syncExec(defaultSetter);
	}
	
	public void start() {
		currentSteps = 0;
		if( !currentThread.isAlive() )
		{	
			currentThread = new Thread(this);
			currentThread.start();
		}
	}
}
