/*
 * Created on 21.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das LAUNCH Kommando.
 * @author Nicos Tegos
 */
public class LaunchSection extends CommandSection {

	private ComboViewer dirCV;
	private Text prognameTxt;
	private Text parametersTxt;
	private ComboViewer waitCV;
	
	private Text waitTxt;
	private VisibilityChanger waitTxtVis;
	private VisibilityChanger waitLblVis;
	
	private Text returnCodeTxt;
	private VisibilityChanger returnTxtVis;
	private VisibilityChanger returnLblVis;
	
	private Text logfileTxt;
		
	private static final String[] dirs = { HTSSyntax.FILEOP_ABS, HTSSyntax.FILEOP_BIN, HTSSyntax.FILEOP_ENV, HTSSyntax.FILEOP_LOG, HTSSyntax.FILEOP_REF, HTSSyntax.FILEOP_TGT };
	private static final String[] waitstates = { HTSSyntax.LAUNCH_FOREVER, HTSSyntax.LAUNCH_NOWAIT, HTSSyntax.LAUNCH_TIME };
	
	public LaunchSection(Composite parent) {
		super(parent);
	
		createContent();
		fillCombos();
	}
	
	private void createContent() 
	{
		// <VERZEICHNIS>, <PROGRAMMNAME>, <PARAMETER>, FOREVER, <RETURNCODE>, [,<LOGFILE>]
		dirCV = createComboViewer(Messages.getString("LaunchSection.0")); //$NON-NLS-1$
		
		
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("LaunchSection.1")); //$NON-NLS-1$
		prognameTxt = new Text(this, SWT.BORDER);
		prognameTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("LaunchSection.2")); //$NON-NLS-1$
		parametersTxt = new Text(this, SWT.BORDER);
		parametersTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		waitCV = createComboViewer(Messages.getString("LaunchSection.3")); //$NON-NLS-1$
		waitCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				String waitState = getSelectedElement(waitCV);
				if( waitState.equals(HTSSyntax.LAUNCH_TIME) )
				{
					returnLblVis.setVisible(true);
					returnTxtVis.setVisible(true);
					waitLblVis.setVisible(true);
					waitTxtVis.setVisible(true);
				}
				else
				if( waitState.equals(HTSSyntax.LAUNCH_FOREVER) )
				{
					returnLblVis.setVisible(true);
					returnTxtVis.setVisible(true);
					waitLblVis.setVisible(false);
					waitTxtVis.setVisible(false);
				}
				else
				if( waitState.equals(HTSSyntax.LAUNCH_NOWAIT) )
				{
					returnLblVis.setVisible(false);
					returnTxtVis.setVisible(false);
					waitLblVis.setVisible(false);
					waitTxtVis.setVisible(false);
				}
				
				layout();
			}
		});
		
		// angabe der wartezeit nur bei auswahl von TIME
		Label waitLbl = new Label(this, SWT.NULL);
		waitLbl.setText(Messages.getString("LaunchSection.4")); //$NON-NLS-1$
		waitLblVis = new VisibilityChanger(waitLbl, false);
		waitLblVis.setVisible(false);
		
		waitTxt = new Text(this, SWT.BORDER);
		waitTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		waitTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				e.doit = Character.isDigit(e.character) 
				|| Character.isISOControl(e.character);
			}
		});
		waitTxtVis = new VisibilityChanger(waitTxt, true);
		waitTxtVis.setVisible(false);
		
		// Eingabe des returncodes nur bei FOREVER oder TIME
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("LaunchSection.5")); //$NON-NLS-1$
		returnLblVis = new VisibilityChanger(lbl, false);
		returnLblVis.setVisible(false);
		returnCodeTxt = new Text(this, SWT.BORDER);
		returnCodeTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		returnCodeTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				e.doit = Character.isDigit(e.character) 
				|| Character.isISOControl(e.character);
			}
		});
		returnTxtVis = new VisibilityChanger(returnCodeTxt, true);
		returnTxtVis.setVisible(false);
		
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("LaunchSection.6")); //$NON-NLS-1$
		logfileTxt = new Text(this, SWT.BORDER);
		logfileTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
	}
	
	private void fillCombos() {
		dirCV.setInput(dirs);
		waitCV.setInput(waitstates);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		// <VERZEICHNIS>, <PROGRAMMNAME>, <PARAMETER>, FOREVER, <RETURNCODE>, [,<LOGFILE>]
		dirCV.setSelection( new StructuredSelection(parameters[0]) );
		prognameTxt.setText( HTSParser.stripQuotes(parameters[1]) );
		parametersTxt.setText( HTSParser.stripQuotes(parameters[2]) );
		waitCV.setSelection( new StructuredSelection(parameters[3]) );
		
		if( parameters[3].equals(HTSSyntax.LAUNCH_TIME) )
		{
			waitTxt.setText( parameters[4] );
			returnCodeTxt.setText( parameters[5] );
			if(parameters.length > 6)
				logfileTxt.setText( HTSParser.stripQuotes(parameters[6]) );
		}
		else
		if( parameters[3].equals(HTSSyntax.LAUNCH_FOREVER) )
		{
			returnCodeTxt.setText( parameters[4] );
			if(parameters.length > 5)
				logfileTxt.setText( HTSParser.stripQuotes(parameters[5]) );
		}
		else
		if( parameters[3].equals(HTSSyntax.LAUNCH_NOWAIT) )
		{
			if(parameters.length > 4)
				logfileTxt.setText( HTSParser.stripQuotes(parameters[4]) );
		}
	}
	
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(2);
		parameters.add( getSelectedElement(dirCV) );
		parameters.add( '"' +prognameTxt.getText() + '"' ); 
		parameters.add( '"' +parametersTxt.getText() + '"' );
		String waitState = getSelectedElement(waitCV);
		parameters.add(waitState);
		
		if( waitState.equals(HTSSyntax.LAUNCH_TIME) )
			parameters.add( waitTxt.getText() );
		
		if( waitState.equals(HTSSyntax.LAUNCH_FOREVER)
			|| waitState.equals(HTSSyntax.LAUNCH_TIME) )
		{
			parameters.add( returnCodeTxt.getText() );
		}
		
		if( !logfileTxt.getText().equals("") ) //$NON-NLS-1$
			parameters.add( '"' + logfileTxt.getText() + '"' );
		
		return parameters;
	}

	public boolean checkIntegrity() 
	{
		if( getSelectedElement(dirCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("LaunchSection.9"), Messages.getString("LaunchSection.10")); //$NON-NLS-1$ //$NON-NLS-2$
			dirCV.getCombo().setFocus();
			return false;
		}
		
		if( prognameTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("LaunchSection.12"), Messages.getString("LaunchSection.13")); //$NON-NLS-1$ //$NON-NLS-2$
			prognameTxt.setFocus();
			return false;
		}
		
		if( getSelectedElement(waitCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("LaunchSection.15"), Messages.getString("LaunchSection.16")); //$NON-NLS-1$ //$NON-NLS-2$
			waitCV.getCombo().setFocus();
			return false;
		}
		
		if( getSelectedElement(waitCV).equals(HTSSyntax.LAUNCH_TIME) )
		{
			try
			{
				int value = Integer.parseInt(waitTxt.getText());
				if(value <= 0)
				{
					MessageDialog.openError(null, Messages.getString("LaunchSection.17"), Messages.getString("LaunchSection.18")); //$NON-NLS-1$ //$NON-NLS-2$
					waitTxt.setFocus();
					return false;
				}
			} catch(NumberFormatException ex) 
			{
				MessageDialog.openError(null, Messages.getString("LaunchSection.19"), Messages.getString("LaunchSection.20")); //$NON-NLS-1$ //$NON-NLS-2$
				waitTxt.setFocus();
				return false;
			}
			
			try
			{
				int value = Integer.parseInt(returnCodeTxt.getText());
				if(value < 0)
				{
					MessageDialog.openError(null, Messages.getString("LaunchSection.21"), Messages.getString("LaunchSection.22")); //$NON-NLS-1$ //$NON-NLS-2$
					returnCodeTxt.setFocus();
					return false;
				}
			} catch(NumberFormatException ex) 
			{
				MessageDialog.openError(null, Messages.getString("LaunchSection.23"), Messages.getString("LaunchSection.24")); //$NON-NLS-1$ //$NON-NLS-2$
				returnCodeTxt.setFocus();
				return false;
			}
		}
		else if( getSelectedElement(waitCV).equals(HTSSyntax.LAUNCH_FOREVER) )
		{
			try
			{
				int value = Integer.parseInt(returnCodeTxt.getText());
				if(value < 0)
				{
					MessageDialog.openError(null, Messages.getString("LaunchSection.25"), Messages.getString("LaunchSection.26")); //$NON-NLS-1$ //$NON-NLS-2$
					returnCodeTxt.setFocus();
					return false;
				}
			} catch(NumberFormatException ex) 
			{
				MessageDialog.openError(null, Messages.getString("LaunchSection.27"), Messages.getString("LaunchSection.28")); //$NON-NLS-1$ //$NON-NLS-2$
				returnCodeTxt.setFocus();
				return false;
			}
		} // Launch_NOWAIT hat weder Zeit noch Rckgabewert
		
		return true;
	}

}
