/*
 * Created on 21.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das QUESTION Kommando.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSQuestion
 */
public class QuestionSection extends CommandSection {

	private Text questionTxt;
	private ComboViewer answerCV;
	
	private static final String[] answers = { HTSSyntax.QUESTION_YES, HTSSyntax.QUESTION_NO };
	
	public QuestionSection(Composite parent) {
		super(parent);
		
		createContent();
	}
	
	private void createContent() {
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("QuestionSection.0")); //$NON-NLS-1$
		lbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		questionTxt = new Text(this, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		questionTxt.setLayoutData(gd);
		
		answerCV = createComboViewer(Messages.getString("QuestionSection.1")); //$NON-NLS-1$
		answerCV.setInput(answers);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		questionTxt.setText( HTSParser.stripQuotes(parameters[0]) );
		answerCV.setSelection( new StructuredSelection(parameters[1]) );
	}
	
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(2);
		parameters.add( '"' + questionTxt.getText() + '"');
		parameters.add( getSelectedElement(answerCV) ); 
		
		return parameters;
	}

	public boolean checkIntegrity() 
	{
		if( questionTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("QuestionSection.3"), Messages.getString("QuestionSection.4")); //$NON-NLS-1$ //$NON-NLS-2$
			questionTxt.setFocus();
			return false;
		}
		
		if( getSelectedElement(answerCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("QuestionSection.6"), Messages.getString("QuestionSection.7")); //$NON-NLS-1$ //$NON-NLS-2$
			answerCV.getCombo().setFocus();
			return false;
		}
		
		return true;
	}

}
