package atosj.dialog.addcmd;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.editors.urf.ClassNameComparator;
import atosj.editors.urf.IdTableLabelProvider;
import atosj.editors.urf.NameComparator;
import atosj.editors.urf.WindowNameComparator;
import atosj.gui.TableResizeListener;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.util.UniversalViewerSorter;

/**
 * Dient zur Eingabe der Parameter fr das READ Kommando.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSRead
 */
public class ReadSection extends CommandSection 
{
	/**
	 * Gibt alle gltigen Zustnde die bei einem bestimmten Elementtyp
	 * ausgelesen werden knnen zurck.
	 */
	private static class StateMap
	{
		private static HashMap typeToState; // HashMap<String, String[]>
		private static HashMap subtypeToState; // HashMap<String, String[]>
		
		static
		{
			typeToState = new HashMap();
			subtypeToState = new HashMap();
			
			// COMPONENT
			String[] states = new String[] { HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_COMPONENT, states);
			
			// BUTTON
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_BUTTON, states);
			
			// CHECKBOX
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_CHECKBOX, states);
			
			// COMBOBOX
			states = new String[] { HTSSyntax.READ_ITEMCOUNT, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_COMBOBOX, states);
			String[] substates = new String[] {};
			subtypeToState.put(HTSSyntax.TYPE_COMBOBOX, substates);
			
			// EDITBOX
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_EDITBOX, states);
			
			// SPINNER
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_SPINNER, states);
			
			// LABEL
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_LABEL, states);
			
			// LIST
			states = new String[] { HTSSyntax.READ_ITEMCOUNT, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_LIST, states);
			// TODO vielleicht auch NUM einfhren in IList
			substates = new String[] { HTSSyntax.READ_TEXT };
			subtypeToState.put(HTSSyntax.TYPE_LIST, substates);
			
			// MENU
			states = new String[] { HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_MENU, states);
			substates = new String[] { HTSSyntax.READ_CHECKSTATE, HTSSyntax.READ_ENABLESTATE };
			subtypeToState.put(HTSSyntax.TYPE_MENU, substates);
			
			// RADIOBUTTON
			states = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_RADIOBUTTON, states);
			
			// TABFOLDER
			states = new String[] { HTSSyntax.READ_ITEMCOUNT, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TABFOLDER, states);
			substates = new String[] { HTSSyntax.READ_TEXT };
			subtypeToState.put(HTSSyntax.TYPE_TABFOLDER, substates);
			
			// TABLE
			states = new String[] { HTSSyntax.READ_ITEMCOUNT, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TABLE, states);
			substates = new String[] { HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE };
			subtypeToState.put(HTSSyntax.TYPE_TABLE, substates);
			
			// TREE
			states = new String[] { HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TREE, states);
			substates = new String[] {};
			subtypeToState.put(HTSSyntax.TYPE_TREE, substates);
			
			// WINDOW
			states = new String[] { HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT };
			typeToState.put(HTSSyntax.TYPE_WINDOW, states);
		}
		
		/**
		 * @param elementType Der Typ des Elements, siehe HTSSyntax.TYPE_* Konstanten.
		 * @param subtype Gibt an, ob die zustnde fr das element selbst oder fr dessen subitem gewnscht sind
		 * @return Alle Zustnde die fr diesen elementtyp, bzw. subtyp, zugelassen sind.
		 */
		public static String[] getStates(String elementType, boolean subtype) 
		{
			if(subtype)
			{
				String[] actions = (String[]) subtypeToState.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
			else
			{
				if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCustomType(elementType) )
					return new String[] { HTSSyntax.READ_PROPERTY, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE };
				
				String[] actions = (String[]) typeToState.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
		}
	}

	/** Liefert zu jeder Zustandsprfung eines HTS-Typs das Format des Subelementpfads. */
	private static class PathTypeMap
	{
		public static final int PATH_TYPE_NUM = 1; // die Pfadelemente mssen natrliche Zahlen >=0 sein 
		public static final int PATH_TYPE_STR = 2; // die Pfadelemente mssen Zeichenketten sein
		public static final int PATH_TYPE_NUM_OR_STR = 3; // die Pfadelemente mssen knnen beides sein
		
		private static final Integer P_T_N = new Integer(PATH_TYPE_NUM);
		private static final Integer P_T_S = new Integer(PATH_TYPE_STR);
		private static final Integer P_T_NS = new Integer(PATH_TYPE_NUM_OR_STR);
		
		public static final int PATH_LEN_INFINITE = -1;
		
		private static final Integer P_L_I = new Integer(PATH_LEN_INFINITE);
		private static final Integer P_L_1 = new Integer(1);
		private static final Integer P_L_2 = new Integer(2);
		
		private static Object[][] map = new Object[][]
		{
			{HTSSyntax.TYPE_LIST, HTSSyntax.READ_TEXT, P_T_N, P_L_1},
			// TODO in IList noch getNum(index) einfhren?
			{HTSSyntax.TYPE_MENU, HTSSyntax.READ_CHECKSTATE, P_T_S, P_L_I},
			{HTSSyntax.TYPE_MENU, HTSSyntax.READ_ENABLESTATE, P_T_S, P_L_I},
			{HTSSyntax.TYPE_TABFOLDER, HTSSyntax.READ_TEXT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.READ_CHECKSTATE, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.READ_TEXT, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.READ_NUM, P_T_N, P_L_2}
			//{HTSSyntax.TYPE_TREE, HTSSyntax.READ_TEXT, P_T_S, P_L_I}, sinnlos weil wir eh den kompletten Pfad angeben mssen
			//{HTSSyntax.TYPE_TREE, HTSSyntax.READ_NUM, P_T_S, P_L_I},
		};
		
		/**
		 * berprft die gltigkeit des subelementpfades bei angegebenem elementtyp
		 * und readstate.
		 * @param subitemPath Vector<StringOderInteger>
		 * @return null fr korrekt, fehlernachricht sonst.
		 */
		public static String checkPath(String type, String state, Vector subitemPath)
		{
			for(int i = 0; i < map.length; ++i)
			{
				if( map[i][0].equals(type) && map[i][1].equals(state) )
				{
					int p_type = ((Integer)map[i][2]).intValue();
					int p_len = ((Integer)map[i][3]).intValue();
					
					if( subitemPath.size() == 0 )
						return Messages.getString("ReadSection.0"); //$NON-NLS-1$
					
					if( p_len != PATH_LEN_INFINITE )
					{
						if( p_len < subitemPath.size() || p_len > subitemPath.size() )
							return Messages.getString("ReadSection.1") + p_len + Messages.getString("ReadSection.2"); //$NON-NLS-1$ //$NON-NLS-2$
					}
					
					if(p_type == PATH_TYPE_NUM_OR_STR)
						return null;
					
					for(int j = 0; j < subitemPath.size(); ++j)
					{
						int currentType = (subitemPath.get(j) instanceof Integer) ? PATH_TYPE_NUM : PATH_TYPE_STR;
						if( (currentType & p_type) == 0 )
							return Messages.getString("ReadSection.3") + (p_type == PATH_TYPE_NUM ? Messages.getString("ReadSection.4") : Messages.getString("ReadSection.5"));  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
				}
			}
			
			return null;
		}
	}

///////////////////////////////////// private classes end //////////////////////	
	
	private TableViewer idTable;
	private ComboViewer stateCV;
	
	private Text variableTxt;
	
	private Button subitemChk;
	private SubItemWidget subitemWidget;
	private VisibilityChanger subitemChkVis;
	private VisibilityChanger subitemChkLblVis;
	private VisibilityChanger subitemLblVis;
	private VisibilityChanger subitemWidgetVis;
	private VisibilityChanger subitemListVis;
	
	/*private String states[] = new String[] { HTSSyntax.READ_CHECKSTATE, HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE,
			HTSSyntax.READ_NUM, HTSSyntax.READ_TEXT, HTSSyntax.READ_VISIBLESTATE };*/
	private static final String[] EMPTY_STATES = new String[0];
	
	public ReadSection(Composite parent) {
		super(parent);
		
		createContent();
		fillCombos();
	}

	private void createContent() 
	{	
		createTableArea();
		
		Label subitemChkLbl = new Label(this, SWT.NULL);
		subitemChkLbl.setText(Messages.getString("ReadSection.6")); //$NON-NLS-1$
		subitemChkLblVis = new VisibilityChanger(subitemChkLbl, false);
		subitemChkLblVis.setVisible(false);
		
		subitemChk = new Button(this, SWT.CHECK);
		subitemChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				onSubitemChkSelection();
			}
		});
		subitemChkVis = new VisibilityChanger(subitemChk, false);
		subitemChkVis.setVisible(false);
		
        // Angabe der Subitems
		Label subitemLbl = new Label(this, SWT.NULL);
		subitemLbl.setText(Messages.getString("ReadSection.7")); //$NON-NLS-1$
		subitemLbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		subitemLblVis = new VisibilityChanger(subitemLbl, false);
		subitemLblVis.setVisible(false);
		
		subitemWidget = new SubItemWidget(this);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		subitemWidget.setLayoutData(gd);
		subitemWidgetVis = new VisibilityChanger(subitemWidget, false);
		subitemWidgetVis.setVisible(false);
		subitemListVis = new VisibilityChanger(subitemWidget.getList(), true);
				
		// Combo der Zustnde
		stateCV = createComboViewer(Messages.getString("ReadSection.8")); //$NON-NLS-1$
		stateCV.getCombo().setVisibleItemCount(8);
				
		// Textfeld zur fr die die Eingabe des Variablennamen
		Label variableLbl = new Label(this, SWT.NULL);
		variableLbl.setText(Messages.getString("ReadSection.9")); //$NON-NLS-1$
				
		variableTxt = new Text(this, SWT.BORDER);
		variableTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
	}
	
	private void createTableArea()
	{
		Label lbl = new Label(this, SWT.NULL);
		lbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		lbl.setText(Messages.getString("ReadSection.10")); //$NON-NLS-1$
		
		idTable = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION);
		idTable.setContentProvider( new ArrayContentProvider() );
		idTable.setLabelProvider( new IdTableLabelProvider() );
		idTable.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onIdTableSelection(e);
				/*if( !e.getSelection().isEmpty() )
				{
					ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
					boolean enabled = ComponentResolver.isCollectionComponent( id.getClassName() );
					subitemLblVis.setVisible(enabled);
					subitemWidgetVis.setVisible(enabled);
					subitemListVis.setVisible(enabled);
				}
				else
				{
					subitemLblVis.setVisible(false);
					subitemWidgetVis.setVisible(false);
					subitemListVis.setVisible(false);
				}
				
				layout();*/
			}
			
		});
				
		Table table = idTable.getTable();
		new TableResizeListener(table, new int[] {25, 25, 25}, 20);
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		table.setLayoutData(gd);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ReadSection.11")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ReadSection.12")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ReadSection.13")); //$NON-NLS-1$
		
		UniversalViewerSorter.setColumnSorters(idTable, 
				new Comparator[] 
				{
					new WindowNameComparator(),
					new NameComparator(),
					new ClassNameComparator()
				},
				0);
	}
	
	private void fillCombos() {
		stateCV.setInput(EMPTY_STATES);
		idTable.setInput( AtosJApp.getInstance().getInput().getURFFile().getIds() );
	}
	
	private void onIdTableSelection(SelectionChangedEvent e)
	{
		if( !e.getSelection().isEmpty() )
		{
			ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
			boolean enabled = AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent( id.getClassName() );
			subitemChk.setSelection(enabled);
			
			subitemChkLblVis.setVisible(enabled);
			subitemChkVis.setVisible(enabled);
			subitemLblVis.setVisible( subitemChk.getSelection() );
			subitemWidgetVis.setVisible( subitemChk.getSelection()  );
			subitemListVis.setVisible( subitemChk.getSelection() );
			
			// preserve the selection
			ISelection sel = stateCV.getSelection();
			stateCV.setInput( StateMap.getStates(id.getClassName(), subitemChk.getSelection()) );
			// try to set the preserved selection again
			stateCV.setSelection(sel);
		}
		else
		{
			subitemChkLblVis.setVisible(false);
			subitemChkVis.setVisible(false);
			subitemLblVis.setVisible(false);
			subitemWidgetVis.setVisible(false);
			subitemListVis.setVisible(false);
			
			stateCV.setInput(EMPTY_STATES);
			// empty selection
			stateCV.setSelection( new StructuredSelection() );
		}
		
		layout();
	}
	
	private void onSubitemChkSelection() 
	{
		boolean selected = subitemChk.getSelection();
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		// preserve the selection
		ISelection sel = stateCV.getSelection();
		stateCV.setInput( StateMap.getStates(id.getClassName(), selected) );
		// try to set the preserved selection again
		stateCV.setSelection(sel);
		
		subitemLblVis.setVisible(selected);
		subitemWidgetVis.setVisible(selected);
		subitemListVis.setVisible(selected);
		
		layout();
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		ComponentId id = new ComponentId( 
				HTSParser.stripQuotes(parameters[2]), 
				parameters[1], 
				HTSParser.stripQuotes(parameters[0]) );
		
		idTable.setSelection( new StructuredSelection(id), true );
		variableTxt.setText( HTSParser.stripQuotes(parameters[4]) );
		
		int possibleSubitemIndex = 5; // mglicher index des Schlsselwortes SUBITEM
		if(possibleSubitemIndex < parameters.length)
		{
			Vector subitemPath = new Vector();
			for(int i = possibleSubitemIndex +1; i < parameters.length; ++i)
				subitemPath.add(parameters[i]);
			
			subitemWidget.setSubitemPath( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
			subitemChk.setSelection(true);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
		}
		else
		{
			subitemChk.setSelection(false);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
		}
		// na ja die Reihenfolge mag merkwrdig anmuten, ist aber nicht anders zu beheben,
		// denn der Inhalt der actionCV wird erst richtig gesetzt, wenn das Hckchen
		// bei subitemChk richtig gesetzt wurde und das geht erst nach dem Parsen eines
		// evtl. Subitempfades.
		stateCV.setSelection( new StructuredSelection(parameters[3]) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(16);
		
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		parameters.add( '"' + id.getWindowName() + '"' );
		parameters.add( id.getClassName() );
		parameters.add( '"' + id.getName() + '"' );
		parameters.add( getSelectedElement(stateCV) );
		parameters.add( '"' + variableTxt.getText() + '"' );
		
		if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent(id.getClassName()) 
				&& subitemChk.getSelection() )
		{
			parameters.add("SUBITEM"); //$NON-NLS-1$
			parameters.addAll( subitemWidget.getSubitemPathForLine() );
		}
		
		return parameters;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( idTable.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("ReadSection.15"), Messages.getString("ReadSection.16")); //$NON-NLS-1$ //$NON-NLS-2$
			idTable.getTable().setFocus();
			return false;
		}
		
		if( subitemChk.getSelection() )
		{
			Vector subitemPath = subitemWidget.getSubitemPath();
			String type = ((ComponentId)((StructuredSelection)idTable.getSelection()).getFirstElement()).getClassName();
			String state = (String) ((StructuredSelection)stateCV.getSelection()).getFirstElement();
			String result = PathTypeMap.checkPath(type, state, subitemPath);
			if(result != null)
			{
				subitemWidget.getList().setFocus();
				MessageDialog.openError(null, Messages.getString("ReadSection.17"), result); //$NON-NLS-1$
				return false;
			}
		}
		
		if( stateCV.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("ReadSection.18"), Messages.getString("ReadSection.19")); //$NON-NLS-1$ //$NON-NLS-2$
			stateCV.getCombo().setFocus();
			return false;
		}
		
		if( variableTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("ReadSection.21"), Messages.getString("ReadSection.22")); //$NON-NLS-1$ //$NON-NLS-2$
			variableTxt.setFocus();
			return false;
		}
		
		return true;
	}

}
