package atosj.dialog.addcmd;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.PositionWidget;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.parser.HTSParser;
import atosj.util.AcceleratorHook;
import atosj.util.DefaultImageDescriptor;

class AddElementAction extends Action
{
	private ListViewer lv;
		
	AddElementAction(ListViewer lv) {
		super(Messages.getString("SubItemWidget.0")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'N');
		setToolTipText(Messages.getString("SubItemWidget.1")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD) );
		
		this.lv = lv;
	}

	private Vector getSubitemPath() {
		return (Vector) lv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		// aktuellen Index bestimmen
		Object currentElement = ((StructuredSelection)lv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentElement != null)
			 currentPos = getSubitemPath().indexOf(currentElement);
		
		AddPathElementDlg dlg = new AddPathElementDlg(
				lv.getList().getShell(), 
				getSubitemPath().size(),
				currentPos);
		
		if( dlg.open() == AddCmdDlg.ID_ADD )
		{
			getSubitemPath().add( dlg.getPosition(), dlg.getPathElement() );
			lv.refresh();
			lv.setSelection( new StructuredSelection( dlg.getPathElement()) );
		}
	}
}

class EditElementAction extends Action implements ISelectionChangedListener
{
	private ListViewer lv;
		
	EditElementAction(ListViewer lv) {
		super(Messages.getString("SubItemWidget.2")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'E');
		setToolTipText(Messages.getString("SubItemWidget.3")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EDIT) );
				
		this.lv = lv;
		
		lv.addSelectionChangedListener(this);
		setEnabled( ((StructuredSelection)lv.getSelection()).size() == 1 );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( ((StructuredSelection) lv.getSelection()).size() == 1);		
	}
	
	private Vector getSubitemPath() {
		return (Vector) lv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		int index = lv.getList().getSelectionIndex();
		
		AddPathElementDlg dlg = new AddPathElementDlg(
				lv.getList().getShell(), 
				getSubitemPath().get(index));
		
		if( dlg.open() == AddCmdDlg.ID_ADD )
		{
			getSubitemPath().remove(index);
			getSubitemPath().add( index, dlg.getPathElement() );
			lv.refresh();
			lv.setSelection( new StructuredSelection( dlg.getPathElement()) );
		}
		
		lv.refresh();
	}
}

class RemoveElementAction extends Action implements ISelectionChangedListener
{
	private ListViewer lv;
		
	RemoveElementAction(ListViewer lv) {
		super(Messages.getString("SubItemWidget.4")); //$NON-NLS-1$
		setAccelerator(SWT.DEL);
		setToolTipText(Messages.getString("SubItemWidget.5")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE_DIS) );
		
		this.lv = lv;
		
		lv.addSelectionChangedListener(this);
		setEnabled( !lv.getSelection().isEmpty() );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( !lv.getSelection().isEmpty() );
	}
	
	private Vector getSubitemPath() {
		return (Vector) lv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		Object[] elements = ((StructuredSelection)lv.getSelection()).toArray();
		for(int i = 0; i < elements.length; ++i)
		{
			Iterator it = getSubitemPath().iterator();
			while( it.hasNext() )
			{
				if( it.next() == elements[i] )
				{
					it.remove();
					break;
				}
			}
		}
		
		lv.refresh();
	}
}

class MoveUpAction extends Action implements ISelectionChangedListener
{
	private ListViewer lv;
		
	MoveUpAction(ListViewer lv) {
		super(Messages.getString("SubItemWidget.6")); //$NON-NLS-1$
		setToolTipText(Messages.getString("SubItemWidget.7")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP_DIS) );
		
		this.lv = lv;
		
		lv.addSelectionChangedListener(this);
		setEnabled( ((StructuredSelection)lv.getSelection()).size() == 1 );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( ((StructuredSelection) lv.getSelection()).size() == 1);		
	}
	
	private Vector getSubitemPath() {
		return (Vector) lv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		Object toMove = ((StructuredSelection)lv.getSelection()).getFirstElement();
		int index = lv.getList().getSelectionIndex();
		if( index > 0 )
		{
			getSubitemPath().remove(index);
			getSubitemPath().add(--index, toMove);
		}
		
		lv.refresh();
	}
}

class MoveDownAction extends Action implements ISelectionChangedListener
{
	private ListViewer lv;
		
	MoveDownAction(ListViewer tv) {
		super(Messages.getString("SubItemWidget.8")); //$NON-NLS-1$
		setToolTipText(Messages.getString("SubItemWidget.9")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN_DIS) );
		
		this.lv = tv;
		
		tv.addSelectionChangedListener(this);
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( ((StructuredSelection) lv.getSelection()).size() == 1);		
	}
	
	private Vector getSubitemPath() {
		return (Vector) lv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() {
		Object toMove = ((StructuredSelection)lv.getSelection()).getFirstElement();
		int index = lv.getList().getSelectionIndex();
		if(index < getSubitemPath().size() -1)
		{
			getSubitemPath().remove(index);
			getSubitemPath().add(++index, toMove);
			lv.refresh();
		}
	}
}

class SubitemTableLabelProvider extends BaseLabelProvider
{
	public String getText(Object element) {
		if(element instanceof String)
			return "\"" + element + "\""; //$NON-NLS-1$ //$NON-NLS-2$
		else
			return element.toString();
	}
}

/**
 * Sektion zur Eingabe von Subelementpfaden fr zusammengesetzte
 * Components, wie Tabellen.
 * @author Nicos Tegos
 */
public class SubItemWidget extends Composite 
{
	private ListViewer subitemLV;
	private MenuManager menuMngr = new MenuManager();
	private AcceleratorHook accHook;
	private Action editAction;
	private Vector subitemPath = new Vector();
	
	public static final String PROP_VALUE = Messages.getString("SubItemWidget.12"); //$NON-NLS-1$
	public static final String PROP_TYPE = Messages.getString("SubItemWidget.13"); //$NON-NLS-1$
	public static final String[] types = new String[] {Messages.getString("SubItemWidget.14"), Messages.getString("SubItemWidget.15")}; //$NON-NLS-1$ //$NON-NLS-2$
	
	public SubItemWidget(Composite parent) 
	{
		super(parent, SWT.NULL);
				
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = 0;
		setLayout(gl);
		
		createContents();
		subitemLV.setInput(subitemPath);
	}
	
	private void createContents() 
	{
		subitemLV = new ListViewer(this, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		subitemLV.setContentProvider( new ArrayContentProvider() );
		subitemLV.setLabelProvider( new SubitemTableLabelProvider() );
		
		final List list = subitemLV.getList();
		list.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				list.setMenu( menuMngr.createContextMenu(list) );
			}
		});
		
		list.addListener(SWT.MouseDoubleClick, new Listener() {
			public void handleEvent(Event e) {
				if( editAction.isEnabled() )
					editAction.run();
			}
		});
		
		list.setLayoutData( new GridData(GridData.FILL_BOTH) );
				
		Action addAction = new AddElementAction(subitemLV);
		editAction = new EditElementAction(subitemLV);
		Action removeAction = new RemoveElementAction(subitemLV);
		Action upAction = new MoveUpAction(subitemLV);
		Action downAction = new MoveDownAction(subitemLV);
		
		menuMngr.add(addAction);
		menuMngr.add(editAction);
		menuMngr.add(removeAction);
		menuMngr.add(upAction);
		menuMngr.add(downAction);
		
		ToolBarManager tobMngr = new ToolBarManager(SWT.VERTICAL);
		tobMngr.add(addAction);
		tobMngr.add(removeAction);
		tobMngr.add(upAction);
		tobMngr.add(downAction);
				
		Control ctrl = tobMngr.createControl(this);
		ctrl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		accHook = new AcceleratorHook(getList());
		accHook.add(addAction);
		accHook.add(editAction);
		accHook.add(removeAction);
	}
	
	/** Setzt einen vorhandenen Pfad zum Editieren. 
	 * Alle Elemente ohne Anfhrungszeichen werden als Integer interpretiert,
	 * alle mit Anfhrungszeichen als String. */
	public void setSubitemPath(String[] origPath) 
	{
		subitemPath.clear();
		
		for(int i = 0; i < origPath.length; ++i)
		{
			if( HTSParser.isQuoted(origPath[i]) )
				subitemPath.add( HTSParser.stripQuotes(origPath[i]) );
			else
				subitemPath.add( new Integer(origPath[i]) );
		}
		
		subitemLV.refresh();
	}

	/**
	 * Liefert die Elemente des Pfades als String oder Integer.
	 */
	public Vector getSubitemPath() {
		return subitemPath;
	}
	
	/**
	 * Liefert den Subelementpfad.
	 * @return Vector<String> Der Subelementpfad (OHNE Schlsselwort SUBITEM am Anfang)
	 * korrekt fr die Erstellung einer CSVLine aufbereitet, d.h. Strings stehen in
	 * Anfhrungszeichen und Zahlen nicht.
	 */
	public Vector getSubitemPathForLine() {
		Vector ret = new Vector( subitemPath.size() );
		if( !subitemPath.isEmpty() )
		{
			for(int i = 0; i < subitemPath.size(); ++i)
			{
				Object pathElement = subitemPath.get(i);
				if(pathElement instanceof String)
					ret.add('"' + (String)pathElement + '"');
				else // Integer
					ret.add( pathElement.toString() );
			}
		}
		
		return ret;
	}
	
	/** Liefert die Liste zur Darstellung des Subelementpfades */
	public List getList() {
		return subitemLV.getList();
	}
}
