/*
 * Created on 27.02.2006
 */
package atosj.dialog.addcmd;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;

/**
 * Setzt die Sichtbarkeit eines Controls. Mit der Mglichkeit die Hintergrundfarbe 
 * bei Sichtbarkeit auf ein von hellblau zu origanal zu faden. Ebenso
 * wird die Gre des Controls bei Sichtbarkeit auf 0 gesetzt.
 * @author Nicos Tegos
 */
public class VisibilityChanger 
{
	private Control ctrl;
	private Fader fader;
	private GridData visibleGd;
	private GridData invisibleGd;
	
	/**
	 * Erstellt einen neuen <code>VisibilityChanger</code>.
	 * @param ctrl Das Control dessen Sichtbarkeit gendert werden soll.
	 * <code>LayoutData</code> dieses Controls <em>MUSS<em> ein <code>GridData</code> Objekt sein.
	 * @param shallFade Gibt an, ob bei nderung auf Sichtbarkeit ein Farbeffekt
	 * zu sehen sein soll.
	 */
	public VisibilityChanger(Control ctrl, boolean shallFade) {
		super();
		
		this.ctrl = ctrl;
		if(shallFade)
			fader = new Fader(ctrl, 100, 10);
		
		visibleGd = (GridData) ctrl.getLayoutData();
		invisibleGd = new GridData();
		invisibleGd.minimumHeight = invisibleGd.heightHint = 0;
		if(visibleGd != null) 
		{
			invisibleGd.widthHint = visibleGd.widthHint;
			invisibleGd.minimumWidth = visibleGd.minimumWidth;
		}
	}

	 /**
	 * Setzt die Sichtbarkeit des Controls. Die Hintergrundfarbe wird 
	 * bei Sichtbarkeit ein gefadet, falls dies im Konstruktor angegeben 
	 * wurde. Die Gre des Controls bei Sichtbarkeit aus auf 0 gesetzt.
	 * Danach muss die Methode <code>layout</code> des Controls oder
	 * eines seiner Vter gerufen werden, damit die Grssennderung umgesetzt wird.
	 **/
	public void setVisible(boolean visible) 
	{
		if(ctrl.getVisible() != visible) 
		{
			ctrl.setVisible(visible);
			
			if(visible)
			{
				ctrl.setLayoutData(visibleGd);
				if(fader != null)
					fader.start();
			}
			else
				ctrl.setLayoutData(invisibleGd);
		}
	}
}
