/*
 * Created on 24.01.2006
 */
package atosj.dialog.addcmd;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Dient zur Eingabe der Parameter fr das WINDOWEXISTS Kommando.
 * @author Nicos Tegos
 *  @see atosj.model.hts.command.HTSWindowExists
 */
public class WindowExistsSection extends CommandSection {

	private Text windowTxt;
	private ComboViewer cv;
	
	private static final String[] answers = { HTSSyntax.WEX_YES, HTSSyntax.WEX_NO };
	
	public WindowExistsSection(Composite parent) {
		super(parent);
		
		createContents();
		fillCombos();
	}
	
	private void createContents() {
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("WindowExistsSection.0")); //$NON-NLS-1$
		windowTxt = new Text(this, SWT.BORDER);
		windowTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		cv = createComboViewer(Messages.getString("WindowExistsSection.1")); //$NON-NLS-1$
	}
	
	private void fillCombos() {
		cv.setInput(answers);
	}
	
	public void setOrigParameters(String[] parameters) 
	{
		windowTxt.setText( HTSParser.stripQuotes(parameters[0]) );
		cv.setSelection( new StructuredSelection(parameters[1]) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() {
		Vector ret = new Vector(2);
		
		ret.add( '"' + windowTxt.getText() + '"');
		ret.add( getSelectedElement(cv) );
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() {
		if( windowTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("WindowExistsSection.3"), Messages.getString("WindowExistsSection.4")); //$NON-NLS-1$ //$NON-NLS-2$
			windowTxt.setFocus();
			return false;
		}
		
		if( getSelectedElement(cv).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("WindowExistsSection.6"), Messages.getString("WindowExistsSection.7")); //$NON-NLS-1$ //$NON-NLS-2$
			cv.getCombo().setFocus();
			return false;
		}
		
		return true;
	}

}
