/*
 * Created on 05.01.2006
 */
package atosj.dialog.addcmd.action;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.dialog.addcmd.CommandSection;
import atosj.dialog.addcmd.EmptySpecificationWidget;
import atosj.dialog.addcmd.SubItemWidget;
import atosj.dialog.addcmd.VisibilityChanger;
import atosj.editors.urf.ClassNameComparator;
import atosj.editors.urf.IdTableLabelProvider;
import atosj.editors.urf.NameComparator;
import atosj.editors.urf.WindowNameComparator;
import atosj.gui.TableResizeListener;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.util.UniversalViewerSorter;

/**
 * Dient der Bearbeitung und Erzeugung eines ACTION Kommandos.
 * @author Nicos Tegos
 */
public class ActionSection extends CommandSection 
{
	/**
	 * Gibt alle gltigen Aktionen zu einem bestimmten Elementtyp
	 * zurck.
	 */
	private static class ActionMap
	{
		private static HashMap typeToActions; // HashMap<String, String[]>
		private static HashMap subtypeToActions; // HashMap<String, String[]>
		
		static
		{
			typeToActions = new HashMap();
			subtypeToActions = new HashMap();
			
			// COMPONENT
			String[] actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_COMPONENT, actions);
			
			// BUTTON
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT};
			typeToActions.put(HTSSyntax.TYPE_BUTTON, actions);
			
			// CHECKBOX
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK };
			typeToActions.put(HTSSyntax.TYPE_CHECKBOX, actions);
			
			// COMBOBOX
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_EDIT };
			typeToActions.put(HTSSyntax.TYPE_COMBOBOX, actions);
			String[] subactions = new String[] { HTSSyntax.ACTION_SELECT };
			subtypeToActions.put(HTSSyntax.TYPE_COMBOBOX, subactions);
			
			// EDITBOX
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_EDIT};
			typeToActions.put(HTSSyntax.TYPE_EDITBOX, actions);
			
			// SPINNER
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_EDIT };
			typeToActions.put(HTSSyntax.TYPE_SPINNER, actions);
			
			// LABEL
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_LABEL, actions);
			
			// LIST
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_LIST, actions);
			subactions = new String[] { HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT };
			subtypeToActions.put(HTSSyntax.TYPE_LIST, subactions);
			
			// MENU
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_MENU, actions);
			subactions = new String[] { HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK };
			subtypeToActions.put(HTSSyntax.TYPE_MENU, subactions);
			
			// RADIOBUTTON
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK };
			typeToActions.put(HTSSyntax.TYPE_RADIOBUTTON, actions);
			
			// TABFOLDER
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_TABFOLDER, actions);
			subactions = new String[] { HTSSyntax.ACTION_SELECT };
			subtypeToActions.put(HTSSyntax.TYPE_TABFOLDER, subactions);
			
			// TABLE
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_TABLE, actions);
			subactions = new String[] { HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK, HTSSyntax.ACTION_EDIT };
			subtypeToActions.put(HTSSyntax.TYPE_TABLE, subactions);
			
			// TREE
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY };
			typeToActions.put(HTSSyntax.TYPE_TREE, actions);
			subactions = new String[] { HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT, HTSSyntax.ACTION_EDIT };
			subtypeToActions.put(HTSSyntax.TYPE_TREE, subactions);
			
			// WINDOW
			actions = new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_MAXIMIZE, HTSSyntax.ACTION_MINIMIZE, HTSSyntax.ACTION_NORMALIZE, HTSSyntax.ACTION_CLOSE };
			typeToActions.put(HTSSyntax.TYPE_WINDOW, actions);
		}
		
		/**
		 * Liefert alle Aktionsarten fr einen bestimmten Componenttyp.
		 * @param elementType Der Typ des Elements, siehe HTSSyntax.TYPE_* Konstanten.
		 * @param subtype Gibt an, ob die Atkionsarten fr das Component 
		 * selbst oder fr dessen Unterelemente gefragt sind.
		 * @return Alle Aktionsarten, die fr diesen Componenttyp bzw. Subtyp, zugelassen sind, siehe
		 * Konstanten in der HTSSyntax.
		 * @see HTSSyntax
		 */
		public static String[] getActions(String elementType, boolean subtype) 
		{
			if(subtype)
			{
				String[] actions = (String[]) subtypeToActions.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
			else
			{
				if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCustomType(elementType) )
					return new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_PROPERTY };
				
				String[] actions = (String[]) typeToActions.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
		}
	}

	/**
	 * Gibt an, welche Parametertypen fr den Subelementpfad zusammengesetzter
	 * Components gltig sind.
	 */
	private static class PathTypeMap
	{
		public static final int PATH_TYPE_NUM = 1; // die Pfadelemente mssen natrliche Zahlen >=0 sein 
		public static final int PATH_TYPE_STR = 2; // die Pfadelemente mssen Zeichenketten sein
		public static final int PATH_TYPE_NUM_OR_STR = 3; // die Pfadelemente mssen knnen beides sein
		
		private static final Integer P_T_N = new Integer(PATH_TYPE_NUM);
		private static final Integer P_T_S = new Integer(PATH_TYPE_STR);
		private static final Integer P_T_NS = new Integer(PATH_TYPE_NUM_OR_STR);
		
		public static final int PATH_LEN_INFINITE = -1;
		
		private static final Integer P_L_I = new Integer(PATH_LEN_INFINITE);
		private static final Integer P_L_1 = new Integer(1);
		private static final Integer P_L_2 = new Integer(2);
		
		private static Object[][] map = new Object[][]
		{
			{HTSSyntax.TYPE_COMBOBOX, HTSSyntax.ACTION_SELECT, P_T_NS, P_L_1},
			{HTSSyntax.TYPE_LIST, HTSSyntax.ACTION_SELECT, P_T_NS, P_L_1},
			{HTSSyntax.TYPE_LIST, HTSSyntax.ACTION_ADDSELECT, P_T_NS, P_L_1},
			{HTSSyntax.TYPE_LIST, HTSSyntax.ACTION_DESELECT, P_T_NS, P_L_1},
			{HTSSyntax.TYPE_MENU, HTSSyntax.ACTION_SELECT, P_T_S, P_L_I},
			{HTSSyntax.TYPE_MENU, HTSSyntax.ACTION_CHECK, P_T_S, P_L_I},
			{HTSSyntax.TYPE_MENU, HTSSyntax.ACTION_UNCHECK, P_T_S, P_L_I},
			{HTSSyntax.TYPE_TABFOLDER, HTSSyntax.ACTION_SELECT, P_T_NS, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_SELECT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_ADDSELECT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_DESELECT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_CHECK, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_UNCHECK, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.ACTION_EDIT, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TREE, HTSSyntax.ACTION_EDIT, P_T_S, P_L_I},
		};
		
		/**
		 * berprft die Gltigkeit des Subelementpfades bei angegebenem Componenttyp
		 * und Aktionsart.
		 * @param type Der Typ des zusammengesetzten Components.
		 * @param action Die Aktionsart, siehe HTSSyntax.
		 * @param subitemPath Vector(String oder Integer) Die Elemente des Subelementpfades.
		 * @return <code>null</code> fr korrekt, Fehlernachricht sonst.
		 */
		public static String checkPath(String type, String action, Vector subitemPath)
		{
			for(int i = 0; i < map.length; ++i)
			{
				if( map[i][0].equals(type) && map[i][1].equals(action) )
				{
					int p_type = ((Integer)map[i][2]).intValue();
					int p_len = ((Integer)map[i][3]).intValue();
					
					if( subitemPath.size() == 0 )
						return Messages.getString("ActionSection.0"); //$NON-NLS-1$
					
					if( p_len != PATH_LEN_INFINITE )
					{
						if( p_len < subitemPath.size() || p_len > subitemPath.size() )
							return Messages.getString("ActionSection.1") + p_len + Messages.getString("ActionSection.2"); //$NON-NLS-1$ //$NON-NLS-2$
					}
					
					if(p_type == PATH_TYPE_NUM_OR_STR)
						return null;
					
					for(int j = 0; j < subitemPath.size(); ++j)
					{
						int currentType = (subitemPath.get(j) instanceof Integer) ? PATH_TYPE_NUM : PATH_TYPE_STR;
						if( (currentType & p_type) == 0 )
							return Messages.getString("ActionSection.3") + (p_type == PATH_TYPE_NUM ? Messages.getString("ActionSection.4") : Messages.getString("ActionSection.5"));  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
				}
			}
			
			return null;
		}
	}
	
/////////////////////////////////private classes end /////////////////////	
	
	private TableViewer idTable;
	
	private Button subitemChk;
	private SubItemWidget subitemWidget;
	private VisibilityChanger subitemChkVis;
	private VisibilityChanger subitemChkLblVis;
	private VisibilityChanger subitemLblVis;
	private VisibilityChanger subitemWidgetVis;
	private VisibilityChanger subitemListVis;
	
	private ComboViewer actionCV;
	
	private Composite specificationArea;
	private StackLayout specificationStack;
	private EditSpecificationWidget editSpecWidget;
	private ClickSpecificationWidget clickSpecWidget;
	private PropertySpecWidget propSpecWidget;
	private PresskeySpecWidget pressKeySpecWidget;
	private EmptySpecificationWidget emptySpecWidget;
	
	private static final String[] EMPTY_ACTIONS = new String[0];
	
	/**
	 * Erzeugt Dialogelemente zum Erstellen/Bearbeiten des Kommandos ACTION.
	 * @param parent Das graphische Vaterelement.
	 */
	public ActionSection(Composite parent) {
		super(parent);

		createContent();
		fillCombos();
	}

	/**
	 * Erstellt alle graph. Elemente.
	 */
	private void createContent() 
	{
		createTableArea();
		createSubitemArea();
		createSpecificationArea();
	}
	
	/**
	 * Erstellt die Tabelle zur Auswahl der ComponentId.
	 * @see ComponentId
	 */
	private void createTableArea()
	{
		Label lbl = new Label(this, SWT.NULL);
		lbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		lbl.setText(Messages.getString("ActionSection.6")); //$NON-NLS-1$
		
		idTable = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION);
		idTable.setContentProvider( new ArrayContentProvider() );
		idTable.setLabelProvider( new IdTableLabelProvider() );
		idTable.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				onIdTableSelection(e);
			}
			
		});
				
		Table table = idTable.getTable();
		new TableResizeListener(table, new int[] {25, 25, 25}, 20);
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		table.setLayoutData(gd);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ActionSection.7")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ActionSection.8")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ActionSection.9")); //$NON-NLS-1$
		
		UniversalViewerSorter.setColumnSorters(idTable, 
				new Comparator[] 
				{
					new WindowNameComparator(),
					new NameComparator(),
					new ClassNameComparator()
				},
				0);
	}
	
	/**
	 * Erstellt die Tabelle zur Eingabe des Subelementpfades.
	 */
	private void createSubitemArea() 
	{
		Label subitemChkLbl = new Label(this, SWT.NULL);
		subitemChkLbl.setText(Messages.getString("ActionSection.10")); //$NON-NLS-1$
		subitemChkLblVis = new VisibilityChanger(subitemChkLbl, false);
		subitemChkLblVis.setVisible(false);
		
		subitemChk = new Button(this, SWT.CHECK);
		subitemChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				onSubitemChkSelection();
			}
		});
		subitemChkVis = new VisibilityChanger(subitemChk, false);
		subitemChkVis.setVisible(false);
		
		// Listbox zur Angabe der Subitems
		Label subitemLbl = new Label(this, SWT.NULL);
		subitemLbl.setText(Messages.getString("ActionSection.11")); //$NON-NLS-1$
		subitemLbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		subitemLblVis = new VisibilityChanger(subitemLbl, false);
		subitemLblVis.setVisible(false);
		
		subitemWidget = new SubItemWidget(this);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		subitemWidget.setLayoutData(gd);
		subitemWidgetVis = new VisibilityChanger(subitemWidget, false);
		subitemWidgetVis.setVisible(false);
		// fr die Liste einen extra mit fader
		subitemListVis = new VisibilityChanger(subitemWidget.getList(), true);
		
		// Combo zur aktionsauswahl
		actionCV = createComboViewer(Messages.getString("ActionSection.12")); //$NON-NLS-1$
		actionCV.getCombo().setVisibleItemCount(8);
		actionCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				onActionSelection(e);
			}
		});
	}
	
	/**
	 * Erstellt ein Stacklayout fr die Eingabe von
	 * Parametern fr die verschiedenen Aktionsarten.
	 */
	private void createSpecificationArea() 
	{
		new Composite(this, SWT.NULL); // spacer
		specificationArea = new Composite(this, SWT.NULL);
		specificationArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		specificationStack = new StackLayout();
		specificationStack.marginHeight = specificationStack.marginWidth = 0;
		specificationArea.setLayout(specificationStack);
		
		editSpecWidget = new EditSpecificationWidget(specificationArea);
		editSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		clickSpecWidget = new ClickSpecificationWidget(specificationArea);
		clickSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		propSpecWidget = new PropertySpecWidget(specificationArea);
		propSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		pressKeySpecWidget = new PresskeySpecWidget(specificationArea);
		pressKeySpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		emptySpecWidget = new EmptySpecificationWidget(specificationArea);
		emptySpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		
		specificationStack.topControl = emptySpecWidget;
	}
	
	private void fillCombos() 
	{
		actionCV.setInput( new String[0] );
		idTable.setInput( AtosJApp.getInstance().getInput().getURFFile().getIds() );
	}
	
	/**
	 * Setzt die gltigen Aktionsarten
	 * anhand des Componenttyps der ausgewhlt wurde.
	 * @param e Auswahlereignis aus der ID Tabelle.
	 */
	private void onIdTableSelection(SelectionChangedEvent e)
	{
		if( !e.getSelection().isEmpty() )
		{
			ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
			boolean enabled = AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent( id.getClassName() );
			subitemChk.setSelection(enabled);
			
			subitemChkLblVis.setVisible(enabled);
			subitemChkVis.setVisible(enabled);
			subitemLblVis.setVisible( subitemChk.getSelection() );
			subitemWidgetVis.setVisible( subitemChk.getSelection()  );
			subitemListVis.setVisible( subitemChk.getSelection() );
			
			// preserve the selection
			ISelection sel = actionCV.getSelection();
			actionCV.setInput( ActionMap.getActions(id.getClassName(), subitemChk.getSelection()) );
			// try to set the preserved selection again
			actionCV.setSelection(sel);
		}
		else
		{
			subitemChkLblVis.setVisible(false);
			subitemChkVis.setVisible(false);
			subitemLblVis.setVisible(false);
			subitemWidgetVis.setVisible(false);
			subitemListVis.setVisible(false);
			
			actionCV.setInput(EMPTY_ACTIONS);
			// empty selection
			actionCV.setSelection( new StructuredSelection() );
		}
		
		layout();
	}
	
	/**
	 * Wird die Checkbox angewhlt, knnen nur Aktionen fr Unterelemente
	 * eingegeben werden und die Tabelle fr die Eingabe
	 * des Subelementpfades wird angezeigt, sonst knnen nur Aktionsarten fr das
	 * zusammengesetzte Component selbst angegeben werden und Tabelle wird ausgeblendet.
	 */
	private void onSubitemChkSelection() 
	{
		boolean selected = subitemChk.getSelection();
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		// preserve the selection TODO null pointer bei unbekannter id
		ISelection sel = actionCV.getSelection();
		actionCV.setInput( ActionMap.getActions(id.getClassName(), selected) );
		// try to set the preserved selection again
		actionCV.setSelection(sel);
		
		subitemLblVis.setVisible(selected);
		subitemWidgetVis.setVisible(selected);
		subitemListVis.setVisible(selected);
		
		layout();
	}
	
	/**
	 * Wird eine bestimmte Aktionsart ausgewhlt, wird
	 * dazu das entsprechende Widget zur Eingabe der Aktionsparameter angezeigt.
	 * @param e Auswahl Ereignis aus der Combobox fr die Aktionsart.
	 */
	private void onActionSelection(SelectionChangedEvent e)
	{
		if( !e.getSelection().isEmpty() )
		{
			String action = (String) ((StructuredSelection)e.getSelection()).getFirstElement();
			if( action.equals( HTSSyntax.ACTION_EDIT) )
			{
				specificationStack.topControl = editSpecWidget;
				specificationArea.layout();
			}
			else
			if( action.equals( HTSSyntax.ACTION_CLICK) )
			{
				specificationStack.topControl = clickSpecWidget;
				specificationArea.layout();
			}
			else
			if( action.equals( HTSSyntax.ACTION_PRESSKEY) )
			{
				specificationStack.topControl = pressKeySpecWidget;
				specificationArea.layout();
			}
			else
			if( action.equals( HTSSyntax.ACTION_PROPERTY) )
			{
				specificationStack.topControl = propSpecWidget;
				specificationArea.layout();
			}
			else
			{
				specificationStack.topControl = emptySpecWidget;
				specificationArea.layout();
			}
		}
		else
		{
			specificationStack.topControl = emptySpecWidget;
			specificationArea.layout();
		}
		
		layout();
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.util.Vector)
	 */
	public void setOrigParameters(String[] parameters) 
	{
		ComponentId id = new ComponentId( 
			HTSParser.stripQuotes(parameters[2]), 
			parameters[1], 
			HTSParser.stripQuotes(parameters[0])
			);
		
		idTable.setSelection( new StructuredSelection(id), true );
		
		String action = parameters[3];
		int possibleSubitemIndex = 4; // mglicher index des Schlsselwortes SUBITEM
		
		if( action.equals(HTSSyntax.ACTION_EDIT) )
		{
			editSpecWidget.setOrigParameters(new String[]{parameters[4]});
			possibleSubitemIndex = 5;
		}
		else
		if( action.equals(HTSSyntax.ACTION_PROPERTY) )
		{
			propSpecWidget.setOrigParameters(new String[]{parameters[4], parameters[5]});
			possibleSubitemIndex = 6;
		}
		else
		if( action.equals(HTSSyntax.ACTION_PRESSKEY) )
		{
			editSpecWidget.setOrigParameters(new String[]{parameters[4]});
			Vector params = new Vector();
			params.add( parameters[4] ); // Zeichen
			int modifierIndex = 5;
			
			while(modifierIndex < parameters.length)
			{
				String modifier = parameters[modifierIndex];
				if(modifier.equals(HTSSyntax.ACTION_ALT)
						|| modifier.equals(HTSSyntax.ACTION_CTRL)
						|| modifier.equals(HTSSyntax.ACTION_SHIFT))
				{
					params.add(modifier);
					++modifierIndex;
				}
				else
					break;
			}
			
			pressKeySpecWidget.setOrigParameters( (String[]) params.toArray( new String[params.size()] ));
			possibleSubitemIndex = modifierIndex;
		}
		else
		if( action.equals(HTSSyntax.ACTION_CLICK) )
		{
			Vector params = new Vector();
			params.add( parameters[4] ); // Mausbutton
			params.add( parameters[5] ); // # Clicks
			int modifierIndex = 6;
			
			while(modifierIndex < parameters.length)
			{
				String modifier = parameters[modifierIndex];
				if(modifier.equals(HTSSyntax.ACTION_ALT)
						|| modifier.equals(HTSSyntax.ACTION_CTRL)
						|| modifier.equals(HTSSyntax.ACTION_SHIFT))
				{
					params.add(modifier);
					++modifierIndex;
				}
				else
					break;
			}
			
			clickSpecWidget.setOrigParameters( (String[]) params.toArray( new String[params.size()] ));
			possibleSubitemIndex = modifierIndex;
		}
		
		if(possibleSubitemIndex < parameters.length)
		{
			Vector subitemPath = new Vector();
			for(int i = possibleSubitemIndex +1; i < parameters.length; ++i)
				subitemPath.add(parameters[i]);
			
			subitemWidget.setSubitemPath( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
			
			subitemChk.setSelection(true);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
			
		}
		else
		{
			subitemChk.setSelection(false);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
		}
		// na ja die Reihenfolge mag merkwrdig anmuten, ist aber nicht anders zu beheben,
		// denn der Inhalt der actionCV wird erst richtig gesetzt, wenn das Hckchen
		// bei subitemChk richtig gesetzt wurde und das geht erst nach dem Parsen eines
		// evtl. Subitempfades.
		actionCV.setSelection( new StructuredSelection(action) );
	}
	
	/**
	 * Liefert den Abschnitt fr die Eingabe der Parameter
	 * der aktuell gewhlten Aktionsart.
	 */
	private CommandSection getSpecWidget() {
		return (CommandSection) specificationStack.topControl;
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(16);
		
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		parameters.add( '"' + id.getWindowName() + '"' );
		parameters.add( id.getClassName() );
		parameters.add( '"' + id.getName() + '"' );
		parameters.add( getSelectedElement(actionCV) );
		parameters.addAll( getSpecWidget().getCommandParameters() );
				
		if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent(id.getClassName()) 
				&& subitemChk.getSelection() )
		{
			parameters.add("SUBITEM"); //$NON-NLS-1$
			parameters.addAll( subitemWidget.getSubitemPathForLine() );
		}
		
		return parameters;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( idTable.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("ActionSection.14"), Messages.getString("ActionSection.15")); //$NON-NLS-1$ //$NON-NLS-2$
			idTable.getTable().setFocus();
			return false;
		}
		
		if( actionCV.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("ActionSection.16"), Messages.getString("ActionSection.17")); //$NON-NLS-1$ //$NON-NLS-2$
			actionCV.getCombo().setFocus();
			return false;
		}
		
		if( subitemChk.getSelection() )
		{
			Vector subitemPath = subitemWidget.getSubitemPath();
			String type = ((ComponentId)((StructuredSelection)idTable.getSelection()).getFirstElement()).getClassName();
			String action = (String) ((StructuredSelection)actionCV.getSelection()).getFirstElement();
			String result = PathTypeMap.checkPath(type, action, subitemPath);
			if(result != null)
			{
				subitemWidget.getList().setFocus();
				MessageDialog.openError(null, Messages.getString("ActionSection.18"), result); //$NON-NLS-1$
				return false;
			}
		}
		
		if( !getSpecWidget().checkIntegrity() )
			return false;
		
		return true;
	}
}
