/*
 * Created on 24.05.2006
 */
package atosj.dialog.addcmd.action;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe von Parametern der Aktionsart CLICK.
 * @author Nicos Tegos
 */
class ClickSpecificationWidget extends CommandSection 
{
	private static final String[] buttons = new String[] { HTSSyntax.ACTION_LEFT, HTSSyntax.ACTION_RIGHT };
	
	private Text clicksTxt;
	private ComboViewer buttonCV;
	private Button altChk, ctrlChk, shiftChk;
	
	/**
	 * Erstellt ein neues Widget zur Eingabe von Parametern der Aktionsart CLICK.
	 * @param parent Das graphische Vaterelement.
	 */
	public ClickSpecificationWidget(Composite parent) 
	{
		super(parent);
				
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("ClickSpecificationWidget.0")); //$NON-NLS-1$
		clicksTxt = new Text(this, SWT.BORDER);
		clicksTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		buttonCV = createComboViewer(Messages.getString("ClickSpecificationWidget.1")); //$NON-NLS-1$
		buttonCV.setInput(buttons);
		
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("ClickSpecificationWidget.2")); //$NON-NLS-1$
		
		Composite modifierArea = new Composite(this, SWT.NULL);
		modifierArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(3, false);
		gl.marginHeight = gl.marginWidth = 0;
		modifierArea.setLayout(gl);
		
		ctrlChk = new Button(modifierArea, SWT.CHECK);
		ctrlChk.setText(Messages.getString("ClickSpecificationWidget.3")); //$NON-NLS-1$
		shiftChk = new Button(modifierArea, SWT.CHECK);
		shiftChk.setText(Messages.getString("ClickSpecificationWidget.4")); //$NON-NLS-1$
		altChk = new Button(modifierArea, SWT.CHECK);
		altChk.setText(Messages.getString("ClickSpecificationWidget.5")); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		buttonCV.setSelection( new StructuredSelection(parameters[0]) );
		clicksTxt.setText( parameters[1] );
		for(int i = 2; i < parameters.length; ++i)
		{
			if( parameters[i].equals(HTSSyntax.ACTION_ALT) )
			{
				altChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.ACTION_SHIFT) )
			{
				shiftChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.ACTION_CTRL) )
			{
				ctrlChk.setSelection(true);
			}
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		try
		{
			int value = Integer.parseInt( clicksTxt.getText() );
			if(value <= 0)
			{
				clicksTxt.setFocus();
				MessageDialog.openError(null, Messages.getString("ClickSpecificationWidget.6"), Messages.getString("ClickSpecificationWidget.7")); //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}
		catch(NumberFormatException e) 
		{
			clicksTxt.setFocus();
			MessageDialog.openError(null, Messages.getString("ClickSpecificationWidget.8"), Messages.getString("ClickSpecificationWidget.9")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if( buttonCV.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("ClickSpecificationWidget.10"), Messages.getString("ClickSpecificationWidget.11")); //$NON-NLS-1$ //$NON-NLS-2$
			buttonCV.getCombo().setFocus();
			return false;
		}
		
		return true;
	}

	public Vector getCommandParameters() 
	{
		Vector ret = new Vector(5);
		ret.add( ((StructuredSelection)buttonCV.getSelection()).getFirstElement() );
		ret.add( clicksTxt.getText() );
		if(ctrlChk.getSelection())
			ret.add(HTSSyntax.ACTION_CTRL);
		if(shiftChk.getSelection())
			ret.add(HTSSyntax.ACTION_SHIFT);
		if(altChk.getSelection())
			ret.add(HTSSyntax.ACTION_ALT);
		
		return ret;
	}
}
