package atosj.dialog.addcmd.action;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe von Parametern der Aktionsart PRESSKEY.
 * Enthlt ein Textfeld zur Eingabe des Zeichens und Checkboxen
 * fr die Modifier.
 * @author nicos
 */
public class PresskeySpecWidget extends CommandSection {

	private Text charTxt;
	private Button altChk, ctrlChk, shiftChk;
	
	/**
	 * Erstellt ein neues Widget zur Eingabe von Parametern der Aktionsart PRESSKEY.
	 * @param parent Das graph. Vaterelement.
	 */
	public PresskeySpecWidget(Composite parent) 
	{
		super(parent);
				
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("PresskeySpecWidget.0")); //$NON-NLS-1$
		charTxt = new Text(this, SWT.BORDER);
		// nur ein zeichen darf angegeben werden
		charTxt.setTextLimit(1);
		
		lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("PresskeySpecWidget.1")); //$NON-NLS-1$
		
		Composite modifierArea = new Composite(this, SWT.NULL);
		modifierArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(3, false);
		gl.marginHeight = gl.marginWidth = 0;
		modifierArea.setLayout(gl);
		
		ctrlChk = new Button(modifierArea, SWT.CHECK);
		ctrlChk.setText(Messages.getString("PresskeySpecWidget.2")); //$NON-NLS-1$
		shiftChk = new Button(modifierArea, SWT.CHECK);
		shiftChk.setText(Messages.getString("PresskeySpecWidget.3")); //$NON-NLS-1$
		altChk = new Button(modifierArea, SWT.CHECK);
		altChk.setText(Messages.getString("PresskeySpecWidget.4")); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		charTxt.setText( HTSParser.stripQuotes(parameters[0]) );
		for(int i = 1; i < parameters.length; ++i)
		{
			if( parameters[i].equals(HTSSyntax.ACTION_ALT) )
			{
				altChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.ACTION_SHIFT) )
			{
				shiftChk.setSelection(true);
			}
			else
			if( parameters[i].equals(HTSSyntax.ACTION_CTRL) )
			{
				ctrlChk.setSelection(true);
			}
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( charTxt.getText().length() != 1)
		{
			charTxt.setFocus();
			MessageDialog.openError(null, Messages.getString("PresskeySpecWidget.5"), Messages.getString("PresskeySpecWidget.6")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		return true;
	}

	public Vector getCommandParameters() 
	{
		Vector ret = new Vector(4);
		ret.add( '"' + charTxt.getText() + '"' );
		if(ctrlChk.getSelection())
			ret.add(HTSSyntax.ACTION_CTRL);
		if(shiftChk.getSelection())
			ret.add(HTSSyntax.ACTION_SHIFT);
		if(altChk.getSelection())
			ret.add(HTSSyntax.ACTION_ALT);
		
		return ret;
	}
}

