/*
 * Created on 24.01.2006
 */
package atosj.dialog.addcmd.test;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.dialog.addcmd.EmptySpecificationWidget;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe der Parameter fr das Kommando COMPARE.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSCompare
 */
public class CompareSection extends CommandSection 
{
	private Text variableNameTxt;
	private ComboViewer typeCV;
	private ComboViewer modeCV;
	
	private Composite specificationArea;
	private StackLayout specificationStack;
	private TextSpecificationWidget textSpecWidget;
	private BoolSpecificationWidget boolSpecWidget;
	private NumSpecificationWidget numSpecWidget;
	private VarSpecificationWidget varSpecWidget;
	private VarNumSpecificationWidget varNumSpecWidget;
	private EmptySpecificationWidget emptySpecWidget; 
		
	private static final String modes[] =  { HTSSyntax.COMPARE_BOOL, HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_STR };
	private static final String types[] = { HTSSyntax.COMPARE_VAL, HTSSyntax.COMPARE_VAR }; 
	
	/**
	 * Erstellt ein neues Widget zur Eingabe der Parameter fr das Kommando COMPARE.
	 * @param parent Das graph. Vaterelement.
	 */
	public CompareSection(Composite parent) {
		super(parent);
		
		createContent();
		fillCombos();
	}

	private void createContent() 
	{
		Label lbl = new Label(this, SWT.NULL);
		lbl.setText(Messages.getString("CompareSection.0")); //$NON-NLS-1$
		variableNameTxt = new Text(this, SWT.BORDER);
		variableNameTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		modeCV = createComboViewer(Messages.getString("CompareSection.1")); //$NON-NLS-1$
		modeCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onTypeOrModeCV();
			}
			
		});
		
		typeCV = createComboViewer(Messages.getString("CompareSection.2")); //$NON-NLS-1$
		typeCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onTypeOrModeCV();
			}
		}); 
		
		createSpecificationArea();
	}
	
	private void createSpecificationArea() 
	{
		new Composite(this, SWT.NULL); // spacer
		specificationArea = new Composite(this, SWT.NULL);
		specificationArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		specificationStack = new StackLayout();
		specificationStack.marginHeight = specificationStack.marginWidth = 0;
		specificationArea.setLayout(specificationStack);
		
		textSpecWidget = new TextSpecificationWidget(specificationArea);
		textSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		numSpecWidget = new NumSpecificationWidget(specificationArea);
		numSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		boolSpecWidget = new BoolSpecificationWidget(specificationArea);
		boolSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		varSpecWidget = new VarSpecificationWidget(specificationArea);
		varSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		varNumSpecWidget = new VarNumSpecificationWidget(specificationArea);
		varNumSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		emptySpecWidget = new EmptySpecificationWidget(specificationArea);
		emptySpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		
		specificationStack.topControl = emptySpecWidget;
	}
	
	private void fillCombos() {
		modeCV.setInput(modes);
		typeCV.setInput(types);
	}
	
	private void onTypeOrModeCV() 
	{
		String mode = getSelectedElement(modeCV);
		String type = getSelectedElement(typeCV);
		
		if( mode.equals(HTSSyntax.COMPARE_BOOL) )
		{
			if( type.equals(HTSSyntax.COMPARE_VAL) )
			{
				specificationStack.topControl = boolSpecWidget;
				specificationArea.layout();
			}
			else
			if( type.equals(HTSSyntax.COMPARE_VAR) )
			{
				specificationStack.topControl = varSpecWidget;
				specificationArea.layout();
			}
			else
			{
				specificationStack.topControl = emptySpecWidget;
				specificationArea.layout();
			}
		}
		else
		if( mode.equals(HTSSyntax.COMPARE_STR) )
		{
			if( type.equals(HTSSyntax.COMPARE_VAL) )
			{
				specificationStack.topControl = textSpecWidget;
				specificationArea.layout();
			}
			else
			if( type.equals(HTSSyntax.COMPARE_VAR) )
			{
				specificationStack.topControl = varSpecWidget;
				specificationArea.layout();
			}
			else
			{
				specificationStack.topControl = emptySpecWidget;
				specificationArea.layout();
			}
		}
		else
		if( mode.equals(HTSSyntax.COMPARE_NUM) )
		{
			if( type.equals(HTSSyntax.COMPARE_VAL) )
			{
				specificationStack.topControl = numSpecWidget;
				specificationArea.layout();
			}
			else
			if( type.equals(HTSSyntax.COMPARE_VAR) )
			{
				specificationStack.topControl = varNumSpecWidget;
				specificationArea.layout();
			}
			else
			{
				specificationStack.topControl = emptySpecWidget;
				specificationArea.layout();
			}
		}
		else
		{
			specificationStack.topControl = emptySpecWidget;
			specificationArea.layout();
		}
		
		layout();
	}
	
	private CommandSection getSpecWidget() {
		return (CommandSection) specificationStack.topControl;
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		variableNameTxt.setText( HTSParser.stripQuotes(parameters[0]) );
		modeCV.setSelection( new StructuredSelection(parameters[1]) );
		typeCV.setSelection( new StructuredSelection(parameters[2]) );
		Vector specParams = new Vector();
		for(int i = 3; i < parameters.length; ++i)
			specParams.add( parameters[i] );
		
		getSpecWidget().setOrigParameters( (String[]) specParams.toArray(new String[specParams.size()]) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(16);
		
		parameters.add( '"' + variableNameTxt.getText() + '"');
		parameters.add( getSelectedElement(modeCV) );
		parameters.add( getSelectedElement(typeCV) );
		parameters.addAll( getSpecWidget().getCommandParameters() );
		
		return parameters;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( variableNameTxt.getText().equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CompareSection.4"), Messages.getString("CompareSection.5")); //$NON-NLS-1$ //$NON-NLS-2$
			variableNameTxt.setFocus();
			return false;
		}
		
		if( getSelectedElement(modeCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CompareSection.7"), Messages.getString("CompareSection.8")); //$NON-NLS-1$ //$NON-NLS-2$
			modeCV.getCombo().setFocus();
			return false;
		}
		
		if( getSelectedElement(typeCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("CompareSection.10"), Messages.getString("CompareSection.11")); //$NON-NLS-1$ //$NON-NLS-2$
			typeCV.getCombo().setFocus();
			return false;
		}
		
		if( !getSpecWidget().checkIntegrity() )
		{
			return false;
		}
		
		return true;
	}

}
