package atosj.dialog.addcmd.test;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe der Parameter zur Prfung der Elementanzahl (ITEMCOUNT).
 * @author Nicos Tegos
 */
public class ItemcountSpecificationWidget extends CommandSection 
{
	private String operators[] = new String[] { HTSSyntax.COMPARE_EQ, HTSSyntax.COMPARE_GEQ, HTSSyntax.COMPARE_GRT, HTSSyntax.COMPARE_LEQ,
			HTSSyntax.COMPARE_LSS, HTSSyntax.COMPARE_NEQ };
	
	private ComboViewer operatorCV;
	private Text valueTxt;
	
	/**
	 * Erstellt ein neues Widget zur Eingabe der Parameter zur Prfung der Elementanzahl (ITEMCOUNT).
	 * @param parent Das graph. Vaterelement.
	 */
	public ItemcountSpecificationWidget(Composite parent) {
		super(parent);
		
		// Sollwert
		Label valueLbl = new Label(this, SWT.NULL);
		valueLbl.setText(Messages.getString("ItemcountSpecificationWidget.0")); //$NON-NLS-1$
						
		valueTxt = new Text(this, SWT.BORDER);
		valueTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		valueTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				// bei numerischem vergleich nur zeichen zulassen, 
				// die fr eine natrliche zahl bentigt werden
				e.doit = Character.isDigit(e.character) 
				|| ( Character.isISOControl(e.character) && !Character.isWhitespace(e.character) );
			}
		});

		// Vergleichsoperator
		Label operatorLbl = new Label(this, SWT.NULL);
		operatorLbl.setText(Messages.getString("ItemcountSpecificationWidget.1")); //$NON-NLS-1$
				
		operatorCV = new ComboViewer(this, SWT.READ_ONLY);
		operatorCV.setContentProvider( new ArrayContentProvider() );
		operatorCV.setLabelProvider( new BaseLabelProvider() );
		operatorCV.getCombo().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		operatorCV.setInput(operators);
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		valueTxt.setText(parameters[0]);
		if(parameters.length > 1)
			operatorCV.setSelection( new StructuredSelection(parameters[1]) );
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		try{
			Integer.parseInt( valueTxt.getText() );
		}
		catch(NumberFormatException e) {
			valueTxt.setFocus();
			MessageDialog.openError(null, Messages.getString("ItemcountSpecificationWidget.2"), Messages.getString("ItemcountSpecificationWidget.3")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		return true;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#getParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(3);
		
		parameters.add( valueTxt.getText() );
				
		// implizites EQ (kein Operator)
		if( !getSelectedElement(operatorCV).equals("") ) //$NON-NLS-1$
			parameters.add( getSelectedElement(operatorCV) );
		
		return parameters;
	}

}
