/*
 * Created on 26.05.2006
 */
package atosj.dialog.addcmd.test;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.dialog.addcmd.VisibilityChanger;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe der Parameter fr den numerischen Vergleich (NUM).
 * @author Nicos Tegos
 */
class NumSpecificationWidget extends CommandSection 
{
	private String operators[] = new String[] { HTSSyntax.COMPARE_EQ, HTSSyntax.COMPARE_GEQ, HTSSyntax.COMPARE_GRT, HTSSyntax.COMPARE_LEQ,
			HTSSyntax.COMPARE_LSS, HTSSyntax.COMPARE_NEQ, HTSSyntax.COMPARE_TOL };
	
	private ComboViewer operatorCV;
		
	private Text valueTxt;
		
	private Text toleranceTxt;
	private VisibilityChanger toleranceTxtVis;
	private VisibilityChanger toleranceLblVis;
	
	/**
	 * Erstellt ein neues Widget zur Eingabe der Parameter fr den numerischen Vergleich (NUM).
	 * @param parent Das graph. Vaterelement.
	 */
	public NumSpecificationWidget(Composite parent) {
		super(parent);
		
		// Sollwert
		Label valueLbl = new Label(this, SWT.NULL);
		valueLbl.setText(Messages.getString("NumSpecificationWidget.0")); //$NON-NLS-1$
						
		valueTxt = new Text(this, SWT.BORDER);
		valueTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		valueTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				// bei numerischem vergleich nur zeichen zulassen, 
				// die fr eine reelle zahl bentigt werden
				e.doit = Character.isDigit(e.character) 
				|| ( Character.isISOControl(e.character) && !Character.isWhitespace(e.character) ) 
				|| e.character == '.' || e.character == '-';
			}
		});

		// Vergleichsoperator
		Label operatorLbl = new Label(this, SWT.NULL);
		operatorLbl.setText(Messages.getString("NumSpecificationWidget.1")); //$NON-NLS-1$
				
		operatorCV = new ComboViewer(this, SWT.READ_ONLY);
		operatorCV.setContentProvider( new ArrayContentProvider() );
		operatorCV.setLabelProvider( new BaseLabelProvider() );
		operatorCV.getCombo().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		operatorCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onOperatorCV();
			}
		});
		operatorCV.setInput(operators);
				
		// Toleranzwert nur sichtbar bei Operator TOL
		Label toleranceLbl = new Label(this, SWT.NULL);
		toleranceLbl.setText(Messages.getString("NumSpecificationWidget.2")); //$NON-NLS-1$
		toleranceLblVis = new VisibilityChanger(toleranceLbl, false);
		toleranceLblVis.setVisible(false);
				
		toleranceTxt = new Text(this, SWT.BORDER);
		toleranceTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		toleranceTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				e.doit = Character.isDigit(e.character) 
				|| Character.isISOControl(e.character) 
				|| e.character == '.' || e.character == '-';
			}
		});
		toleranceTxtVis = new VisibilityChanger(toleranceTxt, true);
		toleranceTxtVis.setVisible(false);
	}
	
	private void onOperatorCV() 
	{
		if( getSelectedElement(operatorCV).equals(HTSSyntax.COMPARE_TOL) )
		{
			// Toleranzeingabe nur bei Vergleichsoperator TOL sichtbar
			toleranceLblVis.setVisible(true);
			toleranceTxtVis.setVisible(true);
		}
		else
		{
			toleranceLblVis.setVisible(false);
			toleranceTxtVis.setVisible(false);
		}
		
		layout();
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		valueTxt.setText(parameters[0]);
		
		if(parameters.length > 1)
		{
			operatorCV.setSelection( new StructuredSelection(parameters[1]) );
			if(parameters.length > 2)
				toleranceTxt.setText(parameters[2]);
		}
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		try{
			Float.parseFloat( valueTxt.getText() );
		}
		catch(NumberFormatException e) {
			valueTxt.setFocus();
			MessageDialog.openError(null, Messages.getString("NumSpecificationWidget.3"), Messages.getString("NumSpecificationWidget.4")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if( getSelectedElement(operatorCV).equals(HTSSyntax.COMPARE_TOL) )
		{
			try{
				Float.parseFloat( toleranceTxt.getText() );
			}
			catch(NumberFormatException e) {
				toleranceTxt.setFocus();
				MessageDialog.openError(null, Messages.getString("NumSpecificationWidget.5"), Messages.getString("NumSpecificationWidget.6")); //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}
		
		return true;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.SpecificationWidget#getParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(3);
		
		parameters.add( valueTxt.getText() );
				
		// implizites EQ (kein Operator)
		String operator = getSelectedElement(operatorCV);
		if( operator.length() > 0 )
		{
			parameters.add(operator);
			
			if( operator.equals(HTSSyntax.COMPARE_TOL) )
				parameters.add( toleranceTxt.getText() );
		}
		
		return parameters;
	}

}
