package atosj.dialog.addcmd.test;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.dialog.addcmd.CommandSection;
import atosj.dialog.addcmd.EmptySpecificationWidget;
import atosj.dialog.addcmd.SubItemWidget;
import atosj.dialog.addcmd.VisibilityChanger;
import atosj.dialog.addcmd.action.PropertySpecWidget;
import atosj.editors.urf.ClassNameComparator;
import atosj.editors.urf.IdTableLabelProvider;
import atosj.editors.urf.NameComparator;
import atosj.editors.urf.WindowNameComparator;
import atosj.gui.TableResizeListener;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.util.UniversalViewerSorter;

/**
 * Widget zur Bearbeitung und Erzeugung eines TEST Kommandos.
 * @author nicos
 * @see atosj.model.hts.command.HTSTest
 */
public class TestSection extends CommandSection 
{
	/**
	 * Liefert alle gltigen Zustnde, die bei einem bestimmten Componenttyp
	 * ausgelesen werden knnen.
	 */
	private static class StateMap
	{
		private static HashMap typeToState; // HashMap<String, String[]>
		private static HashMap subtypeToState; // HashMap<String, String[]>
		
		static
		{
			typeToState = new HashMap();
			subtypeToState = new HashMap();
			
			// COMPONENT
			String[] states = new String[] { HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_COMPONENT, states);
			
			// BUTTON
			states = new String[] { HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_BUTTON, states);
			
			// CHECKBOX
			states = new String[] { HTSSyntax.TEST_CHECKSTATE, HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE};
			typeToState.put(HTSSyntax.TYPE_CHECKBOX, states);
			
			// COMBOBOX
			states = new String[] { HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_COMBOBOX, states);
			String[] substates = new String[] {};
			subtypeToState.put(HTSSyntax.TYPE_COMBOBOX, substates);
			
			// EDITBOX
			states = new String[] { HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_EDITBOX, states);
			
			// SPINNER
			states = new String[] { HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_SPINNER, states);
			
			// LABEL
			states = new String[] { HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_LABEL, states);
			
			// LIST
			states = new String[] { HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_LIST, states);
			// TODO vielleicht auch NUM einfhren in IList
			substates = new String[] { HTSSyntax.TEST_TEXT };
			subtypeToState.put(HTSSyntax.TYPE_LIST, substates);
			
			// MENU
			states = new String[] { HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_MENU, states);
			substates = new String[] { HTSSyntax.TEST_CHECKSTATE, HTSSyntax.TEST_ENABLESTATE };
			subtypeToState.put(HTSSyntax.TYPE_MENU, substates);
			
			// RADIOBUTTON
			states = new String[] { HTSSyntax.TEST_CHECKSTATE, HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_RADIOBUTTON, states);
			
			// TABFOLDER
			states = new String[] { HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TABFOLDER, states);
			substates = new String[] { HTSSyntax.TEST_TEXT };
			subtypeToState.put(HTSSyntax.TYPE_TABFOLDER, substates);
			
			// TABLE
			states = new String[] { HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TABLE, states);
			substates = new String[] { HTSSyntax.TEST_CHECKSTATE, HTSSyntax.TEST_NUM, HTSSyntax.TEST_TEXT, HTSSyntax.TEST_ENABLESTATE };
			subtypeToState.put(HTSSyntax.TYPE_TABLE, substates);
			
			// TREE
			states = new String[] { HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_TREE, states);
			substates = new String[] {};
			subtypeToState.put(HTSSyntax.TYPE_TREE, substates);
			
			// WINDOW
			states = new String[] { HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
			typeToState.put(HTSSyntax.TYPE_WINDOW, states);
		}
		
		/**
		 * Liefert alle Zustandsnamen, die beim angegebenen Componenttyp ausgelesen werden knnen.
		 * @param elementType Der Typ des Components, siehe HTSSyntax.TYPE_* Konstanten.
		 * @param subtype Gibt an, ob die Zustnde fr das Component 
		 * selbst oder fr dessen Unterelemente gefragt sind.
		 * @return Alle Zustnde, die fr diesen Componenttyp bzw. Subtyp, zugelassen sind, siehe
		 * Konstanten in der HTSSyntax.
		 * @see HTSSyntax
		 */
		public static String[] getStates(String elementType, boolean subtype) 
		{
			if(subtype)
			{
				String[] actions = (String[]) subtypeToState.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
			else
			{
				if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCustomType(elementType) )
					return new String[] { HTSSyntax.TEST_PROPERTY, HTSSyntax.TEST_ENABLESTATE, HTSSyntax.TEST_FOCUSSTATE, HTSSyntax.TEST_VISIBLESTATE };
					
				String[] actions = (String[]) typeToState.get(elementType);
				if(actions == null)
					return new String[0];
				return actions;
			}
		}
	}

	/**
	 * Gibt an, welche Parametertypen fr den Subelementpfad zusammengesetzter
	 * Components gltig sind.
	 */
	private static class PathTypeMap
	{
		public static final int PATH_TYPE_NUM = 1; // die Pfadelemente mssen natrliche Zahlen >=0 sein 
		public static final int PATH_TYPE_STR = 2; // die Pfadelemente mssen Zeichenketten sein
		public static final int PATH_TYPE_NUM_OR_STR = 3; // die Pfadelemente mssen knnen beides sein
		
		private static final Integer P_T_N = new Integer(PATH_TYPE_NUM);
		private static final Integer P_T_S = new Integer(PATH_TYPE_STR);
		private static final Integer P_T_NS = new Integer(PATH_TYPE_NUM_OR_STR);
		
		public static final int PATH_LEN_INFINITE = -1;
		
		private static final Integer P_L_I = new Integer(PATH_LEN_INFINITE);
		private static final Integer P_L_1 = new Integer(1);
		private static final Integer P_L_2 = new Integer(2);
		
		private static Object[][] map = new Object[][]
		{
			// TODO in IList noch getNum(index) einfhren?
			{HTSSyntax.TYPE_LIST, HTSSyntax.TEST_TEXT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_MENU, HTSSyntax.TEST_CHECKSTATE, P_T_S, P_L_I},
			{HTSSyntax.TYPE_MENU, HTSSyntax.TEST_ENABLESTATE, P_T_S, P_L_I},
			{HTSSyntax.TYPE_TABFOLDER, HTSSyntax.TEST_TEXT, P_T_N, P_L_1},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.TEST_CHECKSTATE, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.TEST_TEXT, P_T_N, P_L_2},
			{HTSSyntax.TYPE_TABLE, HTSSyntax.TEST_NUM, P_T_N, P_L_2}
			//{HTSSyntax.TYPE_TREE, HTSSyntax.TEST_TEXT, P_T_S, P_L_I}, sinnlos weil wir eh den kompletten pfad angeben mssen
			//{HTSSyntax.TYPE_TREE, HTSSyntax.TEST_NUM, P_T_S, P_L_I},
		};
		
		/**
		 * berprft die Gltigkeit des Subelementpfades bei angegebenem Componenttyp
		 * und Zustand.
		 * @param type Der Typ des zusammengesetzten Components.
		 * @param state Der Zustand, z.B. ENABLESTATE.
		 * @param subitemPath Vector(String oder Integer) Die Elemente des Subelementpfades.
		 * @return <code>null</code> fr korrekt, Fehlernachricht sonst.
		 */
		public static String checkPath(String type, String state, Vector subitemPath)
		{
			for(int i = 0; i < map.length; ++i)
			{
				if( map[i][0].equals(type) && map[i][1].equals(state) )
				{
					int p_type = ((Integer)map[i][2]).intValue();
					int p_len = ((Integer)map[i][3]).intValue();
					
					if( subitemPath.size() == 0 )
						return Messages.getString("TestSection.0"); //$NON-NLS-1$
					
					if( p_len != PATH_LEN_INFINITE )
					{
						if( p_len < subitemPath.size() || p_len > subitemPath.size() )
							return Messages.getString("TestSection.1") + p_len + Messages.getString("TestSection.2"); //$NON-NLS-1$ //$NON-NLS-2$
					}
					
					if(p_type == PATH_TYPE_NUM_OR_STR)
						return null;
					
					for(int j = 0; j < subitemPath.size(); ++j)
					{
						int currentType = (subitemPath.get(j) instanceof Integer) ? PATH_TYPE_NUM : PATH_TYPE_STR;
						if( (currentType & p_type) == 0 )
							return Messages.getString("TestSection.3") + (p_type == PATH_TYPE_NUM ? Messages.getString("TestSection.4") : Messages.getString("TestSection.5"));  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					}
				}
			}
			
			return null;
		}
	}

///////////////////////////////////// private classes end //////////////////////
	
	private TableViewer idTable;
	private ComboViewer stateCV;
		
	private Button subitemChk;
	private SubItemWidget subitemWidget;
	private VisibilityChanger subitemChkVis;
	private VisibilityChanger subitemChkLblVis;
	private VisibilityChanger subitemWidgetVis;
	private VisibilityChanger subitemLblVis;
	private VisibilityChanger subitemListVis;
	
	private Composite specificationArea;
	private StackLayout specificationStack;
	private TextSpecificationWidget textSpecWidget;
	private NumSpecificationWidget numSpecWidget;
	private ItemcountSpecificationWidget itemcountSpecWidget;
	private BoolSpecificationWidget boolSpecWidget;
	private PropertySpecWidget propSpecWidget;
	private EmptySpecificationWidget emptySpecWidget;
	
	private static final String[] EMPTY_STATES = new String[0];
	
	/**
	 * Erstellt ein neues Widget zur Bearbeitung und Erzeugung eines TEST Kommandos.
	 * @param parent Das graph. Vaterelement.
	 */
	public TestSection(Composite parent) {
		super(parent);
		
		createContent();
		fillCombos();
	}

	/**
	 * Erstellt alle graph. Elemente.
	 */
	private void createContent() 
	{
		createTableArea();
		
		Label subitemChkLbl = new Label(this, SWT.NULL);
		subitemChkLbl.setText(Messages.getString("TestSection.6")); //$NON-NLS-1$
		subitemChkLblVis = new VisibilityChanger(subitemChkLbl, false);
		subitemChkLblVis.setVisible(false);
		
		subitemChk = new Button(this, SWT.CHECK);
		subitemChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				onSubitemChkSelection();
			}
		});
		subitemChkVis = new VisibilityChanger(subitemChk, false);
		subitemChkVis.setVisible(false);
		
        // Angabe der Subitems
		Label subitemLbl = new Label(this, SWT.NULL);
		subitemLbl.setText(Messages.getString("TestSection.7")); //$NON-NLS-1$
		subitemLbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		subitemLblVis = new VisibilityChanger(subitemLbl, false);
		subitemLblVis.setVisible(false);
		
		subitemWidget = new SubItemWidget(this);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		subitemWidget.setLayoutData(gd);
		subitemWidgetVis = new VisibilityChanger(subitemWidget, false);
		subitemWidgetVis.setVisible(false);
		
		subitemListVis = new VisibilityChanger(subitemWidget.getList(), true);
		
		stateCV = createComboViewer(Messages.getString("TestSection.8")); //$NON-NLS-1$
		stateCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				String mode = getSelectedElement(stateCV);
				if( mode.equals(HTSSyntax.TEST_TEXT) )
				{
					specificationStack.topControl = textSpecWidget;
					specificationArea.layout();
				}
				else
				if( mode.equals(HTSSyntax.TEST_NUM) )
				{
					specificationStack.topControl = numSpecWidget;
					specificationArea.layout();
				}
				else
				if( mode.equals(HTSSyntax.TEST_ITEMCOUNT) )
				{
					specificationStack.topControl = itemcountSpecWidget;
					specificationArea.layout();
				}
				else
				if( mode.equals(HTSSyntax.TEST_PROPERTY) )
				{
					specificationStack.topControl = propSpecWidget;
					specificationArea.layout();
				}
				else
				if( mode.equals(HTSSyntax.TEST_CHECKSTATE)
						|| mode.equals(HTSSyntax.TEST_ENABLESTATE) 
						|| mode.equals(HTSSyntax.TEST_FOCUSSTATE)
						|| mode.equals(HTSSyntax.TEST_VISIBLESTATE) )
				{
					specificationStack.topControl = boolSpecWidget;
					specificationArea.layout();
				}
				else
				{
					specificationStack.topControl = emptySpecWidget;
					specificationArea.layout();
				}
				
				layout();
			}
			
		});
		
		createSpecificationArea();
	}
	
	/**
	 * Erstellt die Tabelle zur Auswahl der ComponentId.
	 * @see ComponentId
	 */
	private void createTableArea()
	{
		Label lbl = new Label(this, SWT.NULL);
		lbl.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		lbl.setText(Messages.getString("TestSection.9")); //$NON-NLS-1$
		
		idTable = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION);
		idTable.setContentProvider( new ArrayContentProvider() );
		idTable.setLabelProvider( new IdTableLabelProvider() );
		idTable.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onIdTableSelection(e);
			}
			
		});
				
		Table table = idTable.getTable();
		new TableResizeListener(table, new int[] {25, 25, 25}, 20);
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		table.setLayoutData(gd);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("TestSection.10")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("TestSection.11")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("TestSection.12")); //$NON-NLS-1$
		
		UniversalViewerSorter.setColumnSorters(idTable, 
				new Comparator[] 
				{
					new WindowNameComparator(),
					new NameComparator(),
					new ClassNameComparator()
				},
				0);
	}
	
	/**
	 * Erstellt ein Stacklayout fr die Eingabe von
	 * Parametern fr die verschiedenen Zustandsprfungen.
	 */
	private void createSpecificationArea() 
	{
		new Composite(this, SWT.NULL); // spacer
		specificationArea = new Composite(this, SWT.NULL);
		specificationArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		specificationStack = new StackLayout();
		specificationStack.marginHeight = specificationStack.marginWidth = 0;
		specificationArea.setLayout(specificationStack);
		
		textSpecWidget = new TextSpecificationWidget(specificationArea);
		textSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		numSpecWidget = new NumSpecificationWidget(specificationArea);
		numSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		itemcountSpecWidget = new ItemcountSpecificationWidget(specificationArea);
		itemcountSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		boolSpecWidget = new BoolSpecificationWidget(specificationArea);
		boolSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		propSpecWidget = new PropertySpecWidget(specificationArea);
		propSpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		emptySpecWidget = new EmptySpecificationWidget(specificationArea);
		emptySpecWidget.setLayoutData( new GridData(GridData.FILL_BOTH) );
		
		specificationStack.topControl = emptySpecWidget;
	}
	
	private void fillCombos() {
		stateCV.setInput(EMPTY_STATES);
		idTable.setInput( AtosJApp.getInstance().getInput().getURFFile().getIds() );
	}
	
	/**
	 * Setzt die gltigen Arten von Zustandsprfungen
	 * anhand des Componenttyps der ausgewhlt wurde.
	 * @param e Auswahl Ereignis aus der ID Tabelle.
	 */
	private void onIdTableSelection(SelectionChangedEvent e)
	{
		if( !e.getSelection().isEmpty() )
		{
			ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
			boolean enabled = AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent( id.getClassName() );
			subitemChk.setSelection(enabled);
			
			subitemChkLblVis.setVisible(enabled);
			subitemChkVis.setVisible(enabled);
			subitemLblVis.setVisible( subitemChk.getSelection() );
			subitemWidgetVis.setVisible( subitemChk.getSelection()  );
			subitemListVis.setVisible( subitemChk.getSelection() );
			
			// preserve the selection
			ISelection sel = stateCV.getSelection();
			stateCV.setInput( StateMap.getStates(id.getClassName(), subitemChk.getSelection()) );
			// try to set the preserved selection again
			stateCV.setSelection(sel);
		}
		else
		{
			subitemChkLblVis.setVisible(false);
			subitemChkVis.setVisible(false);
			subitemLblVis.setVisible(false);
			subitemWidgetVis.setVisible(false);
			subitemListVis.setVisible(false);
			
			stateCV.setInput(EMPTY_STATES);
			// empty selection
			stateCV.setSelection( new StructuredSelection() );
		}
		
		layout();
	}
	
	/**
	 * Wird die Checkbox angewhlt, knnen nur Zustnde fr Unterelemente
	 * eingegeben werden und die Tabelle fr die Eingabe
	 * des Subelementpfades wird angezeigt, sonst knnen nur Zustnde fr
	 * zusammengesetzte Component selbst angegeben werden und Tabelle wird ausgeblendet.
	 */
	private void onSubitemChkSelection() 
	{
		boolean selected = subitemChk.getSelection();
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		// preserve the selection
		ISelection sel = stateCV.getSelection();
		stateCV.setInput( StateMap.getStates(id.getClassName(), selected) );
		// try to set the preserved selection again
		stateCV.setSelection(sel);
		
		subitemLblVis.setVisible(selected);
		subitemWidgetVis.setVisible(selected);
		subitemListVis.setVisible(selected);
		
		layout();
	}
	
	private CommandSection getSpecWidget() {
		return (CommandSection) specificationStack.topControl;
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#setOrigParameters(java.lang.String[])
	 */
	public void setOrigParameters(String[] parameters) 
	{
		ComponentId id = new ComponentId( 
				HTSParser.stripQuotes(parameters[2]), 
				parameters[1], 
				HTSParser.stripQuotes(parameters[0]) );
		
		idTable.setSelection( new StructuredSelection(id), true );
		String state = parameters[3];
		
		int possibleSubitemIndex = 4; // mglicher index des Schlsselwortes SUBITEM
		
		if( state.equals(HTSSyntax.TEST_TEXT) )
		{
			textSpecWidget.setOrigParameters(new String[]{parameters[4]});
			possibleSubitemIndex = 5;
		}
		else
		if( state.equals(HTSSyntax.TEST_NUM) )
		{
			Vector params = new Vector();
			params.add( parameters[4] ); // sollwert
			if(parameters.length > 5)
			{	
				params.add( parameters[5] ); // vergleichsoperator
				if( parameters[5].equals(HTSSyntax.COMPARE_TOL) )
				{
					params.add( parameters[6] );
					possibleSubitemIndex = 7;
				}
				else
					possibleSubitemIndex = 6;
			}
			
			numSpecWidget.setOrigParameters( (String[]) params.toArray( new String[params.size()] ));
		}
		else
		if( state.equals(HTSSyntax.TEST_ITEMCOUNT) )
		{
			Vector params = new Vector();
			params.add( parameters[4] ); // sollwert
			possibleSubitemIndex = 5;
			if(parameters.length > 5)
			{	
				params.add( parameters[5] ); // vergleichsoperator
				possibleSubitemIndex = 6;
			}
			
			itemcountSpecWidget.setOrigParameters( (String[]) params.toArray( new String[params.size()] ));
		}
		else
		if( state.equals(HTSSyntax.TEST_CHECKSTATE)
				|| state.equals(HTSSyntax.TEST_ENABLESTATE)
				|| state.equals(HTSSyntax.TEST_FOCUSSTATE)
				|| state.equals(HTSSyntax.TEST_VISIBLESTATE)
				)
		{
			boolSpecWidget.setOrigParameters( new String[]{parameters[4]} );
			possibleSubitemIndex = 5;
		}
		else
		if( state.equals(HTSSyntax.TEST_PROPERTY) )
		{
			propSpecWidget.setOrigParameters( new String[]{parameters[4], parameters[5]} );
		}
		
			
		if(possibleSubitemIndex < parameters.length)
		{
			Vector subitemPath = new Vector();
			for(int i = possibleSubitemIndex +1; i < parameters.length; ++i)
				subitemPath.add(parameters[i]);
			
			subitemWidget.setSubitemPath( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
			subitemChk.setSelection(true);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
		}
		else
		{
			subitemChk.setSelection(false);
			onSubitemChkSelection(); // leider wird bei setSelection kein event generiert
		}
		// na ja die Reihenfolge mag merkwrdig anmuten, ist aber nicht anders zu beheben,
		// denn der Inhalt der actionCV wird erst richtig gesetzt, wenn das Hckchen
		// bei subitemChk richtig gesetzt wurde und das geht erst nach dem Parsen eines
		// evtl. Subitempfades.
		stateCV.setSelection( new StructuredSelection(state) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#getCommandParameters()
	 */
	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(16);
		
		ComponentId id = (ComponentId) ((StructuredSelection)idTable.getSelection()).getFirstElement();
		parameters.add( '"' + id.getWindowName() + '"' );
		parameters.add( id.getClassName() );
		parameters.add( '"' + id.getName() + '"' );
		parameters.add( getSelectedElement(stateCV) );
		parameters.addAll( getSpecWidget().getCommandParameters() );
		
		if( AtosJApp.getInstance().getInput().getTestObject().getResolver().isCollectionComponent(id.getClassName())
				&& subitemChk.getSelection() )
		{
			parameters.add("SUBITEM"); //$NON-NLS-1$
			parameters.addAll( subitemWidget.getSubitemPathForLine() );
		}
		
		return parameters;
	}

	/* (non-Javadoc)
	 * @see atosj.dialog.addcmd.CommandSection#checkIntegrity()
	 */
	public boolean checkIntegrity() 
	{
		if( idTable.getSelection().isEmpty() )
		{
			MessageDialog.openError(null, Messages.getString("TestSection.14"), Messages.getString("TestSection.15")); //$NON-NLS-1$ //$NON-NLS-2$
			idTable.getTable().setFocus();
			return false;
		}
		
		if( subitemChk.getSelection() )
		{
			Vector subitemPath = subitemWidget.getSubitemPath();
			String type = ((ComponentId)((StructuredSelection)idTable.getSelection()).getFirstElement()).getClassName();
			String state = (String) ((StructuredSelection)stateCV.getSelection()).getFirstElement();
			String result = PathTypeMap.checkPath(type, state, subitemPath);
			if(result != null)
			{
				subitemWidget.getList().setFocus();
				MessageDialog.openError(null, Messages.getString("TestSection.16"), result); //$NON-NLS-1$
				return false;
			}
		}
		
		if( getSelectedElement(stateCV).equals("") ) //$NON-NLS-1$
		{
			MessageDialog.openError(null, Messages.getString("TestSection.18"), Messages.getString("TestSection.19")); //$NON-NLS-1$ //$NON-NLS-2$
			stateCV.getCombo().setFocus();
			return false;
		}
		
		if( !getSpecWidget().checkIntegrity() )
			return false;
		
		return true;
	}

}
