package atosj.dialog.addcmd.test;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.dialog.addcmd.CommandSection;
import atosj.dialog.addcmd.VisibilityChanger;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;

/**
 * Widget zur Eingabe der Parameter eines Vergleiches mit einer Variablen (Name der Variablen)
 * oder einem numerischen Wert fr das Kommando COMPARE.
 * Leider geht das NumSpecificationWidget davon aus, dass der Wert
 * eine Zahl ist, aber hier gehen wir davon aus, dass der Wert ein
 * Variablenname ist, darum kopieren wir die gesamte Implementation.
 */
class VarNumSpecificationWidget extends CommandSection 
{
	private String operators[] = new String[] { HTSSyntax.COMPARE_EQ, HTSSyntax.COMPARE_GEQ, HTSSyntax.COMPARE_GRT, HTSSyntax.COMPARE_LEQ,
			HTSSyntax.COMPARE_LSS, HTSSyntax.COMPARE_NEQ, HTSSyntax.COMPARE_TOL };
	
	private ComboViewer operatorCV;
		
	private Text variableTxt;
		
	private Text toleranceTxt;
	private VisibilityChanger toleranceTxtVis;
	private VisibilityChanger toleranceLblVis;
	
	/**
	 * Erstellt ein neues Widget zur Eingabe der Parameter eines Vergleiches mit einer Variablen (Name der Variablen)
	 * oder einem numerischen Wert fr das Kommando COMPARE.
	 * @param parent Das graph. Vaterelement.
	 */
	public VarNumSpecificationWidget(Composite parent) {
		super(parent);
		
		// Variablenname
		Label valueLbl = new Label(this, SWT.NULL);
		valueLbl.setText(Messages.getString("VarNumSpecificationWidget.0")); //$NON-NLS-1$
						
		variableTxt = new Text(this, SWT.BORDER);
		variableTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		// Vergleichsoperator
		Label operatorLbl = new Label(this, SWT.NULL);
		operatorLbl.setText(Messages.getString("VarNumSpecificationWidget.1")); //$NON-NLS-1$
				
		operatorCV = new ComboViewer(this, SWT.READ_ONLY);
		operatorCV.setContentProvider( new ArrayContentProvider() );
		operatorCV.setLabelProvider( new BaseLabelProvider() );
		operatorCV.getCombo().setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		operatorCV.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) 
			{
				onOperatorCV();
			}
		});
		operatorCV.setInput(operators);
				
		// Toleranzwert nur sichtbar bei Operator TOL
		Label toleranceLbl = new Label(this, SWT.NULL);
		toleranceLbl.setText(Messages.getString("VarNumSpecificationWidget.2")); //$NON-NLS-1$
		toleranceLblVis = new VisibilityChanger(toleranceLbl, false);
		toleranceLblVis.setVisible(false);
				
		toleranceTxt = new Text(this, SWT.BORDER);
		toleranceTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		toleranceTxt.addKeyListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				e.doit = Character.isDigit(e.character) 
				|| Character.isISOControl(e.character) 
				|| e.character == '.' || e.character == '-';
			}
		});
		toleranceTxtVis = new VisibilityChanger(toleranceTxt, true);
		toleranceTxtVis.setVisible(false);
	}
	
	private void onOperatorCV() 
	{
		if( getSelectedElement(operatorCV).equals(HTSSyntax.COMPARE_TOL) )
		{
			// Toleranzeingabe nur bei Vergleichsoperator TOL sichtbar
			toleranceLblVis.setVisible(true);
			toleranceTxtVis.setVisible(true);
		}
		else
		{
			toleranceLblVis.setVisible(false);
			toleranceTxtVis.setVisible(false);
		}
		
		layout();
	}
	
	public void setOrigParameters(String[] parameters) 
	{
		variableTxt.setText( HTSParser.stripQuotes(parameters[0]) );
		operatorCV.setSelection( new StructuredSelection(parameters[1]) );
		if(parameters.length > 2)
			toleranceTxt.setText(parameters[2]);
	}

	public boolean checkIntegrity() 
	{
		if( variableTxt.getText().equals("") ) //$NON-NLS-1$
		{
			variableTxt.setFocus();
			MessageDialog.openError(null, Messages.getString("VarNumSpecificationWidget.4"), Messages.getString("VarNumSpecificationWidget.5")); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if( getSelectedElement(operatorCV).equals(HTSSyntax.COMPARE_TOL) )
		{
			try{
				Float.parseFloat( toleranceTxt.getText() );
			}
			catch(NumberFormatException e) {
				toleranceTxt.setFocus();
				MessageDialog.openError(null, Messages.getString("VarNumSpecificationWidget.6"), Messages.getString("VarNumSpecificationWidget.7")); //$NON-NLS-1$ //$NON-NLS-2$
				return false;
			}
		}
		
		return true;
	}

	public Vector getCommandParameters() 
	{
		Vector parameters = new Vector(3);
		
		parameters.add( '"' + variableTxt.getText() + '"' );
				
		if( !getSelectedElement(operatorCV).equals("") ) //$NON-NLS-1$
			parameters.add( getSelectedElement(operatorCV) );
		
		if( getSelectedElement(operatorCV).equals(HTSSyntax.COMPARE_TOL) )
			parameters.add( toleranceTxt.getText() );
		
		return parameters;
	}
}