/*
 * Created on 24.11.2005
 */
package atosj.editors;

import atosj.exception.AtosJException;
import atosj.listener.StateChangeListener;

/**
 * Reprsentiert eine nderbare Ressource.
 * Alle Klassen, die dieses Interface implementieren,
 * knden ihren <code>StateChangeListener</code>n von nderungen
 * ihrer selbst.
 * @author Nicos Tegos
 */
public interface Changeable 
{
	/** Status: Ressource gltig */
	public static final int VALID = 0;
	/** Status: Ressource ungltig */
	public static final int INVALID = 1;
	/** Status: Ressource gltig, aber beinhaltet Warnungen */
	public static final int VALID_WITH_WARNING = 2;
	
	/**
	 * Registriert einen Listener fr nderungen an der Ressource, falls
	 * noch nicht registriert.
	 * @param listener Der Listener.
	 */
	public void addStateChangeListener(StateChangeListener listener);
	/**
	 * Entfernt einen Listener.
	 * @param listener Der Listener.
	 */
	public void removeStateChangeListener(StateChangeListener listener);
	
	/**
	 * Speichert die genderte Ressource. Benachrichtigt alle <code>StateChangeListener</code>
	 * ber den genderten Status.
	 * @throws AtosJException Bei einem Fehler, z.B. I/O Fehler.
	 */
	public void save() throws AtosJException;
	
	/**
	 * Verwirft nderungen, durch Laden des letzten gespeicherten Zustandes.
	 * Benachrichtigt alle <code>StateChangeListener</code> ber den genderten Status.
	 * @throws AtosJException Bei einem Fehler, z.B. I/O Fehler.
	 */
	public void discard() throws AtosJException;
	
	/**
	 * Liefert den Namen der Ressource.
	 */
	public String getName();
	
	/**
	 * Gibt an ob die Ressource gendert wurde und die nderungen noch
	 * nicht gespeichert wurden.
	 * @return wenn <code>true</code>, wurde die Ressource gendert. 
	 */
	public boolean isModified();
	
	/**
	 * Liefert den aktuellen Gltigkeitsstatus.
	 * @return Konstante siehe VALID, INVALID oder VALID_WITH_WARNING.
	 */
	public int getValidState();
}
