/*
 * Created on 17.11.2005
 */
package atosj.editors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabFolder2Adapter;
import org.eclipse.swt.custom.CTabFolderEvent;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.dialog.SaveResourcesDlg;
import atosj.listener.EditorListener;

/**
 * Verwaltet alle Editoren des Programms ATOSj.
 * @author Nicos Tegos
 */
public class EditorManager 
{
	private HashMap classToEditorMap = new HashMap(); // HashMap<Class, Editor>
	private HashMap objectToEditorMap = new HashMap(); // HashMap<Object, Editor> alle aktiven editoren
	
	private CTabFolder editorArea;
	private MenuManager menu = new MenuManager(); // bei rechtsklick auf einen tab aktiviert
	
	private Editor currentEditor; // der aktuell aktive Editor
	private Vector editorListeners = new Vector(); // Vector<EditorListener>
		
	public EditorManager(CTabFolder editorArea) {
		this.editorArea =  editorArea;
		
		initListeners();
		initMenu();
	}
	
	/**
	 * Initialisiert das Kontextmen fr die Editoren.
	 */
	private void initMenu() 
	{
		Action closeAction = new Action(Messages.getString("EditorManager.0")) { //$NON-NLS-1$
			public void run() {
				Editor editor = (Editor) editorArea.getSelection().getData();
				if( removeEditor(editor.getInput()) )
				{
					editor.getTabItem().dispose();
					if( objectToEditorMap.isEmpty() )
						fireEditorActivated(null);
				}
			}
		};
		
		Action closeOthersAction = new Action(Messages.getString("EditorManager.1"))  //$NON-NLS-1$
		{
			public void run() 
			{
				Object[] editedResources = getEditedObjects();
				Vector toSave = new Vector(editedResources.length);
				// alle inputs ausser den gerade aktiven auf modified berprfen
				for(int i = 0; i < editedResources.length; ++i)
				{
					Changeable resource = (Changeable)editedResources[i]; 
					if( resource.isModified() && getActiveObject() != resource )
						toSave.add(editedResources[i]);
				}
				
				// speichern
				if( !toSave.isEmpty() )
				{
					SaveResourcesDlg dlg = new SaveResourcesDlg(editorArea.getShell(), toSave);
					if( dlg.open() == SaveResourcesDlg.ID_SAVE )
					{
						
						try {
							new ProgressMonitorDialog(editorArea.getShell()).run( false, false, new SaveRunner( dlg.getElementsToDiscard(), dlg.getElementsToSave() ) );
						} catch (Throwable e) {
							// hopefully this will never happen
						}
					}
					else // nutzer abbruch
						return;
				}
				
				// alle ausser den aktiven editor schlieen
				for(int i = 0; i < editedResources.length; ++i)
				{
					if( getActiveObject() != editedResources[i] )
						closeEditor( editedResources[i] );
				}
			}
		};
		
		Action closeAllAction = new Action(Messages.getString("EditorManager.2"))  //$NON-NLS-1$
		{
			public void run() 
			{
				Object[] editedResources = getEditedObjects();
				Vector toSave = new Vector(editedResources.length);
				// alle inputs auf modified berprfen
				for(int i = 0; i < editedResources.length; ++i)
				{
					if( ((Changeable)editedResources[i]).isModified() )
						toSave.add(editedResources[i]);
				}
				
				if( !toSave.isEmpty() )
				{
					SaveResourcesDlg dlg = new SaveResourcesDlg(editorArea.getShell(), toSave);
					if( dlg.open() == SaveResourcesDlg.ID_SAVE )
					{
						
						try {
							new ProgressMonitorDialog(editorArea.getShell()).run( false, false, new SaveRunner( dlg.getElementsToDiscard(), dlg.getElementsToSave() ) );
						} catch (Throwable e) {
							// hopefully this will never happen
						}
					}
					else // nutzer abbruch
						return;
				}
				
				closeAllEditors();
			}
		};
		
		menu.add(closeAction);
		menu.add(closeOthersAction);
		menu.add(closeAllAction);
		menu.createContextMenu(editorArea);
	}
	
	/**
	 * Initialisiert die Listener, welche auf den Wechsel oder das Schlieen
	 * eines Tabitems reagieren.
	 */
	private void initListeners() {
		editorArea.addCTabFolder2Listener( new CTabFolder2Adapter() {
			public void close(CTabFolderEvent e) 
			{
				Editor editor = (Editor) e.item.getData();
				e.doit = removeEditor( editor.getInput() );
				// benachrichtigen, wenn alle editoren geschlossen wurden
				if( e.doit && objectToEditorMap.isEmpty() )
					fireEditorActivated(null);
			}
			
		});
		
		editorArea.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) 
			{
				fireEditorActivated( (Editor)e.item.getData() );
			}
		});
		
		editorArea.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				if(currentEditor != null && currentEditor.getMenu() != null)
				{
					MenuManager extendedMenu = new MenuManager();
					extendedMenu.add(menu);
					extendedMenu.add( new Separator() );
					extendedMenu.add( currentEditor.getMenu() );

					editorArea.setMenu( extendedMenu.createContextMenu(editorArea) );
				}
				else
				{
					editorArea.setMenu( menu.getMenu() );
				}
			}
		});
	}
	
	/**
	 * Registriert fr eine bestimmte Klasse von Ressourcen
	 * eine Factory, die einen entsprechenden Editor erstellt.
	 * @param edit Die Klasse der Ressource.
	 * @param factory Die Factory zum Erstellen des Editors.
	 */
	public void addEditor(Class edit, IEditorFactory factory) {
		classToEditorMap.put(edit, factory);
	}
	
	public void addEditor(Object editMe) 
	{
		Editor activeEditor = (Editor)objectToEditorMap.get(editMe);
		
		// editor aktivieren falls schon in der map
		if(activeEditor != null)
		{
			editorArea.setSelection( activeEditor.getTabItem() );
			fireEditorActivated(activeEditor);
		}
		else // editor neu erzeugen
		{
			IEditorFactory factory = (IEditorFactory) classToEditorMap.get(editMe.getClass());
			if(factory != null)
			{
				Editor newEditor = factory.createEditor(editorArea);
				objectToEditorMap.put(editMe, newEditor);
				newEditor.setInput( (Changeable) editMe );
				editorArea.setSelection( newEditor.getTabItem() );
				fireEditorActivated(newEditor); // leider wird von setSelection kein selectionEvent erzeugt, deshalb extra
			}
		}
	}
	
	/**
	 * Befragt den Nutzer, ob der Editor der Ressource <code>editMe</code> gespeichert,
	 * verworfen oder offen bleiben soll. Entfernt den Editor aus der Map, wenn er nicht mehr
	 * offen bleiben soll, das dazugehrige TabItem wird nicht disposet.
	 * @param editMe Die Ressource.
	 * @return <code>true</code>, wenn der Editor geschlossen werden darf,
	 * <code>false</code>, wenn der Nutzer wnscht, dass der Editor offen bleibt.
	 */
	protected boolean removeEditor(Object editMe) 
	{
		Editor editor = (Editor) objectToEditorMap.get(editMe);
		if(editor == null)
			return false;
		
		if( editor.isModified() )
		{
			MessageDialog dlg = new MessageDialog(
					Display.getCurrent().getActiveShell(), 
					Messages.getString("EditorManager.3"),  //$NON-NLS-1$
					null, 
					Messages.getString("EditorManager.4") + editor.getInputName() + Messages.getString("EditorManager.5"),  //$NON-NLS-1$ //$NON-NLS-2$
					MessageDialog.QUESTION, 
					new String[] {Messages.getString("EditorManager.6"), Messages.getString("EditorManager.7"), Messages.getString("EditorManager.8")},  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					0); 

			switch(dlg.open())
			{
				case 0:	// speichern
				{
					boolean commited = editor.commit(true);
					if(commited)
						objectToEditorMap.remove(editMe);
					
					return commited;
				}
				case 1: // verwerfen
				{
					boolean commited = editor.commit(false);
					if(commited)
						objectToEditorMap.remove(editMe);
					
					return commited;
				}
				case 2: return false; // editor unverndert lassen
				default: return true;
			}
		}
		else
		{
			objectToEditorMap.remove(editMe);
			return true;
		}
	}
	
	/**
	 * Disposes a single editor, modified or not, without asking.
	 * @param editMe the ressource, whose editor should be closed.
	 */
	public void closeEditor(Object editMe) {
		Editor editor = (Editor) objectToEditorMap.get(editMe);
		if(editor == null)
			return;
		
		editor.getTabItem().dispose();
		objectToEditorMap.remove(editMe);
		// benachrichtegen wenn alle editoren geschlossen
		if( objectToEditorMap.isEmpty() )
			fireEditorActivated(null);
	}
	
	/**
	 * Disposes all editors, modified or not, without asking.
	*/
	public void closeAllEditors() {
		if( objectToEditorMap.isEmpty() )
			return;
		
		Iterator it = objectToEditorMap.keySet().iterator();
		while( it.hasNext() )
		{
			Object editMe = it.next();
			Editor editor = (Editor) objectToEditorMap.get(editMe);
			editor.getTabItem().dispose();
		}
		
		objectToEditorMap.clear();
		// benachrichtigen, dass alle editoren geschlossen.
		fireEditorActivated(null);
	}
	
	/**
	 * Liefert alle Ressourcen, deren Editoren aktuell geffnet sind.
	 */
	public Object[] getEditedObjects() 
	{
		CTabItem[] items = editorArea.getItems();
		Object[] resources = new Object[items.length];
		for(int i = 0; i < items.length; ++i)
			resources[i] = ((Editor)items[i].getData()).getInput();
		
		return resources;
	}
	
	/**
	 * Returns the resource of the currently activated editor, maybe <code>null</code>
	 * if no editor is open.
	 */
	public Object getActiveObject() {
		CTabItem item = editorArea.getSelection();
		if(item == null)
			return null;
		
		return ((Editor)item.getData()).getInput();
	}
	
	/**
	 * Registriert den angegebenen <code>EditorListener</code>.
	 */
	public void addEditorListener(EditorListener listener) {
		editorListeners.add(listener);
	}
	
	/**
	 * Entfernt den angegebenen <code>EditorListener</code>.
	 */
	public void removeEditorListener(EditorListener listener) {
		editorListeners.remove(listener);
	}
	
	/**
	 * Setzt <code>currentEditor</code>, benachrichtigt alle Listener und setzt den Fokus.
	 * @param editor Der aktivierte Editor oder <code>null</code>, falls
	 * der <code>CTabFolder</code> leer ist.
	 */
	private void fireEditorActivated(Editor editor) 
	{
		if(currentEditor != null)
		{
			if( !currentEditor.onSwitch() )
			{
				editorArea.setSelection( currentEditor.getTabItem() );
				return;
			}
		}
		
		currentEditor = editor;
		if(editor != null)
			editor.setFocus();
		
		Iterator it = editorListeners.iterator();
		while( it.hasNext() )
			((EditorListener)it.next()).editorActivated(editor);
	}
}
