/*
 * Created on 01.12.2005
 */
package atosj.editors.pack;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;

import atosj.Messages;
import atosj.listener.ContentChangeListener;
import atosj.model.pack.HTSPackage;

/**
 * Liefert die Elemente zur Anzeige in der Tabelle des <code>PackageEditors</code>.
 * @author Nicos Tegos
 * @see atosj.editors.pack.PackageEditor
 */
public class PackageTableContentProvider implements IStructuredContentProvider, ContentChangeListener
{
	public static final String[] PROPERTIES = new String[] {Messages.getString("PackageTableContentProvider.0"), Messages.getString("PackageTableContentProvider.1")}; //$NON-NLS-1$ //$NON-NLS-2$
	private HTSPackage pack;
	private TableViewer tv;
		
	/**
	 * Liefert alle <code>PackageElement</code>s des Pakets.
	 */
	public Object[] getElements(Object inputElement) 
	{
		return pack.getElements().toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
	}

	/**
	 * Initialisiert den Listener fr Vernderungen des Pakets.
	 * @param viewer Muss vom Typ <code>TableViewer</code> sein.
	 * @param oldInput Dieser Parameter wird ignoriert.
	 * @param newInput Muss vom Typ <code>HTSPackage</code> sein.
	 * @see HTSPackage#addContentChangeListener(ContentChangeListener)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) 
	{
		if(newInput == null)
		{
			// alle listener entfernen
			pack.removeContentChangeListener(this);
			pack = null;
			
			return;
		}
		
		tv = (TableViewer) viewer;
		pack = (HTSPackage) newInput;
		pack.addContentChangeListener(this);
	}
	
	/**
	 * Aktualisiert den <code>TableViewer</code>.
	 */
	public void contentChanged(Object container) {
		tv.refresh();		
	}
}
