/*
 * Created on 01.12.2005
 */
package atosj.editors.pack;

import org.eclipse.swt.graphics.Image;

import atosj.app.AtosJApp;
import atosj.gui.BaseLabelProvider;
import atosj.model.pack.PackageElement;
import atosj.model.script.HTSScript;

/**
 * Liefert die Beschriftungen der Tabelle im <code>PackageEditor</code>.
 * @author Nicos Tegos
 */
public class PackageTableLabelProvider extends BaseLabelProvider 
{
	
	/**
	 * Liefert den Text fr die einzelnen Spalten der Tabelle.
	 * @param element Ein <code>PackageElement</code>.
	 * @param columnIndex Der Spaltenindex mit folgender Bedeutung:<br>
	 * 0 = Nummer<br>
	 * 1 = Skriptname<br>
	 * 2 = Cleanup<br>
	 * 3 = Interaktiv<br>
	 * @see PackageElement
	 */
	public String getColumnText(Object element, int columnIndex) 
	{
		PackageElement packElement = (PackageElement)element;
		switch(columnIndex)
		{
			case 0: return "" + (packElement.getIndex()+1);
			case 1: return packElement.getScript().getName();
			case 2: return packElement.getScript().isInteractive() ? "X" : "";
			case 3: return packElement.getScript().hasCleanup() ? "X" : "";
			default: return "";
		}
	}
	
	/**
	 * Liefert Icons entsprechend der Gltigkeit des angegebenen Skriptes. Es existieren
	 * Icons fr Skripte mit Warnungen und mit Fehlern. 
	 * @param element Ein <code>PackageElement</code>.
	 * @param columnIndex Der Spaltenindex.
	 */
	public Image getColumnImage(Object element, int columnIndex) 
	{
		HTSScript script = ((PackageElement)element).getScript();
		
		if(columnIndex == 0)
		{
			if(script.getValidState() == HTSScript.INVALID)
				return AtosJApp.getInstance().getImage(AtosJApp.IMG_ERROR);
			
			if(script.getValidState() == HTSScript.VALID_WITH_WARNING)
				return AtosJApp.getInstance().getImage(AtosJApp.IMG_WARNING);
			
			return null;
		}
		
		return null;
	}
}
