/*
 * Created on 28.11.2005
 */
package atosj.editors.pack;

import java.util.Vector;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.graphics.Image;

import atosj.Messages;
import atosj.action.pack.NewPackageAction;
import atosj.action.pack.PastePackageAction;
import atosj.app.AtosJApp;
import atosj.gui.TreeNodeContainer;
import atosj.gui.TreeProject;
import atosj.model.pack.HTSPackage;

/**
 * Stellt das Oberelement zu allen Testpaketen im Navigator-Baum dar.
 * @author Nicos Tegos
 * @see atosj.gui.TreeNodeContainer
 * @see atosj.gui.MasterTreeContentProvider
 */
public class PackageTreeDir extends TreeNodeContainer {

	/**
	 * Erstellt einen neuen Knoten als Oberelement zu allen Testpaketen eines Projektes.
	 * @param tv Der Navigator Baum.
	 * @param packages Vector(HTSPackage) die Pakete des Projektes.
	 * @param parent Der Vaterknoten.
	 */
	public PackageTreeDir(TreeViewer tv, Vector packages, TreeProject parent) {
		super(tv, packages, parent);
	}

	protected Vector createChildren() 
	{
		Vector packages = (Vector) getWrappedObject();
		Vector children = new Vector(packages.size());
		for(int i = 0; i < packages.size(); ++i)
			children.add( new PackageTreeEntry(getTreeViewer(), (HTSPackage)packages.get(i), this) );
		
		return children;
	}

	public MenuManager getContextMenu(TreeViewer tv) 
	{
		MenuManager mngr = new MenuManager();
		mngr.add( new NewPackageAction(this) );
		mngr.add( new PastePackageAction(this) );
		
		return mngr;
	}
	
	public String getName() {
		return Messages.getString("PackageTreeDir.0"); //$NON-NLS-1$
	}
	
	public void addPackage(HTSPackage additionalInput)	{
		addChild( new PackageTreeEntry(getTreeViewer(), additionalInput, this) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getImage()
	 */
	public Image getImage() {
		return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE_FOLDER);
	}
}
