/*
 * Created on 28.11.2005
 */
package atosj.editors.pack;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.graphics.Image;

import atosj.action.CopyTreeEntryAction;
import atosj.action.pack.DeletePackageAction;
import atosj.action.pack.RenamePackageAction;
import atosj.action.pack.RunPackageAction;
import atosj.app.AtosJApp;
import atosj.gui.TreeEntry;
import atosj.model.pack.HTSPackage;

/**
 * Ein Knoten fr ein Paket im Navigator-Baum.
 * @author Nicos Tegos
 * @see atosj.gui.MasterTreeContentProvider
 */
public class PackageTreeEntry extends TreeEntry {

	/**
	 * Erstellt einen neuen Knoten fr ein Paket im Navigator-Baum.
	 * @param tv Der Navigator Baum.
	 * @param toWrap Das Paket.
	 * @param parent Der Vaterknoten.
	 */
	public PackageTreeEntry(TreeViewer tv, HTSPackage toWrap,
			PackageTreeDir parent) {
		super(tv, toWrap, parent);
	}
	
	private HTSPackage getPackage() {
		return (HTSPackage) getWrappedObject();
	}

	public String getName() {
		return getPackage().isModified() ? "*" + getPackage().getName() : getPackage().getName();
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getContextMenu()
	 */
	public MenuManager getContextMenu(TreeViewer tv) 
	{
		MenuManager mngr = new MenuManager();
		mngr.add( new RunPackageAction(this) );
		mngr.add( new Separator() );
		mngr.add( new RenamePackageAction(this) );
		mngr.add( new CopyTreeEntryAction(this) );
		mngr.add( new DeletePackageAction(this) );
		
		return mngr;
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getImage()
	 */
	public Image getImage() {
		switch( getPackage().getValidState() )
		{
			case HTSPackage.VALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE);
			case HTSPackage.INVALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE_ERROR);
			case HTSPackage.VALID_WITH_WARNING: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE_WARNING);
			default: return null;
		}
	}
}
