/*
 * Created on 14.04.2006
 */
package atosj.editors.pack;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import atosj.Messages;
import atosj.editors.UndoElement;
import atosj.editors.UndoManager;
import atosj.model.pack.HTSPackage;
import atosj.model.pack.PackageElement;
import atosj.util.Clipboard;

/**
 * Verwaltet das Rckgngigmachen und Wiederherstellen von Aktionen in einem Paketeditor.
 * @author Nicos Tegos
 */
public class PackageUndoManager extends UndoManager {

	private static final int OP_MOVE = 0;
	private static final int OP_CUT = 1;
	private static final int OP_PASTE_FROM_CUT = 2;
	private static final int OP_PASTE_FROM_COPY = 3;
	private static final int OP_DELETE = 4;
	private static final int OP_ADD = 5;
	
	private HTSPackage pack;
	private Comparator indexComparator;
	
	public PackageUndoManager() {
		super();
		
		indexComparator = new Comparator() {

			public int compare(Object arg0, Object arg1) {
				Integer index0 = new Integer( ((PackageElement)arg0).getIndex() );
				Integer index1 = new Integer( ((PackageElement)arg1).getIndex() );
				
				return index0.compareTo(index1);
			}
			
		};
	}
	
	public void setPackage(HTSPackage pack) {
		this.pack = pack;
		clear();
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performUndo(atosj.editors.UndoElement)
	 */
	public void performUndo(UndoElement element) {
		switch( element.getOperation() )
		{
			case OP_MOVE: 
				Object[] content = (Object[]) element.getObject();
				int current = ((PackageElement)content[0]).getIndex();
				pack.moveElement((PackageElement)content[0], ((Integer)content[1]).intValue() );
				// fr redo die position merken
				content[1] = new Integer(current);
			break;
			case OP_CUT:
				Iterator it = ((Collection)element.getObject()).iterator();
				while( it.hasNext() )
					pack.addElement( (PackageElement) it.next() );
				// TODO oder besser abfragen ob es tatschlich dieselben sind
				// bevor man sie rausholt?
				Clipboard.getInstance().getElements();
			break;
			case OP_PASTE_FROM_CUT:
				pack.removeElements((Collection) ((Object[])element.getObject())[0] );
				// wieder ins Clipboard
				Clipboard.getInstance().setElements( (Collection) ((Object[])element.getObject())[0],
						Clipboard.MODE_CUT ); 
			break;
			case OP_PASTE_FROM_COPY:
				pack.removeElements( (Collection) ((Object[])element.getObject())[0] );
				// wieder ins Clipboard
				Clipboard.getInstance().setElements( (Collection) ((Object[])element.getObject())[0], 
						Clipboard.MODE_COPY );
			break;
			case OP_DELETE:
				Iterator it2 = ((Collection)element.getObject()).iterator();
				while( it2.hasNext() )
					pack.addElement( (PackageElement) it2.next() ); 
			break;
			case OP_ADD:
				pack.removeElements( (Collection)((Object[])element.getObject())[0] );
			break;
		}
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performRedo(atosj.editors.UndoElement)
	 */
	public void performRedo(UndoElement element) {
		switch( element.getOperation() )
		{
			case OP_MOVE: 
				Object[] content = (Object[]) element.getObject();
				int current = ((PackageElement)content[0]).getIndex();
				pack.moveElement( (PackageElement)content[0], ((Integer)content[1]).intValue() );
				// fr undo die position merken
				content[1] = new Integer(current);
			break;
			case OP_CUT:
				pack.removeElements( (Collection)element.getObject() );
				Clipboard.getInstance().setElements( (Collection)element.getObject(), Clipboard.MODE_CUT );
			break;
			case OP_PASTE_FROM_CUT:
				pack.addElements( (Collection)((Object[])element.getObject())[0],
						((Integer)((Object[])element.getObject())[1]).intValue() );
				Clipboard.getInstance().setElements( (Collection)((Object[])element.getObject())[0],
						Clipboard.MODE_CUT ); 
			break;
			case OP_PASTE_FROM_COPY:
				pack.addElements( (Collection)((Object[])element.getObject())[0],
						((Integer)((Object[])element.getObject())[1]).intValue() );
				Clipboard.getInstance().setElements( (Collection)((Object[])element.getObject())[0], 
						Clipboard.MODE_COPY );
			break;
			case OP_DELETE:
				//	auf keinen Fall mit removeElement, denn sonst ndern sich die Indizes
				pack.removeElements( (Collection)element.getObject() ); 
			break;
			case OP_ADD:
				pack.addElements( (Collection)((Object[])element.getObject())[0],
					((Integer)((Object[])element.getObject())[1]).intValue());
			break;
		}
	}

	/**
	 * Speichert eine Verschiebeaktion, zum Rckgngigmachen.
	 * @param element Das verschobene Skript.
	 * @param origPos Die Position des Skriptes im Paket vor dem Verschieben.
	 */
	public void addUndoMove(PackageElement element, int origPos) {
		addUndoElement( new UndoElement(OP_MOVE, new Object[] {element, new Integer(origPos)}) );
	}
	
	/**
	 * Speichert das Ausschneiden von Skripten, zum Rckgngigmachen.
	 * @param elements Collection(PackageElement) Die ausgeschnittenen Skripte.
	 */
	public void addUndoCut(Collection elements) 
	{
		// bei undo von cut mssen die Elemente vorher kopiert werden
		// weil sie zwischendurch vielleicht irgendwo in einem anderen Editor
		// gepastet werden und dann nicht mehr zurckgeholt werden knnen.
		Vector copiedElements = new Vector( elements.size() );
		Iterator it = elements.iterator();
		while( it.hasNext() )
		{
			PackageElement origElement = (PackageElement) it.next();
			PackageElement copy = (PackageElement) origElement.clone();
			// der index muss explizit gesetzt werden, sonst ist er -1
			copy.setIndex( origElement.getIndex() );
			copiedElements.add(copy);
		}
		
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(copiedElements, indexComparator);
		addUndoElement( new UndoElement(OP_CUT, copiedElements) );
	}
	
	/**
	 * Speichert eine Einfgeaktion aus der Zwischenablage, zum Rckgngigmachen.
	 * @param elements  List(PackageElement) Die neu eingefgten Skripte.
	 * @param newPos Die Einfgeposition im Paket.
	 * @param copy wenn <code>true</code> wurden die Skripte vorher kopiert, sonst verschoben.
	 */
	public void addUndoPaste(List elements, int newPos, boolean copy) {
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(elements, indexComparator);
		addUndoElement( new UndoElement(copy ? OP_PASTE_FROM_COPY : OP_PASTE_FROM_CUT,
				new Object[] {elements, new Integer(newPos)}) );
	}
	
	/**
	 * Speichert das Lschen von Skripten, zum Rckgngigmachen.
	 * @param elements List(PackageElement) Die aus dem Paket gelschten Skripte.
	 */
	public void addUndoDelete(List elements) {
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(elements, indexComparator);
		addUndoElement( new UndoElement(OP_DELETE, elements) );
	}
	
	/**
	 * Speichert das Hinzufgen von Skripten, zum spteren Rckgngigmachen.
	 * @param elements List(PackageElement) Die neu hinzugefgten Skripte.
	 * @param index Die Einfgeposition im Paket.
	 */
	public void addUndoAdd(List elements, int index) {
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(elements, indexComparator);
		addUndoElement( new UndoElement(OP_ADD,
				new Object[] {elements, new Integer(index)}) );
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#getUndoName(int)
	 */
	public String getOperationName(int operation) {
		switch(operation)
		{
			case OP_MOVE:
				return Messages.getString("PackageUndoManager.0"); //$NON-NLS-1$
			case OP_CUT:
				return Messages.getString("PackageUndoManager.1"); //$NON-NLS-1$
			case OP_PASTE_FROM_CUT:
				return Messages.getString("PackageUndoManager.2"); //$NON-NLS-1$
			case OP_PASTE_FROM_COPY:
				return Messages.getString("PackageUndoManager.3"); //$NON-NLS-1$
			case OP_DELETE:
				return Messages.getString("PackageUndoManager.4"); //$NON-NLS-1$
			case OP_ADD:
				return Messages.getString("PackageUndoManager.5"); //$NON-NLS-1$
			default:
				return ""; //$NON-NLS-1$
		}
	}
}
