/*
 * Created on 17.12.2005
 */
package atosj.editors.script;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;

import atosj.listener.ContentChangeListener;
import atosj.model.script.HTSScript;

/**
 * Liefert die Elemente zur Anzeige in der Tabelle des <code>ScriptEditors</code>.
 * @author Nicos Tegos
 * @see atosj.editors.script.ScriptEditor
 */
public class ScriptTableContentProvider implements IStructuredContentProvider, ContentChangeListener
{
	private HTSScript script;
	private TableViewer tv;
		
	/**
	 * Liefert alle <code>HTSCommand</code>s des Skripts.
	 */
	public Object[] getElements(Object inputElement) 
	{
		return script.getCommands().toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {
	}

	/**
	 * Initialisiert den Listener fr Vernderungen des Skripts.
	 * @param viewer Muss vom Typ <code>TableViewer</code> sein.
	 * @param oldInput Dieser Parameter wird ignoriert.
	 * @param newInput Muss vom Typ <code>HTSScript</code> sein.
	 * @see HTSScript#addContentChangeListener(ContentChangeListener)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		if(newInput == null)
		{
			script.removeContentChangeListener(this);
			
			script = null;
			return;
		}
		
		tv = (TableViewer) viewer;
		script = (HTSScript) newInput;
		script.addContentChangeListener(this);
	}
	
	/**
	 * Aktualisiert den <code>TableViewer</code>.
	 */
	public void contentChanged(Object container) {
		// bei nderungen des Scriptes und der Gltigkeit dessen Kommandos
		tv.refresh();
	}
}
