/*
 * Created on 20.11.2005
 */
package atosj.editors.script;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.TableItem;

import atosj.app.AtosJApp;
import atosj.exception.IntegrityException;
import atosj.gui.BaseLabelProvider;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSComment;
import atosj.model.hts.command.HTSDisable;
import atosj.model.script.HTSScript;

/**
 * Liefert die Beschriftungen der Tabelle im <code>ScriptEditor</code>.
 * @author Nicos Tegos
 */
public class ScriptTableLabelProvider extends BaseLabelProvider 
{
	private static final RGB commentRGB = new RGB(52, 137, 52);
	private static final RGB disabledRGB = new RGB(114, 114, 114);
	private TableViewer tv;
	
	public ScriptTableLabelProvider(TableViewer tv) {
		this.tv = tv;
	}
	
	/**
	 * Liefert Icons entsprechend der Gltigkeit des angegebenen Kommandos. Es existiert ein
	 * Icon fr Kommandos mit Fehlern. 
	 * @param element Ein <code>HTSCommand</code>.
	 * @param column Der Spaltenindex.
	 */
	public Image getColumnImage(Object element, int column) 
	{
		HTSCommand cmd = (HTSCommand) element;
		boolean isValid = true;
		try {
			cmd.checkIntegrity( AtosJApp.getInstance().getInput(), (HTSScript) tv.getInput() );
		}
		catch(IntegrityException ex) {
			isValid = false;
		}
		
		if(column == 0 && !isValid) 
			return AtosJApp.getInstance().getImage(AtosJApp.IMG_ERROR);			
		
		return null;
	}

	/**
	 * Liefert den Text fr die einzelnen Spalten der Tabelle.
	 * @param element Ein <code>HTSCommand</code>.
	 * @param index Der Spaltenindex mit folgender Bedeutung:<br>
	 * 0 = Nummer<br>
	 * 1 = Kommando-String<br>
	 * @see HTSCommand
	 */
	public String getColumnText(Object element, int index) 
	{
		HTSCommand command = (HTSCommand) element;
		switch(index)
		{
			case 0:
				setColor(command);
				return "" + (command.getIndex()+1);
			case 1: return command.toString();
			default: return "";
		}
	}
	
	/**
	 * Setzt die Schriftfarbe der Tabellenzeile fr das angegebene Kommando.
	 * Grn fr COMMENT, grau fr deaktiviert, schwarz sonst.
	 * @param cmd Das Kommando.
	 */
	private void setColor(HTSCommand cmd) {
		TableItem item = (TableItem) tv.testFindItem(cmd);
		if( cmd instanceof HTSComment )
		{
			item.setForeground( AtosJApp.getInstance().getColor(commentRGB) );
		}
		else
		if( cmd instanceof HTSDisable )
		{
			item.setForeground( AtosJApp.getInstance().getColor(disabledRGB) );
		}
		else
			item.setForeground(null);
	}
}
