/*
 * Created on 17.11.2005
 */
package atosj.editors.script;

import java.util.Vector;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.graphics.Image;

import atosj.Messages;
import atosj.action.script.NewScriptAction;
import atosj.action.script.PasteScriptAction;
import atosj.app.AtosJApp;
import atosj.gui.TreeNodeContainer;
import atosj.model.script.HTSScript;

/**
 * Stellt das Oberelement zu allen Skripten im Navigator-Baum dar.
 * @author Nicos Tegos
 * @see atosj.gui.TreeNodeContainer
 * @see atosj.gui.MasterTreeContentProvider
 */
public class ScriptTreeDir extends TreeNodeContainer {

	/**
	 * Erstellt einen neuen Knoten als Oberelement zu allen Skripten eines Projektes.
	 * @param tv Der Navigator Baum.
	 * @param scripts Vector(HTSScript) die Skripte des Projektes.
	 * @param parent Der Vaterknoten.
	 */
	public ScriptTreeDir(TreeViewer tv, Vector scripts, TreeNodeContainer parent) {
		super( tv, scripts, parent);
	}
	
	protected Vector createChildren() 
	{
		Vector scripts = (Vector) getWrappedObject();
		Vector children = new Vector(scripts.size());
		for(int i = 0; i < scripts.size(); ++i)
			children.add( new ScriptTreeEntry(getTreeViewer(), (HTSScript)scripts.get(i), this) );
		
		return children;
	}

	public MenuManager getContextMenu(TreeViewer tv) 
	{
		MenuManager mngr = new MenuManager();
		mngr.add( new NewScriptAction(this) );
		mngr.add( new PasteScriptAction(this) );
		
		return mngr;
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getName()
	 */
	public String getName() {
		return Messages.getString("ScriptTreeDir.0"); //$NON-NLS-1$
	}
	
	public void addScript(HTSScript additionalInput)	{
		addChild( new ScriptTreeEntry(getTreeViewer(), additionalInput, this) );
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getImage()
	 */
	public Image getImage() {
		return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT_FOLDER);
	}
}
