/*
 * Created on 17.11.2005
 */
package atosj.editors.script;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.graphics.Image;

import atosj.action.CopyTreeEntryAction;
import atosj.action.script.DeleteScriptAction;
import atosj.action.script.RenameScriptAction;
import atosj.action.script.RunScriptAction;
import atosj.app.AtosJApp;
import atosj.gui.TreeEntry;
import atosj.gui.TreeNodeContainer;
import atosj.model.script.HTSScript;

/**
 * Ein Knoten fr ein Skript im Navigator-Baum.
 * @author Nicos Tegos
 * @see atosj.gui.MasterTreeContentProvider
 */
public class ScriptTreeEntry extends TreeEntry {

	/**
	 * Erstellt einen neuen Knoten fr ein Paket im Navigator-Baum.
	 * @param tv Der Navigator Baum.
	 * @param toWrap Das Skript.
	 * @param parent Der Vaterknoten.
	 */
	public ScriptTreeEntry(TreeViewer tv, HTSScript toWrap, TreeNodeContainer parent) {
		super(tv, toWrap, parent);
	}

	private HTSScript getScript() {
		return (HTSScript) getWrappedObject();
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getName()
	 */
	public String getName() {
		return getScript().isModified() ? "*" + getScript().getName() : getScript().getName();
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getContextMenu()
	 */
	public MenuManager getContextMenu(TreeViewer tv) 
	{
		MenuManager mngr = new MenuManager();
		mngr.add( new RunScriptAction(this) );
		mngr.add( new Separator() );
		mngr.add( new RenameScriptAction(this) );
		mngr.add( new CopyTreeEntryAction(this) );
		mngr.add( new DeleteScriptAction(this) );
				
		return mngr;
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.TreeEntry#getImage()
	 */
	public Image getImage() {
		switch( getScript().getValidState() )
		{
			case HTSScript.VALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT);
			case HTSScript.INVALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT_ERROR);
			case HTSScript.VALID_WITH_WARNING: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT_WARNING);
			default: return null;
		}
	}
}
