/*
 * Created on 13.04.2006
 */
package atosj.editors.script;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import atosj.Messages;
import atosj.editors.UndoElement;
import atosj.editors.UndoManager;
import atosj.model.hts.command.CommandIndexComparator;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSDisable;
import atosj.model.script.HTSScript;
import atosj.util.Clipboard;

/**
 * Verwaltet das Rckgngigmachen und Wiederherstellen von Aktionen in einem <code>ScriptEditor</code>.
 * @author Nicos Tegos
 */
public class ScriptUndoManager extends UndoManager 
{
	private static final int OP_MOVE = 0;
	private static final int OP_CUT = 1;
	private static final int OP_PASTE_FROM_CUT = 2;
	private static final int OP_PASTE_FROM_COPY = 3;
	private static final int OP_DELETE = 4;
	private static final int OP_ADD = 5;
	private static final int OP_EDIT = 6;
	private static final int OP_DISABLE = 7;
	private static final int OP_ENABLE = 8;
	
	private HTSScript script;
	private Comparator indexComparator;
	
	public ScriptUndoManager() {
		super();
		indexComparator = new CommandIndexComparator();
	}
	
	/**
	 * Setzt das bearbeitete Skript.
	 */
	public void setScript(HTSScript script) {
		this.script = script;
		clear();
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performUndo(atosj.editors.UndoElement)
	 */
	protected void performUndo(UndoElement element) {
		switch( element.getOperation() )
		{
			case OP_MOVE: 
				Object[] content = (Object[]) element.getObject();
				int current = ((HTSCommand)content[0]).getIndex();
				script.moveCommand( (HTSCommand)content[0], ((Integer)content[1]).intValue() );
				// fr redo die position merken
				content[1] = new Integer(current);
			break;
			case OP_CUT:
				Iterator it = ((Collection)element.getObject()).iterator();
				while( it.hasNext() )
				{
					HTSCommand cutted = (HTSCommand) it.next();
					script.addCommand( cutted, cutted.getIndex() );
				}
				// TODO oder besser abfragen ob es tatschlich dieselben sind
				// bevor man sie rausholt?
				Clipboard.getInstance().getElements();
			break;
			case OP_PASTE_FROM_CUT:
				script.removeCommands( (Collection) ((Object[])element.getObject())[0] );
				// wieder ins Clipboard
				Clipboard.getInstance().setElements( (Collection) ((Object[])element.getObject())[0],
						Clipboard.MODE_CUT ); 
			break;
			case OP_PASTE_FROM_COPY:
				script.removeCommands( (Collection) ((Object[])element.getObject())[0] );
				// wieder ins Clipboard
				Clipboard.getInstance().setElements( (Collection) ((Object[])element.getObject())[0], 
						Clipboard.MODE_COPY );
			break;
			case OP_DELETE:
				Iterator it2 = ((Collection)element.getObject()).iterator();
				while( it2.hasNext() )
				{
					HTSCommand cmd = (HTSCommand) it2.next();
					script.addCommand( cmd, cmd.getIndex() );
				}
			break;
			case OP_ADD:
				script.removeCommand( (HTSCommand)((Object[])element.getObject())[0] );
			break;
			case OP_EDIT:
				script.removeCommand( (HTSCommand)((Object[])element.getObject())[1] );
				HTSCommand origCmd = (HTSCommand)((Object[])element.getObject())[0];
				script.addCommand( origCmd, origCmd.getIndex() );
			break;
			case OP_DISABLE:
				script.removeCommands( (Collection)((Object[])element.getObject())[0] );
				List enableds = (List)((Object[])element.getObject())[1];
				for(int i = 0; i < enableds.size(); ++i)
					script.addCommand( (HTSCommand) enableds.get(i), ((HTSCommand)enableds.get(i)).getIndex() );
			break;
			case OP_ENABLE:
				script.removeCommands( (Collection)((Object[])element.getObject())[1] );
				List disableds = (List)((Object[])element.getObject())[0];
				for(int i = 0; i < disableds.size(); ++i)
					script.addCommand( (HTSDisable) disableds.get(i), ((HTSDisable)disableds.get(i)).getIndex() );
			break;
		}
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performRedo(atosj.editors.UndoElement)
	 */
	protected void performRedo(UndoElement element) {
		switch( element.getOperation() )
		{
			case OP_MOVE: 
				Object[] content = (Object[]) element.getObject();
				int current = ((HTSCommand)content[0]).getIndex();
				script.moveCommand( (HTSCommand)content[0], ((Integer)content[1]).intValue() );
				// fr undo die position merken
				content[1] = new Integer(current);
			break;
			case OP_CUT:
				script.removeCommands( (Collection)element.getObject() );
				Clipboard.getInstance().setElements( (Collection)element.getObject(), Clipboard.MODE_CUT );
			break;
			case OP_PASTE_FROM_CUT:
				script.addCommands( (Collection)((Object[])element.getObject())[0],
						((Integer)((Object[])element.getObject())[1]).intValue() );
				Clipboard.getInstance().setElements( (Collection)((Object[])element.getObject())[0],
						Clipboard.MODE_CUT ); 
			break;
			case OP_PASTE_FROM_COPY:
				script.addCommands( (Collection)((Object[])element.getObject())[0],
						((Integer)((Object[])element.getObject())[1]).intValue() );
				Clipboard.getInstance().setElements( (Collection)((Object[])element.getObject())[0], 
						Clipboard.MODE_COPY );
			break;
			case OP_DELETE:
				// auf keinen Fall mit removeCommand, denn sonst ndern sich die Indizes
				script.removeCommands( (Collection)element.getObject() );
				break;
			case OP_ADD:
				script.addCommand( (HTSCommand)((Object[])element.getObject())[0],
						((Integer)((Object[])element.getObject())[1]).intValue());
			break;
			case OP_EDIT:
				script.removeCommand( (HTSCommand)((Object[])element.getObject())[0] );
				HTSCommand newCmd = (HTSCommand)((Object[])element.getObject())[1]; 
				script.addCommand( newCmd, newCmd.getIndex() );
			break;
			case OP_DISABLE:
				script.removeCommands( (Collection)((Object[])element.getObject())[1] );
				List disableds = (List)((Object[])element.getObject())[0];
				for(int i = 0; i < disableds.size(); ++i)
					script.addCommand( (HTSDisable) disableds.get(i), ((HTSDisable)disableds.get(i)).getIndex() );
			break;
			case OP_ENABLE:
				script.removeCommands( (Collection)((Object[])element.getObject())[0] );
				List enableds = (List)((Object[])element.getObject())[1];
				for(int i = 0; i < enableds.size(); ++i)
					script.addCommand( (HTSCommand) enableds.get(i), ((HTSCommand)enableds.get(i)).getIndex() );
			break;
		}
	}

	/**
	 * Speichert eine Verschiebeaktion, zum Rckgngigmachen.
	 * @param cmd Das verschobene Kommando.
	 * @param origPos Die Position des Kommandos im Skript vor dem Verschieben.
	 */
	public void addUndoMove(HTSCommand cmd, int origPos) {
		addUndoElement( new UndoElement(OP_MOVE, new Object[] {cmd, new Integer(origPos)}) );
	}
	
	/**
	 * Speichert das Ausschneiden von Kommandos, zum Rckgngigmachen.
	 * @param cmds Collection(HTSCommand) Die ausgeschnittenen Kommandos.
	 */
	public void addUndoCut(Collection cmds) 
	{
		// bei undo von cut mssen die Elemente vorher kopiert werden
		// weil sie zwischendurch vielleicht irgendwo in einem anderen Editor
		// gepastet werden und dann nicht mehr zurckgeholt werden knnen.
		Vector copiedCmds = new Vector( cmds.size() );
		Iterator it = cmds.iterator();
		while( it.hasNext() )
		{
			HTSCommand origCmd = (HTSCommand) it.next();
			HTSCommand copy = (HTSCommand) origCmd.clone();
			// der index muss explizit gesetzt werden, sonst ist er -1
			copy.setIndex( origCmd.getIndex() );
			
			copiedCmds.add( copy );
		}
		
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(copiedCmds, indexComparator);
		addUndoElement( new UndoElement(OP_CUT, copiedCmds) );
	}
	
	/**
	 * Speichert eine Einfgeaktion aus der Zwischenablage, zum Rckgngigmachen.
	 * @param cmds  List(HTSCommand) Die neu eingefgten Kommandos.
	 * @param newPos Die Einfgeposition im Skript.
	 * @param copy wenn <code>true</code> wurden die Kommandos vorher kopiert, sonst verschoben.
	 */
	public void addUndoPaste(List cmds, int newPos, boolean copy) {
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen
		Collections.sort(cmds, indexComparator);
		addUndoElement( new UndoElement(copy ? OP_PASTE_FROM_COPY : OP_PASTE_FROM_CUT,
				new Object[] {cmds, new Integer(newPos)}) );
	}
	
	/**
	 * Speichert das Lschen von Kommandos, zum Rckgngigmachen.
	 * @param cmds List(HTSCommand) Die aus dem Skript gelschten Kommandos.
	 */
	public void addUndoDelete(List cmds) {
		// wir muessen die richtige Reihenfolge der Elemente sicherstellen, sonst gibt
		// es beim Wiedereinfgen evtl. bse ArrayIndexOutOfBoundsExceptions
		Collections.sort(cmds, indexComparator);
		addUndoElement( new UndoElement(OP_DELETE, cmds) );
	}
	
	/**
	 * Speichert das Hinzufgen eines Kommandos, zum spteren Rckgngigmachen.
	 * @param cmd Das neu hinzugefgte Kommandos.
	 * @param index Die Einfgeposition im Skript.
	 */
	public void addUndoAdd(HTSCommand cmd, int index) {
		addUndoElement( new UndoElement( OP_ADD, new Object[] {cmd, new Integer(index)}) );
	}
	
	public void addUndoEdit(HTSCommand origCmd, HTSCommand newCmd) {
		addUndoElement( new UndoElement(OP_EDIT, new Object[] {origCmd, newCmd}) );
	}
	
	/**
	 * Speichert das Deaktivieren von Kommandos, zum spteren Rckgngigmachen.
	 * @param disableds Vector(HTSDisable) Die deaktivierten Kommandos.
	 */
	public void addUndoDisable(List disableds)
	{
		Collections.sort(disableds, indexComparator);
		Vector enableds = new Vector( disableds.size() );
		for (int i = 0; i < disableds.size(); i++) {
			enableds.add( ((HTSDisable)disableds.get(i)).getOrigCommand() );
		}
		Collections.sort(enableds, indexComparator);
		
		addUndoElement( new UndoElement(OP_DISABLE, new Object[] {disableds, enableds}) );
	}
	
	/**
	 * Speichert das Aktivieren von Kommandos, zum spteren Rckgngigmachen.
	 * @param disableds Vector(HTSDisable) Die vor dem Aktivieren deaktivierten Kommandos.
	 */
	public void addUndoEnable(List disableds)
	{
		Collections.sort(disableds, indexComparator);
		Vector enableds = new Vector( disableds.size() );
		for (int i = 0; i < disableds.size(); i++) {
			enableds.add( ((HTSDisable)disableds.get(i)).getOrigCommand() );
		}
		Collections.sort(enableds, indexComparator);
		
		addUndoElement( new UndoElement(OP_ENABLE, new Object[] {disableds, enableds}) );
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#getUndoName(int)
	 */
	public String getOperationName(int operation) {
		switch(operation)
		{
			case OP_MOVE:
				return Messages.getString("ScriptUndoManager.0"); //$NON-NLS-1$
			case OP_CUT:
				return Messages.getString("ScriptUndoManager.1"); //$NON-NLS-1$
			case OP_PASTE_FROM_CUT:
				return Messages.getString("ScriptUndoManager.2"); //$NON-NLS-1$
			case OP_PASTE_FROM_COPY:
				return Messages.getString("ScriptUndoManager.3"); //$NON-NLS-1$
			case OP_DELETE:
				return Messages.getString("ScriptUndoManager.4"); //$NON-NLS-1$
			case OP_ADD: 
				return Messages.getString("ScriptUndoManager.5"); //$NON-NLS-1$
			case OP_EDIT: 
				return Messages.getString("ScriptUndoManager.6"); //$NON-NLS-1$
			case OP_DISABLE: 
				return Messages.getString("ScriptUndoManager.7"); //$NON-NLS-1$
			case OP_ENABLE: 
				return Messages.getString("ScriptUndoManager.8"); //$NON-NLS-1$
			default:
				return ""; //$NON-NLS-1$
		}
	}
}
