/*
 * Created on 22.11.2005
 */
package atosj.editors.urf;

import java.rmi.RemoteException;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.dialog.ComponentIdDlg;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.exception.AtosJException;
import atosj.gui.TableResizeListener;
import atosj.listener.StateChangeListener;
import atosj.listener.URFReceiverListener;
import atosj.model.urf.URFFile;
import atosj.model.urf.URFReceiver;
import atosj.util.AcceleratorHook;
import atosj.util.DefaultImageDescriptor;
import atosj.util.UniversalViewerSorter;

/**
 * Fgt der URF-Datei eine neue <code>ComponentId</code> hinzu.
 */
class AddIdAction extends Action
{
	private TableViewer tv;
	private URFUndoManager undoMngr;
	
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der <code>ComponentId</code>s.
	 * Dieser muss als Input ein <code>URFFile</code> haben.
	 * @param receiver Der Empfnger fr automatisch generierte <code>ComponentId</code>s.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	AddIdAction(TableViewer tv, URFReceiver receiver, URFUndoManager undoMngr) 
	{
		super(Messages.getString("URFEditor.0")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'N');
		setToolTipText(Messages.getString("URFEditor.1")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD_DIS) );
		
		this.tv = tv;
		this.undoMngr = undoMngr;
		
		receiver.addListener(new URFReceiverListener(){
			public void runningStateChanged(boolean running) {
				setEnabled(!running);
			}

			public void idsPublished(Vector ids) {
			}
		});
	}

	private URFFile getUrf() {
		return (URFFile) tv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		ComponentIdDlg dlg = new ComponentIdDlg( Display.getCurrent().getActiveShell() );
		if( dlg.open() == ComponentIdDlg.OK )
		{
			ComponentId id = dlg.getId();
			getUrf().addId(id);
			undoMngr.addUndoAdd(id);
		
			tv.reveal(id);
			tv.setSelection( new StructuredSelection(id) );
		}
	}
}

/** 
 * Bearbeitet eine vorhandene ID mit Hilfe eines Dialogs. Die nderungen werden in allen
 * Skripten bernommen.
 */
class EditIdAction extends Action
{
	private TableViewer tv;
	private URFUndoManager undoMngr;
		
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der <code>ComponentId</code>s.
	 * Dieser muss als Input ein <code>URFFile</code> haben.
	 * @param receiver Der Empfnger fr automatisch generierte <code>ComponentId</code>s.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	EditIdAction(final TableViewer tv, final URFReceiver receiver, URFUndoManager undoMngr) 
	{
		super(Messages.getString("URFEditor.2")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'E');
		setToolTipText(Messages.getString("URFEditor.3")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EDIT) );
		// TODO bild erstellen setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EDIT_DIS) );
		
		this.tv = tv;
		this.undoMngr = undoMngr;
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 && !receiver.isRunning() );
		
		tv.addSelectionChangedListener( new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 && !receiver.isRunning() );
			}
			
		});
		
		receiver.addListener(new URFReceiverListener(){
			public void runningStateChanged(boolean running) {
				setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 && !running );
			}

			public void idsPublished(Vector ids) {
			}
		});
	}

	private URFFile getUrf() {
		return (URFFile) tv.getInput();
	}
	
	public void run() 
	{
		// ndern der ID bedingt das Speichern aller Resourcen fr konsistente Anzeige
		// der gltigkeit der Scripte
		Vector modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
		if( !modifiedResources.isEmpty() )
		{
			SaveResourcesDlg dlg = new SaveResourcesDlg(Display.getDefault().getActiveShell(), modifiedResources);
			if( dlg.open() == SaveResourcesDlg.ID_SAVE )
			{
				Vector toSave = dlg.getElementsToSave();
				Vector toDiscard = dlg.getElementsToDiscard();
				
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(toDiscard, toSave));
				} catch(Throwable t) {
					
				}
			}
			else // nutzer abbruch
				return;
		}
		
		ComponentId oldId = (ComponentId) ((StructuredSelection)tv.getSelection()).getFirstElement();
		ComponentIdDlg dlg = new ComponentIdDlg( Display.getCurrent().getActiveShell(), oldId );
		if( dlg.open() == ComponentIdDlg.OK )
		{
			ComponentId newId = dlg.getId();
			
			getUrf().removeId(oldId); // hashcode muss neu berechnet werden also raus
			getUrf().addId(newId); // wieder rein
			
			
			AtosJApp.getInstance().getInput().replaceId(oldId, newId);
			
			// Alle resourcen speichern
			modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
			if( !modifiedResources.isEmpty() )
			{
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(new Vector(0), modifiedResources));
				} catch(Throwable t) {
					
				}
			}
			
			undoMngr.addUndoChange(oldId, newId);
			tv.setSelection( new StructuredSelection(newId), true );
		}
	}
}

/**
 * Lscht die ausgewhlten IDs aus dem <code>URFFile</code>.
 */
class RemoveIdAction extends Action implements ISelectionChangedListener
{
	private TableViewer tv;
	private URFReceiver receiver;
	private URFUndoManager undoMngr;
	
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der <code>ComponentId</code>s.
	 * Dieser muss als Input ein <code>URFFile</code> haben.
	 * @param receiver Der Empfnger fr automatisch generierte <code>ComponentId</code>s.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	RemoveIdAction(final TableViewer tv, URFReceiver receiver, URFUndoManager undoMngr) 
	{
		super(Messages.getString("URFEditor.4")); //$NON-NLS-1$
		setAccelerator(SWT.DEL);
		setToolTipText(Messages.getString("URFEditor.5")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE_DIS) );
		
		this.tv = tv;
		this.receiver = receiver;
		this.undoMngr = undoMngr;
		
		setEnabled( !tv.getSelection().isEmpty() && !receiver.isRunning() );
		
		tv.addSelectionChangedListener(this);
		receiver.addListener(new URFReceiverListener(){
			public void runningStateChanged(boolean running) {
				setEnabled( !tv.getSelection().isEmpty() && !running );
			}

			public void idsPublished(Vector ids) {
			}
		});
	}

	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( !tv.getSelection().isEmpty() && !receiver.isRunning());
	}
	
	public URFFile getUrf() {
		return (URFFile) tv.getInput();
	}
	
	public void run() {
		List ids = ((StructuredSelection)tv.getSelection()).toList();
		getUrf().removeIds(ids);
				
		undoMngr.addUndoDelete(ids);
	}
}

/**
 * Zeigt wenn aktiviert nur die IDs an, die Componentnamen beinhalten,
 * die nicht aus der Position des Components generiert wurden.
 */
class FilterAction extends Action
{
	private TableViewer tv;
	private ViewerFilter filter = new IdTableFilter();
	
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der <code>ComponentId</code>s.
	 * Dieser muss als Input ein <code>URFFile</code> haben.
	 */
	FilterAction(TableViewer tv) 
	{
		super(Messages.getString("URFEditor.6"), SWT.TOGGLE); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'A');
		setToolTipText(Messages.getString("URFEditor.7")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EYE) );
		
		this.tv = tv;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		if( isChecked() )
			tv.addFilter(filter);
		else
			tv.removeFilter(filter);
	}
}

/**
 * Startet den Generator zur automatischen Erzeugung von <code>ComponentId</code>s
 * fr das Testobjekt.
 */
class GeneratorAction extends Action implements URFReceiverListener
{
	private URFFile urf;
	private URFReceiver receiver;
	private URFUndoManager undoMngr;
	
	/** 
	 * @param receiver Der Empfnger fr automatisch generierte <code>ComponentId</code>s.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	GeneratorAction(URFReceiver receiver, URFUndoManager undoMngr) 
	{
		super(Messages.getString("URFEditor.8")); //$NON-NLS-1$
		setToolTipText(Messages.getString("URFEditor.9")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_RECORD) );
		
		this.receiver = receiver;
		this.undoMngr = undoMngr;
		receiver.addListener(this);
	}
	
	public void run() 
	{
		if( receiver.isRunning() )
			receiver.stop();
		else
		{
			Thread starter = new Thread() {
				public void run() 
				{
					try {
						receiver.start();
						undoMngr.clear();
					} 
					catch(final AtosJException e) 
					{
						Display.getDefault().asyncExec( new Runnable() {
							public void run() {
								MessageDialog.openError(null, Messages.getString("URFEditor.10"), Messages.getString("URFEditor.11") //$NON-NLS-1$ //$NON-NLS-2$
										+ e.getMessage());
							}
						});
					}
				}
			};
			
			starter.start();
		}
	}

	public void runningStateChanged(boolean running) {
		setText(running ? Messages.getString("URFEditor.12") : Messages.getString("URFEditor.13") ); //$NON-NLS-1$ //$NON-NLS-2$
		setToolTipText(running ? Messages.getString("URFEditor.14") : Messages.getString("URFEditor.15")); //$NON-NLS-1$ //$NON-NLS-2$
		setImageDescriptor( running ? new DefaultImageDescriptor(AtosJApp.IMG_STOP) : new DefaultImageDescriptor(AtosJApp.IMG_RECORD) );
	}

	public void idsPublished(final Vector ids) {
		if(urf == null)
			return;
		
		urf.addIds(ids);
	}
	
	public void setURFFile(URFFile urf) {
		this.urf = urf;
	}
}

/**
 * Editor zum Bearbeiten der Datenbasis fr die Identifikatoren der Components des Testobjekts. 
 * Zeigt eine Tabelle mit allen enthaltenen <code>CompoentId</code>s an.
 * Auerdem knnen <code>ComponentIds</code> automatisch
 * und manuelle erstellt werden. Auch die nderung einzelner IDs ist
 * mglich, diese wird dann automatisch auf alle Kommandos, die
 * die genderte ID nutzen bertragen.
 * @author Nicos Tegos
 * @see atosj.model.urf.URFFile
 */
public class URFEditor extends Editor implements StateChangeListener
{
	public static final String PROP_WINDOWNAME = "WINDOWNAME"; //$NON-NLS-1$
	public static final String PROP_CLASSNAME = "CLASSNAME"; //$NON-NLS-1$
	public static final String PROP_NAME = "NAME"; //$NON-NLS-1$
	public static final String PROP_POSTION = "POSITION"; //$NON-NLS-1$
	
	private URFReceiver receiver;
	private URFTableContentProvider urfProvider;
	
	private TableViewer idTable;
	
	private MenuManager contextMenu;
	
	private Action addAction;
	private Action editAction;
	private Action removeAction;
	private Action filterAction;
	private Action generatorAction;
	
	private AcceleratorHook accHook;
	
	/**
	 * Erstellt einen neuen Editor fr das <code>URFFile</code>.
	 * @param parent Das Tab in dem der Editor erstellt werden soll.
	 */
	public URFEditor(CTabFolder parent) {
		super(parent, new URFUndoManager());
		
		// ACHTUNG variablen die hier oder oben initialisiert werden,
		// knnen nicht in createContents genutzt werden,
		// bitte init benutzen
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.Editor#init()
	 */
	protected void init() {
		urfProvider = new URFTableContentProvider();
		try {
			receiver = new URFReceiver( AtosJApp.getInstance().getInput().getTestObject() );
		}
		catch(RemoteException e) {
			// TODO
		}
	}

	/* (non-Javadoc)
	 * @see atosj.gui.Editor#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) 
	{
		Composite area = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout(1, false);
		gl.marginHeight = gl.marginWidth = 5;
		area.setLayout(gl);
		
		createTableArea(area);
	}

	/* (non-Javadoc)
	 * @see atosj.gui.Editor#commit()
	 */
	public boolean commit(boolean save) 
	{
		receiver.stop();
		
		if(save) // speichern
		{
			try {
				getURFFile().save();
				return true;
			} catch (AtosJException e) 
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("URFEditor.20"), //$NON-NLS-1$
						e.getMessage());
				return false;
			}
		}
		else // verwerfen
		{
			try {
				getURFFile().discard();
				return true;
			}
			catch(AtosJException e)
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("URFEditor.21"), //$NON-NLS-1$
						Messages.getString("URFEditor.22") //$NON-NLS-1$
						+ e.getMessage()
				);
				return false;
			}
		}
	}
	
	public boolean onSwitch() 
	{
		if( isModified() )
		{
			MessageDialog dlg = new MessageDialog(
					Display.getCurrent().getActiveShell(), 
					Messages.getString("URFEditor.23"),  //$NON-NLS-1$
					null, 
					Messages.getString("URFEditor.24") + getInputName() + Messages.getString("URFEditor.25"),  //$NON-NLS-1$ //$NON-NLS-2$
					MessageDialog.QUESTION, 
					new String[] {Messages.getString("URFEditor.26"), Messages.getString("URFEditor.27"), Messages.getString("URFEditor.28")},  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					0); 

			switch(dlg.open())
			{
				case 0:	return commit(true); // speichern
				case 1: return commit(false); // verwerfen
				case 2: return false; // editor geffnet lassen
				default: return true;
			}
		}
		else
			return true;
	}
	
	/**
	 * Liefert die mit diesem Editor assoziierte URF-Datei.
	 */
	private URFFile getURFFile() {
		return (URFFile) getInput();
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.Editor#setInput(atosj.editors.Changeable)
	 */
	public void setInput(Changeable newInput) {
		super.setInput(newInput);
		((URFUndoManager)getUndoManager()).setURFFile( getURFFile() );
		((GeneratorAction)generatorAction).setURFFile( getURFFile() );
	}
	
	/* (non-Javadoc)
	 * @see atosj.gui.Editor#refresh()
	 */
	public void refresh() {
		idTable.setInput(getURFFile());
	}
	
	protected void dispose() {
		super.dispose();
		if (receiver.isRunning()) 
			receiver.stop();
		receiver.dispose();
	}
	
	/**
	 * Erstellt die Tabelle mit Kontextmen und die Buttons zum Bearbeiten der URF-Datei.
	 * @param parent Das graph. Vaterelement.
	 */
	private void createTableArea(Composite parent)
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		tableArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginHeight = gl.marginWidth = 0;
		tableArea.setLayout(gl);
		
		idTable = new TableViewer(tableArea, SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION);
		idTable.setContentProvider(urfProvider);
		idTable.setLabelProvider( new IdTableLabelProvider() );
		idTable.setColumnProperties( new String [] {PROP_WINDOWNAME, PROP_NAME, PROP_CLASSNAME, PROP_POSTION} );
				
		Table table = idTable.getTable();
		table.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				idTable.getTable().setMenu( contextMenu.getMenu() );
			}
		});
		new TableResizeListener(table, new int[] {25, 50, 25}, 20);
		
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("URFEditor.29")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("URFEditor.30")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("URFEditor.31")); //$NON-NLS-1$
		
		UniversalViewerSorter.setColumnSorters(idTable, 
				new Comparator[] 
				{
					new WindowNameComparator(),
					new NameComparator(),
					new ClassNameComparator()
				},
				0);
		
		createActions();
		
		// die Buttonleiste erstellen
		ToolBarManager buttonsMngr = new ToolBarManager(SWT.VERTICAL);
		buttonsMngr.add(addAction);
		buttonsMngr.add(editAction);
		buttonsMngr.add(removeAction);
		buttonsMngr.add(filterAction);
		buttonsMngr.add( new Separator() );
		buttonsMngr.add(generatorAction);
		buttonsMngr.createControl(tableArea).setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		
		createContextMenu();
		initAccelerators();
	}

	/**
	 * Initialisiert alle Aktionen fr das Bearbeiten derm URF-Datei.
	 */
	private void createActions() {
		URFUndoManager undoMngr = (URFUndoManager) getUndoManager();
		addAction = new AddIdAction(idTable, receiver, undoMngr);
		editAction = new EditIdAction(idTable, receiver, undoMngr);
		removeAction = new RemoveIdAction(idTable, receiver, undoMngr);
		filterAction = new FilterAction(idTable);
		generatorAction = new GeneratorAction(receiver, undoMngr);	
	}
	
	/**
	 * Erstellt das Kontextmen fr die Tabelle.
	 */
	private void createContextMenu() {
		contextMenu = new MenuManager();
		
		contextMenu.add(addAction);
		contextMenu.add(editAction);
		contextMenu.add(removeAction);
		contextMenu.add(filterAction);
		contextMenu.add( new Separator() );
		contextMenu.add(generatorAction);
		
		contextMenu.createContextMenu( idTable.getTable() );
	}
	
	/**
	 * Initialisiert die Untersttzung von Tastenkrzeln fr das
	 * Bearbeiten der URF-Datei.
	 * @see AcceleratorHook
	 */
	private void initAccelerators()
	{
		accHook= new AcceleratorHook( idTable.getTable() );
		
		accHook.add(addAction);
		accHook.add(removeAction);
		accHook.add(filterAction);
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.Editor#getTitleImage()
	 */
	public Image getTitleImage() 
	{
		return AtosJApp.getInstance().getImage(AtosJApp.IMG_ID);
	}
	
	public void setFocus() {
		idTable.getTable().setFocus();	
	}
}
