package atosj.editors.urf;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;

import atosj.listener.ContentChangeListener;
import atosj.model.urf.URFFile;

/**
 * Liefert die Elemente zur Anzeige in der Tabelle des <code>URFEditor</code>s.
 * @author Nicos Tegos
 * @see atosj.editors.urf.URFEditor
 */
public class URFTableContentProvider implements IStructuredContentProvider, ContentChangeListener
{
	private TableViewer viewer;
	private URFFile urf;
	
	/**
	 * Liefert alle <code>ComponentId</code>s aus der URF-Datei.
	 */
	public Object[] getElements(Object input) {
		return ((URFFile)input).getIds().toArray();
	}
	
	/**
	 * Initialisiert den Listener fr Vernderungen des <code>URFFile</code>s.
	 * @param viewer Muss vom Typ <code>TableViewer</code> sein.
	 * @param oldInput Dieser Parameter wird ignoriert.
	 * @param newInput Muss vom Typ <code>URFFile</code> sein.
	 * @see URFFile#addContentChangeListener(ContentChangeListener)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) 
	{
		if(newInput == null)
		{
			if(urf != null)
				urf.removeContentChangeListener(this);
			
			urf = null;
			return;
		}
		
		this.viewer = (TableViewer) viewer;
		urf = (URFFile) newInput;
		urf.addContentChangeListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}
	
	/**
	 * Aktualisiert den <code>TableViewer</code>.
	 */
	public void contentChanged(Object container) {
		// bei nderungen des URFFiles
		viewer.refresh();
	}
}