package atosj.event;

import atosj.component.MComponent;
import atosj.exception.ComponentException;

/**
 * Ein Event, welches im Zuge der Simulation von Nutzeraktionen ausgelst werden soll.
 * @author Nicos Tegos
 */
public abstract class FireableEvent 
{
	MComponent component;
	
	/**
	 * Erstellt ein neues Ereignis fr das angegebene Component.
	 * @param component Das Component - die Ereignisquelle.
	 */
	public FireableEvent(MComponent component) {
		this.component = component;
	}
	
	/**
	 * Liefert das assozierte Component.
	 */
	protected Object getComponent() {
		return component.getComponent();
	}
	
	/**
	 * Synchronisiert mit der EventQueue.
	 * @see MComponent#sync()
	 */
	protected void sync() throws ComponentException {
		component.sync();
	}
}
