package atosj.event.awt;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.KeyEvent;

import atosj.component.IComponent;
import atosj.component.swing.SwingComponent;
import atosj.event.FireableEvent;
import atosj.exception.ComponentException;

/**
 * Basisklasse zur Simulation von Events fr die Swing Bibliothek.
 * @author Nicos Tegos
 */
public abstract class FireableSwingEvent extends FireableEvent
{
	private EventQueue eventQueue;
	
	/**
	 * Erstellt ein neues Ereignis.
	 * @param component Das Component - die Ereignisquelle.
	 */
	public FireableSwingEvent(SwingComponent component) {
		super(component);
		eventQueue = component.getAWTComponent().getToolkit().getSystemEventQueue();
	}

	/**
	 * Synchronisiert mit der AWT EventQueue und fgt das angegebene Ereignis in
	 * die EventQueue ein.
	 * @param evt Das Ereignis.
	 */
	protected void postEvent(AWTEvent evt) throws ComponentException {
		sync();
		eventQueue.postEvent(evt);
		sync();
	}

	protected Component getSwingComponent() {
		return (Component) getComponent();
	}
	
	/**
	 * Wandelt Modifer Codes aus der Klasse <code>IComponent</code> in
	 * entsprechende AWT Modifier um.
	 * @param atosjModifiers Sequenz von ATOSj Modifier Codes verknpft
	 * durch bitweises Oder.
	 * @return Sequenz von AWT Modifier Codes verknpft
	 * durch bitweises Oder.
	 */
	public static int convertToSwingModifiers(int atosjModifiers) 
	{
		int awtModifiers = 0;
		if( (atosjModifiers & IComponent.MODIFIER_ALT) > 0 )
			awtModifiers |= KeyEvent.ALT_MASK;
		if( (atosjModifiers & IComponent.MODIFIER_ALT_GR) > 0 )
			awtModifiers |= KeyEvent.ALT_GRAPH_MASK;
		if( (atosjModifiers & IComponent.MODIFIER_CTRL) > 0 )
			awtModifiers |= KeyEvent.CTRL_MASK;
		if( (atosjModifiers & IComponent.MODIFIER_SHIFT) > 0 )
			awtModifiers |= KeyEvent.SHIFT_MASK;
		
		return awtModifiers;
	}
}
