package atosj.event.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;

import atosj.component.swt.SWTComponent;
import atosj.exception.ComponentException;
import atosj.util.Snooze;

/**
 * Simuliert einen Mausklick auf einem Component.
 * @author Nicos Tegos
 */
public class FireableMouseEvent extends FireableSWTEvent 
{
	private FireableKeyEvent modifierEvent;
	
	/**
	 * Erstellt eine neues Mausereignis fr das angegebene Component.
	 * @param component Das Component, fr das ein Mausklick simuliert werden soll.
	 */
	public FireableMouseEvent(SWTComponent component) {
		super(component);
		modifierEvent = new FireableKeyEvent(component);
	}
	
	/**
	 * Simuliert einen Mausklick. Fgt dazu eine entsprechende Sequenz von <code>MouseEvent</code>s in
	 * die AWT EventQueue ein.
	 * @param numClicks Anzahl der Mausklicks.
	 * @param left <code>true</code> = linke Maustaste, <code>false</code> = rechte Maustaste.
	 * @param modifiers Sequenz von Steuertasten, die gleichzeitig gedrckt werden soll. SWT Modifier
	 * fr Steuertasten durch bitweises Oder verknpft, z.B. <code>SWT.CTRL</code>.
	 * @param x Die x-Koordinate des Mausklicks relativ zum <code>Display</code>.
	 * @param y Die y-Koordinate des Mausklicks relativ zum <code>Display</code>.
	 */
	public void fire(final int numClicks, final boolean left, final int modifiers, final int x, final int y) throws ComponentException 
	{
		Event move = generateEvent(SWT.MouseMove);
		move.x = x;
		move.y = y;
		postEvent(move);
		
		modifierEvent.pressModifiers(modifiers, true );
		
		for(int i = 0; i < numClicks; ++i)
		{
			Event down = generateEvent(SWT.MouseDown);
			down.button = left ? 1 : 3;
			postEvent(down);
			
			Event up = generateEvent(SWT.MouseUp);
			up.button = left ? 1 : 3;
			postEvent(up);
		}
		
		modifierEvent.pressModifiers(modifiers, false);
		
		// wir mssen warten damit aufeinander folgende Mausklicks keinen Doppelclick verursachen
		new Snooze( getSWTComponent().getDisplay().getDoubleClickTime() );
	}
}
