package  atosj.exception;

import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentId;

/**
 * Dieser Fehler tritt auf, wenn ein angefordertes Component zur Laufzeit
 * nicht im Testobjekt gefunden werden kann. 
 * @see atosj.component.ComponentFinder
 * @author Nicos Tegos
 */
public class ComponentNotFoundException extends ComponentException 
{
	/**
	 * Neue Ausnahme, wenn das Component mit der angegebenen ID nicht
	 * gefunden wurde.
	 * @param id ID des nicht gefundenen Components.
	 * @param message spezielle Fehlernachricht.
	 */
	public ComponentNotFoundException(ComponentId id, String message) {
		super( Messages.getString("ComponentNotFoundException.0") + id.toString() + "\n" + message ); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**
	 * Neue Ausnahme, wenn das Component mit der angegebenen ID nicht
	 * gefunden wurde.
	 * @param id ID des nicht gefundenen Components.
	 */
	public ComponentNotFoundException(ComponentId id) {
		super( Messages.getString("ComponentNotFoundException.2") + id.toString() ); //$NON-NLS-1$
	}
	
	public ComponentNotFoundException(ComponentId id, Vector subItemPath, String message) {
		super( createSubItemNotFoundMessage(id, subItemPath, message) );
	}
	
	private static String createSubItemNotFoundMessage(ComponentId id, Vector subItemPath, String message) {
		String ret = Messages.getString("ComponentNotFoundException.3"); //$NON-NLS-1$
		for(int i = 0; i < subItemPath.size(); ++i)
		{
			ret += subItemPath.get(i); 
			if(i < subItemPath.size()-1)
				ret +=  ","; //$NON-NLS-1$
		}
		ret	+= "\n" + id.toString() + "\n" + message; //$NON-NLS-1$ //$NON-NLS-2$
		return ret;
	}
}
