/*
 * Created on 05.09.2005
 */
package atosj.exception;

import atosj.Messages;
import atosj.component.MComponent;

/**
 * Diese Klasse reprsentiert einen Fehler der auftritt, wenn an einem
 * Component eine Operation ausgefhrt werden soll, zu der dieses Component
 * nicht fhig ist. Diese Klasse reprsentiert ebenfalls eine nicht ausfhrbare Operation
 * zur Laufzeit aufgrund eines internen Programmfehlers im Testobjekt,
 * z.B.: Ein Aufruf von getText() an einem org.elipse.swt.Button nachdem
 * dieser disposed wurde (fhrt zu ein SWTException). 
 * @author Nicos Tegos
 */
public class InvalidOperationException extends ComponentException
{
	/**
	 * @param comp Das <code>MComponent</code> an dem die Operation ausgefhrt werden sollte.
	 * @param operation Die Operationsart, siehe <code>MComponent.OP_*</code>  Konstanten.
	 */
	public InvalidOperationException(MComponent comp, int operation, boolean always) {
		super( createMessage(comp, operation, always) );
	}
	
	private static String createMessage(MComponent comp, int operation, boolean always)
	{
		String message = ""; //$NON-NLS-1$
		message += Messages.getString("InvalidOperationException.1"); //$NON-NLS-1$
		
		switch(operation)
		{
			case MComponent.OP_GET_ENABLED: message+= Messages.getString("InvalidOperationException.2"); break; //$NON-NLS-1$
			case MComponent.OP_GET_VISIBLE: message+= Messages.getString("InvalidOperationException.3"); break; //$NON-NLS-1$
			case MComponent.OP_GET_CHECKED: message+= Messages.getString("InvalidOperationException.4"); break; //$NON-NLS-1$
			case MComponent.OP_GET_FOCUS: message+= Messages.getString("InvalidOperationException.5"); break; //$NON-NLS-1$
			case MComponent.OP_GET_TEXT: message+= Messages.getString("InvalidOperationException.6"); break; //$NON-NLS-1$
			case MComponent.OP_GET_NUM: message+= Messages.getString("InvalidOperationException.7"); break; //$NON-NLS-1$
			case MComponent.OP_GET_ITEMCOUNT: message+= Messages.getString("InvalidOperationException.8"); break; //$NON-NLS-1$
			case MComponent.OP_GET_COLCOUNT: message+= Messages.getString("InvalidOperationException.9"); break; //$NON-NLS-1$
			case MComponent.OP_CLOSE: message+= Messages.getString("InvalidOperationException.10"); break; //$NON-NLS-1$
			case MComponent.OP_MAXIMIZE: message+= Messages.getString("InvalidOperationException.11"); break; //$NON-NLS-1$
			case MComponent.OP_MINIMIZE: message+= Messages.getString("InvalidOperationException.12"); break; //$NON-NLS-1$
			case MComponent.OP_NORMALIZE: message+= Messages.getString("InvalidOperationException.13"); break; //$NON-NLS-1$
			case MComponent.OP_CLICK: message+= Messages.getString("InvalidOperationException.14"); break; //$NON-NLS-1$
			case MComponent.OP_SELECT: message+= Messages.getString("InvalidOperationException.15"); break; //$NON-NLS-1$
			case MComponent.OP_SELECTLAST: message+= Messages.getString("InvalidOperationException.16"); break; //$NON-NLS-1$
			case MComponent.OP_DESELECT: message+= Messages.getString("InvalidOperationException.17"); break; //$NON-NLS-1$
			case MComponent.OP_ADDSELECT: message+= Messages.getString("InvalidOperationException.18"); break; //$NON-NLS-1$
			case MComponent.OP_UP: message+= Messages.getString("InvalidOperationException.19"); break; //$NON-NLS-1$
			case MComponent.OP_DOWN: message+= Messages.getString("InvalidOperationException.20"); break; //$NON-NLS-1$
			case MComponent.OP_LEFT: message+= Messages.getString("InvalidOperationException.21"); break; //$NON-NLS-1$
			case MComponent.OP_RIGHT: message+= Messages.getString("InvalidOperationException.22"); break; //$NON-NLS-1$
			case MComponent.OP_SET_TEXT: message+= Messages.getString("InvalidOperationException.23"); break; //$NON-NLS-1$
			case MComponent.OP_SET_CHECKED: message+= Messages.getString("InvalidOperationException.24"); break; //$NON-NLS-1$
			case MComponent.OP_SET_FOCUS: message+= Messages.getString("InvalidOperationException.25"); break; //$NON-NLS-1$
		}
		
		if(always) 
			message += Messages.getString("InvalidOperationException.26");  //$NON-NLS-1$
		else
			message += Messages.getString("InvalidOperationException.27"); //$NON-NLS-1$
		
		message += "\n" + comp.getComponentId().toString(); //$NON-NLS-1$
		
		return message;
	}
	
	/**
	 * @param comp Das <code>MComponent</code> an dem die Operation ausgefhrt werden sollte.
	 * @param operation Die Operationsart, siehe <code>MComponent.OP_*</code>  Konstanten.
	 * @param t interner Fehler, der diese Ausnahme ausgelst hat.
	 */
	public InvalidOperationException(MComponent comp, int operation, Throwable t) {
		super( createMessage(comp, operation, false) 
				+ Messages.getString("InvalidOperationException.29")  //$NON-NLS-1$
				+ t.getClass().getName()
				+ " \"" + t.getMessage() + "\"" //$NON-NLS-1$ //$NON-NLS-2$
		);
	}
}
