package atosj.exception;

import java.util.Arrays;

import atosj.Messages;
import atosj.util.CSVLine;

/**
 * Ausnahme aufgrund eines ungltigen Parameters, der beim Parsevorgang
 * eines Kommandos gefunden wurde.
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSCommand
 */
public class InvalidParamException extends ParseException {

	/**
	 * @param line Die kommaseparierte Liste des Kommandos.
	 * @param paramIndex Der Index des ungltigen Parameters.
	 * @param current Der aktuelle Wert des Parameters.
	 * @param expected Der erwartetet Wert des Parameters.
	 */
	public InvalidParamException(CSVLine line, int paramIndex, String current, 
			String expected) {
		super(line, createMessage(paramIndex, current, expected) );
	}
	
	/**
	 * Ausnahme fr einen ungltigen Parameter, der eigentlich
	 * eine bestimmtes Schlsselwort enthalten sollte.
	 * @param line Die kommaseparierte Liste des Kommandos.
	 * @param paramIndex Der Index des ungltigen Parameters.
	 * @param current Der aktuelle Wert des Parameters.
	 * @param keywords Die erwarteten Werte, d.h. HTS Schlsselwrter, werden bei
	 * der Fehlerausgabe sortiert.
	 */
	public InvalidParamException(CSVLine line, int paramIndex, String current, 
			String[] keywords) {
		super(line, createMessage(paramIndex, current, keywords) );
	}
	
	private static String createMessage(int paramIndex, String current, String expected)
	{
		return (Messages.getString("InvalidParamException.0") + (paramIndex + 1)  //$NON-NLS-1$
				+ Messages.getString("InvalidParamException.1")	+ current  //$NON-NLS-1$
				+ Messages.getString("InvalidParamException.2") + expected); //$NON-NLS-1$
	}
	
	private static String createMessage(int paramIndex, String current, String[] keywords) 
	{
		Arrays.sort(keywords);
		String expected = ""; //$NON-NLS-1$
		for(int i = 0; i < keywords.length; ++i)
		{
			expected += keywords[i];
			if(keywords.length > 1)
			{
				if(i < keywords.length - 2)
					expected += ", "; //$NON-NLS-1$
				if(i == keywords.length - 2)
					expected += Messages.getString("InvalidParamException.5"); //$NON-NLS-1$
			}
		}
		
		return createMessage(paramIndex, current, expected);
	}
}
