package atosj.exception;

import atosj.Messages;
import atosj.util.CSVLine;

/**
 * Ausnahme, die beim Parsevorgang eines HTS Kommandos auftritt.
 * @author Nicos Tegos
 */
public class ParseException extends AtosJException
{
	private String origLine = ""; //$NON-NLS-1$
	
	/**
	 * Ausnahme, die aufgrund einer falschen Syntax einer kommaseparierten Zeile entsteht.
	 * @param errorLine Zeilennummer in der Skriptdatei.
	 * @param scriptLocation Pfad der Skriptdatei.
	 * @param message Fehlernachricht.
	 * @param origLine Die fehlerhafte Zeile.
	 */
	public ParseException(int errorLine, String scriptLocation, String message, String origLine)	{
		super( createMessage(errorLine, scriptLocation, message) );
		this.origLine = origLine;
	}
	
	/**
	 * Ausnahme, die entsteht beim Parsevorgang eines Kommandos.
	 * @param line Die kommaseparierte Liste des Kommandos.
	 * @param message Fehlernachricht.
	 */
	public ParseException(CSVLine line, String message) {
		super( createMessage(line, message) );
		this.origLine = line.toString();
	}

	private static String createMessage(int errorLine, String scriptLocation, String message)
	{
		if(errorLine == -1 || scriptLocation == null) // die Line wurde nicht aus einer Datei sondern aus einem String geparsed
		{
			return Messages.getString("ParseException.1") //$NON-NLS-1$
			+ "\n" + message; //$NON-NLS-1$
		}
		else
		{
			return Messages.getString("ParseException.3") + scriptLocation + "\"" //$NON-NLS-1$ //$NON-NLS-2$
			+ Messages.getString("ParseException.5") + errorLine //$NON-NLS-1$
			+ "\n" + message; //$NON-NLS-1$
		}
	}
	
	private static String createMessage(CSVLine line, String message) {
		return createMessage(line.getLineNumber(), line.getFilePath(), message);
	}
	
	/**
	 * Liefert die Fehlerhafte Zeile in der Skriptdatei, die
	 * zu dieser Ausnahme fhrte.
	 */
	public String getOrigLine() {
		return origLine;
	}
}
