/*
 * Created on 16.11.2005
 */
package atosj.gui;

import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import atosj.Messages;
import atosj.action.editor.DiscardEditorAction;
import atosj.action.editor.RedoEditorAction;
import atosj.action.editor.SaveEditorAction;
import atosj.action.editor.UndoEditorAction;
import atosj.action.help.HelpAction;
import atosj.action.help.InfoAction;
import atosj.action.project.ChangeProjectAction;
import atosj.action.project.CloseProjectAction;
import atosj.action.project.CoverageAction;
import atosj.action.project.NewProjectAction;
import atosj.action.project.OpenProjectAction;
import atosj.action.project.OptionsAction;
import atosj.app.AtosJApp;
import atosj.app.RecentFileList;
import atosj.app.UserPreferences;
import atosj.listener.EditorListener;
import atosj.rmi.RMIConnectionManager;

/**
 * Das Hauptfenster des Programms ATOSj. Beinhaltet entweder den Startbildschirm oder
 * die Projektansicht.
 * @author Nicos Tegos
 * @see atosj.gui.MasterDetailsBlock
 * @see atosj.gui.SplashScreen
 */
public class AtosJMainWindow extends ApplicationWindow 
{
	private Composite stackArea;
	private StackLayout stack;
	private MasterDetailsBlock mdb;
	private Composite mdbParent;
	private Composite splashParent;
	private RecentFileList recentList;
	private Vector editorActions = new Vector(); // Vector<EditorActions> Listener fr die Editoren
	
	private Action saveAction = new SaveEditorAction();
	private Action discardAction = new DiscardEditorAction();
	private Action undoAction = new UndoEditorAction();
	private Action redoAction = new RedoEditorAction();
	private Action optionsAction = new OptionsAction();
	
	/**
	 * Erstellt ein neues Hauptfenster, mit einer Menleist und einer Toolbar.
	 * Dieses muss explizit mit der Methode
	 * <code>open</code> geffnet werden.
	 */
	public AtosJMainWindow() 
	{
		super(null);
		
		setBlockOnOpen(true);
		addMenuBar();
		addToolBar(SWT.FLAT);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) 
	{
		// ein stack entweder fr splash screen oder mdb
		stackArea = new Composite(parent, SWT.NULL);
		stack = new StackLayout();
		stackArea.setLayout(stack);
		
		// splashScreen
		splashParent = new Composite(stackArea, SWT.NULL);
		new SplashScreen(splashParent);
		
		mdbParent = new Composite(stackArea, SWT.NULL);
		mdbParent.setLayoutData( new GridData(GridData.FILL_BOTH) );
		mdb = new MasterDetailsBlock(mdbParent);
		
		// die editor actions als listener anhngen
		for(int i = 0; i < editorActions.size(); ++i)
			mdb.addEditorListener( (EditorListener) editorActions.get(i) );
		
		showMDB(false);
		
		return mdbParent;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.ApplicationWindow#createMenuManager()
	 */
	protected MenuManager createMenuManager() 
	{
		MenuManager menuBar = new MenuManager();
		MenuManager fileMenu = new MenuManager(Messages.getString("AtosJMainWindow.0")); //$NON-NLS-1$
		fileMenu.add( new NewProjectAction() );
		fileMenu.add( new OpenProjectAction() );
		fileMenu.add( new ChangeProjectAction() );
		fileMenu.add( new CloseProjectAction() );
		fileMenu.add( new CoverageAction() );
		fileMenu.add( new Separator() );
		// recentList initialisieren
		MenuManager recentMenu = new MenuManager(Messages.getString("AtosJMainWindow.1")); //$NON-NLS-1$
		recentList = new RecentFileList(recentMenu);
		fileMenu.add(recentMenu);
		
		MenuManager editMenu = new MenuManager(Messages.getString("AtosJMainWindow.2")); //$NON-NLS-1$
		editMenu.add(saveAction);
		editMenu.add(discardAction);
		editMenu.add(undoAction);
		editMenu.add(redoAction);
		editMenu.add( new Separator() );
		editMenu.add(optionsAction);
		
		MenuManager helpMenu = new MenuManager(Messages.getString("AtosJMainWindow.3")); //$NON-NLS-1$
		helpMenu.add( new InfoAction() );
		helpMenu.add( new HelpAction() );
		
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(helpMenu);
		
		return menuBar;
	}
	
	protected ToolBarManager createToolBarManager(int style) {
		
		ToolBarManager toolbarMngr = super.createToolBarManager(style);
		
		toolbarMngr.add(saveAction);
		editorActions.add(saveAction);
		
		toolbarMngr.add(discardAction);
		editorActions.add(discardAction);
		
		toolbarMngr.add(undoAction);
		editorActions.add(undoAction);
		
		toolbarMngr.add(redoAction);
		editorActions.add(redoAction);
		
		return toolbarMngr;
	}
	
	/**
	 * Kreiert das Fenster und setzt den initialen Fenstertitel und die initiale Fenstergre.
	 * Die Gre des Fensters wird beim Start wieder auf den Wert vor dem letzten Schlieen gesetzt.
	 * @see UserPreferences
	 */
	public void create() {
		super.create();
		getShell().setText(AtosJApp.APP_NAME);
		getShell().setImage( AtosJApp.getInstance().getImage(AtosJApp.IMG_ICON));
		
		getShell().setBounds( (Rectangle) UserPreferences.getInstance().getPreferedValue(UserPreferences.MAIN_WND_RECT) );
		if( UserPreferences.getInstance().getPreferedValue(UserPreferences.MAIN_WND_STATE).equals(UserPreferences.STATE_WND_MAXIMIZED) )
			getShell().setMaximized(true);
	}
	
	/**
	 * Vermerkt das Projekt in der Liste der zuletzt geffneten Projekte.
	 * @param path Pfad der Projektdatei.
	 * @see RecentFileList#addPath(String)
	 */
	public void addPathToRecentList(String path) {
		recentList.addPath(path);
	}
	
	/**
	 *  ffnet den Editor zur entsprechenden Ressource im Details Teil
	 *  des <code>MasterDetailsBlocks</code>.
	 *  @see MasterDetailsBlock#openEditor(Object)
	 */
	public void openEditor(Object editMe) 
	{
		// nur wenn der mdb auch sichtbar ist
		if( stack.topControl == mdbParent )
		{
			mdb.openEditor(editMe);
		}
	}
	
	/**
	 *  Liefert alle Ressourcen, die derzeit in
	 *  einem Editor geffnet sind.
	 *  @see MasterDetailsBlock#getEditedObjects()
	 */
	public Object[] getEditedObjects() 
	{
		// nur wenn der mdb auch sichtbar ist
		if( stack.topControl == mdbParent )
		{
			return mdb.getEditedObjects();
		}
		
		return null;
	}
	
	/**
	 * Liefert die Ressource, deren Editor zur Zeit
	 * aktiv ist.
	 * @see MasterDetailsBlock#getActiveObject()
	 */
	public Object getActiveObject() 
	{
		// nur wenn der mdb auch sichtbar ist
		if( stack.topControl == mdbParent )
		{
			return mdb.getActiveObject();
		}
		
		return null;
	}
	
	/**
	 * Zeigt den <code>MasterDetailsBlock</code>.
	 * @param show wenn <code>true</code> wird der MasterDetailsBlock sichtbar,
	 * bei <code>false</code> wird er ausgeblendet und stattdessen 
	 * der Begrungsbildschirm gezeigt.
	 */
	public void showMDB(boolean show) {
		/*if(mdbParent != null)
		mdbParent.setVisible(show);*/
		if(show) // showMdb
		{
			if(mdbParent != null)
			{
				stack.topControl = mdbParent;
				stackArea.layout();
			}
		}
		else // showSplash
		{
			if(splashParent != null)
			{
				stack.topControl = splashParent;
				stackArea.layout();
			}
		}
	}
	
	/**
	 * Schliet das Hauptfenster, sofern das Schlieen des aktuellen
	 * Projektes durch den Nutzer besttigt wird. 
	 * @see CloseProjectAction#closeProject()
	 */
	protected void handleShellCloseEvent() 
	{
		if( CloseProjectAction.closeProject() )
		{
			recentList.save();
			
			if( getShell().getMaximized() )
				UserPreferences.getInstance().setPreferedValue(UserPreferences.MAIN_WND_STATE, UserPreferences.STATE_WND_MAXIMIZED);
			else
				UserPreferences.getInstance().setPreferedValue(UserPreferences.MAIN_WND_RECT, getShell().getBounds());
			
			UserPreferences.getInstance().save();
			RMIConnectionManager.getInstance().dispose();
			AtosJApp.getInstance().dispose();
			super.handleShellCloseEvent();
		}
	}
}
