/*
 * Created on 17.12.2005
 */
package atosj.gui;

import java.util.Arrays;

import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

import atosj.Messages;

/**
 * Reagiert auf Grssenvernderungen einer Tabelle und
 * setzt die Spaltenbreiten anhand von vorgegebenen Prozentstzen.
 * @author Nicos Tegos
 */
public class TableResizeListener extends ControlAdapter{

	private int[] percentages;
	private int[] absolutes; 
	private Table table;
	
	/**
	 * Erstellt einen Manager fr die Spaltenbreiten einer Tabelle.
	 * @param table Die Tabelle.
	 * @param percentages Die relativen Werte fr die gewnschten Spaltenbreiten in Summe 100%. 
	 * Die Lnge des Arrays muss der Anzahl der Spalten in <code>table</code> entsprechen.
	 * @param absolutes Die absoluten minimalen Werte fr die Spaltenbreiten. 
	 * Diese Minima werden niemals unterschritten. 
	 * Die Lnge des Arrays muss der Anzahl der Spalten in <code>table</code> entsprechen.
	 */
	public TableResizeListener(Table table, int[] percentages, int[] absolutes) 
	{
		this.table = table;
		this.percentages = percentages;
		this.absolutes = absolutes;
		
		addResizeListener();
	}

	/**
	 * Erstellt einen Manager fr die Spaltenbreiten einer Tabelle.
	 * @param table Die Tabelle.
	 * @param percentages Die relativen Werte fr die gewnschten Spaltenbreiten in Summe 100%. 
	 * Die Lnge des Arrays muss der Anzahl der Spalten in <code>table</code> entsprechen.
	 * @param absoluteMinimum Das absolute Minimum fr die Spaltenbreiten. 
	 * Dieser Wert wird niemals unterschritten und gilt fr alle Spalten gleichermaen.
	 */
	public TableResizeListener(Table table, int[] percentages, int absoluteMinimum)
	{
		this.table = table;
		this.percentages = percentages;
		this.absolutes = new int[percentages.length];
		Arrays.fill(this.absolutes, absoluteMinimum);
		
		addResizeListener();
	}
	
	/**
	 * Registriert einen Listener fr die Grennderung der Tabelle.
	 */
	private void addResizeListener() 
	{
		Composite tableParent = table.getParent();
		if(tableParent == null)
			return;
		
		tableParent.addControlListener( new ControlAdapter() {
			public void controlResized(ControlEvent e) {
				resizeColumns();
			}
		});
	}
	
	/** 
	 * Setzt die Spaltenbreiten der Tabelle an Hand der Vorgaben im
	 * Konstruktor.
	 */
	private void resizeColumns() 
	{
		if(table.getColumnCount() < percentages.length || table.getColumnCount() < absolutes.length)
			throw new ArrayIndexOutOfBoundsException(Messages.getString("TableResizeListener.0")); //$NON-NLS-1$
		
		// force that freaky bastard to layout, so our tables size cannot be 0
		table.getParent().layout(false);
		int total = table.getClientArea().width;
		int left = total;
		for(int i = 0; i < percentages.length; ++i)
		{
			int currentWidth = (int) (total * percentages[i] / 100.0);
			if(currentWidth < absolutes[i])
				currentWidth = absolutes[i];
		
			left -= currentWidth;
			if(i == percentages.length -1 && left > 0)
				currentWidth += left;
			
			table.getColumn(i).setWidth(currentWidth);
		}
	}
}