package atosj.gui;

import java.util.Vector;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;

import atosj.editors.Changeable;

/**
 * Ein Knoten des Baumes der andere Knoten enthalten kann.
 * @author Nicos Tegos
 */
public abstract class TreeNodeContainer extends TreeEntry {
	
	private Vector/*<TreeEntry>*/ children;

	/**
	 * Ummantelt ein Element, zur Anzeige von Informationen 
	 * in einem <code>TreeViewer</code>. Stellt einen Vaterknoten im Baumes dar.
	 * @param tv Der <code>TreeViewer</code> fr diesen Knoten.
	 * @param toWrap Das ummantelte Element. Implementiert <code>toWrap</code>
	 * das Interface <code>Changeable</code>, dann werden Vernderungen von Eigenschaften,
	 * wie z.B. die Gltigkeit im Baum visualisiert.
	 * @param parent Der Vaterknoten oder, <code>null</code> fr Wurzelelemente.
	 * @see Changeable
	 */
	public TreeNodeContainer(TreeViewer tv, Object toWrap, TreeNodeContainer parent) {
		super(tv, toWrap, parent);
		children = this.createChildren();
	}
	
	/**
	 * Initialisiert die Kinderknoten.
	 * @return Vector(TreeEntry) die Kinderknoten.
	 */
	protected abstract Vector createChildren();
	
	public void addChild(TreeEntry child)	{
		children.add(child);
		getTreeViewer().refresh(this);
		getTreeViewer().setSelection( new StructuredSelection(child), true );
	}
	
	/**
	 * Entfernt einen Kindknoten aus dem Baum und aktualisiert den Baum.
	 * @param child Der zu entfernende Knoten.
	 * @return <code>true</code> wenn der Knoten tatschlich
	 * in der Liste der Kinder enthalten war.
	 */
	public boolean removeChild(TreeEntry child)	{
		boolean ret =  children.remove(child);
		child.unbind();
		getTreeViewer().refresh(this);
		return ret;
	}
	
	/**
	 * Gibt an ob der Knoten Kinder hat.
	 */
	public boolean hasChildren()	{
		return (children.size()>0);
	}
	
	/**
	 * Liefert alle Kinder dieses Knotens.
	 */
	public TreeEntry[] getChildren()	{
		return (TreeEntry[]) children.toArray( new TreeEntry[children.size()] );
	}
}
