package atosj.gui;

import java.util.Vector;

import org.eclipse.jface.viewers.TreeViewer;

import atosj.app.Project;
import atosj.editors.pack.PackageTreeDir;
import atosj.editors.script.ScriptTreeDir;
import atosj.editors.urf.URFTreeEntry;

/**
 * Diese Klasse reprsentiert ein Projekt im Baum.
 * @author Nicos Tegos
 */
public class TreeProject extends TreeNodeContainer 
{
	/**
	 * Erstellt einen neuen Knonten fr ein Projekt zur Anzeige im Baum.
	 * @param tv Der Baum.
	 * @param toWrap Das zu ummantelnde Projekt.
	 */
	public TreeProject(TreeViewer tv, Project toWrap) {
		super(tv, toWrap, null);
	}
	
	private Project getProject() {
		return (Project) getWrappedObject();
	}
	
	/** 
	 * Erstellt die Kinder eine Projektknotens.
	 * Das sind ein <code>ScriptTreeDir</code>, ein <code>PackageTreeDir</code>
	 * und ein <code>URFTreeEntry</code>.
	 * @see ScriptTreeDir
	 * @see PackageTreeDir
	 * @see URFTreeEntry
	 */
	protected Vector createChildren() 
	{
		Vector children = new Vector(4);
		children.add( new ScriptTreeDir( getTreeViewer(), getProject().getScripts(), this) );
		children.add( new PackageTreeDir( getTreeViewer(), getProject().getPackages(), this) );
		children.add( new URFTreeEntry( getTreeViewer(), getProject().getURFFile(), this) );
		
		return children;
	}
	
	/** Liefert den Projektnamen */
	public String getName() { return ((Project) getWrappedObject()).getName(); }
}
