/*
 * Created on 28.04.2006
 */
package atosj.model.hts.command;

import java.io.Serializable;
import java.util.Vector;

import atosj.component.IComponent;
import atosj.exception.ComponentException;

/**
 * Hilfsklasse fr die Ausfhrung von <code>HTSAction</code> und <code>HTSRead</code>.
 * Fhrt eine Operation auf einem Component aus.
 * Die Parameter fr die Operation mssen schon vorher vollstndig
 * geparsed sein.
 * @author Nicos Tegos
 */
abstract class Executor implements Serializable
{
	protected Vector params; // evtl. Parameter die bei der Ausfhrung bentigt werden
	protected Vector subitemPath; // der Subelementpfad, falls die auzufhrende Aktion auf einem Subelement ausgefhrt werden soll
	
	/**
	 * Erstellt einen neuen Executor.
	 */
	public Executor() {}
	
	/**
	 * Fhrt die Operation auf einem Component des Testobjekts aus.
	 * @param component Das Component auf dem die Aktion ausgefhrt werden
	 * soll.
	 * @return Rckgabewert der Ausfhrung, oder <code>null</code>, falls
	 * die Operation keinen Rckgabewert liefert.
	 */
	public abstract Object execute(IComponent component) throws ComponentException;
	
	/**
	 * Setzt die Operationsparameter. Diese werden in <code>execute</code> bentigt. 
	 * @param params Vector(Object) Enthlt Operationsparameter, z.B. der Text
	 * fr die Aktion EDIT.
	 */
	public void setParams(Vector params) {
		this.params = params;
	}
	
	/**
	 * Setzt den Subelementpfad fr eine Operation auf einem Unterelement eines
	 * zusammengesetzten Components. Dieser wirden ggf. in <code>execute</code> bentigt. 
	 * @param subitemPath Vector(Object) Der Subelementpfad, z.B. der Index einer
	 * Tabellenzeile.
	 */
	public void setSubitemPath(Vector subitemPath) {
		this.subitemPath = subitemPath;
	}
}
