/*
 * Created on 22.04.2006
 */
package atosj.model.hts.command;

import java.util.Vector;

import atosj.exception.ParseException;
import atosj.util.CSVLine;

/**
 * Ein Kommando das mehrere andere Kommandos enthalten kann z.B.
 * WHILE enthlt alle Kommandos bis einschliesslich ENDWHILE.
 * 
 * @author Nicos Tegos
 */
public abstract class HTSCollectionCommand extends HTSCommand {

	protected Vector subCommands = new Vector(); // Vector<HTSCommand>
	private HTSEndCommand end; //  gibt an ob dieses Kommando auch ein dazugehriges endkommando enthlt
	
	/**
	 * Erstellt ein neues Blockkommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSCollectionCommand(CSVLine line) throws ParseException {
		super(line);
	}
	
	/**
	 * Gibt an, ob das entsprechende Endkommando gesetzt wurde.
	 * @return <code>true</code>, wenn das Kommando ein Endkommando hat.
	 */
	protected boolean hasEnd() {
		return end !=  null;
	}
	
	/**
	 * Setzt das entsprechende Endkommando fr dieses Kommando, z.B. ENDWHILE fr WHILE.
	 * Ist <code>end</code> <code>null</code>, so muss eine entsprechende Fehlermeldung gesetzt werden.
	 * @param end Das Endkommando (Blockende) oder <code>null</code>, falls kein Endkommando verfgbar.
	 * @see HTSCommand#setError(String)
	 */
	protected void setEnd(HTSEndCommand end) {
		this.end = end;
	}
	
	/**
	 * Liefert das Endkommando.
	 */
	public HTSEndCommand getEnd() {
		return end;
	}
}
