/*
 * Created on 25.10.2005
 */
package atosj.model.hts.command;

import java.io.File;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS DELETE Kommandos.
 * @author Nicos Tegos
 */
public class HTSDelete extends HTSFileOperation
{
	private boolean force = false;
	private boolean ifSrcExists = false;
	private String dir;
	private String fileName;
	
	/**
	 * Erstellt ein neues DELETE-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSDelete(CSVLine line) throws ParseException
	{
		super(line);
		
		//<VERZEICHNIS>, <DATEI> [,FORCE] [,IFEXISTS]
		dir = getDir(0);
		fileName = getParamUnquoted(1, Messages.getString("HTSDelete.0")); //$NON-NLS-1$
				
		if( getParamSize() == 3)
		{
			String param2 = getParam(2, Messages.getString("HTSDelete.1")); //$NON-NLS-1$
			if( param2.equals(HTSSyntax.DELETE_FORCE) )
			{
				force = true;
			}
			else
			if( param2.equals(HTSSyntax.DELETE_IFEXISTS) )
			{
				ifSrcExists = true;
			}
			else
			{
				throw new InvalidParamException(line, 2, param2,
						new String[] {HTSSyntax.DELETE_FORCE, HTSSyntax.DELETE_IFEXISTS} );
			}
		}
		else
		if(getParamSize() == 4)
		{
			String param2 = getParam(2, Messages.getString("HTSDelete.2")); //$NON-NLS-1$
			if( param2.equals(HTSSyntax.DELETE_FORCE) )
			{
				force = true;
			}
			else
			{
				throw new InvalidParamException(line, 2, param2,
						HTSSyntax.DELETE_FORCE);
			}
			
			String param3 = getParam(3, Messages.getString("HTSDelete.3")); //$NON-NLS-1$
			if( param3.equals(HTSSyntax.DELETE_IFEXISTS) )
			{
				ifSrcExists = true;
			}
			else
			{
				throw new InvalidParamException(line, 3, param3,
						HTSSyntax.DELETE_IFEXISTS);
			}
		}
		else
		{
			assertParamSize(2);
		}
	}
	
	/** 
	 * Fhrt das Lschen einer Datei aus.
	 */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		String deletePath = makePath( dir,
						fileName,
						localPlayer.getTestObject() );
		File src = new File(deletePath);
		
		if( !ifSrcExists  && !src.exists() )
			throw new ExecutionException(this, Messages.getString("HTSDelete.4") + deletePath + Messages.getString("HTSDelete.5")); //$NON-NLS-1$ //$NON-NLS-2$

		if( src.exists() )
		{
			if( !force && src.isHidden() )
				throw new ExecutionException(this, Messages.getString("HTSDelete.6") + deletePath + Messages.getString("HTSDelete.7")); //$NON-NLS-1$ //$NON-NLS-2$
			
			if( !force && !src.canWrite() )
				throw new ExecutionException(this, Messages.getString("HTSDelete.8") + deletePath + Messages.getString("HTSDelete.9")); //$NON-NLS-1$ //$NON-NLS-2$
			
			// perform delete
			src.delete();
		}
	}
	
	public Object clone() {
		try {
			return new HTSDelete( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
