/*
 * Created on 22.04.2006
 */
package atosj.model.hts.command;

import atosj.exception.ParseException;
import atosj.util.CSVLine;

/**
 * Bildet den syntaktischen abschluss eines Blockkommandos 
 * (</code>HTSCollectionCommand</code>).
 * @author Nicos Tegos
 * @see atosj.model.hts.command.HTSCollectionCommand
 */
public abstract class HTSEndCommand extends HTSCommand 
{
	private HTSCommand start;
	
	/**
	 * Erstellt ein neues syntaktisches Ende.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 * @throws ParseException
	 */
	public HTSEndCommand(CSVLine line) throws ParseException {
		super(line);
	}
	
	/**
	 * Setzt das entsprechende Startkommando fr dieses Kommando, z.B. WHILE fre ENDWHILE.
	 * Ist start <code>null</code>, so muss die entsprechende Fehlermeldung gesetzt werden.
	 * @param start Das Startkommando (Blockanfang) oder <code>null</code> falls
	 * kein Startkommando verfgbar.
	 * @see HTSCollectionCommand
	 * @see HTSCommand#setError(String)
	 */
	protected void setStart(HTSCommand start) {
		this.start = start;
	}
	
	/**
	 * Gibt an, ob das Startkommando gesetzt wurde.
	 * @return <code>true</code> wenn gesetzt.
	 * @see #setStart(HTSCommand)
	 */
	public boolean hasStart() {
		return start != null;
	}
	
	/**
	 * Liefert das Startkommando.
	 */
	public HTSCommand getStart() {
		return start;
	}
}
