package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;


/**
 * Syntaktisches Ende eines LOOP Kommandos.
 * @author Nicos Tegos
 */
public class HTSEndLoop extends HTSEndCommand
{
	/**
	 * Erstellt ein neues ENDLOOP-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSEndLoop(CSVLine line) throws ParseException
	{
		// keine ParamterListe und auch nix zu tun
		super(line);
		assertParamSize(0);
	}
	
	/** Macht nichts. */
	public void execute(IHTSPlayer player) throws ExecutionException{
	}

	/**
	 * Setzt den syntaktischen Start, d.h. LOOP.
	 * @param start wenn <code>null</code> wird, das
	 * Kommando auf fehlerhaft gesetzt.
	 * @see HTSCommand#setError(String)
	 */
	public void setStart(HTSLoop start) {
		super.setStart(start);
		if(start == null)
			setError(HTSSyntax.CMD_ENDLOOP + Messages.getString("HTSEndLoop.0") + HTSSyntax.CMD_LOOP + "."); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public Object clone() {
		try {
			return new HTSEndLoop( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
