package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Syntaktisches Ende eines WHILE Kommandos.
 * @author Nicos Tegos
 */
public class HTSEndWhile extends HTSEndCommand {

	/**
	 * Erstellt ein neues ENDWHILE-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSEndWhile(CSVLine line) throws ParseException {
		super(line);
		assertParamSize(0);
	}

	/** Macht nichts. */
	public void execute(IHTSPlayer player) throws ExecutionException {
	}
	
	/**
	 * Setzt den syntaktischen Start, d.h. WHILE.
	 * @param start wenn <code>null</code> wird, das
	 * Kommando auf fehlerhaft gesetzt.
	 * @see HTSCommand#setError(String)
	 */
	public void setStart(HTSCommand start) {
		super.setStart(start);
		if(start == null)
			setError(HTSSyntax.CMD_ENDWHILE + Messages.getString("HTSEndWhile.0") + HTSSyntax.CMD_WHILE+ "."); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public Object clone() {
		try {
			return new HTSEndWhile( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}

}
