/*
 * Created on 08.11.2005
 */
package atosj.model.hts.command;

import java.io.File;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS EXIST Kommandos.
 * @author Nicos Tegos
 */
public class HTSExists extends HTSFileOperation
{
	private String dir;
	private String fileName;
	private boolean isDir = false; // Existenz eines Verzeichnisses
	
	/**
	 * Erstellt ein neues EXIST-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 * */
	public HTSExists(CSVLine line) throws ParseException 
	{
		super(line);
		
		// <VERZEICHNIS>, <ZIEL> [,DIRECTORY]
		dir = getDir(0);
		fileName = getParamUnquoted(1, Messages.getString("HTSExists.0")); //$NON-NLS-1$
		
		if( getParamSize() > 2)
		{
			if( getParam(2, Messages.getString("HTSExists.1")).equals(HTSSyntax.EXISTS_DIRECTORY) ) //$NON-NLS-1$
			{
				isDir = true;
				assertParamSize(3);
			}
			else
				throw new InvalidParamException(line, 2, getParam(2, Messages.getString("HTSExists.2")), //$NON-NLS-1$
						HTSSyntax.EXISTS_DIRECTORY);
		}
		else
		{
			assertParamSize(2);
		}
	}

	/**
	 * berprft die Existenz einer Datei/Verzeichnis.
	 */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		String path = makePath( dir, fileName, localPlayer.getTestObject() );
		
		if( isDir && !path.endsWith("\\") ) //$NON-NLS-1$
			path += "\\"; //$NON-NLS-1$
			
		File f = new File(path);
		if( !f.exists() )
		{
			throw new ExecutionException(this, Messages.getString("HTSExists.5") //$NON-NLS-1$
					+ f.getAbsolutePath() + "\""); //$NON-NLS-1$
		}
	}
	
	public Object clone() {
		try {
			return new HTSExists( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
