/*
 * Created on 08.11.2005
 */
package atosj.model.hts.command;

import java.io.File;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.util.CSVLine;

/**
 * Basisklasse fr alle Kommandos, die Dateioperationen ausfhren.
 * @author Nicos Tegos
 */
public abstract class HTSFileOperation extends HTSCommand
{
	/**
	 * Erstellt eine neue Dateioperation.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSFileOperation(CSVLine line) throws ParseException {
		super(line);
	}
	
	/**
	 * berprft ob der angegebene Paramter ein Verzeichnisschlsselwort ist
	 * und liefert das Verzeichnisschlselwort.
	 * @param index Der Parameterindex, der ein Verzeichnis beinhalten soll.
	 * @return Das Verzeichnisschlsselwort.
	 * @exception ParseException wenn das Verzeichnis nicht den Schlsselworten
	 * aus der HTSSyntax entspricht.
	 */
	protected String getDir(int index) throws ParseException
	{
		String dir = getParam(index, Messages.getString("HTSFileOperation.0")); //$NON-NLS-1$
		if(dir.equalsIgnoreCase(HTSSyntax.FILEOP_ABS) 
				|| dir.equals(HTSSyntax.FILEOP_TGT)
				|| dir.equals(HTSSyntax.FILEOP_REF)
				|| dir.equals(HTSSyntax.FILEOP_BIN)
				|| dir.equals(HTSSyntax.FILEOP_ENV)
				|| dir.equals(HTSSyntax.FILEOP_LOG)
				)
		{
			return dir;
		}
		else
		{
			throw new InvalidParamException(line, 0, dir, 
				new String[] {HTSSyntax.FILEOP_ABS, HTSSyntax.FILEOP_BIN 
				,HTSSyntax.FILEOP_ENV, HTSSyntax.FILEOP_LOG
				, HTSSyntax.FILEOP_REF, HTSSyntax.FILEOP_TGT});
		}
	}
	
	/**
	 * Erstellt den absoluten Pfad zum Verzeichnisschlsselwort aus der HTS-Syntax
	 * @param testee Das Testobjekt, wird bentigt zum Auflsen der Pfadnamen.
	 * @see HTSSyntax
	 */
	protected String makePath(String dir, String fileName, TestObject testee)
	{
		String path = ""; //$NON-NLS-1$
		
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_ABS) )
		{
			// kompletter pfad mit dateiName
			path = fileName;
		}
		else
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_TGT) )
		{
			path = testee.getMainClassDir() + File.separatorChar + fileName;
		}
		else
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_REF) )
		{
			path = testee.getProjectDir() + File.separatorChar + HTSSyntax.FILEOP_REF.toLowerCase() + File.separator + fileName;
		}
		else
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_BIN) )
		{
			path = testee.getProjectDir() + File.separatorChar + HTSSyntax.FILEOP_BIN.toLowerCase() + File.separator + fileName;
		}
		else
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_ENV) )
		{
			// kompletter pfad mit dateiName
			path = testee.getProjectDir() + File.separatorChar + HTSSyntax.FILEOP_ENV.toLowerCase() + File.separator + fileName;
		}
		else
		if( dir.equalsIgnoreCase(HTSSyntax.FILEOP_LOG) )
		{
			path = testee.getProjectDir() + File.separatorChar + HTSSyntax.FILEOP_LOG.toLowerCase() + File.separator + fileName;
		}
		
		return path;
	}
}
