/*
 * Created on 24.01.2006
 */
package atosj.model.hts.command;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.swt.SWTThreadSafeAction;
import atosj.dialog.MessageWindow;
import atosj.exception.ComponentException;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;
import atosj.util.Snooze;

/**
 * Klasse zum Parsen und Ausfhren eines HTS MESSAGE Kommandos.
 * @author Nicos Tegos
 */
public class HTSMessage extends HTSCommand 
{
	private String message; // anzuzeigende Nachricht
	
	private MessageWindow msgWnd; // zur Laufzeit erstelltes Dialogfenster
	
	/**
	 * Erstellt ein neues MESSAGE-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSMessage(CSVLine line) throws ParseException {
		super(line);

		message = getParamUnquoted(0, Messages.getString("HTSMessage.0")); //$NON-NLS-1$
		assertParamSize(1);
	}

	/**
	 * Zeigt ein nicht modales Fenster mit einer Nachricht im ATOSj-Prozess an.
	 */
	public void execute(final IHTSPlayer player) throws ExecutionException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			public Object performAction() throws Throwable
			{
				msgWnd = new MessageWindow(((HTSPlayer)player).getRunDlgShell(), Messages.getString("HTSMessage.1"),  //$NON-NLS-1$
						AtosJApp.getInstance().getImage(AtosJApp.IMG_ICON),
						message, MessageWindow.INFORMATION,
						new String[] {Messages.getString("HTSMessage.2")},  //$NON-NLS-1$
						0);
				
				msgWnd.open();
				msgWnd.getShell().forceActive();
				
				return msgWnd;
			}
		};
		
		
		try 
		{
			action.go();
			while( !msgWnd.isClosed() )
				new Snooze(100);
		} catch (ComponentException e) {
			throw new ExecutionException(this, e.getMessage());
		}
	}
	
	/**
	 * Schliet ein eventuell geffnetes Nachrichtenfenster.
	 */
	public void reset() 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			public Object performAction() throws Throwable
			{
				if(msgWnd != null)
					msgWnd.close();
				
				return null;
			}
		};
		
		try {
			action.go();
		} catch (ComponentException e) {
			/* egal */
		}
	}

	public Object clone() {
		try {
			return new HTSMessage( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
