/*
 * Created on 25.10.2005
 */
package atosj.model.hts.command;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.swt.SWTThreadSafeAction;
import atosj.dialog.MessageWindow;
import atosj.exception.ComponentException;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;
import atosj.util.Snooze;

/**
 * Klasse zum Parsen und Ausfhren eines HTS QUESTION Kommandos.
 * @author Nicos Tegos
 */
public class HTSQuestion extends HTSCommand
{
	private String question; // darzustellende Frage
	private boolean yes; // erwartete Antwort true = ja
	
	private MessageWindow msgWnd; // zur Laufzeit erstelltes Dialogfenster.
	
	/**
	 * Erstellt ein neues QUESTION-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSQuestion(CSVLine line) throws ParseException
	{
		super(line);
		
		// QUESTION "Frage", <ANSWER>
		question = getParamUnquoted(0, Messages.getString("HTSQuestion.0")); //$NON-NLS-1$
		String answer = getParam(1, Messages.getString("HTSQuestion.1")); //$NON-NLS-1$
		if( answer.equals(HTSSyntax.QUESTION_YES) )
		{
			yes = true;
		}
		else
		if( answer.equals(HTSSyntax.QUESTION_NO) )
		{
			yes = false;
		}
		else
		{
			throw new InvalidParamException(line, 1, answer,
					new String[] {HTSSyntax.QUESTION_YES, HTSSyntax.QUESTION_NO});
		}
		
		assertParamSize(2);
	}

	/**
	 * Zeigt ein nicht modales Fenster mit einer Frage im ATOSj-Prozess an.
	 */
	public void execute(final IHTSPlayer player) throws ExecutionException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			public Object performAction() throws Throwable
			{
				msgWnd = new MessageWindow(((HTSPlayer)player).getRunDlgShell(), Messages.getString("HTSQuestion.2"),  //$NON-NLS-1$
						AtosJApp.getInstance().getImage(AtosJApp.IMG_ICON),
						question, MessageWindow.QUESTION,
						new String[] {Messages.getString("HTSQuestion.3"), Messages.getString("HTSQuestion.4")},  //$NON-NLS-1$ //$NON-NLS-2$
						0);
						
				msgWnd.open();
				msgWnd.getShell().forceActive();
				
				return msgWnd;
			}
		};
		
		
		try 
		{
			action.go();	
		} catch (ComponentException e) {
			throw new ExecutionException(this, e.getMessage());
		}
		
		while( !msgWnd.isClosed() )
			new Snooze(100);
		
		if(yes)
		{
			if(msgWnd.getReturnCode() != 0)
				throw new ExecutionException(this, Messages.getString("HTSQuestion.5")); //$NON-NLS-1$
		}
		else
		{
			if(msgWnd.getReturnCode() != 1)
				throw new ExecutionException(this, Messages.getString("HTSQuestion.6")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Schliet ein eventuell geffnetes Nachrichtenfenster.
	 */
	public void reset() 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() 
		{
			public Object performAction() throws Throwable
			{
				if(msgWnd != null)
					msgWnd.close();
				
				return null;
			}
		};
		
		try {
			action.go();
		} catch (ComponentException e) {
			/* egal */
		}
	}
	
	public Object clone() {
		try {
			return new HTSQuestion( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
