/*
 * Created on 19.11.2005
 */
package atosj.model.hts.command;

import java.rmi.RemoteException;
import java.util.Vector;

import atosj.Messages;
import atosj.exception.AtosJException;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.rmi.HTSCommandExecutor;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS START Kommandos.
 * TODO die Definition des START Kommandos evtl ndern, und die Argumente einzeln als beliebig lange Liste zulassen.
 * @author Nicos Tegos
 */
public class HTSStart extends HTSCommand 
{
	private String[] args = new String[0]; // die Programmargumente
	
	/**
	 * Erstellt ein neues START-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSStart(CSVLine line) throws ParseException 
	{
		super(line);
		
		if(getParamSize() == 1)
		{
			String str = getParamUnquoted(0, Messages.getString("HTSStart.0")); //$NON-NLS-1$
			// wir zerhacken den Argumentstring so das alle Argumente einzeln stehen
			Vector argsVec = new Vector();
			int i = 0;
			while( i < str.length() )
			{
				char c = str.charAt(i);
				// eatup whitespace
				while(c == ' ' && i < str.length())
				{
					++i;
					if(i < str.length())
						c = str.charAt(i);
				}
				
				if(i == str.length())
					break;
				
				StringBuffer current = new StringBuffer();
				// innerhalb von Argumenten die in Anfhrungszeichen stehen wird nicht
				// an Leerzeichen getrennt
				if(c == '"')
				{
					++i;
					if(i < str.length())
						c = str.charAt(i);
					
					while( c!= '"' && i < str.length() )
					{
						current.append(c);
						++i;
						if(i < str.length())
							c = str.charAt(i);
					}
					++i;
				}
				else
				{
					while(c != ' ' && i < str.length())
					{
						current.append(c);
						++i;
						if(i < str.length())
							c = str.charAt(i);
					}
				}
				
				String currentStr = current.toString();
				if(currentStr.trim().length() > 0)
					argsVec.add(currentStr);
			}
			
			args = (String[]) argsVec.toArray( new String[argsVec.size()] );
		}
		else
			assertParamSize(0);
	}

	/** Startet das Testobjekt mit den angegebenen Argumenten. */
	public void execute(IHTSPlayer player) throws ExecutionException 
	{
		try {
			// wir mssen auch noch den HTSCommandExecutor als Listener, weil der sich sonst nicht beendet
			// und somit der Prozess des Testobjektes am Ende nicht geschlossen wird.
			player.getTestObject().start(20000, args, null, HTSCommandExecutor.instance, null);
		}
		catch(AtosJException e) {
			throw new ExecutionException( this, e.getMessage() );
		}
		catch(RemoteException e) // der Player im AtosJ Prozess ist tot
		{
			// wir machen einfach nix
			// throw new ExecutionException( this, e.getMessage() );
		}
	}
	
	public Object clone() {
		try {
			return new HTSStart( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
