/*
 * Created on 08.11.2005
 */
package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;
import atosj.util.WindowMonitor;

/**
 * Klasse zum Parsen und Ausfhren eines HTS WINDOWEXISTS Kommandos.
 * @author Nicos Tegos
 */
public class HTSWindowExists extends HTSCommand
{
	private String windowTitle; // Fenstertitel
	private boolean isVisible; // Sollwert fr die Existenz
	
	/**
	 * Erstellt ein neues WINDOWEXISTS-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSWindowExists(CSVLine line) throws ParseException 
	{
		super(line);
		
		// <FENSTER>, <ABFRAGE>
		windowTitle = getParamUnquoted(0, Messages.getString("HTSWindowExists.0")); //$NON-NLS-1$
		
		if( getParam(1, Messages.getString("HTSWindowExists.1")).equals(HTSSyntax.WEX_YES) ) //$NON-NLS-1$
			isVisible = true;
		else
		if( getParam(1, Messages.getString("HTSWindowExists.2")).equals(HTSSyntax.WEX_NO) ) //$NON-NLS-1$
			isVisible = false;
		else
		{
			throw new InvalidParamException(line, 1, getParam(1, Messages.getString("HTSWindowExists.3")), //$NON-NLS-1$
					Messages.getString("HTSWindowExists.4") + HTSSyntax.WEX_YES + Messages.getString("HTSWindowExists.5") + HTSSyntax.WEX_NO + "."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		
		assertParamSize(2);
	}

	/** berprft, ob das Fenster existiert. Momentan ist das eher eine berprfung,
	 * ob das Fenster schon mal geffnet wurde und sichtbar ist.
	 * TODO vielleicht Kommando streichen */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		if(isVisible)
		{
			if( WindowMonitor.getInstance().getWindow(windowTitle) == null )
				throw new ExecutionException(this, Messages.getString("HTSWindowExists.7") + windowTitle + Messages.getString("HTSWindowExists.8")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		else
		{
			if( WindowMonitor.getInstance().getWindow(windowTitle) != null )
				throw new ExecutionException(this, Messages.getString("HTSWindowExists.9") + windowTitle + Messages.getString("HTSWindowExists.10")); //$NON-NLS-1$ //$NON-NLS-2$
		}

	}
	
	public Object clone() {
		try {
			return new HTSWindowExists( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
