package atosj.model.hts.command;

import java.util.Vector;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Diese Klasse reprsentiert ein ungltiges Kommando.
 * Ein Kommando ist ungltig, wenn die Syntax der
 * kommaseparierten Liste verletzt wurde oder, wenn
 * die Parameter des Kommandos fehlerhaft sind. 
 * @author Nicos Tegos
 */
public class InvalidCommand extends HTSCommand 
{
	private ParseException error; // Fehler der dieses Kommando ungltig gemacht hat
	
	/**
	 * Erstellt ein neues ungltiges Kommando.
	 * @param e Die Ausnahme, die das Kommando ungltig gemacht hat.
	 * @throws ParseException kann nie passieren, ist aber auf Grund
	 * der Vererbung da.
	 */
	public InvalidCommand(ParseException e) throws ParseException {
		super( new CSVLine(new Vector()) ); // leere Zeile
		error = e;
		
		setError( e.getMessage() );
	}

	/** Wirt immer eine Ausnahme. */
	public void execute(IHTSPlayer player) throws ExecutionException {
		throw new ExecutionException( this, Messages.getString("InvalidCommand.0") + error.getMessage() ); //$NON-NLS-1$
	}
	
	/** Liefert den String des Kommandos, so wie es eingegeben wurde. */
	public String toString() {
		return error.getOrigLine();
	}
	
	public Object clone() {
		try {
			return new InvalidCommand(error);
		} catch(ParseException e) {
			return null;
		}
	}
}
