package atosj.model.recorder;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.CommandReceiverListener;
import atosj.listener.StreamListener;
import atosj.model.hts.command.HTSCommand;
import atosj.rmi.BidirectionalReceiver;

/**
 * Diese Klasse empfngt zur Laufzeit des Testobjekts durch Capture
 * generierte TEST und ACTION Kommandos und leitet sie weiter.
 * @author Nicos Tegos
 * @see atosj.model.recorder.ScriptRecorder
 */
public class HTSCommandReceiver	extends BidirectionalReceiver implements IHTSCommandReceiver 
{
	private boolean pausePublish = false; // wenn true werden empfangene Kommandos nicht in das Skript eingefgt
	
	public HTSCommandReceiver(TestObject testee) throws RemoteException	{
		super(testee);
	}

	/**
	 * Startet das Testobjekt und initialisiert das Capture im Remoteprozess.
	 * @throws AtosJException wenn Fehler beim Start auftreten, z.B. Main-Klasse nicht
	 * gefunden, Hauptfenster hat sich nicht geffnet usw. Im Remoteprozess
	 * wird eine Instanz des Interfaces <code>IScriptRecorder</code> in der RMI-Registry
	 * registriert. Diese sorgt fr die bertragung der gecaptureten Kommandos.
	 * @see ScriptRecorder
	 */
	public void start() throws AtosJException {
		StreamListener listenerStd  = new StreamListener() {
			private StringBuffer buff = new StringBuffer();
			public void charsReceived(char[] chars) {
				buff.append(chars);					
			}

			public void streamFinished() {
				System.out.println(buff);
			}
			
		};
		StreamListener listenerErr  = new StreamListener() {
			private StringBuffer buff = new StringBuffer();
			public void charsReceived(char[] chars) {
				buff.append(chars);					
			}

			public void streamFinished() {
				System.out.println(buff);
			}
			
		};
		
		super.start(ScriptRecorder.class, null, listenerStd, listenerErr);
		try 
		{
			IScriptRecorder recorder = (IScriptRecorder) getRemoteObject();
			recorder.start( getTestObject() );
		} 
		catch(AtosJException e) {
			stop();
			throw e;
		} catch(RemoteException e) {
			stop();
			throw new AtosJException(Messages.getString("HTSCommandReceiver.0") + e.getMessage()); //$NON-NLS-1$
		}
	}
	
	/**
	 * Beendet den <code>CommandReceiver</code>.
	 * @see BidirectionalReceiver#stop()
	 */
	public void clientShutdown() throws RemoteException {
		stop();
	}

	/**
	 * Empfngt ein gecapturetes Kommando und informiert alle Listener darber.
	 * @param command Das gecapturete Kommando.
	 */
	public void publishHTSCommand(final HTSCommand command) throws RemoteException {
		if(pausePublish)
			return;
		
		Display.getDefault().syncExec( 
				new Runnable() {
					public void run() {	
						// listener benachrichtigen
						for(int i = 0; i < listeners.size(); ++i)
						{
							if( listeners.get(i) instanceof CommandReceiverListener )
								((CommandReceiverListener)listeners.get(i)).commandReceived(command);
						}
					}
				}
			);
	}
	
	/** Verhindert die Weiterleitung empfangener Kommandos an die Listener.
	 * @param pause wenn <code>true</code> wird die Weiterleitung blockiert,
	 * sonst wird sie wieder aufgenommen.
	 */
	public void pausePublish(boolean pause) {
		pausePublish = pause;
	}
	
	public void debugMessage(String debugString) throws RemoteException {
		System.out.println(Messages.getString("HTSCommandReceiver.1")+debugString); //$NON-NLS-1$
	}
}
 