/*
 * Created on 25.09.2005
 */
package atosj.model.recorder;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.FinishedListener;

/**
 * Diese Klasse wird durch einen <code>HTSCommandReceiver</code> gestartet.
 * Sie startet das Testobjekt und initialisiert den Capture-Mechanismus.
 * Gecapturete Kommandos werden an den Partner im ATOSj Prozess weitergeleitet.
 * This class is not intended to be run solely.
 * @see atosj.model.recorder.HTSCommandReceiver
 */
public class ScriptRecorder extends UnicastRemoteObject implements IScriptRecorder, FinishedListener
{
	protected TestObject testee;
	protected IHTSCommandReceiver receiver;
	
	/**
	 * Einen neuen Recorder fr Kommandos.
	 * @param _receiver Der Partner im ATOSj Prozess, dem gecapturete Kommandos
	 * bermittelt werden sollen.
	 */
	public ScriptRecorder(IHTSCommandReceiver _receiver) throws RemoteException
	{
		super();
		this.receiver = _receiver;
		// dem receiver melden wenn der Prozess beendet wird.
		Runtime.getRuntime().addShutdownHook(new Thread(){
			public void run() {
				try {
					if(receiver != null)
						receiver.clientShutdown();
				} catch (RemoteException e) {System.err.println(e.getMessage());}
			}
		});
	}
		
	public void start(TestObject testee) throws AtosJException {
		this.testee = testee;
		testee.start(10000, null, null, this, receiver);
	}
	
	/**
	 * Ist der letzte Thread des Testobjekts beendet, wird dieses Objekt
	 * aus der RMI-Registry entfernt, damit der Prozess beendet werden kann.
	 */
	public void testObjectFinished() 
	{
		try {
			// wir mssen uns abmelden, sonst wird der Prozess nicht beendet
			UnicastRemoteObject.unexportObject(this, true);
		} 
		catch (Exception e) {/*egal*/}
	}
	
	/**
	 * Startet das Testobjekt und den Recorder. Der Rekorder registriert
	 * sich unter dem angegebenen Namen in der RMI-Registry.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Der Name des Kommunikationspartners
	 * (<code>IHTSCommandReceiver</code> in der RMI-Registry.</li>
	 * <li>args[1] - Der Name unter dem sich der Recorder in der
	 * RMI-Registry registrieren soll.</li>
	 * </ul>
	 * @see IHTSCommandReceiver
	 */
	public static void main(String[] args) 	{
		if (args.length != 2)	{
			System.err.println(Messages.getString("ScriptRecorder.0")); //$NON-NLS-1$
			System.exit(1);	
		}
		
		try 
		{
			IHTSCommandReceiver receiver = (IHTSCommandReceiver) Naming.lookup(args[0]);
			ScriptRecorder recorder = new ScriptRecorder(receiver);
			// wir fgen den recorder in die RMI Registry ein.
			Naming.rebind(args[1], recorder);
		} 
		catch (Exception e) 
		{
			System.err.println(e.getMessage());
			System.exit(1);
		}
	}
}
