package atosj.model.urf;

import java.rmi.RemoteException;
import java.util.Vector;

import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.StreamListener;
import atosj.listener.URFReceiverListener;
import atosj.rmi.BidirectionalReceiver;

/**
 * Empfng alle <code>ComponentIds</code>, die durch den <code>URFGenerator</code>
 * erstellt wurden.
 * @author Nicos Tegos
 */
public class URFReceiver 
		extends BidirectionalReceiver
		implements IURFReceiver {
	
	/**
	 * Erstellt einen neuen Empfnger fr <code>ComponentIds</code>.
	 * @param testee Das Testobjekt, fr das IDs ermittelt werden sollen.
	 * @throws RemoteException
	 */
	public URFReceiver(TestObject testee) throws RemoteException {
		super(testee);
	}
	
	/**
	 * Startet den <code>URFGenerator</code> und registriert sich in der RMI-Registry.
	 */
	public void start() throws AtosJException
	{
		super.start(URFGenerator.class, null,  new StreamListener() {

			StringBuffer str = new StringBuffer();
			public void charsReceived(char[] chars) {
				str.append(chars);
				
			}

			public void streamFinished() {
				System.out.println(str);
			}
			
		}, new StreamListener() {

			StringBuffer str = new StringBuffer();
			public void charsReceived(char[] chars) {
				str.append(chars);
				
			}

			public void streamFinished() {
				System.out.println(str);
			}
			
		});
		IURFGenerator generator = (IURFGenerator) getRemoteObject();
		try {
			generator.start( getTestObject() );
		} catch(AtosJException e) {
			stop();
			throw e;
		} catch(RemoteException e) {
			stop();
			throw new AtosJException(Messages.getString("URFReceiver.0") + e.getMessage()); //$NON-NLS-1$
		}
	}

	/**
	 * Empfngt durch den <code>URFGenerator</code> erstellte IDs
	 * unt leitet sie an die <code>URFReceiverListener</code> weiter.
	 * @param ids Vector(ComponentId) Generierte IDs.
	 */
	public void publishIds(final Vector ids) throws RemoteException
	{
		Runnable run = new Runnable() {
			public void run() {
				for(int i = 0; i < listeners.size(); ++i)
					((URFReceiverListener)listeners.get(i)).idsPublished(ids);
			}
		};
		
		Display.getDefault().asyncExec(run);
	}

	/**
	 * Beendet den <code>URFReceiver</code>.
	 * @see BidirectionalReceiver#stop()
	 */
	public void clientShutdown() throws RemoteException {
		stop();
	}
}
