/*
 * Created on 05.09.2005
 */
package atosj.rmi;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.listener.FinishedListener;
import atosj.model.hts.command.HTSCommand;

/**
 * Diese Klasse fhrt einzelne HTS Kommandos im Prozess des Testobjektes aus.
 * @author Nicos Tegos
 * @see atosj.rmi.HTSPlayer
 */
public class HTSCommandExecutor extends UnicastRemoteObject implements ICommandExecutor, FinishedListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private IHTSPlayer receiver;
	public static HTSCommandExecutor instance;
	
	/**
	 * Erstellt einen neuen Kommando Ausfhrer.
	 * @param receiver Ein <code>IHTSPlayer</code> aus dem ATOSj Prozess,
	 * der die Ausfhrung von Kommandos veranlat.
	 */
	public HTSCommandExecutor(IHTSPlayer receiver) throws RemoteException	{
		this.receiver = receiver;
		
		instance = this;
	}
	
	public void executeCommand(HTSCommand cmd) throws RemoteException, ExecutionException {
		cmd.execute(receiver);		
	}
	
	/** Entfernt dieses Objekt aus der RMI-Registry damit der Prozess beendet wird. */
	public void testObjectFinished() 
	{
		try {
			// wir mssen uns abmelden, sonst wird der Prozess nicht beendet
			UnicastRemoteObject.unexportObject(this, true);
		} 
		catch (Exception e) {/*egal */}
	}
		
	/**
	 * Startet den <code>HTSCommandExecutor</code> und registriert
	 * ihn unter dem angegebenen Namen in der RMI-Registry.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Der RMI-Name des Kommunikationspartners
	 * (<code>IHTSPlayer</code> im ATOSj Prozess.</li>
	 * <li>args[1] - Der Name unter dem sich <code>HTSCommandExecutor</code> in der
	 * RMI-Registry registrieren soll.</li>
	 * </ul>
	 * @see IHTSPlayer
	 */
	public static void main(String[] args) 
	{
		if (args.length != 2)	
		{
			System.err.println(Messages.getString("HTSCommandExecutor.0")); //$NON-NLS-1$
			System.exit(1);	
		}
		
		try 
		{
			final IHTSPlayer receiver = (IHTSPlayer) Naming.lookup(args[0]);
			final HTSCommandExecutor player = new HTSCommandExecutor(receiver);
			Naming.rebind(args[1], player);
		}
		catch(Throwable e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	/**
	 * Setzen des Pfades zu 'user.prf' fr die eigene UserPreferences-Instanz
	 */
	public void setPrefPath(String path) throws RemoteException {
			Messages.setPrefPath(path);
		
	}
}
