package atosj.rmi;

import java.rmi.Remote;
import java.rmi.RemoteException;

import atosj.exception.ExecutionException;
import atosj.model.hts.command.HTSCommand;

/**
 * Sorgt fr die Ausfhrung einzelner HTS Kommandos im Prozess des Testobjektes.
 * @author Nicos Tegos
 * @see atosj.rmi.IHTSPlayer
 */
public interface ICommandExecutor extends Remote 
{	
	/**
	 * Fhrt Kommandos im Prozess des Testobjektes aus, die nicht im ATOSj Prozess
	 * ausgefhrt werden knnen, weil sie direkt auf GUI Elemente zugreifen. Dazu
	 * gehren z.B. <code>HTSRead</code> oder <code>HTSTest</code>.
	 * @param cmd Das auszufhrende Kommando.
	 * @throws RemoteException Bei Problemen mit RMI oder wenn <code>System.exit</code> gerufen wurde.
	 * @throws ExecutionException Weitergeleitet von <code>HTSCommand.execute(IHTSPlayer)</code>
	 */
	void executeCommand(HTSCommand cmd) throws RemoteException, ExecutionException;
	
	/**
	 * Setzt den Pfad zu 'user.prf' in der Testobjektumgebung.
	 * @param path
	 */
	void setPrefPath(String path) throws RemoteException;
}
