/*
 * Created on 11.04.2006
 */
package atosj.rmi;

import java.lang.reflect.Method;
import java.util.Vector;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.Loader;
import javassist.NotFoundException;
import javassist.Translator;
import acover.RuntimeCoverageGenerator;
import acover.RuntimeCoverageTranslator;
import atosj.util.swteventmonitor.SWTElementTranslator;

/**
 * Diese Klasse delegiert die Aufrufe des Interfaces Translator an
 * die enthaltenen Translatoren. Es wird in der Reihenfolge vorgegangen,
 * in der die Translatoren hinzugefgt wurden. Es sind Wechselwirkungen
 * zu beachten, wenn zwei Translatoren nderungen an derselben Klasse vornehmen.
 * @author Nicos Tegos
 * @see javassist.Translator
 */
class MultiTranslator implements Translator
{
	private Vector translators = new Vector(); // Vector<Translator>
	
	public void addTranslator(Translator t) {
		translators.add(t);
	}
	
	public void start(ClassPool pool) throws NotFoundException, CannotCompileException {
		for(int i = 0; i < translators.size(); ++i)
			((Translator)translators.get(i)).start(pool);
	}

	public void onLoad(ClassPool pool, String className) throws NotFoundException, CannotCompileException {
		for(int i = 0; i < translators.size(); ++i)
			((Translator)translators.get(i)).onLoad(pool, className);
	}
}

/**
 * Diese Klasse startet alle entfernten Prozesse, die das Testobjekt starten sollen.
 * Sie initialisiert bei Bedarf den javassist Klassenlader und startet danach die
 * eigentliche Main-Funktion. Grund fr diesen zustzlichen Schritt ist, dass
 * der javassist Klassenlader vor allen anderen Klassen geladen werden muss.
 * 
 * @author Nicos Tegos
 * @see javassist.Loader
 */
public class ProcessRunner {

	/**
	 * @param args 
	 * <ul>
	 * <li>args[0] Projekttyp "swt" oder "awt"</li>
	 * <li>args[1] Option "-cover", wenn berdeckungsanalyse gewnscht. 
	 * Sonst der Name der auszhrenden Main-Klasse.
	 * </li>
	 * <li>args[2] wenn die Option "-cover" aktiv ist, dann der vollstndige Pfad der XML-Datei.</li>
	 * <li> nach dem Namen der Main-Klasse folgen die Parameter fr die Ausfhrung der Main-Klasse.</li>
	 * </ul>
	 */
	public static void main(String[] args) {
		try	
		{
			// soll eine berdeckungsanalyse stattfinden muss der Pfad der XML Datenbasis 
			//angegeben und der entsprechende Translator gesetzt werden.
			String projectType;
			String mainClassName;
			String[] realArgs;
			
			String coverPath = null;
			if(args.length > 2)
			{
				if(args[0].equals("-cover") )
					coverPath = args[1];
			}
			
			if(coverPath != null)
			{
				projectType = args[2];
				mainClassName = args[3];
				
				realArgs = new String[args.length-4];
				System.arraycopy(args, 4, realArgs, 0, realArgs.length);
			}
			else
			{
				projectType = args[0];
				mainClassName = args[1];
				
				realArgs = new String[args.length-2];
				System.arraycopy(args, 2, realArgs, 0, realArgs.length);
			}
			
			if( projectType.equals("swt") )
			{
				Loader loader = new Loader();
								
				if(coverPath != null)
				{
					new RuntimeCoverageGenerator(coverPath);
					MultiTranslator mt = new MultiTranslator();
					mt.addTranslator( new SWTElementTranslator() );
					mt.addTranslator( new RuntimeCoverageTranslator() );
					
					loader.delegateLoadingOf( RuntimeCoverageGenerator.class.getName() );
					loader.addTranslator( ClassPool.getDefault(), mt );
				}
				else
				{
					loader.addTranslator( ClassPool.getDefault(), new SWTElementTranslator() );
				}
				
				
				loader.run(mainClassName, realArgs);
			}
			else // swing
			{
				if(coverPath != null)
				{
					Loader loader = new Loader();
					new RuntimeCoverageGenerator(coverPath);
					loader.addTranslator( ClassPool.getDefault(), new RuntimeCoverageTranslator() );
					loader.delegateLoadingOf( RuntimeCoverageGenerator.class.getName() );
					loader.run(mainClassName, realArgs);
				}
				else
				{
					Class mainClass = Class.forName(mainClassName);
					Method mainMethod = mainClass.getMethod( "main", new Class[] {String[].class} );
					mainMethod.invoke( null, new Object[] {realArgs} );
				}
			}
		}
		catch(Throwable e) { 
			e.printStackTrace();
		}
	}

}
