/*
 * Created on 13.06.2006
 */
package atosj.rmi;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.FinishedListener;

/**
 * Diese Klasse hat die Aufgabe das Testobjekt probehalber zu starten.
 * Luft im Remote Prozess.
 * @author Nicos Tegos
 */
public class TestStarter extends UnicastRemoteObject implements ITestStarter, FinishedListener
{
	private ITestStarterReceiver receiver;
	
	/**
	 * Erstellt einen neuen Startet fr das Testobjekt.
	 * @param _receiver Der Kommunikationspartner im ATOSj Prozess.
	 */
	public TestStarter(ITestStarterReceiver _receiver) throws RemoteException	
	{
		super();
		this.receiver = _receiver;
		// dem receiver melden wenn der Prozess beendet wird.
		Runtime.getRuntime().addShutdownHook(new Thread(){
			public void run() {
				try {
					if(receiver != null)
						receiver.clientShutdown();
				} catch (RemoteException e) {System.err.println(e.getMessage());}
			}
		});
	}

	public void start(TestObject testee) throws AtosJException, RemoteException {
		testee.start(10000, null, null, this, null);		
	}
	
	/** Entfernt sich aus der RMI-Registry, damit sich der Prozess des Testobjekts
	 * beendet kann.
	 */
	public void testObjectFinished() 
	{
		try {
			// wir mssen uns abmelden, sonst wird der Prozess nicht beendet
			UnicastRemoteObject.unexportObject(this, true);
		} 
		catch (Exception e) {/*egal*/}
	}
	
	/**
	 * Startet einen <code>TestStarter</code> und registriert diesen
	 * unter dem angegebenen Namen in der RMI-Registry.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Der Name des Kommunikationspartners
	 * (<code>ITestStarterReceiver</code> in der RMI-Registry.</li>
	 * <li>args[1] - Der Name unter dem sich der <code>TestStarter</code> in der
	 * RMI-Registry registrieren soll.</li>
	 * </ul>
	 * @see ITestStarterReceiver
	 */
	public static void main(String[] args) 
	{
		if (args.length != 2)	{
			System.err.println(Messages.getString("TestStarter.0")); //$NON-NLS-1$
			System.exit(1);	
		}
		
		try 
		{
			ITestStarterReceiver remoteObject = (ITestStarterReceiver) Naming.lookup(args[0]);
			TestStarter starter = new TestStarter(remoteObject);
			// wir fgen den starter in die RMI Registry ein.
			Naming.rebind(args[1], starter);
		}  
		catch (Exception e) 
		{
			System.err.println(e.getMessage());
			System.exit(1);
		}
	}
}

