/*
 * Created on 13.06.2006
 */
package atosj.rmi;

import java.rmi.RemoteException;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;

/**
 * Startet ein Testobjekt probehalber in einem entfernten Prozess.
 * Fehler beim Start geben Aufschluss ber inkorrekte Angabe bzgl.
 * main-Klasse oder Klassenpfad. Bildet den Kommunikationsendpunkt
 * im ATOSj Prozess. Ein <code>TestStarter</code> bildet den
 * Kommunikationspunkt im entfernten Prozess.
 * @author Nicos Tegos
 * @see atosj.rmi.TestStarter
 */
public class TestStarterReceiver extends BidirectionalReceiver implements ITestStarterReceiver 
{
	public TestStarterReceiver() throws RemoteException {
		super(null);
	}

	/**
	 * Stoppt die Ausfhrung des Testobjekts.
	 */
	public void clientShutdown() throws RemoteException {
		stop();
	}
	
	/**
	 * Startet die Ausfhrung des Testobjekts in einem Remote-Prozess.
	 * @param testee Das Testobjekt.
	 * @throws AtosJException Fehler beim Start des Testobjekts, z.B.
	 * falsche main-Klasse, falscher Hauptfenstertitel usw.
	 */
	public void start(TestObject testee) throws AtosJException {
		setTestObject(testee);
		super.start(TestStarter.class, null, null, null);
		
		try 
		{
			ITestStarter starter = (ITestStarter) getRemoteObject();
			starter.start(testee);
		}
		catch(AtosJException e) {
			stop();
			throw e;
		} catch(RemoteException e) {
			stop();
			throw new AtosJException(Messages.getString("TestStarterReceiver.0") + e.getMessage()); //$NON-NLS-1$
		}
	}
}
