/*
 * Created on 09.04.2006
 */
package atosj.util;

import java.awt.Window;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import atosj.component.swing.SwingComponentFinder;
import atosj.component.swing.SwingComponentResolver;
import atosj.component.swing.SwingEventMonitorWrapper;
import atosj.model.recorder.IHTSCommandReceiver;

import com.sun.java.accessibility.util.EventQueueMonitor;
import com.sun.java.accessibility.util.SwingEventMonitor;
import com.sun.java.accessibility.util.TopLevelWindowListener;

/**
 * Ein <code>WindowMonitor</code> fr die AWT/Swing Architektur.
 * 
 * @author Nicos Tegos
 */
public class AWTWindowMonitor extends WindowMonitor implements TopLevelWindowListener, ComponentListener
{
	/**
	 * Erstellt den Monitor.
	 * @param resolver bernimmt die Zuordnung der Components eines Fensters
	 * zu den entsprechenden ATOSj Components.
	 * @param receiver Falls das Testobjekt im Capture Modus gestartet wurde, muss
	 * hier der Empfnger der generierten Kommandos aus dem ATOSj Prozess bergeben werden,
	 * sonst <code>null</code>.
	 */
	protected AWTWindowMonitor(SwingComponentResolver resolver, IHTSCommandReceiver receiver) {
		super( new SwingComponentFinder(resolver, receiver), receiver != null ? new SwingEventMonitorWrapper() : null);
		
		EventQueueMonitor.maybeInitialize();
		EventQueueMonitor.addTopLevelWindowListener(this);
		SwingEventMonitor.addComponentListener(this);
		Window[] initialWindows = EventQueueMonitor.getTopLevelWindows();
		for(int i = 0; i < initialWindows.length; ++i)
			addWindow(initialWindows[i]);
	}
	
	/**
	 * Initialisiert die Singleton-Instanz des Monitors. Dieser Aufruf muss
	 * vor dem Start des Testobjekts geschehen.
	 * @param resolver bernimmt die Zuordnung der Components eines Fensters
	 * zu den entsprechenden ATOSj Components.
	 * @param receiver Falls das Testobjekt im Capture Modus gestartet wurde, muss
	 * hier der Empfnger der generierten Kommandos aus dem ATOSj Prozess bergeben werden,
	 * sonst <code>null</code>.
	 */
	public static void createInstance(SwingComponentResolver resolver, IHTSCommandReceiver receiver) {
		setInstance( new AWTWindowMonitor(resolver, receiver) );
	}

	/**
	 * siehe <code>addWindow</code>.
	 * @see WindowMonitor#addWindow(Object)
	 */
	public void topLevelWindowCreated(Window w) {
		addWindow(w);
	}
	
	/**
	 * siehe <code>removeWindow</code>.
	 * @see WindowMonitor#removeWindow(Object)
	 */
	public void topLevelWindowDestroyed(Window w) {
		removeWindow(w);
	}
	
	/**
	 * Wenn es sich bei dem versteckten Component
	 * um ein Fenster (<code>java.awt.Window)</code>handelt,
	 * dann wird es aus der Liste der Fenster entfernt.
	 * @see WindowMonitor#removeWindow(Object)
	 */
	public void componentHidden(ComponentEvent e) 
	{
		if (e.getSource() instanceof Window) 
			removeWindow( e.getSource() );
	}
	
	/**
	 * Wenn es sich bei dem sichtbar gemachten Component
	 * um ein Fenster (<code>java.awt.Window)</code>handelt,
	 * dann wird es aus der Liste der Fenster entfernt.
	 * @see WindowMonitor#addWindow(Object)
	 */
	public void componentShown(ComponentEvent e) {
		if (e.getSource() instanceof Window)
			addWindow( e.getSource() );
	}
	
	/** Leere Implementation */
	public void componentMoved(ComponentEvent e) {}
	
	/** Leere Implementation */
	public void componentResized(ComponentEvent e) {}
}
