/*
 * Created on 09.04.2006
 */
package atosj.util;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.TableTreeItem;
import org.eclipse.swt.widgets.Caret;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;

import atosj.component.swt.SWTComponentFinder;
import atosj.component.swt.SWTComponentResolver;
import atosj.component.swt.SWTEventMonitorWrapper;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.swteventmonitor.SWTEventMonitor;
import atosj.util.swteventmonitor.TopLevelWindowListener;

/**
 * Ein <code>WindowMonitor</code>fr die SWT Architektur.
 * @author Nicos Tegos
 */
public class SWTWindowMonitor extends WindowMonitor implements TopLevelWindowListener
{
	/**
	 * Erstellt den Monitor.
	 * @param resolver bernimmt die Zuordnung der Components eines Fensters
	 * zu den entsprechenden ATOSj Components.
	 * @param receiver Falls das Testobjekt im Capture Modus gestartet wurde, muss
	 * hier der Empfnger der generierten Kommandos aus dem ATOSj Prozess bergeben werden,
	 * sonst <code>null</code>.
	 */
	protected SWTWindowMonitor(SWTComponentResolver resolver, IHTSCommandReceiver receiver) 
	{
		super( new SWTComponentFinder(resolver, receiver), receiver != null ? new SWTEventMonitorWrapper() : null );
		
		SWTEventMonitor.addTopLevelWindowListener(this);
		Set initialWindows = SWTEventMonitor.getTopLevelWindows();
		for(Iterator it = initialWindows.iterator(); it.hasNext(); )
			addWindow( it.next() );
	}
	
	/**
	 * Initialisiert die Singleton-Instanz des Monitors. Dieser Aufruf muss
	 * vor dem Start des Testobjekts geschehen.
	 * @param resolver bernimmt die Zuordnung der Components eines Fensters
	 * zu den entsprechenden ATOSj Components.
	 * @param receiver Falls das Testobjekt im Capture Modus gestartet wurde, muss
	 * hier der Empfnger der generierten Kommandos aus dem ATOSj Prozess bergeben werden,
	 * sonst <code>null</code>.
	 */
	public static void createInstance(SWTComponentResolver resolver, IHTSCommandReceiver receiver) {
		setInstance( new SWTWindowMonitor(resolver, receiver) );
	}

	/**
	 * Liefert das Fenster zu einem Component.
	 * @param widget Das Component.
	 * @return Das Fenster, in dem sich das Component befindet.
	 */
	public static Shell getShell(Widget widget)	{
		Control control = null;
		
		if (widget instanceof Control)
			control = (Control) widget;
		else if (widget instanceof Caret)	{
			control = ((Caret)widget).getParent();
		}
		else if (widget instanceof CoolItem)	{
			control = ((CoolItem)widget).getParent();
		}
		else if (widget instanceof Menu)	{
			control = ((Menu)widget).getParent();
		}
		else if (widget instanceof MenuItem)	{
			control = ((MenuItem)widget).getParent().getParent();
		}
		else if (widget instanceof ScrollBar)	{
			control = ((ScrollBar)widget).getParent();
		}
		else if (widget instanceof TabItem)	{
			control = ((TabItem)widget).getParent();
		}
		else if (widget instanceof TableColumn)	{
			control = ((TableColumn)widget).getParent();
		}
		else if (widget instanceof TableItem)	{
			control = ((TableItem)widget).getParent();
		}
		else if (widget instanceof ToolItem)	{
			control = ((ToolItem)widget).getParent();
		}
		else if (widget instanceof TreeColumn)	{
			control = ((TreeColumn)widget).getParent();
		}
		else if (widget instanceof TreeItem)	{
			control = ((TreeItem)widget).getParent();
		}
		else if (widget instanceof CTabItem)	{
			control = ((CTabItem)widget).getParent();
		}
		else if (widget instanceof TableTreeItem)	{
			control = ((TableTreeItem)widget).getParent();
		}
		
		//CTabItem, ControlEditor, TableTreeItem
		//FIXME was bei Tracker, Tray, TrayItem, PopupList?
		
		//neue Funktion getShell(ControlEditor)
		
		return (Shell) control.getShell();
	}

	/**
	 * siehe <code>addWindow</code>.
	 * @see WindowMonitor#addWindow(Object)
	 */
	public void newTopLevelWindowActivated(Shell shell) {
		addWindow(shell);
	}

	/**
	 * siehe <code>removeWindow</code>.
	 * @see WindowMonitor#removeWindow(Object)
	 */
	public void topLevelWindowDisposed(Shell shell) {
		removeWindow(shell);
	}

}
