/*
 * Created on 27.11.2005
 */
package atosj.util;

import java.io.Serializable;

/**
 * Das Resultat der Ausfhrung eines Objektes.
 * @author Nicos Tegos
 */
public class Statistics implements Serializable
{
	public static final int STAT_ERR = 0;
	public static final int STAT_PASSED = 1;
	public static final int STAT_NOT_RUN = 2;
	public static final int STAT_IS_RUNNING = 3;
	
	private int line;
	private int duration = 0;
	private int status = STAT_NOT_RUN;
	private String error = "";
	
	/**
	 * Ein leeres Resultat, das keine Ausfhrung anzeigt.
	 * @param line Der Index des ausgefhrten Elementes 0-basiert, z.B. der Index
	 * des Kommandos in seinem Script.
	 */
	public Statistics(int line) 
	{
		this.line = line;
	}
	
	/**
	 * Ein Resultat, das eine fehlerhafte Ausfhrung anzeigt.
	 * @param line Der Index des ausgefhrten Elementes 0-basiert, z.B. der Index
	 * des Kommandos in seinem Script.
	 * @param duration Die Dauer der Ausfhrung in ms.
	 * @param error Eine Beschreibung des Fehlers.
	 */
	public Statistics(int line, int duration, String error) {
		this(line);
		status = STAT_ERR;
		this.error = error;
		this.duration = duration;
	}
	
	/**
	 * Ein Resultat, das eine korrekte Ausfhrung anzeigt.
	 * @param line Der Index des ausgefhrten Elementes 0-basiert, z.B. der Index
	 * des Kommandos in seinem Script.
	 * @param duration Die Dauer der Ausfhrung in ms.
	 */
	public Statistics(int line, int duration) {
		this(line);
		status = STAT_PASSED;
		this.duration = duration;
	}
	
	/**
	 * Liefert die Ausfhrungszeit.
	 */
	public int getDuration() {
		return duration;
	}
	
	/**
	 * Gibt an ob die Ausfhrung fehlerhaft war.
	 * @return <code>true</code>, wenn dieses dieses Resultat einen Fehler anzeigt.
	 * @see #getError()
	 */
	public boolean hasError() {
		return status == STAT_ERR;
	}
	
	/**
	 * Liefert die Fehlerbeschreibung falls vorhanden, <code>null</code> sonst.
	 */
	public String getError() {
		return error;
	}
	/**
	 * Liefert den Index des Kommandos, 0-basiert.
	 */
	public int getLine() {
		return line;
	}
}
