package atosj.util.swteventmonitor;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.Modifier;
import javassist.NotFoundException;
import javassist.Translator;

/**
 * Modifiziert die SWT Klassen beim Laden so, dass
 * ein universaler und globaler Listener in jedem Konstruktor
 * angehangen wird.
 * @author Volker Janetschek
 */
public class SWTElementTranslator implements Translator {
	
	String start = "try{";
	String end ="}catch(Throwable t){}";
	
	private String getTryCatchString(String s)	{ return start + s + end; }
	
    public void start(ClassPool pool)	throws NotFoundException, CannotCompileException {}
    public void onLoad(ClassPool pool, String classname)	throws NotFoundException, CannotCompileException
    {
   	try	{

////////////////////////////////////////   		
   		
   		if (classname.equals("org.eclipse.swt.widgets.Display"))	{

    		CtClass cc = pool.get(classname);
    		CtMethod method = cc.getMethod("post", "(Lorg/eclipse/swt/widgets/Event;)Z");

    		method.insertAt(2871, "inputs.wVk = 0;" +
    				"inputs.wScan = key;");

 		
    		method.insertAt(2877, "inputs.dwFlags |= 4;");
   		}
//////////////////////////////////////////////
   		
   		if (classname.equals("org.eclipse.swt.widgets.Widget"))	{
    		CtClass cc = pool.get(classname);
    		CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Widget;I)V");
			constructor.insertAfter(getTryCatchString("addDisposeListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackDisposeListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Shell"))	{  
            CtClass cc = pool.get(classname);
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Display;I)V");
			constructor1.insertAfter(getTryCatchString("atosj.util.swteventmonitor.SWTEventMonitor.Internal.newShellInstance(this);" +
					"org.eclipse.swt.graphics.Rectangle bounds = getBounds();bounds.y = 0;setBounds(bounds);"));
			//constructor1.insertAfter(getTryCatchString("addShellListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackShellListener);");

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Shell;I)V");
			constructor2.insertAfter(getTryCatchString("atosj.util.swteventmonitor.SWTEventMonitor.Internal.newShellInstance(this);" +
					"org.eclipse.swt.graphics.Rectangle bounds = getBounds();bounds.y = 0;setBounds(bounds);"));
			//constructor2.insertAfter(getTryCatchString("addShellListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackShellListener);");
    	}
   		
   		else if (classname.equals("org.eclipse.swt.widgets.Button"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Combo"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addModifyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackModifyListener);"));
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
			constructor.insertAfter(getTryCatchString("addVerifyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackVerifyListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Control"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor.insertAfter(getTryCatchString("addFocusListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackFocusListener);"));
			constructor.insertAfter(getTryCatchString("addHelpListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackHelpListener);"));
			constructor.insertAfter(getTryCatchString("addKeyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackKeyListener);"));
			constructor.insertAfter(getTryCatchString("addMouseListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackMouseListener);"));
			constructor.insertAfter(getTryCatchString("addMouseMoveListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackMouseMoveListener);"));
			constructor.insertAfter(getTryCatchString("addMouseTrackListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackMouseTrackListener);"));
			constructor.insertAfter(getTryCatchString("addPaintListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackPaintListener);"));
			constructor.insertAfter(getTryCatchString("addTraverseListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackTraverseListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.CoolItem"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/CoolBar;I)V");
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/CoolBar;II)V");
			constructor2.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Link"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.List"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Menu"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Decorations;I)V");
			constructor.insertAfter(getTryCatchString("addHelpListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackHelpListener);"));
			constructor.insertAfter(getTryCatchString("addMenuListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackMenuListener);"));
			
			// wir ndern die sichtbarkeit der methode getBounds auf true, weil wir die zum clicken brauchen
			cc.getMethod("getBounds", "()Lorg/eclipse/swt/graphics/Rectangle;").setModifiers(Modifier.PUBLIC);
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.MenuItem"))	{
    		CtClass cc = pool.get(classname);

    		CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Menu;I)V");
			constructor1.insertAfter(getTryCatchString("addArmListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackArmListener);"));
			constructor1.insertAfter(getTryCatchString("addHelpListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackHelpListener);"));
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Menu;II)V");
			constructor2.insertAfter(getTryCatchString("addArmListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackArmListener);"));
			constructor2.insertAfter(getTryCatchString("addHelpListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackHelpListener);"));
			constructor2.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
			
			// wir ndern die sichtbarkeit der methode getBounds auf true, weil wir die zum clicken brauchen
			cc.getMethod("getBounds", "()Lorg/eclipse/swt/graphics/Rectangle;").setModifiers(Modifier.PUBLIC);
   		}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Sash"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Scale"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.ScrollBar"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Scrollable;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Slider"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Spinner"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addModifyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackModifyListener);"));
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.TabFolder"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Table"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.TableColumn"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Table;I)V");
			constructor1.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Table;II)V");
			constructor2.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor2.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
	   	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.Text"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addModifyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackModifyListener);"));
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
			constructor.insertAfter(getTryCatchString("addVerifyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackVerifyListener);"));
    	}
    
  		else if (classname.equals("org.eclipse.swt.widgets.ToolItem"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/ToolBar;I)V");
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/ToolBar;II)V");
			constructor2.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
    
  		else if (classname.equals("org.eclipse.swt.widgets.Tracker"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor1.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor1.insertAfter(getTryCatchString("addKeyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackKeyListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Display;I)V");
			constructor2.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor2.insertAfter(getTryCatchString("addKeyListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackKeyListener);"));
    	}
    
   		else if (classname.equals("org.eclipse.swt.widgets.TrayItem"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Tray;I)V");
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}

   		else if (classname.equals("org.eclipse.swt.widgets.Tree"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor = cc.getConstructor("(Lorg/eclipse/swt/widgets/Composite;I)V");
			constructor.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
			constructor.insertAfter(getTryCatchString("addTreeListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackTreeListener);"));
    	}

   		else if (classname.equals("org.eclipse.swt.widgets.TreeColumn"))	{
    		CtClass cc = pool.get(classname);
    		
			CtConstructor constructor1 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Tree;I)V");
			constructor1.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor1.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));

			CtConstructor constructor2 = cc.getConstructor("(Lorg/eclipse/swt/widgets/Tree;II)V");
			constructor2.insertAfter(getTryCatchString("addControlListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackControlListener);"));
			constructor2.insertAfter(getTryCatchString("addSelectionListener(atosj.util.swteventmonitor.SWTEventMonitor.Internal.callBackSelectionListener);"));
    	}
   		
   	//TODO: gesamtes Package org.eclipse.swt.custom 

   	}catch (Exception e){
	   e.printStackTrace();
	   /*bei Fehler Klasse "normal" laden*/
	   }
    }
}