package atosj.util.swteventmonitor;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.swt.events.ArmEvent;
import org.eclipse.swt.events.ArmListener;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.HelpEvent;
import org.eclipse.swt.events.HelpListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.events.ShellListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.events.TreeEvent;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Shell;

/**
 * Zentrale Instanz fr das Hinzufgen beliebiger Listener zu den SWT Components.
 * Das Hinzufgen von Listenern kann nur funktionieren, wenn zuvor die SWT-Klassen
 * mit Hilfe des <code>SWTElementTranslator</code> modifiziert wurden. Ein Listener
 * der sich beim <code>SWTEventMonitor</code> registriert wird von jedem Ereignis
 * des entsprechenden Typs informiert, das auf einem beliebigen Component des
 * Testobjekts aufgetreten ist. Fr alle hier registrierten Listener wird
 * garantiert, dass sie vor allen Listener informiert werden, die sich direkt bei
 * einem Component registriert haben.
 * 
 * @author Volker Janetschek.
 */
public class SWTEventMonitor	{

	/** 
	 * Der Grund, warum die Klasse Internal eine "nested top-Level"-Klasse von 
	 * SWTEventMonitor ist, ist der, dass die Implementations-Funktionen nicht 
	 * direkt von aussen sichtbar sein sollen. Allerdings muss Internal dennoch
	 * public sein, da es in Java keine Friends gibt.
	 * 
	 * In den ...Created()-Funktionen dieser Klasse drfen keine instanceof berprfungen
	 * mit anschlieendem UpCast vorgenommen werden, da der Konstruktor der abgeleiteten
	 * Klasse noch nicht gelaufen ist. Nur die Konstruktoren der super-Klassen und
	 * der Klasse selbst sind fertig!
	 **/
	public static class Internal	{
		
		public static ArmListener callBackArmListener = new ArmListener(){

			public void widgetArmed(ArmEvent event) {
				Object[]listeners = armListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ArmListener currentListener = (ArmListener) listeners[i];
					currentListener.widgetArmed(event);
				}	
			}};
		public static DisposeListener callBackDisposeListener = new DisposeListener(){

			public void widgetDisposed(DisposeEvent event) {
				Object[]listeners = disposeListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					DisposeListener currentListener = (DisposeListener) listeners[i];
					currentListener.widgetDisposed(event);
				}	
			}
		};
		public static ControlListener callBackControlListener = new ControlListener(){

			public void controlMoved(ControlEvent event) {
				Object[]listeners = controlListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ControlListener currentListener = (ControlListener) listeners[i];
					currentListener.controlMoved(event);
				}	
			}

			public void controlResized(ControlEvent event) {
				Object[]listeners = controlListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ControlListener currentListener = (ControlListener) listeners[i];
					currentListener.controlResized(event);
				}	
			}};
		public static FocusListener callBackFocusListener = new FocusListener(){

			public void focusGained(FocusEvent event) {
				Object[]listeners = focusListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					FocusListener currentListener = (FocusListener) listeners[i];
					currentListener.focusGained(event);
				}	
			}

			public void focusLost(FocusEvent event) {
				Object[]listeners = focusListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					FocusListener currentListener = (FocusListener) listeners[i];
					currentListener.focusLost(event);
				}	
			}};
		public static HelpListener callBackHelpListener = new HelpListener(){

			public void helpRequested(HelpEvent event) {
				Object[]listeners = helpListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					HelpListener currentListener = (HelpListener) listeners[i];
					currentListener.helpRequested(event);
				}	
			}};
		public static KeyListener callBackKeyListener = new KeyListener(){

			public void keyPressed(KeyEvent event) {
				Object[]listeners = keyListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					KeyListener currentListener = (KeyListener) listeners[i];
					currentListener.keyPressed(event);
				}	
			}

			public void keyReleased(KeyEvent event) {
				Object[]listeners = keyListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					KeyListener currentListener = (KeyListener) listeners[i];
					currentListener.keyReleased(event);
				}	
			}};
		public static MenuListener callBackMenuListener = new MenuListener(){

			public void menuHidden(MenuEvent event) {
				Object[]listeners = menuListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MenuListener currentListener = (MenuListener) listeners[i];
					currentListener.menuHidden(event);
				}	
			}

			public void menuShown(MenuEvent event) {
				Object[]listeners = menuListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MenuListener currentListener = (MenuListener) listeners[i];
					currentListener.menuShown(event);
				}	
			}};
		public static ModifyListener callBackModifyListener = new ModifyListener(){

			public void modifyText(ModifyEvent event) {
				Object[]listeners = modifyListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ModifyListener currentListener = (ModifyListener) listeners[i];
					currentListener.modifyText(event);
				}	
			}};
		public static MouseListener callBackMouseListener = new MouseListener(){

			public void mouseDoubleClick(MouseEvent event) {
				Object[]listeners = mouseListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseListener currentListener = (MouseListener) listeners[i];
					currentListener.mouseDoubleClick(event);
				}	
			}

			public void mouseDown(MouseEvent event) {
				Object[]listeners = mouseListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseListener currentListener = (MouseListener) listeners[i];
					currentListener.mouseDown(event);
				}	
			}

			public void mouseUp(MouseEvent event) {
				Object[]listeners = mouseListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseListener currentListener = (MouseListener) listeners[i];
					currentListener.mouseUp(event);
				}	
			}};
		public static MouseMoveListener callBackMouseMoveListener = new MouseMoveListener(){

			public void mouseMove(MouseEvent event) {
				Object[]listeners = mouseMoveListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseMoveListener currentListener = (MouseMoveListener) listeners[i];
					currentListener.mouseMove(event);
				}	
			}};
		public static MouseTrackListener callBackMouseTrackListener = new MouseTrackListener(){

			public void mouseEnter(MouseEvent event) {
				Object[]listeners = mouseTrackListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseTrackListener currentListener = (MouseTrackListener) listeners[i];
					currentListener.mouseEnter(event);
				}	
			}

			public void mouseExit(MouseEvent event) {
				Object[]listeners = mouseTrackListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseTrackListener currentListener = (MouseTrackListener) listeners[i];
					currentListener.mouseExit(event);
				}	
			}

			public void mouseHover(MouseEvent event) {
				Object[]listeners = mouseTrackListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					MouseTrackListener currentListener = (MouseTrackListener) listeners[i];
					currentListener.mouseHover(event);
				}	
			}};
		public static PaintListener callBackPaintListener = new PaintListener(){

			public void paintControl(PaintEvent event) {
				Object[]listeners = paintListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					PaintListener currentListener = (PaintListener) listeners[i];
					currentListener.paintControl(event);
				}	
			}};
		public static SelectionListener callBackSelectionListener = new SelectionListener(){

			public void widgetSelected(SelectionEvent event) {
				Object[]listeners = selectionListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					SelectionListener currentListener = (SelectionListener) listeners[i];
					currentListener.widgetSelected(event);
				}	
			}

			public void widgetDefaultSelected(SelectionEvent event) {
				Object[]listeners = selectionListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					SelectionListener currentListener = (SelectionListener) listeners[i];
					currentListener.widgetDefaultSelected(event);
				}	
			}};
		public static ShellListener callBackShellListener = new ShellListener(){

			public void shellActivated(ShellEvent event) {
				Object[]listeners = shellListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ShellListener currentListener = (ShellListener) listeners[i];
					currentListener.shellActivated(event);
				}	
			}

			public void shellClosed(ShellEvent event) {
				Object[]listeners = shellListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ShellListener currentListener = (ShellListener) listeners[i];
					currentListener.shellClosed(event);
				}	
			}

			public void shellDeactivated(ShellEvent event) {
				Object[]listeners = shellListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ShellListener currentListener = (ShellListener) listeners[i];
					currentListener.shellDeactivated(event);
				}	
			}

			public void shellDeiconified(ShellEvent event) {
				Object[]listeners = shellListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ShellListener currentListener = (ShellListener) listeners[i];
					currentListener.shellDeiconified(event);
				}	
			}

			public void shellIconified(ShellEvent event) {
				Object[]listeners = shellListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					ShellListener currentListener = (ShellListener) listeners[i];
					currentListener.shellIconified(event);
				}	
			}};
		public static TraverseListener callBackTraverseListener = new TraverseListener(){

			public void keyTraversed(TraverseEvent event) {
				Object[]listeners = traverseListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					TraverseListener currentListener = (TraverseListener) listeners[i];
					currentListener.keyTraversed(event);
				}	
			}};
		public static TreeListener callBackTreeListener = new TreeListener(){

			public void treeCollapsed(TreeEvent event) {
				Object[]listeners = treeListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					TreeListener currentListener = (TreeListener) listeners[i];
					currentListener.treeCollapsed(event);
				}	
			}

			public void treeExpanded(TreeEvent event) {
				Object[]listeners = treeListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					TreeListener currentListener = (TreeListener) listeners[i];
					currentListener.treeExpanded(event);
				}	
			}};
		public static VerifyListener callBackVerifyListener = new VerifyListener(){

			public void verifyText(VerifyEvent event) {
				Object[]listeners = verifyListeners.toArray();
				for (int i = 0; i < listeners.length; i++) {
					VerifyListener currentListener = (VerifyListener) listeners[i];
					currentListener.verifyText(event);
				}	
			}};


	
		public static void newShellInstance(final Shell shell)	{
			//die Aktionen an der Shell erst vornehmen, wenn sie zum ersten mal aktiviert wurde
			shell.addShellListener(new ShellAdapter(){

				public void shellActivated(ShellEvent e) {
					shell.removeShellListener(this);

					// auch alle Listener der Subclassen hinzufgen, weil Shells eigentlich
					// keine Widgets sin. Es wird nur ein kaskadiertes super() ohne Funktion
					// ausgefhrt.

					// von Widget
					shell.addDisposeListener(SWTEventMonitor.Internal.callBackDisposeListener);
					
					// von Control
					shell.addControlListener(SWTEventMonitor.Internal.callBackControlListener);
					shell.addFocusListener(SWTEventMonitor.Internal.callBackFocusListener);
					shell.addHelpListener(SWTEventMonitor.Internal.callBackHelpListener);
					shell.addKeyListener(SWTEventMonitor.Internal.callBackKeyListener);
					shell.addMouseListener(SWTEventMonitor.Internal.callBackMouseListener);
					shell.addMouseMoveListener(SWTEventMonitor.Internal.callBackMouseMoveListener);
					shell.addMouseTrackListener(SWTEventMonitor.Internal.callBackMouseTrackListener);
					shell.addPaintListener(SWTEventMonitor.Internal.callBackPaintListener);
					shell.addTraverseListener(SWTEventMonitor.Internal.callBackTraverseListener);
					
					// von Shell
					shell.addShellListener(SWTEventMonitor.Internal.callBackShellListener);
										
					shells.add(shell);
					fireTopLevelWindowEvent(shell, true);
					
					shell.addDisposeListener(new DisposeListener(){
						
						public void widgetDisposed(DisposeEvent arg0) {
							shell.removeDisposeListener(this);
							
							shells.remove(shell);
							fireTopLevelWindowEvent(shell, false);
						}});
				}
			});
		}
	}
	
	private static Set armListeners = new HashSet();
	private static Set disposeListeners = new HashSet();
	private static Set controlListeners = new HashSet();
	private static Set focusListeners = new HashSet();
	private static Set helpListeners = new HashSet();
	private static Set keyListeners = new HashSet();
	private static Set menuListeners = new HashSet();
	private static Set modifyListeners = new HashSet();
	private static Set mouseListeners = new HashSet();
	private static Set mouseMoveListeners = new HashSet();
	private static Set mouseTrackListeners = new HashSet();
	private static Set paintListeners = new HashSet();
	private static Set selectionListeners = new HashSet();
	private static Set shellListeners = new HashSet();
	private static Set traverseListeners = new HashSet();
	private static Set treeListeners = new HashSet();
	private static Set verifyListeners = new HashSet();
	
	private static Set shells = new HashSet();
	
	private static Set topLevelWindowListeners = new HashSet();
	
	public static Set getTopLevelWindows()	{
		return shells;
	}
		
	public static void addTopLevelWindowListener(TopLevelWindowListener listener)	{
		topLevelWindowListeners.add(listener);
	}
	
	public static void removeTopLevelWindowListener(TopLevelWindowListener listener)	{
			topLevelWindowListeners.remove(listener);
	}
	
	private static void fireTopLevelWindowEvent(Shell shell, boolean created)	{
		Object[] tLWs = topLevelWindowListeners.toArray();
		for (int i = 0; i<tLWs.length; i++)	{
			TopLevelWindowListener currentListener = (TopLevelWindowListener) tLWs[i];	
			if (created)	{
				currentListener.newTopLevelWindowActivated(shell);
			}
			else {
				currentListener.topLevelWindowDisposed(shell);
			}
		}
	}

	
	public static void addArmListener(ArmListener listener)	{
		armListeners.add(listener);
	}
	
	public static void removeArmListener(ArmListener listener)	{
		armListeners.remove(listener);
	}
	
	public static void addControlListener(ControlListener listener)	{
		controlListeners.add(listener);
	}
	
	public static void removeControlListener(ControlListener listener)	{
		controlListeners.remove(listener);
	}
	
	public static void addFocusListener(FocusListener listener)	{
		focusListeners.add(listener);
	}
	
	public static void removeFocusListener(FocusListener listener)	{
		focusListeners.remove(listener);
	}
	
	public static void addHelpListener(HelpListener listener)	{
		helpListeners.add(listener);
	}
	
	public static void removeHelpListener(HelpListener listener)	{
		helpListeners.remove(listener);
	}
	
	public static void addKeyListener(KeyListener listener)	{
		keyListeners.add(listener);
	}
	
	public static void removeKeyListener(KeyListener listener)	{
		keyListeners.remove(listener);
	}
	
	public static void addMenuListener(MenuListener listener)	{
		menuListeners.add(listener);
	}
	
	public static void removeMenuListener(MenuListener listener)	{
		menuListeners.remove(listener);
	}
	
	public static void addModifyListener(ModifyListener listener)	{
		modifyListeners.add(listener);
	}
	
	public static void removeModifyListener(ModifyListener listener)	{
		modifyListeners.remove(listener);
	}
	
	public static void addMouseListener(MouseListener listener)	{
		mouseListeners.add(listener);
	}
	
	public static void removeMouseListener(MouseListener listener)	{
		mouseListeners.remove(listener);
	}
	
	public static void addMouseMoveListener(MouseMoveListener listener)	{
		mouseMoveListeners.add(listener);
	}
	
	public static void removeMouseMoveListener(MouseMoveListener listener)	{
		mouseMoveListeners.remove(listener);
	}
	
	public static void addMouseTrackListener(MouseTrackListener listener)	{
		mouseTrackListeners.add(listener);
	}
	
	public static void removeMouseTrackListener(MouseTrackListener listener)	{
		mouseTrackListeners.remove(listener);
	}
	
	public static void addPaintListener(PaintListener listener)	{
		paintListeners.add(listener);
	}
	
	public static void removePaintListener(PaintListener listener)	{
		paintListeners.remove(listener);
	}
	
	public static void addSelectionListener(SelectionListener listener)	{
		selectionListeners.add(listener);
	}
	
	public static void removeSelectionListener(SelectionListener listener)	{
		selectionListeners.remove(listener);
	}
	
	public static void addShellListener(ShellListener listener)	{
		shellListeners.add(listener);
	}
	
	public static void removeShellListener(ShellListener listener)	{
		shellListeners.remove(listener);
	}
	
	public static void addTraverseListener(TraverseListener listener)	{
		traverseListeners.add(listener);
	}
	
	public static void removeTraverseListener(TraverseListener listener)	{
		traverseListeners.remove(listener);
	}
	
	public static void addTreeListener(TreeListener listener)	{
		treeListeners.add(listener);
	}
	
	public static void removeTreeListener(TreeListener listener)	{
		treeListeners.remove(listener);
	}
	
	public static void addVerifyListener(VerifyListener listener)	{
		verifyListeners.add(listener);
	}
	
	public static void removeVerifyListener(VerifyListener listener)	{
		verifyListeners.remove(listener);
	}
}

