package sampleapp;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;

import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.SpinnerDateModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

public class SampleApp extends JFrame {

	private JPanel simpleWidgetPanel;
	private JPanel treePanel;
	private JPanel tablePanel;
	private GridBagConstraints gbc;

	public SampleApp() {
		setTitle("Marathon Demo");
		final JTabbedPane pane = new JTabbedPane();
		pane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				String name = pane.getTitleAt(pane.getSelectedIndex());
				setTitle(name);
			}
		});
		pane.setName("tabbedpane");
		pane.addTab("Simple Widgets", createSimpleWidgetPanel());
		pane.addTab("Table Demo", createTablePanel());
		pane.addTab("Tree Demo", createTreePanel());
		JToolBar toolbar = new JToolBar() ;
		toolbar.setFloatable(false);
		URL url = getClass().getClassLoader().getResource("sampleapp/images/save.gif");
		Icon icon = new ImageIcon(url);
		
		JButton saveButton = new JButton (icon);
		saveButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				JOptionPane.showMessageDialog(SampleApp.this, "To save or not to save is the question ?", "Simple Widgets", JOptionPane.QUESTION_MESSAGE);
			}
		});
		saveButton.setName("Save");
		url = getClass().getClassLoader().getResource("sampleapp/images/load.gif");
		icon = new ImageIcon(url);
		JButton loadButton = new JButton (icon);
		loadButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				JOptionPane.showMessageDialog(SampleApp.this, "Open the Pandora's Box", "Load", JOptionPane.INFORMATION_MESSAGE);
			}
		});

		loadButton.setName("Load");
		toolbar.add (saveButton);
		toolbar.add (loadButton);
		getContentPane().add(toolbar, BorderLayout.NORTH);
		getContentPane().add(pane);
		setJMenuBar(createMenuBar());
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		pack();
		show();
	}

	private JMenuBar createMenuBar() {
		JMenuBar bar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");

		JMenuItem item = new JMenuItem("Exit");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}

		});
		
		JMenu test = new JMenu("Test");
		
		JMenuItem testitem = new JMenuItem("Exit1");
		testitem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}

		});
		test.add(testitem);
		fileMenu.add(item);
		fileMenu.add(test);

		bar.add(fileMenu);

		JMenu helpMenu = new JMenu("Help");
		item = new JMenuItem("About");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(SampleApp.this,
						"Marathon Test Demo version 1.0",
						"About Marathon Demo", JOptionPane.INFORMATION_MESSAGE);
			}
		});
		helpMenu.add(item);

		bar.add(helpMenu);

		return bar;

	}

	private JPanel createTablePanel() {
		JPanel panel = new JPanel(new BorderLayout());
		final JTable table = new JTable();
		table.setRowHeight(20);
		table.setName("Table");
		final SampleAppTableModel model = new SampleAppTableModel();
		model.addRow();
		table.setModel(model);
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JButton addButton = new JButton("Add");
		addButton.setName("addButton");
		addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				model.addRow();
			}
		});

		final JButton removeButton = new JButton("Remove");
		removeButton.setEnabled(false);
		removeButton.setName("removeButton");
		removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selectedRow = table.getSelectedRow();
				if (selectedRow != -1)
					model.removeRow(selectedRow);
			}
		});

		table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						if (e.getValueIsAdjusting())
							return;
						int row = table.getSelectedRow();
						removeButton.setEnabled(row != -1);
					}
				});
		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);
		panel.add(new JScrollPane(table));
		panel.add(buttonPanel, BorderLayout.SOUTH);
		return panel;
	}

	private JPanel createSimpleWidgetPanel() {
		
		JPanel topPanel = new JPanel(new BorderLayout());
		
		JPanel panel = new JPanel(new GridBagLayout());
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(3, 3, 3, 3);

		JLabel label = new JLabel("First Name: ");
		addToPanel(0, 0, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		JTextField textField = new JTextField(15);
		label.setLabelFor(textField);
		addToPanel(1, 0, 1.0f, 2, 1, GridBagConstraints.NONE, panel,
				textField);

		label = new JLabel("Password: ");
		addToPanel(0, 1, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		JPasswordField passwordField = new JPasswordField(15);
		label.setLabelFor(passwordField);
		addToPanel(1, 1, 1.0f, 2, 1, GridBagConstraints.NONE, panel,
				passwordField);

		label = new JLabel("Email: ");
		addToPanel(0, 2, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		textField = new JTextField(20);
		textField.setName("email");
		label.setLabelFor(textField);
		addToPanel(1, 2, 1.0f, 2, 1, GridBagConstraints.HORIZONTAL, panel,
				textField);

		label = new JLabel("Address: ");
		addToPanel(0, 3, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		JTextArea area = new JTextArea();
		area.setName("address");
		JScrollPane pane = new JScrollPane(area);
		pane.setPreferredSize(new Dimension(150, 80));
		area.setLineWrap(true);
		area.setWrapStyleWord(true);
		label.setLabelFor(area);
		addToPanel(1, 3, 1.0f, 2, 1, GridBagConstraints.BOTH, panel, pane);

		label = new JLabel("Country: ");
		addToPanel(0, 4, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		String[] country = {"India", "Mexico", "USA", "Pakistan", "Srilanka",
				"France", "England"};
		JComboBox countryBox = new JComboBox(country);
		countryBox.setEditable(true);
		countryBox.setName("country");
		label.setLabelFor(countryBox);
		addToPanel(1, 4, 1.0f, 2, 1, GridBagConstraints.NONE, panel,
				countryBox);

		label = new JLabel("Gender: ");
		addToPanel(0, 5, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);

		JPanel radioPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JRadioButton maleButton = new JRadioButton("Male");
		JRadioButton femaleButton = new JRadioButton("Female");
		maleButton.setSelected(true);
		ButtonGroup grp = new ButtonGroup();
		grp.add(maleButton);
		grp.add(femaleButton);
		radioPanel.add(maleButton);
		radioPanel.add(femaleButton);

		addToPanel(1, 5, 1.0f, 2, 1, GridBagConstraints.NONE, panel,
				radioPanel);

		label = new JLabel("Annual Income ($)");
		addToPanel(0, 6, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		
		String[] income = {"<10K", "Between 10K and 20K", "Between 20K and 30K", "Between 30K and 50K", ">50K"};
		JList list = new JList(income);
		list.addMouseListener( new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount() == 2)
				{
					JOptionPane.showMessageDialog(
							null,
							"Die Liste wurde doppelt angeclickt.",
							"Doppelclick",
							JOptionPane.INFORMATION_MESSAGE,
							null);
				}
			}
		});
		label.setLabelFor(list);
		addToPanel(1, 6, 1.0f, 2, 1, GridBagConstraints.HORIZONTAL, panel,
				new JScrollPane(list));
		label = new JLabel("Languages: ");
		addToPanel(0, 7, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);

		JPanel checkBoxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		String[] languages = {"English", "Hindi", "French", "German", "Urdu"};
		for (int i = 0; i < languages.length; i++) {
			JCheckBox box = new JCheckBox(languages[i]);
			checkBoxPanel.add(box);
		}

		addToPanel(1, 7, 1.0f, 2, 1, GridBagConstraints.HORIZONTAL, panel,
				checkBoxPanel);

		label = new JLabel("Password Expiry: ");
		addToPanel(0, 8, 0.0f, 1, 1, GridBagConstraints.NONE, panel, label);
		JPanel spinnerPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		SpinnerNumberModel model = new SpinnerNumberModel(10, 1, 45, 1);
		JSpinner spinner = new JSpinner(new SpinnerDateModel());
		spinner.setName("passwordexpiry");
		spinnerPanel.add(spinner);
		spinnerPanel.add(new JLabel(" days"));

		addToPanel(1, 8, 1.0f, 2, 1, GridBagConstraints.HORIZONTAL, panel,
				spinnerPanel);

		topPanel.add(panel, BorderLayout.NORTH);
		return topPanel;
	}

	private JPanel createTreePanel() {
		JPanel panel = new JPanel(new BorderLayout());
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Root");

		DefaultMutableTreeNode child = new DefaultMutableTreeNode("Colors");
		child.add(new DefaultMutableTreeNode("Red"));
		child.add(new DefaultMutableTreeNode("Blue"));
		child.add(new DefaultMutableTreeNode("Green"));
		child.add(new DefaultMutableTreeNode("Yellow"));
		child.add(new DefaultMutableTreeNode("Magenta"));
		child.add(new DefaultMutableTreeNode("Cyan"));

		root.add(child);

		child = new DefaultMutableTreeNode("Sports");
		DefaultMutableTreeNode individual = new DefaultMutableTreeNode(
				"Individual Sports");
		individual.add(new DefaultMutableTreeNode("Tennis"));
		individual.add(new DefaultMutableTreeNode("Badminton"));

		DefaultMutableTreeNode team = new DefaultMutableTreeNode("Team Sports");
		team.add(new DefaultMutableTreeNode("Cricket"));
		team.add(new DefaultMutableTreeNode("Hockey"));
		team.add(new DefaultMutableTreeNode("Football"));
		team.add(new DefaultMutableTreeNode("Volley Ball"));
		child.add(individual);
		child.add(team);

		root.add(child);

		JTree tree = new JTree(root);
		final JLabel status = new JLabel("Root");

		tree.setEditable(true);
		tree.setDragEnabled(true);
		tree.getSelectionModel().addTreeSelectionListener(
				new TreeSelectionListener() {
					public void valueChanged(TreeSelectionEvent e) {
						TreePath path = e.getPath();
						Object[] data = path.getPath();
						String statusText = "";
						for (int i = 0; i < data.length - 1; i++) {
							statusText += data[i].toString() + "/";
						}
						statusText += data[data.length - 1].toString();

						status.setText(statusText);
					}

				});
		panel.add(new JScrollPane(tree));

		JPanel statusPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		status.setName("status");
		statusPanel.add(status);

		panel.add(statusPanel, BorderLayout.SOUTH);
		return panel;
	}

	private void addToPanel(int x, int y, float weightx, int gridWidth,
			int gridHeight, int fill, JPanel parent, JComponent comp) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.weightx = weightx;
		gbc.gridwidth = gridWidth;
		gbc.gridheight = gridHeight;
		gbc.fill = fill;
		parent.add(comp, gbc);
	}

	public static void main(String[] args) throws Exception {
		System.out.println("SampleApp started!");
		new SampleApp();
	}

}

