package sampleapp;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

class SampleAppTableModel extends AbstractTableModel{
    
    private List dataList;
    private String[] columnNames = {"Name", "Age", "Marital Status"};
    
    public SampleAppTableModel() {
        dataList = new ArrayList();
    }

    public void removeRow(int selectedRow) {
        dataList.remove(selectedRow);
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return 3;
    }

    public int getRowCount() {
        return dataList.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        SampleData data = (SampleData) dataList.get(rowIndex);
        
        switch(columnIndex) {
        	case 0:
        	    return data.getName();
        	case 1:
        	    return data.getAge();
        	case 2:
        	    return data.isMarried();
        	default:
        	    return "";
        }
    }
    
    
    public String getColumnName(int column) {
        return columnNames[column];
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }
    
    public Class getColumnClass(int columnIndex) {
        switch(columnIndex) {
        	case 0:
        	default:
        	    return String.class;
        	case 1:
        	    return Integer.class;
        	case 2:
        	    return Boolean.class;
        }
    }
    
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        SampleData data = (SampleData) dataList.get(rowIndex);
        switch(columnIndex) {
        	case 0:
        	    data.setName(aValue.toString());
        	    break;
        	case 1:
        	  data.setAge((Integer) aValue);
        	  break;
        	case 2:
        	  data.setMarried((Boolean) aValue);
        	  break;
        }
    }
    
    public void addRow() {
        dataList.add(new SampleData());
        fireTableDataChanged();
    }
    
}

class SampleData {
    private String name;
    private Integer age;
    private Boolean married;
    
    public SampleData() {
        name = "";
        age = new Integer(0);
        married = Boolean.FALSE;
    }
    
    public SampleData(String name, int age, boolean married) {
        this.name = name;
        this.age = new Integer(age);
        this.married = Boolean.valueOf(married);
    }
    
    
    public Integer getAge() {
        return age;
    }
    
    public Boolean isMarried() {
        return married;
    }
    
    public String getName() {
        return name;
    }
    
    public void setAge(Integer age) {
        this.age = age;
    }
    
    public void setMarried(Boolean married) {
        this.married = married;
    }
    
    public void setName(String name) {
        this.name = name;
    }
}
