/*
 * Created on 31.08.2005
 *
 */
package sampleapp.custom;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;


/**
 * @author tegos
 */
public class BaseDateWidget extends Composite 
{

	private class DayWidget extends Composite
	{
		private int dayOfMonth;
		//private Label dayLbl;
			
		public DayWidget(Composite parent, int dayOfMonth) 
		{
			super(parent, SWT.NULL);
			this.dayOfMonth = dayOfMonth;
			setBackground( getDisplay().getSystemColor(SWT.COLOR_WHITE) );
			
			/*dayLbl = new Label(this, SWT.NULL);
			dayLbl.setForeground( getDisplay().getSystemColor(SWT.COLOR_BLACK) );
			dayLbl.setText( "" + dayOfMonth);*/
			
			// nummer des Tages zentriert zeichnen
			addPaintListener(new PaintListener() {
				public void paintControl(PaintEvent e) {
					GC gc = e.gc;
					
					String text = "" + DayWidget.this.dayOfMonth;
					gc.drawString( text, (getClientArea().width - gc.stringExtent(text).x) / 2, 
							(getClientArea().height - gc.stringExtent(text).y) / 2 );
				}
			});
		}
		
		public void setDayOfMonth(int dayOfMonth)
		{
			this.dayOfMonth = dayOfMonth;
			//dayLbl.setText( "" + dayOfMonth);
		}
		
		public int getDayOfMonth() {
			return dayOfMonth;
		}
	}
	
	private static final int defaultYear = 2000;
	private static final int maxDays = 31;
	private Vector modificationListeners = new Vector(2);
	
	private SimpleDateFormat dfWithYear = new SimpleDateFormat("dd.MM.yyyy");
	private SimpleDateFormat dfWithoutYear = new SimpleDateFormat("dd.MM.");
	private boolean withYear;
	private Calendar calCurrentDate;
	
	private static final int MAX_ROWS = 6;
	private static final int MAX_COLS = 7;
	private DayWidget days[][] = new DayWidget[MAX_ROWS][MAX_COLS];
	private DayWidget currentDay = null;
		
	/**
	 * Das Kalenderblatt wird mit den Iniatialwerten angezeigt.
	 * @param date Datum in der Form: 
	 * 	- "dd.MM" fr jahresunabhngige Darstellung
	 *  - "dd.MM.yy" fr jahresabhngige Darstellung
	 */
	public BaseDateWidget(Composite parent, int style, String date) 
	{
		super(parent, style);
		calCurrentDate = new GregorianCalendar();
		
		// layout management
		GridLayout gl = new GridLayout(1, false);
		gl.marginHeight = gl.marginWidth = 0;
		gl.horizontalSpacing = 20;
		setLayout(gl);
		
		createDaysArea();

		// aktueller Tag grau bei focuslost
		addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				currentDay.setBackground( getDisplay().getSystemColor(SWT.COLOR_DARK_BLUE) );
			}

			public void focusLost(FocusEvent e) {
				currentDay.setBackground( getDisplay().getSystemColor(SWT.COLOR_GRAY) );
			} 
		});
		
		setDate(date);
	}
	
	public BaseDateWidget(Composite parent, String date) 
	{
		this(parent, SWT.NULL, date);
	}
	
	/**
	 *  Standardkonstruktor ohne Jahr mit dem Datum 01.01
	 */
	public BaseDateWidget(Composite parent)
	{
		this(parent, "01.01");
	}
	
	/**
	 * @param date
	 */
	public void setDate(String date) 
	{
		try
		{
			calCurrentDate.setTime( dfWithYear.parse(date) );
			withYear = true;
		}
		catch(ParseException exWithYear)
		{
			try
			{
				calCurrentDate.setTime( dfWithoutYear.parse(date) );
				calCurrentDate.set(Calendar.YEAR, defaultYear);
				withYear = false;
			}
			catch(ParseException exWithoutYear)
			{
				calCurrentDate.set(Calendar.YEAR, defaultYear);
				calCurrentDate.set(Calendar.MONTH, Calendar.JANUARY);
				calCurrentDate.set(Calendar.DAY_OF_MONTH, 1);
			}
		}
		
		setVisibleDays();
	}
	
	public void setWithYear(boolean withYear)
	{
		this.withYear = withYear;
		setVisibleDays();
		fireDateWigetEvent();
	}
	
	
	private void setDay(int day)
	{
		calCurrentDate.set(Calendar.DAY_OF_MONTH, day);
		fireDateWigetEvent();
	}
	
	
	/**
	 * @param month 0 basierter Monat
	 */
	public void setMonth(int month)
	{
		Calendar calCopy = new GregorianCalendar();
		calCopy.setTime( calCurrentDate.getTime() );
		calCopy.set( Calendar.DAY_OF_MONTH, 1);
		calCopy.set( Calendar.MONTH, month );
		
		// DATUM korrigieren falls tag ausserhalb des neuen monats
		int day = Math.min(
				calCurrentDate.get(Calendar.DAY_OF_MONTH), calCopy.getActualMaximum(Calendar.DAY_OF_MONTH)
		);
		calCurrentDate.set( Calendar.MONTH, month );
		calCurrentDate.set( Calendar.DAY_OF_MONTH, day);
		
		setVisibleDays();
		fireDateWigetEvent();
	}

	public void setYear(int year)
	{
		withYear = true;
		
		Calendar calCopy = new GregorianCalendar();
		calCopy.setTime( calCurrentDate.getTime() );
		calCopy.set( Calendar.DAY_OF_MONTH, 1);
		calCopy.set(Calendar.YEAR,  year);
		
		int day = Math.min(
				calCurrentDate.get(Calendar.DAY_OF_MONTH), calCopy.getActualMaximum(Calendar.DAY_OF_MONTH)
		);
		
		calCurrentDate.set( Calendar.YEAR, year );
		calCurrentDate.set( Calendar.DAY_OF_MONTH, day);
		
		setVisibleDays();
		fireDateWigetEvent();
	}
	
	private void createDaysArea()
	{
		Composite daysArea = new Composite(this, SWT.NULL);
		daysArea.setLayoutData( new GridData() );
		daysArea.setBackground( getDisplay().getSystemColor(SWT.COLOR_WHITE) );
				
		GridLayout gl = new GridLayout(MAX_COLS, false);
		gl.marginHeight = gl.marginWidth = gl.horizontalSpacing = gl.verticalSpacing = 0;
		daysArea.setLayout(gl);
		
		
		// DayWidgets initialisierten
		for(int i = 0; i < MAX_ROWS; ++i)
		{
			for(int j = 0; j < MAX_COLS; ++j)
			{
				days[i][j] = new DayWidget(daysArea, 1);
				days[i][j].setLayoutData( new GridData(30, 30) );
				// blauen hintergrund bei selektiertem Tag
				days[i][j].addMouseListener(new MouseAdapter() {
					public void mouseDown(MouseEvent e) 
					{
						DayWidget day = (DayWidget) e.widget; 
						setDay( day.getDayOfMonth() );
						setCurrent(day);
					}
				});
			}
		}
	}
	
	/**
	 * Aktualisiert die DayWidgets mit den entsprechenden Tagesnummern und
	 * Positionen im Kalenderblatt.
	 */
	private void setVisibleDays()
	{
		// alle tage erstmal auf nicht visible		
		for(int i = 0; i < MAX_ROWS; ++i)
		{
			for(int j = 0; j < MAX_COLS; ++j)
			{
				days[i][j].setVisible(false);
			}
		}
		
		// die sichtbaren Tage setzen
		Calendar calCopy = new GregorianCalendar();
		calCopy.setTime( calCurrentDate.getTime() );
		int daysPerMonth = calCopy.getActualMaximum(Calendar.DAY_OF_MONTH);
		calCopy.set(Calendar.DAY_OF_MONTH, 1);
		int startIndex = 0;
		if(this.withYear)
		{
			int dayOfWeek = calCopy.get(Calendar.DAY_OF_WEEK);
			//	die Calendar woche beginnt mit Sonntag = 1
			startIndex = dayOfWeek == 1 ? 6 : dayOfWeek - 2;
		}
		int index = startIndex;
		
		for(int i = 0; i < daysPerMonth; ++i)
		{
			int x = index % MAX_COLS;
			int y = index / MAX_COLS;
			
			days[y][x].setVisible(true);
			days[y][x].setDayOfMonth(calCopy.get(Calendar.DAY_OF_MONTH));
			int d1 = calCopy.get(Calendar.DAY_OF_MONTH);
			int d2 = calCurrentDate.get(Calendar.DAY_OF_MONTH);
			
			if( calCopy.get(Calendar.DAY_OF_MONTH) == calCurrentDate.get(Calendar.DAY_OF_MONTH) )
			{	
				setCurrent( days[y][x] );
				DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, Locale.GERMANY);
			}
			calCopy.add(Calendar.DAY_OF_MONTH, 1);
			++index;
		}
	}
	
	/**
	 * Setzt Hintergrund- und Vordergrundfarbe fr das
	 * currentDay Element auf defaultwerte zurck.
	 * Setzt day als neues currentDay Element und
	 * setzt neue Hintergrund- und Vordergrundfarbe fr day.
	 * Benachrichtigt alle IDateWidgetListener.
	 * @param day Das aktuell ausgewhlte Widget
	 */
	private void setCurrent(DayWidget day)
	{
		if(currentDay != null)
		{
			currentDay.setBackground( getDisplay().getSystemColor(SWT.COLOR_WHITE) );
			currentDay.setForeground(  getDisplay().getSystemColor(SWT.COLOR_BLACK) );
		}
		
		currentDay = day;
		currentDay.setBackground( getDisplay().getSystemColor(SWT.COLOR_DARK_BLUE) );
		currentDay.setForeground(  getDisplay().getSystemColor(SWT.COLOR_WHITE) );
		
		fireDateWigetEvent();
	}
	
	private DateFormat getDateFormat()
	{
		return withYear ? dfWithYear : dfWithoutYear;
	}
	
	/**
	 * @return Das aktuell ausgewhlte Datum in der Form dd.MM.[yyyy]
	 */
	public String getDate()
	{
		String ret = null;
		try
		{
			if(withYear)
				ret = dfWithYear.format( calCurrentDate.getTime() );
			else
				ret = dfWithoutYear.format( calCurrentDate.getTime() );
		}
		catch(Exception ex)
		{
			ret = "01.01." + (withYear ? "2000" : "");
		}
		
		return ret;
	}
	
	public int getDay()
	{
		return calCurrentDate.get(Calendar.DAY_OF_MONTH);
	}
	
	/**
	 * @return 0 basierter Monat.
	 */
	public int getMonth()
	{
		return calCurrentDate.get(Calendar.MONTH);
	}
	
	public int getYear()
	{
		return calCurrentDate.get(Calendar.YEAR);
	}
	
	public boolean hasYear() 
	{
		return withYear;
	}
	
	public void addDateWidgetListener(IDateWidgetListener listener) {
		modificationListeners.add(listener);
	}
	
	protected void fireDateWigetEvent() 
	{
		DateWidgetEvent e = new DateWidgetEvent(this);
		Iterator it = modificationListeners.iterator();
		
		while( it.hasNext() )
			( (IDateWidgetListener)it.next() ).onChangeDate(e);
	}

}
