/*
 * Created on 18.03.2005
 */
package sampleapp.custom;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

/**
 * @author Nicos Tegos
 * Diese Klasse soll es ermglichen graphisch aus einer Art Kalender,
 * ein Datum auszuwhlen. Es werden aber nur der Monat 
 * und die verfgbaren Tage angezeigt.
 */
public class DateWidget extends Composite {

	private static final String months[] = new String[] { "Januar", "Februar", "Mrz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember" };
	public static final String dayNames[] = new String[] { "Mo", "Di", "Mi", "Do", "Fr", "Sa", "So" };
	
	private BaseDateWidget base;
	private Combo monthCombo; // listbox fuer die monatsauswahl

	/**
	 * Das Kalenderblatt wird mit den Iniatialwerten angezeigt.
	 * @param dayOfMonth Initialer Tag
	 * @param month Initialer Monat
	 */
	public DateWidget(Composite parent, String date) 
	{
		super(parent, SWT.NULL);
		
		// layout management
		GridLayout gl = new GridLayout(1, false);
		gl.marginHeight = gl.marginWidth = 0;
		gl.verticalSpacing = 10;
		setLayout(gl);
		
		createButtonArea();
		createDaysArea(date);
		
		initialUpdate();
	}
	
	public DateWidget(Composite parent) 
	{	
		this (parent, "01.01");
	}
	
	private void createDaysArea(String date)
	{
		base = new BaseDateWidget(this, SWT.BORDER, date);
	}
	
	private void createButtonArea()
	{
		// MonatsCombo
		monthCombo = new Combo(this, SWT.READ_ONLY);
		monthCombo.setVisibleItemCount(months.length);
		monthCombo.setItems(months);
		monthCombo.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				onMonthCombo();
			}
		});
	}
	
	private void initialUpdate()
	{
		monthCombo.select( base.getMonth() );
	}
	
	private void onMonthCombo() 
	{
		base.setMonth( monthCombo.getSelectionIndex() );
	}
	
	/**
	 * @return Das aktuell ausgewhlte Datum in der Form dd.MM.[yyyy]
	 */
	public String getDate()
	{
		return base.getDate();
	}
	
	public void addDateWidgetListener(IDateWidgetListener listener) {
		base.addDateWidgetListener(listener);
	}
}
