/*
 * Created on 25.08.2005
 */
package sampleapp.custom;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

/**
 * @author Nicos Tegos
 * Diese Klasse soll es ermglichen graphisch aus einer Art Kalender,
 * ein Datum auszuwhlen. Es werden der Monat 
 * und die verfgbaren Tage angezeigt, wahlweise kann man auch das Jahr whlen.
 */
public class DateWidgetWithYear extends Composite 
{
	private static final String months[] = new String[] { "Januar", "Februar", "Mrz", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember" };
	
	private BaseDateWidget base;
	private Combo monthCombo; // listbox fuer die monatsauswahl
	private Combo yearCombo; // listbox fuer die monatsauswahl
	private Button withYearChk; // listbox fuer die monatsauswahl
	private WeekWidget week;
	
	/**
	 * Das Kalenderblatt wird mit den Iniatialwerten angezeigt.
	 * @param dayOfMonth Initialer Tag
	 * @param month Initialer Monat
	 */
	public DateWidgetWithYear(Composite parent, String date) 
	{
		super(parent, SWT.NULL);
		
		// layout management
		GridLayout gl = new GridLayout(1, false);
		gl.verticalSpacing = 10;
		setLayout(gl);
		
		createButtonArea();
		createDaysArea(date);
		
		initialUpdate();
	}
	
	public DateWidgetWithYear(Composite parent) 
	{	
		this (parent, "01.01");
	}
	
	private void createDaysArea(String date)
	{
		Composite daysArea  = new Composite(this, SWT.BORDER);
		GridLayout gl = new GridLayout(1, true);
		gl.marginWidth = gl.marginHeight = gl.horizontalSpacing = gl.verticalSpacing = 0;
		daysArea.setLayout(gl);
		
		week = new WeekWidget(daysArea);
		week.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		base = new BaseDateWidget(daysArea, date);
	}
	
	private void createButtonArea()
	{
		Composite buttonArea = new Composite(this, SWT.NULL);
		buttonArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight =  0;
		gl.horizontalSpacing = 10;
		buttonArea.setLayout(gl);
		
		// MonatsCombo
		monthCombo = new Combo(buttonArea, SWT.READ_ONLY);
		monthCombo.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		monthCombo.setVisibleItemCount(months.length);
		monthCombo.setItems(months);
		monthCombo.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				onMonthCombo();
			}
		});
		
		
		Composite	withYearArea = new Composite(buttonArea, SWT.NULL);
		withYearArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		gl = new GridLayout(3, false);
		gl.marginHeight = gl.marginWidth = 0;
		withYearArea.setLayout(gl);
		
		// JahresCheckbox
		withYearChk = new Button(withYearArea, SWT.CHECK);
		withYearChk.addSelectionListener( new SelectionAdapter() {
			
			public void widgetSelected(SelectionEvent e) {
				onWithYearChk();
			}
		});
		Label lbl = new Label(withYearArea, SWT.NULL);
		lbl.setText("mit Jahr");
		
		// JahresCombo
		yearCombo = new Combo(withYearArea, SWT.READ_ONLY);
		yearCombo.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		yearCombo.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				onYearCombo();
			}
		});
	}
	
	private void initialUpdate()
	{
		withYearChk.setSelection( base.hasYear() );
		monthCombo.select( base.getMonth() );
		week.showNames( base.hasYear() );
		
		int startYear = base.getYear();
		for(int i = startYear; i < startYear + 30; ++i)
			yearCombo.add("" + i);
		
		yearCombo.select(0);
		yearCombo.setEnabled( base.hasYear() );
		
	}
	
	private void onWithYearChk() 
	{
		yearCombo.setEnabled( withYearChk.getSelection() );
		week.showNames( withYearChk.getSelection() );
		base.setWithYear( withYearChk.getSelection() );
	}
	
	private void onMonthCombo() 
	{
		base.setMonth( monthCombo.getSelectionIndex() );
	}
	
	private void onYearCombo()
	{
		base.setYear( Integer.parseInt(yearCombo.getText()) );
	}
	
	/**
	 * @return Das aktuell ausgewhlte Datum in der Form dd.MM.[yyyy]
	 */
	public String getDate()
	{
		return base.getDate();
	}
	
	public void addDateWidgetListener(IDateWidgetListener listener) {
		base.addDateWidgetListener(listener);
	}
	
	Combo getYearCombo() {
		return yearCombo;
	}
	
	Combo getMonthCombo() {
		return monthCombo;
	}
}
